=== modified file '.bzrignore'
--- .bzrignore	2019-08-24 13:40:34 +0000
+++ .bzrignore	2020-07-05 12:50:01 +0000
@@ -76,3 +76,4 @@
 .coverage
 doc/developers/api/*.txt
 __pycache__
+.mypy_cache

=== added file 'CODE_OF_CONDUCT.md'
--- CODE_OF_CONDUCT.md	1970-01-01 00:00:00 +0000
+++ CODE_OF_CONDUCT.md	2020-07-04 00:13:42 +0000
@@ -0,0 +1,76 @@
+# Contributor Covenant Code of Conduct
+
+## Our Pledge
+
+In the interest of fostering an open and welcoming environment, we as
+contributors and maintainers pledge to making participation in our project and
+our community a harassment-free experience for everyone, regardless of age, body
+size, disability, ethnicity, sex characteristics, gender identity and expression,
+level of experience, education, socio-economic status, nationality, personal
+appearance, race, religion, or sexual identity and orientation.
+
+## Our Standards
+
+Examples of behavior that contributes to creating a positive environment
+include:
+
+* Using welcoming and inclusive language
+* Being respectful of differing viewpoints and experiences
+* Gracefully accepting constructive criticism
+* Focusing on what is best for the community
+* Showing empathy towards other community members
+
+Examples of unacceptable behavior by participants include:
+
+* The use of sexualized language or imagery and unwelcome sexual attention or
+ advances
+* Trolling, insulting/derogatory comments, and personal or political attacks
+* Public or private harassment
+* Publishing others' private information, such as a physical or electronic
+ address, without explicit permission
+* Other conduct which could reasonably be considered inappropriate in a
+ professional setting
+
+## Our Responsibilities
+
+Project maintainers are responsible for clarifying the standards of acceptable
+behavior and are expected to take appropriate and fair corrective action in
+response to any instances of unacceptable behavior.
+
+Project maintainers have the right and responsibility to remove, edit, or
+reject comments, commits, code, wiki edits, issues, and other contributions
+that are not aligned to this Code of Conduct, or to ban temporarily or
+permanently any contributor for other behaviors that they deem inappropriate,
+threatening, offensive, or harmful.
+
+## Scope
+
+This Code of Conduct applies both within project spaces and in public spaces
+when an individual is representing the project or its community. Examples of
+representing a project or community include using an official project e-mail
+address, posting via an official social media account, or acting as an appointed
+representative at an online or offline event. Representation of a project may be
+further defined and clarified by project maintainers.
+
+## Enforcement
+
+Instances of abusive, harassing, or otherwise unacceptable behavior may be
+reported by contacting the project team at core@breezy-vcs.org. All
+complaints will be reviewed and investigated and will result in a response that
+is deemed necessary and appropriate to the circumstances. The project team is
+obligated to maintain confidentiality with regard to the reporter of an incident.
+Further details of specific enforcement policies may be posted separately.
+
+Project maintainers who do not follow or enforce the Code of Conduct in good
+faith may face temporary or permanent repercussions as determined by other
+members of the project's leadership.
+
+## Attribution
+
+This Code of Conduct is adapted from the [Contributor Covenant][homepage], version 1.4,
+available at https://www.contributor-covenant.org/version/1/4/code-of-conduct.html
+
+[homepage]: https://www.contributor-covenant.org
+
+For answers to common questions about this code of conduct, see
+https://www.contributor-covenant.org/faq

=== modified file 'breezy/_rio_py.py'
--- breezy/_rio_py.py	2020-02-18 01:57:45 +0000
+++ breezy/_rio_py.py	2020-07-18 23:14:00 +0000
@@ -36,7 +36,7 @@
         for line in line_iter:
             if not isinstance(line, bytes):
                 raise TypeError(line)
-            yield line.decode('utf-8')
+            yield line.decode('utf-8', 'surrogateescape')
     return _read_stanza_unicode(iter_unicode_lines())
 
 

=== modified file 'breezy/_rio_pyx.pyx'
--- breezy/_rio_pyx.pyx	2020-06-08 19:42:53 +0000
+++ breezy/_rio_pyx.pyx	2020-07-18 23:14:00 +0000
@@ -175,7 +175,7 @@
                     raise ValueError("invalid rio tag %r" % (tag,))
         if tag is not None: # add last tag-value
             PyList_Append(pairs,
-                (tag, PyUnicode_DecodeUTF8(accum_value, accum_len-1, "strict")))
+                (tag, PyUnicode_DecodeUTF8(accum_value, accum_len-1, "surrogateescape")))
             return Stanza.from_pairs(pairs)
         else:     # didn't see any content
             return None

=== modified file 'breezy/builtins.py'
--- breezy/builtins.py	2020-06-23 01:02:30 +0000
+++ breezy/builtins.py	2020-07-18 23:14:00 +0000
@@ -2824,23 +2824,23 @@
             else:
                 revision = change
 
-        file_ids = []
+        files = []
         filter_by_dir = False
         if file_list:
             # find the file ids to log and check for directory filtering
             b, file_info_list, rev1, rev2 = _get_info_for_log_files(
                 revision, file_list, self._exit_stack)
-            for relpath, file_id, kind in file_info_list:
-                if file_id is None:
+            for relpath, kind in file_info_list:
+                if not kind:
                     raise errors.CommandError(gettext(
                         "Path unknown at end or start of revision range: %s") %
                         relpath)
                 # If the relpath is the top of the tree, we log everything
                 if relpath == '':
-                    file_ids = []
+                    files = []
                     break
                 else:
-                    file_ids.append(file_id)
+                    files.append(relpath)
                 filter_by_dir = filter_by_dir or (
                     kind in ['directory', 'tree-reference'])
         else:
@@ -2871,7 +2871,7 @@
             delta_type = 'full'
         if not show_diff:
             diff_type = None
-        elif file_ids:
+        elif files:
             diff_type = 'partial'
         else:
             diff_type = 'full'
@@ -2903,7 +2903,7 @@
         # original algorithm - per-file-graph - for the "single
         # file that isn't a directory without showing a delta" case.
         partial_history = revision and b.repository._format.supports_chks
-        match_using_deltas = (len(file_ids) != 1 or filter_by_dir
+        match_using_deltas = (len(files) != 1 or filter_by_dir
                               or delta_type or partial_history)
 
         match_dict = {}
@@ -2919,10 +2919,10 @@
             match_dict['bugs'] = match_bugs
 
         # Build the LogRequest and execute it
-        if len(file_ids) == 0:
-            file_ids = None
+        if len(files) == 0:
+            files = None
         rqst = make_log_request_dict(
-            direction=direction, specific_fileids=file_ids,
+            direction=direction, specific_files=files,
             start_revision=rev1, end_revision=rev2, limit=limit,
             message_search=message, delta_type=delta_type,
             diff_type=diff_type, _match_using_deltas=match_using_deltas,

=== modified file 'breezy/bzr/__init__.py'
--- breezy/bzr/__init__.py	2020-06-23 01:02:30 +0000
+++ breezy/bzr/__init__.py	2020-07-18 23:14:00 +0000
@@ -40,6 +40,12 @@
             format_string = transport.get_bytes(".bzr/branch-format")
         except errors.NoSuchFile:
             raise errors.NotBranchError(path=transport.base)
+        except errors.BadHttpRequest as e:
+            if e.reason == 'no such method: .bzr':
+                # hgweb 
+                raise errors.NotBranchError(path=transport.base)
+            raise
+
         try:
             first_line = format_string[:format_string.index(b"\n") + 1]
         except ValueError:

=== modified file 'breezy/bzr/bundle/bundle_data.py'
--- breezy/bzr/bundle/bundle_data.py	2020-02-18 01:57:45 +0000
+++ breezy/bzr/bundle/bundle_data.py	2020-07-18 23:14:00 +0000
@@ -38,6 +38,7 @@
     InventoryFile,
     InventoryLink,
     )
+from ..inventorytree import InventoryTree
 from ...osutils import sha_string, sha_strings, pathjoin
 from ...revision import Revision, NULL_REVISION
 from ..testament import StrictTestament
@@ -471,7 +472,7 @@
         return None, self.target, 'inapplicable'
 
 
-class BundleTree(Tree):
+class BundleTree(InventoryTree):
 
     def __init__(self, base_tree, revision_id):
         self.base_tree = base_tree

=== modified file 'breezy/bzr/bundle/serializer/v4.py'
--- breezy/bzr/bundle/serializer/v4.py	2020-02-18 01:57:45 +0000
+++ breezy/bzr/bundle/serializer/v4.py	2020-07-18 23:14:00 +0000
@@ -687,7 +687,7 @@
                         delta = target_inv._make_delta(parent_inv)
                         self._repository.add_inventory_by_delta(parent_ids[0],
                                                                 delta, revision_id, parent_ids)
-                except errors.UnsupportedInventoryKind:
+                except serializer.UnsupportedInventoryKind:
                     raise errors.IncompatibleRevision(repr(self._repository))
                 inventory_cache[revision_id] = target_inv
 

=== modified file 'breezy/bzr/bzrdir.py'
--- breezy/bzr/bzrdir.py	2020-06-19 21:26:53 +0000
+++ breezy/bzr/bzrdir.py	2020-07-18 23:14:00 +0000
@@ -430,6 +430,8 @@
                 # actually useful?
                 # Not especially, but it's part of the contract.
                 result_branch = result.create_branch()
+                if revision_id is not None:
+                    result_branch.generate_revision_history(revision_id)
             else:
                 result_branch = source_branch.sprout(
                     result, revision_id=revision_id,

=== modified file 'breezy/bzr/chk_serializer.py'
--- breezy/bzr/chk_serializer.py	2020-02-18 01:57:45 +0000
+++ breezy/bzr/chk_serializer.py	2020-07-18 23:14:00 +0000
@@ -24,6 +24,7 @@
 lazy_import.lazy_import(globals(),
                         """
 from breezy.bzr import (
+    serializer,
     xml_serializer,
     )
 """)
@@ -41,7 +42,7 @@
 def _validate_properties(props, _decode=cache_utf8._utf8_decode):
     # TODO: we really want an 'isascii' check for key
     # Cast the utf8 properties into Unicode 'in place'
-    return {_decode(key)[0]: _decode(value)[0] for key, value in props.items()}
+    return {_decode(key)[0]: _decode(value, 'surrogateescape')[0] for key, value in props.items()}
 
 
 def _is_format_10(value):
@@ -89,7 +90,7 @@
         # which changes infrequently.
         revprops = {}
         for key, value in rev.properties.items():
-            revprops[encode_utf8(key)[0]] = encode_utf8(value)[0]
+            revprops[encode_utf8(key)[0]] = encode_utf8(value, 'surrogateescape')[0]
         ret.append((b'properties', revprops))
         ret.extend([
             (b"timestamp", b"%.3f" % rev.timestamp),
@@ -182,7 +183,7 @@
                 entry_cache=entry_cache,
                 return_from_cache=return_from_cache)
         except xml_serializer.ParseError as e:
-            raise errors.UnexpectedInventoryFormat(e)
+            raise serializer.UnexpectedInventoryFormat(e)
 
     def read_inventory(self, f, revision_id=None):
         """Read an inventory from a file-like object."""
@@ -193,7 +194,7 @@
             finally:
                 f.close()
         except xml_serializer.ParseError as e:
-            raise errors.UnexpectedInventoryFormat(e)
+            raise serializer.UnexpectedInventoryFormat(e)
 
     def write_inventory_to_lines(self, inv):
         """Return a list of lines with the encoded inventory."""

=== modified file 'breezy/bzr/dirstate.py'
--- breezy/bzr/dirstate.py	2020-02-18 01:57:45 +0000
+++ breezy/bzr/dirstate.py	2020-07-18 23:14:00 +0000
@@ -523,7 +523,7 @@
                     file_id_entry[0][0], file_id_entry[0][1])
                 kind = DirState._minikind_to_kind[file_id_entry[1][0][0]]
                 info = '%s:%s' % (kind, path)
-                raise errors.DuplicateFileId(file_id, info)
+                raise inventory.DuplicateFileId(file_id, info)
         first_key = (dirname, basename, b'')
         block_index, present = self._find_block_index_from_key(first_key)
         if present:

=== modified file 'breezy/bzr/groupcompress.py'
--- breezy/bzr/groupcompress.py	2020-02-18 01:57:45 +0000
+++ breezy/bzr/groupcompress.py	2020-07-18 23:14:00 +0000
@@ -49,8 +49,10 @@
     adapter_registry,
     AbsentContentFactory,
     ChunkedContentFactory,
+    ExistingContent,
     FulltextContentFactory,
     VersionedFilesWithFallbacks,
+    UnavailableRepresentation,
     )
 
 # Minimum number of uncompressed bytes to try fetch at once when retrieving
@@ -482,7 +484,7 @@
                 return self._chunks
             else:
                 return osutils.chunks_to_lines(self._chunks)
-        raise errors.UnavailableRepresentation(self.key, storage_kind,
+        raise UnavailableRepresentation(self.key, storage_kind,
                                                self.storage_kind)
 
     def iter_bytes_as(self, storage_kind):
@@ -492,7 +494,7 @@
             return iter(self._chunks)
         elif storage_kind == 'lines':
             return iter(osutils.chunks_to_lines(self._chunks))
-        raise errors.UnavailableRepresentation(self.key, storage_kind,
+        raise UnavailableRepresentation(self.key, storage_kind,
                                                self.storage_kind)
 
 
@@ -863,7 +865,7 @@
         """
         if length == 0:  # empty, like a dir entry, etc
             if nostore_sha == _null_sha1:
-                raise errors.ExistingContent()
+                raise ExistingContent()
             return _null_sha1, 0, 0, 'fulltext'
         # we assume someone knew what they were doing when they passed it in
         if expected_sha is not None:
@@ -872,7 +874,7 @@
             sha1 = osutils.sha_strings(chunks)
         if nostore_sha is not None:
             if sha1 == nostore_sha:
-                raise errors.ExistingContent()
+                raise ExistingContent()
         if key[-1] is None:
             key = key[:-1] + (b'sha1:' + sha1,)
 
@@ -1833,7 +1835,7 @@
                     continue
             try:
                 chunks = record.get_bytes_as('chunked')
-            except errors.UnavailableRepresentation:
+            except UnavailableRepresentation:
                 adapter_key = record.storage_kind, 'chunked'
                 adapter = get_adapter(adapter_key)
                 chunks = adapter.get_bytes(record, 'chunked')

=== modified file 'breezy/bzr/inventory.py'
--- breezy/bzr/inventory.py	2020-02-18 01:57:45 +0000
+++ breezy/bzr/inventory.py	2020-07-18 23:14:00 +0000
@@ -60,6 +60,16 @@
         self.name = name
 
 
+class DuplicateFileId(errors.BzrError):
+
+    _fmt = "File id {%(file_id)s} already exists in inventory as %(entry)s"
+
+    def __init__(self, file_id, entry):
+        errors.BzrError.__init__(self)
+        self.file_id = file_id
+        self.entry = entry
+
+
 class InventoryEntry(object):
     """Description of a versioned file.
 
@@ -1131,7 +1141,7 @@
                 new_entry = replacement
             try:
                 self.add(new_entry)
-            except errors.DuplicateFileId:
+            except DuplicateFileId:
                 raise errors.InconsistentDelta(new_path, new_entry.file_id,
                                                "New id is already present in target.")
             except AttributeError:
@@ -1233,8 +1243,7 @@
         :return: entry
         """
         if entry.file_id in self._byid:
-            raise errors.DuplicateFileId(entry.file_id,
-                                         self._byid[entry.file_id])
+            raise DuplicateFileId(entry.file_id, self._byid[entry.file_id])
         if entry.parent_id is None:
             self.root = entry
         else:

=== modified file 'breezy/bzr/inventorytree.py'
--- breezy/bzr/inventorytree.py	2020-02-18 01:57:45 +0000
+++ breezy/bzr/inventorytree.py	2020-07-18 23:14:00 +0000
@@ -17,9 +17,15 @@
 """Tree classes, representing directory at point in time.
 """
 
+try:
+    from collections.abc import deque
+except ImportError:  # python < 3.7
+    from collections import deque
+
 import os
 import re
 
+
 from .. import (
     branch as _mod_branch,
     debug,
@@ -57,6 +63,7 @@
     MissingNestedTree,
     Tree,
     TreeChange,
+    TreeFile,
     )
 
 
@@ -312,6 +319,10 @@
             vf.fallback_versionedfiles.append(base_vf)
         return last_revision
 
+    def preview_transform(self, pb=None):
+        from .transform import TransformPreview
+        return TransformPreview(self, pb=pb)
+
 
 def find_ids_across_trees(filenames, trees, require_versioned=True):
     """Find the ids corresponding to specified filenames.
@@ -493,9 +504,9 @@
                                          inventory, new_revid)
         self.set_parent_trees([(new_revid, rev_tree)])
 
-    def get_transform(self, pb=None):
-        from ..transform import TreeTransform
-        return TreeTransform(self, pb=pb)
+    def transform(self, pb=None):
+        from .transform import InventoryTreeTransform
+        return InventoryTreeTransform(self, pb=pb)
 
 
 class _SmartAddHelper(object):
@@ -951,7 +962,398 @@
         raise ValueError('not hashable')
 
 
-class InterCHKRevisionTree(InterTree):
+class InterInventoryTree(InterTree):
+    """InterTree implementation for InventoryTree objects.
+
+    """
+    # Formats that will be used to test this InterTree. If both are
+    # None, this InterTree will not be tested (e.g. because a complex
+    # setup is required)
+    _matching_from_tree_format = None
+    _matching_to_tree_format = None
+
+    @classmethod
+    def is_compatible(kls, source, target):
+        # The default implementation is naive and uses the public API, so
+        # it works for all trees.
+        return (isinstance(source, InventoryTree) and
+                isinstance(target, InventoryTree))
+
+    def _changes_from_entries(self, source_entry, target_entry, source_path,
+                              target_path):
+        """Generate a iter_changes tuple between source_entry and target_entry.
+
+        :param source_entry: An inventory entry from self.source, or None.
+        :param target_entry: An inventory entry from self.target, or None.
+        :param source_path: The path of source_entry.
+        :param target_path: The path of target_entry.
+        :return: A tuple, item 0 of which is an iter_changes result tuple, and
+            item 1 is True if there are any changes in the result tuple.
+        """
+        if source_entry is None:
+            if target_entry is None:
+                return None
+            file_id = target_entry.file_id
+        else:
+            file_id = source_entry.file_id
+        if source_entry is not None:
+            source_versioned = True
+            source_name = source_entry.name
+            source_parent = source_entry.parent_id
+            source_kind, source_executable, source_stat = \
+                self.source._comparison_data(source_entry, source_path)
+        else:
+            source_versioned = False
+            source_name = None
+            source_parent = None
+            source_kind = None
+            source_executable = None
+        if target_entry is not None:
+            target_versioned = True
+            target_name = target_entry.name
+            target_parent = target_entry.parent_id
+            target_kind, target_executable, target_stat = \
+                self.target._comparison_data(target_entry, target_path)
+        else:
+            target_versioned = False
+            target_name = None
+            target_parent = None
+            target_kind = None
+            target_executable = None
+        versioned = (source_versioned, target_versioned)
+        kind = (source_kind, target_kind)
+        changed_content = False
+        if source_kind != target_kind:
+            changed_content = True
+        elif source_kind == 'file':
+            if not self.file_content_matches(
+                    source_path, target_path,
+                    source_stat, target_stat):
+                changed_content = True
+        elif source_kind == 'symlink':
+            if (self.source.get_symlink_target(source_path) !=
+                    self.target.get_symlink_target(target_path)):
+                changed_content = True
+        elif source_kind == 'tree-reference':
+            if (self.source.get_reference_revision(source_path)
+                    != self.target.get_reference_revision(target_path)):
+                changed_content = True
+        parent = (source_parent, target_parent)
+        name = (source_name, target_name)
+        executable = (source_executable, target_executable)
+        if (changed_content is not False or versioned[0] != versioned[1] or
+            parent[0] != parent[1] or name[0] != name[1] or
+                executable[0] != executable[1]):
+            changes = True
+        else:
+            changes = False
+        return TreeChange(
+            file_id, (source_path, target_path), changed_content,
+            versioned, parent, name, kind, executable), changes
+
+    def iter_changes(self, include_unchanged=False,
+                     specific_files=None, pb=None, extra_trees=[],
+                     require_versioned=True, want_unversioned=False):
+        """Generate an iterator of changes between trees.
+
+        A tuple is returned:
+        (file_id, (path_in_source, path_in_target),
+         changed_content, versioned, parent, name, kind,
+         executable)
+
+        Changed_content is True if the file's content has changed.  This
+        includes changes to its kind, and to a symlink's target.
+
+        versioned, parent, name, kind, executable are tuples of (from, to).
+        If a file is missing in a tree, its kind is None.
+
+        Iteration is done in parent-to-child order, relative to the target
+        tree.
+
+        There is no guarantee that all paths are in sorted order: the
+        requirement to expand the search due to renames may result in children
+        that should be found early being found late in the search, after
+        lexically later results have been returned.
+        :param require_versioned: Raise errors.PathsNotVersionedError if a
+            path in the specific_files list is not versioned in one of
+            source, target or extra_trees.
+        :param specific_files: An optional list of file paths to restrict the
+            comparison to. When mapping filenames to ids, all matches in all
+            trees (including optional extra_trees) are used, and all children
+            of matched directories are included. The parents in the target tree
+            of the specific files up to and including the root of the tree are
+            always evaluated for changes too.
+        :param want_unversioned: Should unversioned files be returned in the
+            output. An unversioned file is defined as one with (False, False)
+            for the versioned pair.
+        """
+        if not extra_trees:
+            extra_trees = []
+        else:
+            extra_trees = list(extra_trees)
+        # The ids of items we need to examine to insure delta consistency.
+        precise_file_ids = set()
+        changed_file_ids = []
+        if specific_files == []:
+            target_specific_files = []
+            source_specific_files = []
+        else:
+            target_specific_files = self.target.find_related_paths_across_trees(
+                specific_files, [self.source] + extra_trees,
+                require_versioned=require_versioned)
+            source_specific_files = self.source.find_related_paths_across_trees(
+                specific_files, [self.target] + extra_trees,
+                require_versioned=require_versioned)
+        if specific_files is not None:
+            # reparented or added entries must have their parents included
+            # so that valid deltas can be created. The seen_parents set
+            # tracks the parents that we need to have.
+            # The seen_dirs set tracks directory entries we've yielded.
+            # After outputting version object in to_entries we set difference
+            # the two seen sets and start checking parents.
+            seen_parents = set()
+            seen_dirs = set()
+        if want_unversioned:
+            all_unversioned = sorted([(p.split('/'), p) for p in
+                                      self.target.extras()
+                                      if specific_files is None or
+                                      osutils.is_inside_any(specific_files, p)])
+            all_unversioned = deque(all_unversioned)
+        else:
+            all_unversioned = deque()
+        to_paths = {}
+        from_entries_by_dir = list(self.source.iter_entries_by_dir(
+            specific_files=source_specific_files))
+        from_data = dict(from_entries_by_dir)
+        to_entries_by_dir = list(self.target.iter_entries_by_dir(
+            specific_files=target_specific_files))
+        path_equivs = self.find_source_paths([p for p, e in to_entries_by_dir])
+        num_entries = len(from_entries_by_dir) + len(to_entries_by_dir)
+        entry_count = 0
+        # the unversioned path lookup only occurs on real trees - where there
+        # can be extras. So the fake_entry is solely used to look up
+        # executable it values when execute is not supported.
+        fake_entry = TreeFile()
+        for target_path, target_entry in to_entries_by_dir:
+            while (all_unversioned and
+                   all_unversioned[0][0] < target_path.split('/')):
+                unversioned_path = all_unversioned.popleft()
+                target_kind, target_executable, target_stat = \
+                    self.target._comparison_data(
+                        fake_entry, unversioned_path[1])
+                yield TreeChange(
+                    None, (None, unversioned_path[1]), True, (False, False),
+                    (None, None),
+                    (None, unversioned_path[0][-1]),
+                    (None, target_kind),
+                    (None, target_executable))
+            source_path = path_equivs[target_path]
+            if source_path is not None:
+                source_entry = from_data.get(source_path)
+            else:
+                source_entry = None
+            result, changes = self._changes_from_entries(
+                source_entry, target_entry, source_path=source_path, target_path=target_path)
+            to_paths[result.file_id] = result.path[1]
+            entry_count += 1
+            if result.versioned[0]:
+                entry_count += 1
+            if pb is not None:
+                pb.update('comparing files', entry_count, num_entries)
+            if changes or include_unchanged:
+                if specific_files is not None:
+                    precise_file_ids.add(result.parent_id[1])
+                    changed_file_ids.append(result.file_id)
+                yield result
+            # Ensure correct behaviour for reparented/added specific files.
+            if specific_files is not None:
+                # Record output dirs
+                if result.kind[1] == 'directory':
+                    seen_dirs.add(result.file_id)
+                # Record parents of reparented/added entries.
+                if not result.versioned[0] or result.is_reparented():
+                    seen_parents.add(result.parent_id[1])
+        while all_unversioned:
+            # yield any trailing unversioned paths
+            unversioned_path = all_unversioned.popleft()
+            to_kind, to_executable, to_stat = \
+                self.target._comparison_data(fake_entry, unversioned_path[1])
+            yield TreeChange(
+                None, (None, unversioned_path[1]), True, (False, False),
+                (None, None),
+                (None, unversioned_path[0][-1]),
+                (None, to_kind),
+                (None, to_executable))
+        # Yield all remaining source paths
+        for path, from_entry in from_entries_by_dir:
+            file_id = from_entry.file_id
+            if file_id in to_paths:
+                # already returned
+                continue
+            to_path = self.find_target_path(path)
+            entry_count += 1
+            if pb is not None:
+                pb.update('comparing files', entry_count, num_entries)
+            versioned = (True, False)
+            parent = (from_entry.parent_id, None)
+            name = (from_entry.name, None)
+            from_kind, from_executable, stat_value = \
+                self.source._comparison_data(from_entry, path)
+            kind = (from_kind, None)
+            executable = (from_executable, None)
+            changed_content = from_kind is not None
+            # the parent's path is necessarily known at this point.
+            changed_file_ids.append(file_id)
+            yield TreeChange(
+                file_id, (path, to_path), changed_content, versioned, parent,
+                name, kind, executable)
+        changed_file_ids = set(changed_file_ids)
+        if specific_files is not None:
+            for result in self._handle_precise_ids(precise_file_ids,
+                                                   changed_file_ids):
+                yield result
+
+    @staticmethod
+    def _get_entry(tree, path):
+        """Get an inventory entry from a tree, with missing entries as None.
+
+        If the tree raises NotImplementedError on accessing .inventory, then
+        this is worked around using iter_entries_by_dir on just the file id
+        desired.
+
+        :param tree: The tree to lookup the entry in.
+        :param path: The path to look up
+        """
+        # No inventory available.
+        try:
+            iterator = tree.iter_entries_by_dir(specific_files=[path])
+            return next(iterator)[1]
+        except StopIteration:
+            return None
+
+    def _handle_precise_ids(self, precise_file_ids, changed_file_ids,
+                            discarded_changes=None):
+        """Fill out a partial iter_changes to be consistent.
+
+        :param precise_file_ids: The file ids of parents that were seen during
+            the iter_changes.
+        :param changed_file_ids: The file ids of already emitted items.
+        :param discarded_changes: An optional dict of precalculated
+            iter_changes items which the partial iter_changes had not output
+            but had calculated.
+        :return: A generator of iter_changes items to output.
+        """
+        # process parents of things that had changed under the users
+        # requested paths to prevent incorrect paths or parent ids which
+        # aren't in the tree.
+        while precise_file_ids:
+            precise_file_ids.discard(None)
+            # Don't emit file_ids twice
+            precise_file_ids.difference_update(changed_file_ids)
+            if not precise_file_ids:
+                break
+            # If the there was something at a given output path in source, we
+            # have to include the entry from source in the delta, or we would
+            # be putting this entry into a used path.
+            paths = []
+            for parent_id in precise_file_ids:
+                try:
+                    paths.append(self.target.id2path(parent_id))
+                except errors.NoSuchId:
+                    # This id has been dragged in from the source by delta
+                    # expansion and isn't present in target at all: we don't
+                    # need to check for path collisions on it.
+                    pass
+            for path in paths:
+                old_id = self.source.path2id(path)
+                precise_file_ids.add(old_id)
+            precise_file_ids.discard(None)
+            current_ids = precise_file_ids
+            precise_file_ids = set()
+            # We have to emit all of precise_file_ids that have been altered.
+            # We may have to output the children of some of those ids if any
+            # directories have stopped being directories.
+            for file_id in current_ids:
+                # Examine file_id
+                if discarded_changes:
+                    result = discarded_changes.get(file_id)
+                    source_entry = None
+                else:
+                    result = None
+                if result is None:
+                    try:
+                        source_path = self.source.id2path(file_id)
+                    except errors.NoSuchId:
+                        source_path = None
+                        source_entry = None
+                    else:
+                        source_entry = self._get_entry(
+                            self.source, source_path)
+                    try:
+                        target_path = self.target.id2path(file_id)
+                    except errors.NoSuchId:
+                        target_path = None
+                        target_entry = None
+                    else:
+                        target_entry = self._get_entry(
+                            self.target, target_path)
+                    result, changes = self._changes_from_entries(
+                        source_entry, target_entry, source_path, target_path)
+                else:
+                    changes = True
+                # Get this parents parent to examine.
+                new_parent_id = result.parent_id[1]
+                precise_file_ids.add(new_parent_id)
+                if changes:
+                    if (result.kind[0] == 'directory' and
+                            result.kind[1] != 'directory'):
+                        # This stopped being a directory, the old children have
+                        # to be included.
+                        if source_entry is None:
+                            # Reusing a discarded change.
+                            source_entry = self._get_entry(
+                                self.source, result.path[0])
+                        precise_file_ids.update(
+                            child.file_id
+                            for child in self.source.iter_child_entries(result.path[0]))
+                    changed_file_ids.add(result.file_id)
+                    yield result
+
+    def find_target_path(self, path, recurse='none'):
+        """Find target tree path.
+
+        :param path: Path to search for (exists in source)
+        :return: path in target, or None if there is no equivalent path.
+        :raise NoSuchFile: If the path doesn't exist in source
+        """
+        file_id = self.source.path2id(path)
+        if file_id is None:
+            raise errors.NoSuchFile(path)
+        try:
+            return self.target.id2path(file_id, recurse=recurse)
+        except errors.NoSuchId:
+            return None
+
+    def find_source_path(self, path, recurse='none'):
+        """Find the source tree path.
+
+        :param path: Path to search for (exists in target)
+        :return: path in source, or None if there is no equivalent path.
+        :raise NoSuchFile: if the path doesn't exist in target
+        """
+        file_id = self.target.path2id(path)
+        if file_id is None:
+            raise errors.NoSuchFile(path)
+        try:
+            return self.source.id2path(file_id, recurse=recurse)
+        except errors.NoSuchId:
+            return None
+
+
+InterTree.register_optimiser(InterInventoryTree)
+
+
+class InterCHKRevisionTree(InterInventoryTree):
     """Fast path optimiser for RevisionTrees with CHK inventories."""
 
     @staticmethod

=== modified file 'breezy/bzr/knit.py'
--- breezy/bzr/knit.py	2020-02-18 01:57:45 +0000
+++ breezy/bzr/knit.py	2020-07-18 23:14:00 +0000
@@ -100,7 +100,9 @@
     adapter_registry,
     ConstantMapper,
     ContentFactory,
+    ExistingContent,
     sort_groupcompress,
+    UnavailableRepresentation,
     VersionedFilesWithFallbacks,
     )
 
@@ -218,7 +220,7 @@
 
     def get_bytes(self, factory, target_storage_kind):
         if target_storage_kind != 'knit-ft-gz':
-            raise errors.UnavailableRepresentation(
+            raise UnavailableRepresentation(
                 factory.key, target_storage_kind, factory.storage_kind)
         annotated_compressed_bytes = factory._raw_record
         rec, contents = \
@@ -234,7 +236,7 @@
 
     def get_bytes(self, factory, target_storage_kind):
         if target_storage_kind != 'knit-delta-gz':
-            raise errors.UnavailableRepresentation(
+            raise UnavailableRepresentation(
                 factory.key, target_storage_kind, factory.storage_kind)
         annotated_compressed_bytes = factory._raw_record
         rec, contents = \
@@ -259,7 +261,7 @@
             return b''.join(content.text())
         elif target_storage_kind in ('chunked', 'lines'):
             return content.text()
-        raise errors.UnavailableRepresentation(
+        raise UnavailableRepresentation(
             factory.key, target_storage_kind, factory.storage_kind)
 
 
@@ -288,7 +290,7 @@
             return b''.join(basis_content.text())
         elif target_storage_kind in ('chunked', 'lines'):
             return basis_content.text()
-        raise errors.UnavailableRepresentation(
+        raise UnavailableRepresentation(
             factory.key, target_storage_kind, factory.storage_kind)
 
 
@@ -305,7 +307,7 @@
             return b''.join(content.text())
         elif target_storage_kind in ('chunked', 'lines'):
             return content.text()
-        raise errors.UnavailableRepresentation(
+        raise UnavailableRepresentation(
             factory.key, target_storage_kind, factory.storage_kind)
 
 
@@ -333,7 +335,7 @@
             return b''.join(content.text())
         elif target_storage_kind in ('chunked', 'lines'):
             return content.text()
-        raise errors.UnavailableRepresentation(
+        raise UnavailableRepresentation(
             factory.key, target_storage_kind, factory.storage_kind)
 
 
@@ -411,7 +413,7 @@
                 return self._knit.get_lines(self.key[0])
             elif storage_kind == 'fulltext':
                 return self._knit.get_text(self.key[0])
-        raise errors.UnavailableRepresentation(self.key, storage_kind,
+        raise UnavailableRepresentation(self.key, storage_kind,
                                                self.storage_kind)
 
     def iter_bytes_as(self, storage_kind):
@@ -459,7 +461,7 @@
                 return chunks
             else:
                 return b''.join(chunks)
-        raise errors.UnavailableRepresentation(self.key, storage_kind,
+        raise UnavailableRepresentation(self.key, storage_kind,
                                                self.storage_kind)
 
     def iter_bytes_as(self, storage_kind):
@@ -1061,7 +1063,7 @@
         # that out.
         digest = sha_string(line_bytes)
         if nostore_sha == digest:
-            raise errors.ExistingContent
+            raise ExistingContent
 
         present_parents = []
         if parent_texts is None:
@@ -1786,7 +1788,7 @@
                 try:
                     # Try getting a fulltext directly from the record.
                     lines = record.get_bytes_as('lines')
-                except errors.UnavailableRepresentation:
+                except UnavailableRepresentation:
                     adapter_key = record.storage_kind, 'lines'
                     adapter = get_adapter(adapter_key)
                     lines = adapter.get_bytes(record, 'lines')

=== modified file 'breezy/bzr/knitrepo.py'
--- breezy/bzr/knitrepo.py	2020-02-18 01:57:45 +0000
+++ breezy/bzr/knitrepo.py	2020-07-18 23:14:00 +0000
@@ -187,7 +187,6 @@
 
     def get_revision(self, revision_id):
         """Return the Revision object for a named revision"""
-        revision_id = osutils.safe_revision_id(revision_id)
         with self.lock_read():
             return self.get_revision_reconcile(revision_id)
 

=== modified file 'breezy/bzr/remote.py'
--- breezy/bzr/remote.py	2020-05-06 02:56:16 +0000
+++ breezy/bzr/remote.py	2020-07-18 23:14:00 +0000
@@ -2267,9 +2267,9 @@
         identifiers = []
         for (file_id, revid, identifier) in desired_files:
             lines.append(b''.join([
-                osutils.safe_file_id(file_id),
+                file_id,
                 b'\0',
-                osutils.safe_revision_id(revid)]))
+                revid]))
             identifiers.append(identifier)
         (response_tuple, response_handler) = (
             self._call_with_body_bytes_expecting_body(
@@ -2557,47 +2557,10 @@
             filtered_inv = inv.filter(file_ids)
             yield InventoryRevisionTree(self, filtered_inv, filtered_inv.revision_id)
 
-    def get_deltas_for_revisions(self, revisions, specific_fileids=None):
-        with self.lock_read():
-            medium = self._client._medium
-            if medium._is_remote_before((1, 2)):
-                self._ensure_real()
-                for delta in self._real_repository.get_deltas_for_revisions(
-                        revisions, specific_fileids):
-                    yield delta
-                return
-            # Get the revision-ids of interest
-            required_trees = set()
-            for revision in revisions:
-                required_trees.add(revision.revision_id)
-                required_trees.update(revision.parent_ids[:1])
-
-            # Get the matching filtered trees. Note that it's more
-            # efficient to pass filtered trees to changes_from() rather
-            # than doing the filtering afterwards. changes_from() could
-            # arguably do the filtering itself but it's path-based, not
-            # file-id based, so filtering before or afterwards is
-            # currently easier.
-            if specific_fileids is None:
-                trees = dict((t.get_revision_id(), t) for
-                             t in self.revision_trees(required_trees))
-            else:
-                trees = dict((t.get_revision_id(), t) for
-                             t in self._filtered_revision_trees(required_trees,
-                                                                specific_fileids))
-
-            # Calculate the deltas
-            for revision in revisions:
-                if not revision.parent_ids:
-                    old_tree = self.revision_tree(_mod_revision.NULL_REVISION)
-                else:
-                    old_tree = trees[revision.parent_ids[0]]
-                yield trees[revision.revision_id].changes_from(old_tree)
-
     def get_revision_delta(self, revision_id):
         with self.lock_read():
             r = self.get_revision(revision_id)
-            return list(self.get_deltas_for_revisions([r]))[0]
+            return list(self.get_revision_deltas([r]))[0]
 
     def revision_trees(self, revision_ids):
         with self.lock_read():
@@ -4523,6 +4486,9 @@
                            lambda err, find, get_path: errors.UnresumableWriteGroup(
                                repository=find('repository'), write_groups=err.error_args[0],
                                reason=err.error_args[1]))
+error_translators.register(b'AlreadyControlDir',
+                           lambda err, find, get_path: errors.AlreadyControlDirError(get_path()))
+
 no_context_error_translators.register(b'GhostRevisionsHaveNoRevno',
                                       lambda err: errors.GhostRevisionsHaveNoRevno(*err.error_args))
 no_context_error_translators.register(b'IncompatibleRepositories',

=== modified file 'breezy/bzr/serializer.py'
--- breezy/bzr/serializer.py	2020-02-18 01:57:45 +0000
+++ breezy/bzr/serializer.py	2020-07-18 23:14:00 +0000
@@ -16,7 +16,28 @@
 
 """Inventory/revision serialization."""
 
-from .. import registry
+from .. import errors, registry
+
+
+class BadInventoryFormat(errors.BzrError):
+
+    _fmt = "Root class for inventory serialization errors"
+
+
+class UnexpectedInventoryFormat(BadInventoryFormat):
+
+    _fmt = "The inventory was not in the expected format:\n %(msg)s"
+
+    def __init__(self, msg):
+        BadInventoryFormat.__init__(self, msg=msg)
+
+
+class UnsupportedInventoryKind(errors.BzrError):
+
+    _fmt = """Unsupported entry kind %(kind)s"""
+
+    def __init__(self, kind):
+        self.kind = kind
 
 
 class Serializer(object):

=== modified file 'breezy/bzr/smart/request.py'
--- breezy/bzr/smart/request.py	2020-02-18 01:57:45 +0000
+++ breezy/bzr/smart/request.py	2020-07-18 23:14:00 +0000
@@ -472,6 +472,8 @@
     elif isinstance(err, MemoryError):
         # GZ 2011-02-24: Copy breezy.trace -Dmem_dump functionality here?
         return (b'MemoryError',)
+    elif isinstance(err, errors.AlreadyControlDirError):
+        return (b'AlreadyControlDir', err.path)
     # Unserialisable error.  Log it, and return a generic error
     trace.log_exception_quietly()
     return (b'error',

=== modified file 'breezy/bzr/tests/__init__.py'
--- breezy/bzr/tests/__init__.py	2020-06-17 17:41:09 +0000
+++ breezy/bzr/tests/__init__.py	2020-06-30 00:19:29 +0000
@@ -81,9 +81,11 @@
         'test_serializer',
         'test_tag',
         'test_testament',
+        'test_transform',
         'test_versionedfile',
         'test_vf_search',
         'test_vfs_ratchet',
+        'test_workingtree',
         'test_workingtree_4',
         'test_weave',
         'test_xml',

=== modified file 'breezy/bzr/tests/per_bzrdir/test_bzrdir.py'
--- breezy/bzr/tests/per_bzrdir/test_bzrdir.py	2020-06-10 03:30:11 +0000
+++ breezy/bzr/tests/per_bzrdir/test_bzrdir.py	2020-07-18 19:28:51 +0000
@@ -516,6 +516,23 @@
         self.assertRepositoryHasSameItems(
             tree.branch.repository, target.open_repository())
 
+    def test_sprout_branch_no_source_branch(self):
+        try:
+            repo = self.make_repository('source', shared=True)
+        except errors.IncompatibleFormat:
+            return
+        if isinstance(self.bzrdir_format, RemoteBzrDirFormat):
+            self.skipTest('remote formats not supported')
+        branch = controldir.ControlDir.create_branch_convenience('source/trunk')
+        tree = branch.controldir.open_workingtree()
+        self.build_tree(['source/trunk/foo'])
+        tree.add('foo')
+        tree.commit('revision 1')
+        rev2 = tree.commit('revision 2', allow_pointless=True)
+        target = self.sproutOrSkip(
+            repo.controldir, self.get_url('target'), revision_id=rev2)
+        self.assertEqual([rev2], target.open_workingtree().get_parent_ids())
+
     def test_retire_bzrdir(self):
         bd = self.make_controldir('.')
         transport = bd.root_transport

=== modified file 'breezy/bzr/tests/per_repository_vf/test_fileid_involved.py'
--- breezy/bzr/tests/per_repository_vf/test_fileid_involved.py	2020-06-10 02:56:53 +0000
+++ breezy/bzr/tests/per_repository_vf/test_fileid_involved.py	2020-07-04 12:29:00 +0000
@@ -430,6 +430,6 @@
     os.chmod() doesn't work on windows. But TreeTransform can mark or
     unmark a file as executable.
     """
-    with wt.get_transform() as tt:
+    with wt.transform() as tt:
         tt.set_executability(executable, tt.trans_id_tree_path(path))
         tt.apply()

=== modified file 'breezy/bzr/tests/per_repository_vf/test_repository.py'
--- breezy/bzr/tests/per_repository_vf/test_repository.py	2020-06-10 02:56:53 +0000
+++ breezy/bzr/tests/per_repository_vf/test_repository.py	2020-06-28 19:19:25 +0000
@@ -357,15 +357,15 @@
         for tree1, tree2 in zip(trees1, trees2):
             self.assertFalse(tree2.changes_from(tree1).has_changed())
 
-    def test_get_deltas_for_revisions(self):
+    def test_get_revision_deltas(self):
         repository = self.controldir.open_repository()
         repository.lock_read()
         self.addCleanup(repository.unlock)
         revisions = [repository.get_revision(r) for r in
                      [b'rev1', b'rev2', b'rev3', b'rev4']]
-        deltas1 = list(repository.get_deltas_for_revisions(revisions))
-        deltas2 = [repository.get_revision_delta(r.revision_id) for r in
-                   revisions]
+        deltas1 = list(repository.get_revision_deltas(revisions))
+        deltas2 = [repository.get_revision_delta(r.revision_id)
+                   for r in revisions]
         self.assertEqual(deltas1, deltas2)
 
     def test_all_revision_ids(self):

=== modified file 'breezy/bzr/tests/per_versionedfile.py'
--- breezy/bzr/tests/per_versionedfile.py	2020-06-19 21:26:53 +0000
+++ breezy/bzr/tests/per_versionedfile.py	2020-07-18 23:14:00 +0000
@@ -58,8 +58,10 @@
 from ..versionedfile import (
     ChunkedContentFactory,
     ConstantMapper,
+    ExistingContent,
     HashEscapedPrefixMapper,
     PrefixMapper,
+    UnavailableRepresentation,
     VirtualVersionedFiles,
     make_versioned_files_factory,
     )
@@ -303,7 +305,7 @@
         # we now have a copy of all the lines in the vf.
         for sha, (version, lines) in zip(
                 shas, (empty_text, sample_text_nl, sample_text_no_nl)):
-            self.assertRaises(errors.ExistingContent,
+            self.assertRaises(ExistingContent,
                               vf.add_lines, version + b"2", [], lines,
                               nostore_sha=sha)
             # and no new version should have been added.
@@ -328,7 +330,7 @@
             raise TestSkipped("add_lines_with_ghosts is optional")
         for sha, (version, lines) in zip(
                 shas, (empty_text, sample_text_nl, sample_text_no_nl)):
-            self.assertRaises(errors.ExistingContent,
+            self.assertRaises(ExistingContent,
                               vf.add_lines_with_ghosts, version + b"2", [], lines,
                               nostore_sha=sha)
             # and no new version should have been added.
@@ -1644,10 +1646,10 @@
         for sha, (version, lines) in zip(
                 shas, (empty_text, sample_text_nl, sample_text_no_nl)):
             new_key = self.get_simple_key(version + b"2")
-            self.assertRaises(errors.ExistingContent,
+            self.assertRaises(ExistingContent,
                               vf.add_lines, new_key, [], lines,
                               nostore_sha=sha)
-            self.assertRaises(errors.ExistingContent,
+            self.assertRaises(ExistingContent,
                               vf.add_lines, new_key, [], lines,
                               nostore_sha=sha)
             # and no new version should have been added.
@@ -1982,7 +1984,7 @@
                                  factory.sha1)
             self.assertEqual(parent_map[factory.key], factory.parents)
             # currently no stream emits mpdiff
-            self.assertRaises(errors.UnavailableRepresentation,
+            self.assertRaises(UnavailableRepresentation,
                               factory.get_bytes_as, 'mpdiff')
             self.assertIsInstance(factory.get_bytes_as(factory.storage_kind),
                                   bytes)

=== modified file 'breezy/bzr/tests/test_bundle.py'
--- breezy/bzr/tests/test_bundle.py	2020-06-17 19:04:08 +0000
+++ breezy/bzr/tests/test_bundle.py	2020-07-04 12:29:00 +0000
@@ -39,6 +39,7 @@
 from ..bundle.serializer.v09 import BundleSerializerV09
 from ..bundle.serializer.v4 import BundleSerializerV4
 from ..import knitrepo
+from ..inventorytree import InventoryTree
 from ...tests import (
     features,
     test_commit,
@@ -59,7 +60,7 @@
         return get_text(repo.inventories, (revision_id,))
 
 
-class MockTree(object):
+class MockTree(InventoryTree):
 
     def __init__(self):
         from ..inventory import InventoryDirectory, ROOT_ID
@@ -592,7 +593,7 @@
             ])
         self.build_tree_contents([('b1/sub/sub/emptyfile.txt', b''),
                                   ('b1/dir/nolastnewline.txt', b'bloop')])
-        tt = self.tree1.get_transform()
+        tt = self.tree1.transform()
         tt.new_file('executable', tt.root, [b'#!/bin/sh\n'], b'exe-1', True)
         tt.apply()
         # have to fix length of file-id so that we can predictably rewrite
@@ -612,7 +613,7 @@
         self.tree1.remove(
             ['sub/sub/nonempty.txt', 'sub/sub/emptyfile.txt', 'sub/sub'
              ])
-        tt = self.tree1.get_transform()
+        tt = self.tree1.transform()
         trans_id = tt.trans_id_tree_path('executable')
         tt.set_executability(False, trans_id)
         tt.apply()
@@ -673,7 +674,7 @@
         self.tree1 = self.make_branch_and_tree('b1')
         self.b1 = self.tree1.branch
 
-        tt = self.tree1.get_transform()
+        tt = self.tree1.transform()
         tt.new_symlink(link_name, tt.root, link_target, link_id)
         tt.apply()
         self.tree1.commit('add symlink', rev_id=b'l@cset-0-1')
@@ -684,7 +685,7 @@
             self.assertEqual(
                 link_target, bund_tree.get_symlink_target(link_name))
 
-        tt = self.tree1.get_transform()
+        tt = self.tree1.transform()
         trans_id = tt.trans_id_tree_path(link_name)
         tt.adjust_path('link2', tt.root, trans_id)
         tt.delete_contents(trans_id)
@@ -698,7 +699,7 @@
             self.assertEqual(new_link_target,
                              bund_tree.get_symlink_target('link2'))
 
-        tt = self.tree1.get_transform()
+        tt = self.tree1.transform()
         trans_id = tt.trans_id_tree_path('link2')
         tt.delete_contents(trans_id)
         tt.create_symlink('jupiter', trans_id)
@@ -706,7 +707,7 @@
         self.tree1.commit('just change symlink target', rev_id=b'l@cset-0-3')
         bundle = self.get_valid_bundle(b'l@cset-0-2', b'l@cset-0-3')
 
-        tt = self.tree1.get_transform()
+        tt = self.tree1.transform()
         trans_id = tt.trans_id_tree_path('link2')
         tt.delete_contents(trans_id)
         tt.apply()
@@ -725,7 +726,7 @@
     def test_binary_bundle(self):
         self.tree1 = self.make_branch_and_tree('b1')
         self.b1 = self.tree1.branch
-        tt = self.tree1.get_transform()
+        tt = self.tree1.transform()
 
         # Add
         tt.new_file('file', tt.root, [
@@ -737,7 +738,7 @@
         self.get_valid_bundle(b'null:', b'b@cset-0-1')
 
         # Delete
-        tt = self.tree1.get_transform()
+        tt = self.tree1.transform()
         trans_id = tt.trans_id_tree_path('file')
         tt.delete_contents(trans_id)
         tt.apply()
@@ -745,7 +746,7 @@
         self.get_valid_bundle(b'b@cset-0-1', b'b@cset-0-2')
 
         # Rename & modify
-        tt = self.tree1.get_transform()
+        tt = self.tree1.transform()
         trans_id = tt.trans_id_tree_path('file2')
         tt.adjust_path('file3', tt.root, trans_id)
         tt.delete_contents(trans_id)
@@ -755,7 +756,7 @@
         self.get_valid_bundle(b'b@cset-0-2', b'b@cset-0-3')
 
         # Modify
-        tt = self.tree1.get_transform()
+        tt = self.tree1.transform()
         trans_id = tt.trans_id_tree_path('file3')
         tt.delete_contents(trans_id)
         tt.create_file([b'\x00file\rcontents'], trans_id)
@@ -769,12 +770,12 @@
     def test_last_modified(self):
         self.tree1 = self.make_branch_and_tree('b1')
         self.b1 = self.tree1.branch
-        tt = self.tree1.get_transform()
+        tt = self.tree1.transform()
         tt.new_file('file', tt.root, [b'file'], b'file')
         tt.apply()
         self.tree1.commit('create file', rev_id=b'a@lmod-0-1')
 
-        tt = self.tree1.get_transform()
+        tt = self.tree1.transform()
         trans_id = tt.trans_id_tree_path('file')
         tt.delete_contents(trans_id)
         tt.create_file([b'file2'], trans_id)
@@ -782,7 +783,7 @@
         self.tree1.commit('modify text', rev_id=b'a@lmod-0-2a')
 
         other = self.get_checkout(b'a@lmod-0-1')
-        tt = other.get_transform()
+        tt = other.transform()
         trans_id = tt.trans_id_tree_path('file2')
         tt.delete_contents(trans_id)
         tt.create_file([b'file2'], trans_id)

=== modified file 'breezy/bzr/tests/test_inv.py'
--- breezy/bzr/tests/test_inv.py	2020-06-11 01:25:02 +0000
+++ breezy/bzr/tests/test_inv.py	2020-07-18 23:14:00 +0000
@@ -30,6 +30,7 @@
     )
 from ..inventory import (
     CHKInventory,
+    DuplicateFileId,
     Inventory,
     ROOT_ID,
     InventoryFile,
@@ -1601,3 +1602,12 @@
         inv.get_entry(b'thefileid').executable = True
         self.assertFalse(tree.root_inventory.get_entry(
             b'thefileid').executable)
+
+
+class ErrorTests(TestCase):
+
+    def test_duplicate_file_id(self):
+        error = DuplicateFileId('a_file_id', 'foo')
+        self.assertEqualDiff(
+            'File id {a_file_id} already exists in inventory as foo',
+            str(error))

=== modified file 'breezy/bzr/tests/test_testament.py'
--- breezy/bzr/tests/test_testament.py	2020-06-10 21:13:00 +0000
+++ breezy/bzr/tests/test_testament.py	2020-07-04 12:29:00 +0000
@@ -50,7 +50,7 @@
                                   ('src/foo.c', b'int main()\n{\n}\n')])
         self.wt.add(['hello', 'src', 'src/foo.c'],
                     [b'hello-id', b'src-id', b'foo.c-id'])
-        tt = self.wt.get_transform()
+        tt = self.wt.transform()
         trans_id = tt.trans_id_tree_path('hello')
         tt.set_executability(True, trans_id)
         tt.apply()

=== added file 'breezy/bzr/tests/test_transform.py'
--- breezy/bzr/tests/test_transform.py	1970-01-01 00:00:00 +0000
+++ breezy/bzr/tests/test_transform.py	2020-07-03 00:43:31 +0000
@@ -0,0 +1,49 @@
+# Copyright (C) 2006-2012, 2016 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+from . import TestCaseWithTransport
+
+
+class TestInventoryAltered(TestCaseWithTransport):
+
+    def test_inventory_altered_unchanged(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/foo'])
+        tree.add('foo', b'foo-id')
+        with tree.preview_transform() as tt:
+            self.assertEqual([], tt._inventory_altered())
+
+    def test_inventory_altered_changed_parent_id(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/foo'])
+        tree.add('foo', b'foo-id')
+        with tree.preview_transform() as tt:
+            tt.unversion_file(tt.root)
+            tt.version_file(tt.root, file_id=b'new-id')
+            foo_trans_id = tt.trans_id_tree_path('foo')
+            foo_tuple = ('foo', foo_trans_id)
+            root_tuple = ('', tt.root)
+            self.assertEqual([root_tuple, foo_tuple], tt._inventory_altered())
+
+    def test_inventory_altered_noop_changed_parent_id(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/foo'])
+        tree.add('foo', b'foo-id')
+        with tree.preview_transform() as tt:
+            tt.unversion_file(tt.root)
+            tt.version_file(tt.root, file_id=tree.path2id(''))
+            tt.trans_id_tree_path('foo')
+            self.assertEqual([], tt._inventory_altered())

=== modified file 'breezy/bzr/tests/test_versionedfile.py'
--- breezy/bzr/tests/test_versionedfile.py	2020-06-10 02:56:53 +0000
+++ breezy/bzr/tests/test_versionedfile.py	2020-06-25 21:13:23 +0000
@@ -137,3 +137,13 @@
             multiparent.MultiParent([multiparent.NewText([b'first\n'])]),
             ]
         self.assertEqual(expected_diffs, diffs)
+
+
+class ErrorTests(tests.TestCase):
+
+    def test_unavailable_representation(self):
+        error = versionedfile.UnavailableRepresentation(
+            ('key',), "mpdiff", "fulltext")
+        self.assertEqualDiff("The encoding 'mpdiff' is not available for key "
+                             "('key',) which is encoded as 'fulltext'.",
+                             str(error))

=== added file 'breezy/bzr/tests/test_workingtree.py'
--- breezy/bzr/tests/test_workingtree.py	1970-01-01 00:00:00 +0000
+++ breezy/bzr/tests/test_workingtree.py	2020-06-25 21:13:23 +0000
@@ -0,0 +1,33 @@
+# Copyright (C) 2007-2012, 2016 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Tests for InventoryWorkingTree"""
+
+from . import TestCase
+
+from ..workingtree import InventoryModified
+
+
+class ErrorTests(TestCase):
+
+    def test_inventory_modified(self):
+        error = InventoryModified("a tree to be repred")
+        self.assertEqualDiff("The current inventory for the tree 'a tree to "
+                             "be repred' has been modified, so a clean inventory cannot be "
+                             "read without data loss.",
+                             str(error))
+
+

=== modified file 'breezy/bzr/tests/test_xml.py'
--- breezy/bzr/tests/test_xml.py	2020-06-10 23:47:24 +0000
+++ breezy/bzr/tests/test_xml.py	2020-07-18 23:14:00 +0000
@@ -22,6 +22,7 @@
     )
 from .. import (
     inventory,
+    serializer,
     xml6,
     xml7,
     xml8,
@@ -406,10 +407,10 @@
         """Can't accidentally open a file with wrong serializer"""
         s_v6 = breezy.bzr.xml6.serializer_v6
         s_v7 = xml7.serializer_v7
-        self.assertRaises(errors.UnexpectedInventoryFormat,
+        self.assertRaises(serializer.UnexpectedInventoryFormat,
                           s_v7.read_inventory_from_lines,
                           breezy.osutils.split_lines(_expected_inv_v5))
-        self.assertRaises(errors.UnexpectedInventoryFormat,
+        self.assertRaises(serializer.UnexpectedInventoryFormat,
                           s_v6.read_inventory_from_lines,
                           breezy.osutils.split_lines(_expected_inv_v7))
 
@@ -421,9 +422,9 @@
         inv.root.revision = b'root-rev'
         inv.add(inventory.TreeReference(b'nested-id', 'nested', b'tree-root-321',
                                         b'rev-outer', b'rev-inner'))
-        self.assertRaises(errors.UnsupportedInventoryKind,
+        self.assertRaises(serializer.UnsupportedInventoryKind,
                           s_v5.write_inventory_to_lines, inv)
-        self.assertRaises(errors.UnsupportedInventoryKind,
+        self.assertRaises(serializer.UnsupportedInventoryKind,
                           s_v6.write_inventory_to_lines, inv)
         lines = s_v7.write_inventory_to_chunks(inv)
         inv2 = s_v7.read_inventory_from_lines(lines)
@@ -525,7 +526,7 @@
     def test_serialization_error(self):
         s_v5 = breezy.bzr.xml5.serializer_v5
         e = self.assertRaises(
-            errors.UnexpectedInventoryFormat,
+            serializer.UnexpectedInventoryFormat,
             s_v5.read_inventory_from_lines, [b"<Notquitexml"])
         self.assertEqual(str(e), "unclosed token: line 1, column 0")
 

=== added file 'breezy/bzr/transform.py'
--- breezy/bzr/transform.py	1970-01-01 00:00:00 +0000
+++ breezy/bzr/transform.py	2020-07-07 22:21:51 +0000
@@ -0,0 +1,2148 @@
+# Copyright (C) 2006-2011 Canonical Ltd
+# Copyright (C) 2020 Breezy Developers
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+from __future__ import absolute_import
+
+import errno
+import os
+from stat import S_IEXEC, S_ISREG
+import time
+
+from .. import (
+    annotate,
+    errors,
+    lock,
+    multiparent,
+    osutils,
+    revision as _mod_revision,
+    trace,
+    tree,
+    ui,
+    urlutils,
+    )
+
+from ..i18n import gettext
+from ..mutabletree import MutableTree
+from ..sixish import text_type, viewvalues, viewitems
+from ..transform import (
+    ROOT_PARENT,
+    _FileMover,
+    _TransformResults,
+    joinpath,
+    NoFinalPath,
+    FinalPaths,
+    unique_add,
+    TreeTransform,
+    TransformRenameFailed,
+    ImmortalLimbo,
+    ReusingTransform,
+    MalformedTransform,
+    PreviewTree,
+    )
+from ..tree import TreeChange
+from . import (
+    inventory,
+    inventorytree,
+    )
+
+
+class TreeTransformBase(TreeTransform):
+    """The base class for TreeTransform and its kin."""
+
+    def __init__(self, tree, pb=None, case_sensitive=True):
+        """Constructor.
+
+        :param tree: The tree that will be transformed, but not necessarily
+            the output tree.
+        :param pb: ignored
+        :param case_sensitive: If True, the target of the transform is
+            case sensitive, not just case preserving.
+        """
+        super(TreeTransformBase, self).__init__(tree, pb=pb)
+        # mapping of trans_id => (sha1 of content, stat_value)
+        self._observed_sha1s = {}
+        # Mapping of trans_id -> new file_id
+        self._new_id = {}
+        # Mapping of old file-id -> trans_id
+        self._non_present_ids = {}
+        # Mapping of new file_id -> trans_id
+        self._r_new_id = {}
+        # The trans_id that will be used as the tree root
+        if tree.is_versioned(''):
+            self._new_root = self.trans_id_tree_path('')
+        else:
+            self._new_root = None
+        # Whether the target is case sensitive
+        self._case_sensitive_target = case_sensitive
+
+    def finalize(self):
+        """Release the working tree lock, if held.
+
+        This is required if apply has not been invoked, but can be invoked
+        even after apply.
+        """
+        if self._tree is None:
+            return
+        for hook in MutableTree.hooks['post_transform']:
+            hook(self._tree, self)
+        self._tree.unlock()
+        self._tree = None
+
+    def __get_root(self):
+        return self._new_root
+
+    root = property(__get_root)
+
+    def create_path(self, name, parent):
+        """Assign a transaction id to a new path"""
+        trans_id = self._assign_id()
+        unique_add(self._new_name, trans_id, name)
+        unique_add(self._new_parent, trans_id, parent)
+        return trans_id
+
+    def adjust_root_path(self, name, parent):
+        """Emulate moving the root by moving all children, instead.
+
+        We do this by undoing the association of root's transaction id with the
+        current tree.  This allows us to create a new directory with that
+        transaction id.  We unversion the root directory and version the
+        physically new directory, and hope someone versions the tree root
+        later.
+        """
+        old_root = self._new_root
+        old_root_file_id = self.final_file_id(old_root)
+        # force moving all children of root
+        for child_id in self.iter_tree_children(old_root):
+            if child_id != parent:
+                self.adjust_path(self.final_name(child_id),
+                                 self.final_parent(child_id), child_id)
+            file_id = self.final_file_id(child_id)
+            if file_id is not None:
+                self.unversion_file(child_id)
+            self.version_file(child_id, file_id=file_id)
+
+        # the physical root needs a new transaction id
+        self._tree_path_ids.pop("")
+        self._tree_id_paths.pop(old_root)
+        self._new_root = self.trans_id_tree_path('')
+        if parent == old_root:
+            parent = self._new_root
+        self.adjust_path(name, parent, old_root)
+        self.create_directory(old_root)
+        self.version_file(old_root, file_id=old_root_file_id)
+        self.unversion_file(self._new_root)
+
+    def fixup_new_roots(self):
+        """Reinterpret requests to change the root directory
+
+        Instead of creating a root directory, or moving an existing directory,
+        all the attributes and children of the new root are applied to the
+        existing root directory.
+
+        This means that the old root trans-id becomes obsolete, so it is
+        recommended only to invoke this after the root trans-id has become
+        irrelevant.
+
+        """
+        new_roots = [k for k, v in viewitems(self._new_parent)
+                     if v == ROOT_PARENT]
+        if len(new_roots) < 1:
+            return
+        if len(new_roots) != 1:
+            raise ValueError('A tree cannot have two roots!')
+        if self._new_root is None:
+            self._new_root = new_roots[0]
+            return
+        old_new_root = new_roots[0]
+        # unversion the new root's directory.
+        if self.final_kind(self._new_root) is None:
+            file_id = self.final_file_id(old_new_root)
+        else:
+            file_id = self.final_file_id(self._new_root)
+        if old_new_root in self._new_id:
+            self.cancel_versioning(old_new_root)
+        else:
+            self.unversion_file(old_new_root)
+        # if, at this stage, root still has an old file_id, zap it so we can
+        # stick a new one in.
+        if (self.tree_file_id(self._new_root) is not None
+                and self._new_root not in self._removed_id):
+            self.unversion_file(self._new_root)
+        if file_id is not None:
+            self.version_file(self._new_root, file_id=file_id)
+
+        # Now move children of new root into old root directory.
+        # Ensure all children are registered with the transaction, but don't
+        # use directly-- some tree children have new parents
+        list(self.iter_tree_children(old_new_root))
+        # Move all children of new root into old root directory.
+        for child in self.by_parent().get(old_new_root, []):
+            self.adjust_path(self.final_name(child), self._new_root, child)
+
+        # Ensure old_new_root has no directory.
+        if old_new_root in self._new_contents:
+            self.cancel_creation(old_new_root)
+        else:
+            self.delete_contents(old_new_root)
+
+        # prevent deletion of root directory.
+        if self._new_root in self._removed_contents:
+            self.cancel_deletion(self._new_root)
+
+        # destroy path info for old_new_root.
+        del self._new_parent[old_new_root]
+        del self._new_name[old_new_root]
+
+    def trans_id_file_id(self, file_id):
+        """Determine or set the transaction id associated with a file ID.
+        A new id is only created for file_ids that were never present.  If
+        a transaction has been unversioned, it is deliberately still returned.
+        (this will likely lead to an unversioned parent conflict.)
+        """
+        if file_id is None:
+            raise ValueError('None is not a valid file id')
+        if file_id in self._r_new_id and self._r_new_id[file_id] is not None:
+            return self._r_new_id[file_id]
+        else:
+            try:
+                path = self._tree.id2path(file_id)
+            except errors.NoSuchId:
+                if file_id in self._non_present_ids:
+                    return self._non_present_ids[file_id]
+                else:
+                    trans_id = self._assign_id()
+                    self._non_present_ids[file_id] = trans_id
+                    return trans_id
+            else:
+                return self.trans_id_tree_path(path)
+
+    def version_file(self, trans_id, file_id=None):
+        """Schedule a file to become versioned."""
+        raise NotImplementedError(self.version_file)
+
+    def cancel_versioning(self, trans_id):
+        """Undo a previous versioning of a file"""
+        raise NotImplementedError(self.cancel_versioning)
+
+    def new_paths(self, filesystem_only=False):
+        """Determine the paths of all new and changed files.
+
+        :param filesystem_only: if True, only calculate values for files
+            that require renames or execute bit changes.
+        """
+        new_ids = set()
+        if filesystem_only:
+            stale_ids = self._needs_rename.difference(self._new_name)
+            stale_ids.difference_update(self._new_parent)
+            stale_ids.difference_update(self._new_contents)
+            stale_ids.difference_update(self._new_id)
+            needs_rename = self._needs_rename.difference(stale_ids)
+            id_sets = (needs_rename, self._new_executability)
+        else:
+            id_sets = (self._new_name, self._new_parent, self._new_contents,
+                       self._new_id, self._new_executability)
+        for id_set in id_sets:
+            new_ids.update(id_set)
+        return sorted(FinalPaths(self).get_paths(new_ids))
+
+    def tree_file_id(self, trans_id):
+        """Determine the file id associated with the trans_id in the tree"""
+        path = self.tree_path(trans_id)
+        if path is None:
+            return None
+        # the file is old; the old id is still valid
+        if self._new_root == trans_id:
+            return self._tree.path2id('')
+        return self._tree.path2id(path)
+
+    def final_is_versioned(self, trans_id):
+        return self.final_file_id(trans_id) is not None
+
+    def final_file_id(self, trans_id):
+        """Determine the file id after any changes are applied, or None.
+
+        None indicates that the file will not be versioned after changes are
+        applied.
+        """
+        try:
+            return self._new_id[trans_id]
+        except KeyError:
+            if trans_id in self._removed_id:
+                return None
+        return self.tree_file_id(trans_id)
+
+    def inactive_file_id(self, trans_id):
+        """Return the inactive file_id associated with a transaction id.
+        That is, the one in the tree or in non_present_ids.
+        The file_id may actually be active, too.
+        """
+        file_id = self.tree_file_id(trans_id)
+        if file_id is not None:
+            return file_id
+        for key, value in viewitems(self._non_present_ids):
+            if value == trans_id:
+                return key
+
+    def find_conflicts(self):
+        """Find any violations of inventory or filesystem invariants"""
+        if self._done is True:
+            raise ReusingTransform()
+        conflicts = []
+        # ensure all children of all existent parents are known
+        # all children of non-existent parents are known, by definition.
+        self._add_tree_children()
+        by_parent = self.by_parent()
+        conflicts.extend(self._unversioned_parents(by_parent))
+        conflicts.extend(self._parent_loops())
+        conflicts.extend(self._duplicate_entries(by_parent))
+        conflicts.extend(self._parent_type_conflicts(by_parent))
+        conflicts.extend(self._improper_versioning())
+        conflicts.extend(self._executability_conflicts())
+        conflicts.extend(self._overwrite_conflicts())
+        return conflicts
+
+    def _check_malformed(self):
+        conflicts = self.find_conflicts()
+        if len(conflicts) != 0:
+            raise MalformedTransform(conflicts=conflicts)
+
+    def _add_tree_children(self):
+        """Add all the children of all active parents to the known paths.
+
+        Active parents are those which gain children, and those which are
+        removed.  This is a necessary first step in detecting conflicts.
+        """
+        parents = list(self.by_parent())
+        parents.extend([t for t in self._removed_contents if
+                        self.tree_kind(t) == 'directory'])
+        for trans_id in self._removed_id:
+            path = self.tree_path(trans_id)
+            if path is not None:
+                if self._tree.stored_kind(path) == 'directory':
+                    parents.append(trans_id)
+            elif self.tree_kind(trans_id) == 'directory':
+                parents.append(trans_id)
+
+        for parent_id in parents:
+            # ensure that all children are registered with the transaction
+            list(self.iter_tree_children(parent_id))
+
+    def _has_named_child(self, name, parent_id, known_children):
+        """Does a parent already have a name child.
+
+        :param name: The searched for name.
+
+        :param parent_id: The parent for which the check is made.
+
+        :param known_children: The already known children. This should have
+            been recently obtained from `self.by_parent.get(parent_id)`
+            (or will be if None is passed).
+        """
+        if known_children is None:
+            known_children = self.by_parent().get(parent_id, [])
+        for child in known_children:
+            if self.final_name(child) == name:
+                return True
+        parent_path = self._tree_id_paths.get(parent_id, None)
+        if parent_path is None:
+            # No parent... no children
+            return False
+        child_path = joinpath(parent_path, name)
+        child_id = self._tree_path_ids.get(child_path, None)
+        if child_id is None:
+            # Not known by the tree transform yet, check the filesystem
+            return osutils.lexists(self._tree.abspath(child_path))
+        else:
+            raise AssertionError('child_id is missing: %s, %s, %s'
+                                 % (name, parent_id, child_id))
+
+    def _available_backup_name(self, name, target_id):
+        """Find an available backup name.
+
+        :param name: The basename of the file.
+
+        :param target_id: The directory trans_id where the backup should
+            be placed.
+        """
+        known_children = self.by_parent().get(target_id, [])
+        return osutils.available_backup_name(
+            name,
+            lambda base: self._has_named_child(
+                base, target_id, known_children))
+
+    def _parent_loops(self):
+        """No entry should be its own ancestor"""
+        conflicts = []
+        for trans_id in self._new_parent:
+            seen = set()
+            parent_id = trans_id
+            while parent_id != ROOT_PARENT:
+                seen.add(parent_id)
+                try:
+                    parent_id = self.final_parent(parent_id)
+                except KeyError:
+                    break
+                if parent_id == trans_id:
+                    conflicts.append(('parent loop', trans_id))
+                if parent_id in seen:
+                    break
+        return conflicts
+
+    def _unversioned_parents(self, by_parent):
+        """If parent directories are versioned, children must be versioned."""
+        conflicts = []
+        for parent_id, children in viewitems(by_parent):
+            if parent_id == ROOT_PARENT:
+                continue
+            if self.final_is_versioned(parent_id):
+                continue
+            for child_id in children:
+                if self.final_is_versioned(child_id):
+                    conflicts.append(('unversioned parent', parent_id))
+                    break
+        return conflicts
+
+    def _improper_versioning(self):
+        """Cannot version a file with no contents, or a bad type.
+
+        However, existing entries with no contents are okay.
+        """
+        conflicts = []
+        for trans_id in self._new_id:
+            kind = self.final_kind(trans_id)
+            if kind == 'symlink' and not self._tree.supports_symlinks():
+                # Ignore symlinks as they are not supported on this platform
+                continue
+            if kind is None:
+                conflicts.append(('versioning no contents', trans_id))
+                continue
+            if not self._tree.versionable_kind(kind):
+                conflicts.append(('versioning bad kind', trans_id, kind))
+        return conflicts
+
+    def _executability_conflicts(self):
+        """Check for bad executability changes.
+
+        Only versioned files may have their executability set, because
+        1. only versioned entries can have executability under windows
+        2. only files can be executable.  (The execute bit on a directory
+           does not indicate searchability)
+        """
+        conflicts = []
+        for trans_id in self._new_executability:
+            if not self.final_is_versioned(trans_id):
+                conflicts.append(('unversioned executability', trans_id))
+            else:
+                if self.final_kind(trans_id) != "file":
+                    conflicts.append(('non-file executability', trans_id))
+        return conflicts
+
+    def _overwrite_conflicts(self):
+        """Check for overwrites (not permitted on Win32)"""
+        conflicts = []
+        for trans_id in self._new_contents:
+            if self.tree_kind(trans_id) is None:
+                continue
+            if trans_id not in self._removed_contents:
+                conflicts.append(('overwrite', trans_id,
+                                  self.final_name(trans_id)))
+        return conflicts
+
+    def _duplicate_entries(self, by_parent):
+        """No directory may have two entries with the same name."""
+        conflicts = []
+        if (self._new_name, self._new_parent) == ({}, {}):
+            return conflicts
+        for children in viewvalues(by_parent):
+            name_ids = []
+            for child_tid in children:
+                name = self.final_name(child_tid)
+                if name is not None:
+                    # Keep children only if they still exist in the end
+                    if not self._case_sensitive_target:
+                        name = name.lower()
+                    name_ids.append((name, child_tid))
+            name_ids.sort()
+            last_name = None
+            last_trans_id = None
+            for name, trans_id in name_ids:
+                kind = self.final_kind(trans_id)
+                if kind is None and not self.final_is_versioned(trans_id):
+                    continue
+                if name == last_name:
+                    conflicts.append(('duplicate', last_trans_id, trans_id,
+                                      name))
+                last_name = name
+                last_trans_id = trans_id
+        return conflicts
+
+    def _parent_type_conflicts(self, by_parent):
+        """Children must have a directory parent"""
+        conflicts = []
+        for parent_id, children in viewitems(by_parent):
+            if parent_id == ROOT_PARENT:
+                continue
+            no_children = True
+            for child_id in children:
+                if self.final_kind(child_id) is not None:
+                    no_children = False
+                    break
+            if no_children:
+                continue
+            # There is at least a child, so we need an existing directory to
+            # contain it.
+            kind = self.final_kind(parent_id)
+            if kind is None:
+                # The directory will be deleted
+                conflicts.append(('missing parent', parent_id))
+            elif kind != "directory":
+                # Meh, we need a *directory* to put something in it
+                conflicts.append(('non-directory parent', parent_id))
+        return conflicts
+
+    def _set_executability(self, path, trans_id):
+        """Set the executability of versioned files """
+        if self._tree._supports_executable():
+            new_executability = self._new_executability[trans_id]
+            abspath = self._tree.abspath(path)
+            current_mode = os.stat(abspath).st_mode
+            if new_executability:
+                umask = os.umask(0)
+                os.umask(umask)
+                to_mode = current_mode | (0o100 & ~umask)
+                # Enable x-bit for others only if they can read it.
+                if current_mode & 0o004:
+                    to_mode |= 0o001 & ~umask
+                if current_mode & 0o040:
+                    to_mode |= 0o010 & ~umask
+            else:
+                to_mode = current_mode & ~0o111
+            osutils.chmod_if_possible(abspath, to_mode)
+
+    def _new_entry(self, name, parent_id, file_id):
+        """Helper function to create a new filesystem entry."""
+        trans_id = self.create_path(name, parent_id)
+        if file_id is not None:
+            self.version_file(trans_id, file_id=file_id)
+        return trans_id
+
+    def new_file(self, name, parent_id, contents, file_id=None,
+                 executable=None, sha1=None):
+        """Convenience method to create files.
+
+        name is the name of the file to create.
+        parent_id is the transaction id of the parent directory of the file.
+        contents is an iterator of bytestrings, which will be used to produce
+        the file.
+        :param file_id: The inventory ID of the file, if it is to be versioned.
+        :param executable: Only valid when a file_id has been supplied.
+        """
+        trans_id = self._new_entry(name, parent_id, file_id)
+        # TODO: rather than scheduling a set_executable call,
+        # have create_file create the file with the right mode.
+        self.create_file(contents, trans_id, sha1=sha1)
+        if executable is not None:
+            self.set_executability(executable, trans_id)
+        return trans_id
+
+    def new_directory(self, name, parent_id, file_id=None):
+        """Convenience method to create directories.
+
+        name is the name of the directory to create.
+        parent_id is the transaction id of the parent directory of the
+        directory.
+        file_id is the inventory ID of the directory, if it is to be versioned.
+        """
+        trans_id = self._new_entry(name, parent_id, file_id)
+        self.create_directory(trans_id)
+        return trans_id
+
+    def new_symlink(self, name, parent_id, target, file_id=None):
+        """Convenience method to create symbolic link.
+
+        name is the name of the symlink to create.
+        parent_id is the transaction id of the parent directory of the symlink.
+        target is a bytestring of the target of the symlink.
+        file_id is the inventory ID of the file, if it is to be versioned.
+        """
+        trans_id = self._new_entry(name, parent_id, file_id)
+        self.create_symlink(target, trans_id)
+        return trans_id
+
+    def new_orphan(self, trans_id, parent_id):
+        """Schedule an item to be orphaned.
+
+        When a directory is about to be removed, its children, if they are not
+        versioned are moved out of the way: they don't have a parent anymore.
+
+        :param trans_id: The trans_id of the existing item.
+        :param parent_id: The parent trans_id of the item.
+        """
+        raise NotImplementedError(self.new_orphan)
+
+    def _get_potential_orphans(self, dir_id):
+        """Find the potential orphans in a directory.
+
+        A directory can't be safely deleted if there are versioned files in it.
+        If all the contained files are unversioned then they can be orphaned.
+
+        The 'None' return value means that the directory contains at least one
+        versioned file and should not be deleted.
+
+        :param dir_id: The directory trans id.
+
+        :return: A list of the orphan trans ids or None if at least one
+             versioned file is present.
+        """
+        orphans = []
+        # Find the potential orphans, stop if one item should be kept
+        for child_tid in self.by_parent()[dir_id]:
+            if child_tid in self._removed_contents:
+                # The child is removed as part of the transform. Since it was
+                # versioned before, it's not an orphan
+                continue
+            if not self.final_is_versioned(child_tid):
+                # The child is not versioned
+                orphans.append(child_tid)
+            else:
+                # We have a versioned file here, searching for orphans is
+                # meaningless.
+                orphans = None
+                break
+        return orphans
+
+    def _affected_ids(self):
+        """Return the set of transform ids affected by the transform"""
+        trans_ids = set(self._removed_id)
+        trans_ids.update(self._new_id)
+        trans_ids.update(self._removed_contents)
+        trans_ids.update(self._new_contents)
+        trans_ids.update(self._new_executability)
+        trans_ids.update(self._new_name)
+        trans_ids.update(self._new_parent)
+        return trans_ids
+
+    def _get_file_id_maps(self):
+        """Return mapping of file_ids to trans_ids in the to and from states"""
+        trans_ids = self._affected_ids()
+        from_trans_ids = {}
+        to_trans_ids = {}
+        # Build up two dicts: trans_ids associated with file ids in the
+        # FROM state, vs the TO state.
+        for trans_id in trans_ids:
+            from_file_id = self.tree_file_id(trans_id)
+            if from_file_id is not None:
+                from_trans_ids[from_file_id] = trans_id
+            to_file_id = self.final_file_id(trans_id)
+            if to_file_id is not None:
+                to_trans_ids[to_file_id] = trans_id
+        return from_trans_ids, to_trans_ids
+
+    def _from_file_data(self, from_trans_id, from_versioned, from_path):
+        """Get data about a file in the from (tree) state
+
+        Return a (name, parent, kind, executable) tuple
+        """
+        from_path = self._tree_id_paths.get(from_trans_id)
+        if from_versioned:
+            # get data from working tree if versioned
+            from_entry = next(self._tree.iter_entries_by_dir(
+                specific_files=[from_path]))[1]
+            from_name = from_entry.name
+            from_parent = from_entry.parent_id
+        else:
+            from_entry = None
+            if from_path is None:
+                # File does not exist in FROM state
+                from_name = None
+                from_parent = None
+            else:
+                # File exists, but is not versioned.  Have to use path-
+                # splitting stuff
+                from_name = os.path.basename(from_path)
+                tree_parent = self.get_tree_parent(from_trans_id)
+                from_parent = self.tree_file_id(tree_parent)
+        if from_path is not None:
+            from_kind, from_executable, from_stats = \
+                self._tree._comparison_data(from_entry, from_path)
+        else:
+            from_kind = None
+            from_executable = False
+        return from_name, from_parent, from_kind, from_executable
+
+    def _to_file_data(self, to_trans_id, from_trans_id, from_executable):
+        """Get data about a file in the to (target) state
+
+        Return a (name, parent, kind, executable) tuple
+        """
+        to_name = self.final_name(to_trans_id)
+        to_kind = self.final_kind(to_trans_id)
+        to_parent = self.final_file_id(self.final_parent(to_trans_id))
+        if to_trans_id in self._new_executability:
+            to_executable = self._new_executability[to_trans_id]
+        elif to_trans_id == from_trans_id:
+            to_executable = from_executable
+        else:
+            to_executable = False
+        return to_name, to_parent, to_kind, to_executable
+
+    def iter_changes(self):
+        """Produce output in the same format as Tree.iter_changes.
+
+        Will produce nonsensical results if invoked while inventory/filesystem
+        conflicts (as reported by TreeTransform.find_conflicts()) are present.
+
+        This reads the Transform, but only reproduces changes involving a
+        file_id.  Files that are not versioned in either of the FROM or TO
+        states are not reflected.
+        """
+        final_paths = FinalPaths(self)
+        from_trans_ids, to_trans_ids = self._get_file_id_maps()
+        results = []
+        # Now iterate through all active file_ids
+        for file_id in set(from_trans_ids).union(to_trans_ids):
+            modified = False
+            from_trans_id = from_trans_ids.get(file_id)
+            # find file ids, and determine versioning state
+            if from_trans_id is None:
+                from_versioned = False
+                from_trans_id = to_trans_ids[file_id]
+            else:
+                from_versioned = True
+            to_trans_id = to_trans_ids.get(file_id)
+            if to_trans_id is None:
+                to_versioned = False
+                to_trans_id = from_trans_id
+            else:
+                to_versioned = True
+
+            if not from_versioned:
+                from_path = None
+            else:
+                from_path = self._tree_id_paths.get(from_trans_id)
+            if not to_versioned:
+                to_path = None
+            else:
+                to_path = final_paths.get_path(to_trans_id)
+
+            from_name, from_parent, from_kind, from_executable = \
+                self._from_file_data(from_trans_id, from_versioned, from_path)
+
+            to_name, to_parent, to_kind, to_executable = \
+                self._to_file_data(to_trans_id, from_trans_id, from_executable)
+
+            if from_kind != to_kind:
+                modified = True
+            elif to_kind in ('file', 'symlink') and (
+                    to_trans_id != from_trans_id
+                    or to_trans_id in self._new_contents):
+                modified = True
+            if (not modified and from_versioned == to_versioned
+                and from_parent == to_parent and from_name == to_name
+                    and from_executable == to_executable):
+                continue
+            results.append(
+                TreeChange(
+                    file_id, (from_path, to_path), modified,
+                    (from_versioned, to_versioned),
+                    (from_parent, to_parent),
+                    (from_name, to_name),
+                    (from_kind, to_kind),
+                    (from_executable, to_executable)))
+
+        def path_key(c):
+            return (c.path[0] or '', c.path[1] or '')
+        return iter(sorted(results, key=path_key))
+
+    def get_preview_tree(self):
+        """Return a tree representing the result of the transform.
+
+        The tree is a snapshot, and altering the TreeTransform will invalidate
+        it.
+        """
+        raise NotImplementedError(self.get_preview)
+
+    def commit(self, branch, message, merge_parents=None, strict=False,
+               timestamp=None, timezone=None, committer=None, authors=None,
+               revprops=None, revision_id=None):
+        """Commit the result of this TreeTransform to a branch.
+
+        :param branch: The branch to commit to.
+        :param message: The message to attach to the commit.
+        :param merge_parents: Additional parent revision-ids specified by
+            pending merges.
+        :param strict: If True, abort the commit if there are unversioned
+            files.
+        :param timestamp: if not None, seconds-since-epoch for the time and
+            date.  (May be a float.)
+        :param timezone: Optional timezone for timestamp, as an offset in
+            seconds.
+        :param committer: Optional committer in email-id format.
+            (e.g. "J Random Hacker <jrandom@example.com>")
+        :param authors: Optional list of authors in email-id format.
+        :param revprops: Optional dictionary of revision properties.
+        :param revision_id: Optional revision id.  (Specifying a revision-id
+            may reduce performance for some non-native formats.)
+        :return: The revision_id of the revision committed.
+        """
+        self._check_malformed()
+        if strict:
+            unversioned = set(self._new_contents).difference(set(self._new_id))
+            for trans_id in unversioned:
+                if not self.final_is_versioned(trans_id):
+                    raise errors.StrictCommitFailed()
+
+        revno, last_rev_id = branch.last_revision_info()
+        if last_rev_id == _mod_revision.NULL_REVISION:
+            if merge_parents is not None:
+                raise ValueError('Cannot supply merge parents for first'
+                                 ' commit.')
+            parent_ids = []
+        else:
+            parent_ids = [last_rev_id]
+            if merge_parents is not None:
+                parent_ids.extend(merge_parents)
+        if self._tree.get_revision_id() != last_rev_id:
+            raise ValueError('TreeTransform not based on branch basis: %s' %
+                             self._tree.get_revision_id().decode('utf-8'))
+        from .. import commit
+        revprops = commit.Commit.update_revprops(revprops, branch, authors)
+        builder = branch.get_commit_builder(parent_ids,
+                                            timestamp=timestamp,
+                                            timezone=timezone,
+                                            committer=committer,
+                                            revprops=revprops,
+                                            revision_id=revision_id)
+        preview = self.get_preview_tree()
+        list(builder.record_iter_changes(preview, last_rev_id,
+                                         self.iter_changes()))
+        builder.finish_inventory()
+        revision_id = builder.commit(message)
+        branch.set_last_revision_info(revno + 1, revision_id)
+        return revision_id
+
+    def _text_parent(self, trans_id):
+        path = self.tree_path(trans_id)
+        try:
+            if path is None or self._tree.kind(path) != 'file':
+                return None
+        except errors.NoSuchFile:
+            return None
+        return path
+
+    def _get_parents_texts(self, trans_id):
+        """Get texts for compression parents of this file."""
+        path = self._text_parent(trans_id)
+        if path is None:
+            return ()
+        return (self._tree.get_file_text(path),)
+
+    def _get_parents_lines(self, trans_id):
+        """Get lines for compression parents of this file."""
+        path = self._text_parent(trans_id)
+        if path is None:
+            return ()
+        return (self._tree.get_file_lines(path),)
+
+    def serialize(self, serializer):
+        """Serialize this TreeTransform.
+
+        :param serializer: A Serialiser like pack.ContainerSerializer.
+        """
+        from .. import bencode
+        new_name = {k.encode('utf-8'): v.encode('utf-8')
+                    for k, v in viewitems(self._new_name)}
+        new_parent = {k.encode('utf-8'): v.encode('utf-8')
+                      for k, v in viewitems(self._new_parent)}
+        new_id = {k.encode('utf-8'): v
+                  for k, v in viewitems(self._new_id)}
+        new_executability = {k.encode('utf-8'): int(v)
+                             for k, v in viewitems(self._new_executability)}
+        tree_path_ids = {k.encode('utf-8'): v.encode('utf-8')
+                         for k, v in viewitems(self._tree_path_ids)}
+        non_present_ids = {k: v.encode('utf-8')
+                           for k, v in viewitems(self._non_present_ids)}
+        removed_contents = [trans_id.encode('utf-8')
+                            for trans_id in self._removed_contents]
+        removed_id = [trans_id.encode('utf-8')
+                      for trans_id in self._removed_id]
+        attribs = {
+            b'_id_number': self._id_number,
+            b'_new_name': new_name,
+            b'_new_parent': new_parent,
+            b'_new_executability': new_executability,
+            b'_new_id': new_id,
+            b'_tree_path_ids': tree_path_ids,
+            b'_removed_id': removed_id,
+            b'_removed_contents': removed_contents,
+            b'_non_present_ids': non_present_ids,
+            }
+        yield serializer.bytes_record(bencode.bencode(attribs),
+                                      ((b'attribs',),))
+        for trans_id, kind in sorted(viewitems(self._new_contents)):
+            if kind == 'file':
+                with open(self._limbo_name(trans_id), 'rb') as cur_file:
+                    lines = cur_file.readlines()
+                parents = self._get_parents_lines(trans_id)
+                mpdiff = multiparent.MultiParent.from_lines(lines, parents)
+                content = b''.join(mpdiff.to_patch())
+            if kind == 'directory':
+                content = b''
+            if kind == 'symlink':
+                content = self._read_symlink_target(trans_id)
+                if not isinstance(content, bytes):
+                    content = content.encode('utf-8')
+            yield serializer.bytes_record(
+                content, ((trans_id.encode('utf-8'), kind.encode('ascii')),))
+
+    def deserialize(self, records):
+        """Deserialize a stored TreeTransform.
+
+        :param records: An iterable of (names, content) tuples, as per
+            pack.ContainerPushParser.
+        """
+        from .. import bencode
+        names, content = next(records)
+        attribs = bencode.bdecode(content)
+        self._id_number = attribs[b'_id_number']
+        self._new_name = {k.decode('utf-8'): v.decode('utf-8')
+                          for k, v in viewitems(attribs[b'_new_name'])}
+        self._new_parent = {k.decode('utf-8'): v.decode('utf-8')
+                            for k, v in viewitems(attribs[b'_new_parent'])}
+        self._new_executability = {
+            k.decode('utf-8'): bool(v)
+            for k, v in viewitems(attribs[b'_new_executability'])}
+        self._new_id = {k.decode('utf-8'): v
+                        for k, v in viewitems(attribs[b'_new_id'])}
+        self._r_new_id = {v: k for k, v in viewitems(self._new_id)}
+        self._tree_path_ids = {}
+        self._tree_id_paths = {}
+        for bytepath, trans_id in viewitems(attribs[b'_tree_path_ids']):
+            path = bytepath.decode('utf-8')
+            trans_id = trans_id.decode('utf-8')
+            self._tree_path_ids[path] = trans_id
+            self._tree_id_paths[trans_id] = path
+        self._removed_id = {trans_id.decode('utf-8')
+                            for trans_id in attribs[b'_removed_id']}
+        self._removed_contents = set(
+            trans_id.decode('utf-8')
+            for trans_id in attribs[b'_removed_contents'])
+        self._non_present_ids = {
+            k: v.decode('utf-8')
+            for k, v in viewitems(attribs[b'_non_present_ids'])}
+        for ((trans_id, kind),), content in records:
+            trans_id = trans_id.decode('utf-8')
+            kind = kind.decode('ascii')
+            if kind == 'file':
+                mpdiff = multiparent.MultiParent.from_patch(content)
+                lines = mpdiff.to_lines(self._get_parents_texts(trans_id))
+                self.create_file(lines, trans_id)
+            if kind == 'directory':
+                self.create_directory(trans_id)
+            if kind == 'symlink':
+                self.create_symlink(content.decode('utf-8'), trans_id)
+
+    def create_file(self, contents, trans_id, mode_id=None, sha1=None):
+        """Schedule creation of a new file.
+
+        :seealso: new_file.
+
+        :param contents: an iterator of strings, all of which will be written
+            to the target destination.
+        :param trans_id: TreeTransform handle
+        :param mode_id: If not None, force the mode of the target file to match
+            the mode of the object referenced by mode_id.
+            Otherwise, we will try to preserve mode bits of an existing file.
+        :param sha1: If the sha1 of this content is already known, pass it in.
+            We can use it to prevent future sha1 computations.
+        """
+        raise NotImplementedError(self.create_file)
+
+    def create_directory(self, trans_id):
+        """Schedule creation of a new directory.
+
+        See also new_directory.
+        """
+        raise NotImplementedError(self.create_directory)
+
+    def create_symlink(self, target, trans_id):
+        """Schedule creation of a new symbolic link.
+
+        target is a bytestring.
+        See also new_symlink.
+        """
+        raise NotImplementedError(self.create_symlink)
+
+    def create_hardlink(self, path, trans_id):
+        """Schedule creation of a hard link"""
+        raise NotImplementedError(self.create_hardlink)
+
+    def cancel_creation(self, trans_id):
+        """Cancel the creation of new file contents."""
+        raise NotImplementedError(self.cancel_creation)
+
+    def apply(self, no_conflicts=False, precomputed_delta=None, _mover=None):
+        """Apply all changes to the inventory and filesystem.
+
+        If filesystem or inventory conflicts are present, MalformedTransform
+        will be thrown.
+
+        If apply succeeds, finalize is not necessary.
+
+        :param no_conflicts: if True, the caller guarantees there are no
+            conflicts, so no check is made.
+        :param precomputed_delta: An inventory delta to use instead of
+            calculating one.
+        :param _mover: Supply an alternate FileMover, for testing
+        """
+        raise NotImplementedError(self.apply)
+
+
+class DiskTreeTransform(TreeTransformBase):
+    """Tree transform storing its contents on disk."""
+
+    def __init__(self, tree, limbodir, pb=None, case_sensitive=True):
+        """Constructor.
+        :param tree: The tree that will be transformed, but not necessarily
+            the output tree.
+        :param limbodir: A directory where new files can be stored until
+            they are installed in their proper places
+        :param pb: ignored
+        :param case_sensitive: If True, the target of the transform is
+            case sensitive, not just case preserving.
+        """
+        TreeTransformBase.__init__(self, tree, pb, case_sensitive)
+        self._limbodir = limbodir
+        self._deletiondir = None
+        # A mapping of transform ids to their limbo filename
+        self._limbo_files = {}
+        self._possibly_stale_limbo_files = set()
+        # A mapping of transform ids to a set of the transform ids of children
+        # that their limbo directory has
+        self._limbo_children = {}
+        # Map transform ids to maps of child filename to child transform id
+        self._limbo_children_names = {}
+        # List of transform ids that need to be renamed from limbo into place
+        self._needs_rename = set()
+        self._creation_mtime = None
+        self._create_symlinks = osutils.supports_symlinks(self._limbodir)
+
+    def finalize(self):
+        """Release the working tree lock, if held, clean up limbo dir.
+
+        This is required if apply has not been invoked, but can be invoked
+        even after apply.
+        """
+        if self._tree is None:
+            return
+        try:
+            limbo_paths = list(viewvalues(self._limbo_files))
+            limbo_paths.extend(self._possibly_stale_limbo_files)
+            limbo_paths.sort(reverse=True)
+            for path in limbo_paths:
+                try:
+                    osutils.delete_any(path)
+                except OSError as e:
+                    if e.errno != errno.ENOENT:
+                        raise
+                    # XXX: warn? perhaps we just got interrupted at an
+                    # inconvenient moment, but perhaps files are disappearing
+                    # from under us?
+            try:
+                osutils.delete_any(self._limbodir)
+            except OSError:
+                # We don't especially care *why* the dir is immortal.
+                raise ImmortalLimbo(self._limbodir)
+            try:
+                if self._deletiondir is not None:
+                    osutils.delete_any(self._deletiondir)
+            except OSError:
+                raise errors.ImmortalPendingDeletion(self._deletiondir)
+        finally:
+            TreeTransformBase.finalize(self)
+
+    def _limbo_supports_executable(self):
+        """Check if the limbo path supports the executable bit."""
+        return osutils.supports_executable(self._limbodir)
+
+    def _limbo_name(self, trans_id):
+        """Generate the limbo name of a file"""
+        limbo_name = self._limbo_files.get(trans_id)
+        if limbo_name is None:
+            limbo_name = self._generate_limbo_path(trans_id)
+            self._limbo_files[trans_id] = limbo_name
+        return limbo_name
+
+    def _generate_limbo_path(self, trans_id):
+        """Generate a limbo path using the trans_id as the relative path.
+
+        This is suitable as a fallback, and when the transform should not be
+        sensitive to the path encoding of the limbo directory.
+        """
+        self._needs_rename.add(trans_id)
+        return osutils.pathjoin(self._limbodir, trans_id)
+
+    def adjust_path(self, name, parent, trans_id):
+        previous_parent = self._new_parent.get(trans_id)
+        previous_name = self._new_name.get(trans_id)
+        super(DiskTreeTransform, self).adjust_path(name, parent, trans_id)
+        if (trans_id in self._limbo_files
+                and trans_id not in self._needs_rename):
+            self._rename_in_limbo([trans_id])
+            if previous_parent != parent:
+                self._limbo_children[previous_parent].remove(trans_id)
+            if previous_parent != parent or previous_name != name:
+                del self._limbo_children_names[previous_parent][previous_name]
+
+    def _rename_in_limbo(self, trans_ids):
+        """Fix limbo names so that the right final path is produced.
+
+        This means we outsmarted ourselves-- we tried to avoid renaming
+        these files later by creating them with their final names in their
+        final parents.  But now the previous name or parent is no longer
+        suitable, so we have to rename them.
+
+        Even for trans_ids that have no new contents, we must remove their
+        entries from _limbo_files, because they are now stale.
+        """
+        for trans_id in trans_ids:
+            old_path = self._limbo_files[trans_id]
+            self._possibly_stale_limbo_files.add(old_path)
+            del self._limbo_files[trans_id]
+            if trans_id not in self._new_contents:
+                continue
+            new_path = self._limbo_name(trans_id)
+            os.rename(old_path, new_path)
+            self._possibly_stale_limbo_files.remove(old_path)
+            for descendant in self._limbo_descendants(trans_id):
+                desc_path = self._limbo_files[descendant]
+                desc_path = new_path + desc_path[len(old_path):]
+                self._limbo_files[descendant] = desc_path
+
+    def _limbo_descendants(self, trans_id):
+        """Return the set of trans_ids whose limbo paths descend from this."""
+        descendants = set(self._limbo_children.get(trans_id, []))
+        for descendant in list(descendants):
+            descendants.update(self._limbo_descendants(descendant))
+        return descendants
+
+    def _set_mode(self, trans_id, mode_id, typefunc):
+        raise NotImplementedError(self._set_mode)
+
+    def create_file(self, contents, trans_id, mode_id=None, sha1=None):
+        """Schedule creation of a new file.
+
+        :seealso: new_file.
+
+        :param contents: an iterator of strings, all of which will be written
+            to the target destination.
+        :param trans_id: TreeTransform handle
+        :param mode_id: If not None, force the mode of the target file to match
+            the mode of the object referenced by mode_id.
+            Otherwise, we will try to preserve mode bits of an existing file.
+        :param sha1: If the sha1 of this content is already known, pass it in.
+            We can use it to prevent future sha1 computations.
+        """
+        name = self._limbo_name(trans_id)
+        with open(name, 'wb') as f:
+            unique_add(self._new_contents, trans_id, 'file')
+            f.writelines(contents)
+        self._set_mtime(name)
+        self._set_mode(trans_id, mode_id, S_ISREG)
+        # It is unfortunate we have to use lstat instead of fstat, but we just
+        # used utime and chmod on the file, so we need the accurate final
+        # details.
+        if sha1 is not None:
+            self._observed_sha1s[trans_id] = (sha1, osutils.lstat(name))
+
+    def _read_symlink_target(self, trans_id):
+        return os.readlink(self._limbo_name(trans_id))
+
+    def _set_mtime(self, path):
+        """All files that are created get the same mtime.
+
+        This time is set by the first object to be created.
+        """
+        if self._creation_mtime is None:
+            self._creation_mtime = time.time()
+        os.utime(path, (self._creation_mtime, self._creation_mtime))
+
+    def create_hardlink(self, path, trans_id):
+        """Schedule creation of a hard link"""
+        name = self._limbo_name(trans_id)
+        try:
+            os.link(path, name)
+        except OSError as e:
+            if e.errno != errno.EPERM:
+                raise
+            raise errors.HardLinkNotSupported(path)
+        try:
+            unique_add(self._new_contents, trans_id, 'file')
+        except BaseException:
+            # Clean up the file, it never got registered so
+            # TreeTransform.finalize() won't clean it up.
+            os.unlink(name)
+            raise
+
+    def create_directory(self, trans_id):
+        """Schedule creation of a new directory.
+
+        See also new_directory.
+        """
+        os.mkdir(self._limbo_name(trans_id))
+        unique_add(self._new_contents, trans_id, 'directory')
+
+    def create_symlink(self, target, trans_id):
+        """Schedule creation of a new symbolic link.
+
+        target is a bytestring.
+        See also new_symlink.
+        """
+        if self._create_symlinks:
+            os.symlink(target, self._limbo_name(trans_id))
+        else:
+            try:
+                path = FinalPaths(self).get_path(trans_id)
+            except KeyError:
+                path = None
+            trace.warning(
+                'Unable to create symlink "%s" on this filesystem.' % (path,))
+        # We add symlink to _new_contents even if they are unsupported
+        # and not created. These entries are subsequently used to avoid
+        # conflicts on platforms that don't support symlink
+        unique_add(self._new_contents, trans_id, 'symlink')
+
+    def cancel_creation(self, trans_id):
+        """Cancel the creation of new file contents."""
+        del self._new_contents[trans_id]
+        if trans_id in self._observed_sha1s:
+            del self._observed_sha1s[trans_id]
+        children = self._limbo_children.get(trans_id)
+        # if this is a limbo directory with children, move them before removing
+        # the directory
+        if children is not None:
+            self._rename_in_limbo(children)
+            del self._limbo_children[trans_id]
+            del self._limbo_children_names[trans_id]
+        osutils.delete_any(self._limbo_name(trans_id))
+
+    def new_orphan(self, trans_id, parent_id):
+        conf = self._tree.get_config_stack()
+        handle_orphan = conf.get('transform.orphan_policy')
+        handle_orphan(self, trans_id, parent_id)
+
+
+class InventoryTreeTransform(DiskTreeTransform):
+    """Represent a tree transformation.
+
+    This object is designed to support incremental generation of the transform,
+    in any order.
+
+    However, it gives optimum performance when parent directories are created
+    before their contents.  The transform is then able to put child files
+    directly in their parent directory, avoiding later renames.
+
+    It is easy to produce malformed transforms, but they are generally
+    harmless.  Attempting to apply a malformed transform will cause an
+    exception to be raised before any modifications are made to the tree.
+
+    Many kinds of malformed transforms can be corrected with the
+    resolve_conflicts function.  The remaining ones indicate programming error,
+    such as trying to create a file with no path.
+
+    Two sets of file creation methods are supplied.  Convenience methods are:
+     * new_file
+     * new_directory
+     * new_symlink
+
+    These are composed of the low-level methods:
+     * create_path
+     * create_file or create_directory or create_symlink
+     * version_file
+     * set_executability
+
+    Transform/Transaction ids
+    -------------------------
+    trans_ids are temporary ids assigned to all files involved in a transform.
+    It's possible, even common, that not all files in the Tree have trans_ids.
+
+    trans_ids are used because filenames and file_ids are not good enough
+    identifiers; filenames change, and not all files have file_ids.  File-ids
+    are also associated with trans-ids, so that moving a file moves its
+    file-id.
+
+    trans_ids are only valid for the TreeTransform that generated them.
+
+    Limbo
+    -----
+    Limbo is a temporary directory use to hold new versions of files.
+    Files are added to limbo by create_file, create_directory, create_symlink,
+    and their convenience variants (new_*).  Files may be removed from limbo
+    using cancel_creation.  Files are renamed from limbo into their final
+    location as part of TreeTransform.apply
+
+    Limbo must be cleaned up, by either calling TreeTransform.apply or
+    calling TreeTransform.finalize.
+
+    Files are placed into limbo inside their parent directories, where
+    possible.  This reduces subsequent renames, and makes operations involving
+    lots of files faster.  This optimization is only possible if the parent
+    directory is created *before* creating any of its children, so avoid
+    creating children before parents, where possible.
+
+    Pending-deletion
+    ----------------
+    This temporary directory is used by _FileMover for storing files that are
+    about to be deleted.  In case of rollback, the files will be restored.
+    FileMover does not delete files until it is sure that a rollback will not
+    happen.
+    """
+
+    def __init__(self, tree, pb=None):
+        """Note: a tree_write lock is taken on the tree.
+
+        Use TreeTransform.finalize() to release the lock (can be omitted if
+        TreeTransform.apply() called).
+        """
+        tree.lock_tree_write()
+        try:
+            limbodir = urlutils.local_path_from_url(
+                tree._transport.abspath('limbo'))
+            osutils.ensure_empty_directory_exists(
+                limbodir,
+                errors.ExistingLimbo)
+            deletiondir = urlutils.local_path_from_url(
+                tree._transport.abspath('pending-deletion'))
+            osutils.ensure_empty_directory_exists(
+                deletiondir,
+                errors.ExistingPendingDeletion)
+        except BaseException:
+            tree.unlock()
+            raise
+
+        # Cache of realpath results, to speed up canonical_path
+        self._realpaths = {}
+        # Cache of relpath results, to speed up canonical_path
+        self._relpaths = {}
+        DiskTreeTransform.__init__(self, tree, limbodir, pb,
+                                   tree.case_sensitive)
+        self._deletiondir = deletiondir
+
+    def canonical_path(self, path):
+        """Get the canonical tree-relative path"""
+        # don't follow final symlinks
+        abs = self._tree.abspath(path)
+        if abs in self._relpaths:
+            return self._relpaths[abs]
+        dirname, basename = os.path.split(abs)
+        if dirname not in self._realpaths:
+            self._realpaths[dirname] = os.path.realpath(dirname)
+        dirname = self._realpaths[dirname]
+        abs = osutils.pathjoin(dirname, basename)
+        if dirname in self._relpaths:
+            relpath = osutils.pathjoin(self._relpaths[dirname], basename)
+            relpath = relpath.rstrip('/\\')
+        else:
+            relpath = self._tree.relpath(abs)
+        self._relpaths[abs] = relpath
+        return relpath
+
+    def tree_kind(self, trans_id):
+        """Determine the file kind in the working tree.
+
+        :returns: The file kind or None if the file does not exist
+        """
+        path = self._tree_id_paths.get(trans_id)
+        if path is None:
+            return None
+        try:
+            return osutils.file_kind(self._tree.abspath(path))
+        except errors.NoSuchFile:
+            return None
+
+    def _set_mode(self, trans_id, mode_id, typefunc):
+        """Set the mode of new file contents.
+        The mode_id is the existing file to get the mode from (often the same
+        as trans_id).  The operation is only performed if there's a mode match
+        according to typefunc.
+        """
+        if mode_id is None:
+            mode_id = trans_id
+        try:
+            old_path = self._tree_id_paths[mode_id]
+        except KeyError:
+            return
+        try:
+            mode = os.stat(self._tree.abspath(old_path)).st_mode
+        except OSError as e:
+            if e.errno in (errno.ENOENT, errno.ENOTDIR):
+                # Either old_path doesn't exist, or the parent of the
+                # target is not a directory (but will be one eventually)
+                # Either way, we know it doesn't exist *right now*
+                # See also bug #248448
+                return
+            else:
+                raise
+        if typefunc(mode):
+            osutils.chmod_if_possible(self._limbo_name(trans_id), mode)
+
+    def iter_tree_children(self, parent_id):
+        """Iterate through the entry's tree children, if any"""
+        try:
+            path = self._tree_id_paths[parent_id]
+        except KeyError:
+            return
+        try:
+            children = os.listdir(self._tree.abspath(path))
+        except OSError as e:
+            if not (osutils._is_error_enotdir(e) or
+                    e.errno in (errno.ENOENT, errno.ESRCH)):
+                raise
+            return
+
+        for child in children:
+            childpath = joinpath(path, child)
+            if self._tree.is_control_filename(childpath):
+                continue
+            yield self.trans_id_tree_path(childpath)
+
+    def _generate_limbo_path(self, trans_id):
+        """Generate a limbo path using the final path if possible.
+
+        This optimizes the performance of applying the tree transform by
+        avoiding renames.  These renames can be avoided only when the parent
+        directory is already scheduled for creation.
+
+        If the final path cannot be used, falls back to using the trans_id as
+        the relpath.
+        """
+        parent = self._new_parent.get(trans_id)
+        # if the parent directory is already in limbo (e.g. when building a
+        # tree), choose a limbo name inside the parent, to reduce further
+        # renames.
+        use_direct_path = False
+        if self._new_contents.get(parent) == 'directory':
+            filename = self._new_name.get(trans_id)
+            if filename is not None:
+                if parent not in self._limbo_children:
+                    self._limbo_children[parent] = set()
+                    self._limbo_children_names[parent] = {}
+                    use_direct_path = True
+                # the direct path can only be used if no other file has
+                # already taken this pathname, i.e. if the name is unused, or
+                # if it is already associated with this trans_id.
+                elif self._case_sensitive_target:
+                    if (self._limbo_children_names[parent].get(filename)
+                            in (trans_id, None)):
+                        use_direct_path = True
+                else:
+                    for l_filename, l_trans_id in viewitems(
+                            self._limbo_children_names[parent]):
+                        if l_trans_id == trans_id:
+                            continue
+                        if l_filename.lower() == filename.lower():
+                            break
+                    else:
+                        use_direct_path = True
+
+        if not use_direct_path:
+            return DiskTreeTransform._generate_limbo_path(self, trans_id)
+
+        limbo_name = osutils.pathjoin(self._limbo_files[parent], filename)
+        self._limbo_children[parent].add(trans_id)
+        self._limbo_children_names[parent][filename] = trans_id
+        return limbo_name
+
+    def version_file(self, trans_id, file_id=None):
+        """Schedule a file to become versioned."""
+        if file_id is None:
+            raise ValueError()
+        unique_add(self._new_id, trans_id, file_id)
+        unique_add(self._r_new_id, file_id, trans_id)
+
+    def cancel_versioning(self, trans_id):
+        """Undo a previous versioning of a file"""
+        file_id = self._new_id[trans_id]
+        del self._new_id[trans_id]
+        del self._r_new_id[file_id]
+
+    def _duplicate_ids(self):
+        """Each inventory id may only be used once"""
+        conflicts = []
+        try:
+            all_ids = self._tree.all_file_ids()
+        except errors.UnsupportedOperation:
+            # it's okay for non-file-id trees to raise UnsupportedOperation.
+            return []
+        removed_tree_ids = set((self.tree_file_id(trans_id) for trans_id in
+                                self._removed_id))
+        active_tree_ids = all_ids.difference(removed_tree_ids)
+        for trans_id, file_id in viewitems(self._new_id):
+            if file_id in active_tree_ids:
+                path = self._tree.id2path(file_id)
+                old_trans_id = self.trans_id_tree_path(path)
+                conflicts.append(('duplicate id', old_trans_id, trans_id))
+        return conflicts
+
+    def find_conflicts(self):
+        conflicts = super(InventoryTreeTransform, self).find_conflicts()
+        conflicts.extend(self._duplicate_ids())
+        return conflicts
+
+    def apply(self, no_conflicts=False, precomputed_delta=None, _mover=None):
+        """Apply all changes to the inventory and filesystem.
+
+        If filesystem or inventory conflicts are present, MalformedTransform
+        will be thrown.
+
+        If apply succeeds, finalize is not necessary.
+
+        :param no_conflicts: if True, the caller guarantees there are no
+            conflicts, so no check is made.
+        :param precomputed_delta: An inventory delta to use instead of
+            calculating one.
+        :param _mover: Supply an alternate FileMover, for testing
+        """
+        for hook in MutableTree.hooks['pre_transform']:
+            hook(self._tree, self)
+        if not no_conflicts:
+            self._check_malformed()
+        self.rename_count = 0
+        with ui.ui_factory.nested_progress_bar() as child_pb:
+            if precomputed_delta is None:
+                child_pb.update(gettext('Apply phase'), 0, 2)
+                inventory_delta = self._generate_inventory_delta()
+                offset = 1
+            else:
+                inventory_delta = precomputed_delta
+                offset = 0
+            if _mover is None:
+                mover = _FileMover()
+            else:
+                mover = _mover
+            try:
+                child_pb.update(gettext('Apply phase'), 0 + offset, 2 + offset)
+                self._apply_removals(mover)
+                child_pb.update(gettext('Apply phase'), 1 + offset, 2 + offset)
+                modified_paths = self._apply_insertions(mover)
+            except BaseException:
+                mover.rollback()
+                raise
+            else:
+                mover.apply_deletions()
+        if self.final_file_id(self.root) is None:
+            inventory_delta = [e for e in inventory_delta if e[0] != '']
+        self._tree.apply_inventory_delta(inventory_delta)
+        self._apply_observed_sha1s()
+        self._done = True
+        self.finalize()
+        return _TransformResults(modified_paths, self.rename_count)
+
+    def _apply_removals(self, mover):
+        """Perform tree operations that remove directory/inventory names.
+
+        That is, delete files that are to be deleted, and put any files that
+        need renaming into limbo.  This must be done in strict child-to-parent
+        order.
+
+        If inventory_delta is None, no inventory delta generation is performed.
+        """
+        tree_paths = sorted(viewitems(self._tree_path_ids), reverse=True)
+        with ui.ui_factory.nested_progress_bar() as child_pb:
+            for num, (path, trans_id) in enumerate(tree_paths):
+                # do not attempt to move root into a subdirectory of itself.
+                if path == '':
+                    continue
+                child_pb.update(gettext('removing file'), num, len(tree_paths))
+                full_path = self._tree.abspath(path)
+                if trans_id in self._removed_contents:
+                    delete_path = os.path.join(self._deletiondir, trans_id)
+                    mover.pre_delete(full_path, delete_path)
+                elif (trans_id in self._new_name or
+                      trans_id in self._new_parent):
+                    try:
+                        mover.rename(full_path, self._limbo_name(trans_id))
+                    except TransformRenameFailed as e:
+                        if e.errno != errno.ENOENT:
+                            raise
+                    else:
+                        self.rename_count += 1
+
+    def _apply_insertions(self, mover):
+        """Perform tree operations that insert directory/inventory names.
+
+        That is, create any files that need to be created, and restore from
+        limbo any files that needed renaming.  This must be done in strict
+        parent-to-child order.
+
+        If inventory_delta is None, no inventory delta is calculated, and
+        no list of modified paths is returned.
+        """
+        new_paths = self.new_paths(filesystem_only=True)
+        modified_paths = []
+        with ui.ui_factory.nested_progress_bar() as child_pb:
+            for num, (path, trans_id) in enumerate(new_paths):
+                if (num % 10) == 0:
+                    child_pb.update(gettext('adding file'),
+                                    num, len(new_paths))
+                full_path = self._tree.abspath(path)
+                if trans_id in self._needs_rename:
+                    try:
+                        mover.rename(self._limbo_name(trans_id), full_path)
+                    except TransformRenameFailed as e:
+                        # We may be renaming a dangling inventory id
+                        if e.errno != errno.ENOENT:
+                            raise
+                    else:
+                        self.rename_count += 1
+                    # TODO: if trans_id in self._observed_sha1s, we should
+                    #       re-stat the final target, since ctime will be
+                    #       updated by the change.
+                if (trans_id in self._new_contents
+                        or self.path_changed(trans_id)):
+                    if trans_id in self._new_contents:
+                        modified_paths.append(full_path)
+                if trans_id in self._new_executability:
+                    self._set_executability(path, trans_id)
+                if trans_id in self._observed_sha1s:
+                    o_sha1, o_st_val = self._observed_sha1s[trans_id]
+                    st = osutils.lstat(full_path)
+                    self._observed_sha1s[trans_id] = (o_sha1, st)
+        for path, trans_id in new_paths:
+            # new_paths includes stuff like workingtree conflicts. Only the
+            # stuff in new_contents actually comes from limbo.
+            if trans_id in self._limbo_files:
+                del self._limbo_files[trans_id]
+        self._new_contents.clear()
+        return modified_paths
+
+    def _apply_observed_sha1s(self):
+        """After we have finished renaming everything, update observed sha1s
+
+        This has to be done after self._tree.apply_inventory_delta, otherwise
+        it doesn't know anything about the files we are updating. Also, we want
+        to do this as late as possible, so that most entries end up cached.
+        """
+        # TODO: this doesn't update the stat information for directories. So
+        #       the first 'bzr status' will still need to rewrite
+        #       .bzr/checkout/dirstate. However, we at least don't need to
+        #       re-read all of the files.
+        # TODO: If the operation took a while, we could do a time.sleep(3) here
+        #       to allow the clock to tick over and ensure we won't have any
+        #       problems. (we could observe start time, and finish time, and if
+        #       it is less than eg 10% overhead, add a sleep call.)
+        paths = FinalPaths(self)
+        for trans_id, observed in viewitems(self._observed_sha1s):
+            path = paths.get_path(trans_id)
+            self._tree._observed_sha1(path, observed)
+
+    def get_preview_tree(self):
+        """Return a tree representing the result of the transform.
+
+        The tree is a snapshot, and altering the TreeTransform will invalidate
+        it.
+        """
+        return InventoryPreviewTree(self)
+
+    def _inventory_altered(self):
+        """Determine which trans_ids need new Inventory entries.
+
+        An new entry is needed when anything that would be reflected by an
+        inventory entry changes, including file name, file_id, parent file_id,
+        file kind, and the execute bit.
+
+        Some care is taken to return entries with real changes, not cases
+        where the value is deleted and then restored to its original value,
+        but some actually unchanged values may be returned.
+
+        :returns: A list of (path, trans_id) for all items requiring an
+            inventory change. Ordered by path.
+        """
+        changed_ids = set()
+        # Find entries whose file_ids are new (or changed).
+        new_file_id = set(t for t in self._new_id
+                          if self._new_id[t] != self.tree_file_id(t))
+        for id_set in [self._new_name, self._new_parent, new_file_id,
+                       self._new_executability]:
+            changed_ids.update(id_set)
+        # removing implies a kind change
+        changed_kind = set(self._removed_contents)
+        # so does adding
+        changed_kind.intersection_update(self._new_contents)
+        # Ignore entries that are already known to have changed.
+        changed_kind.difference_update(changed_ids)
+        #  to keep only the truly changed ones
+        changed_kind = (t for t in changed_kind
+                        if self.tree_kind(t) != self.final_kind(t))
+        # all kind changes will alter the inventory
+        changed_ids.update(changed_kind)
+        # To find entries with changed parent_ids, find parents which existed,
+        # but changed file_id.
+        # Now add all their children to the set.
+        for parent_trans_id in new_file_id:
+            changed_ids.update(self.iter_tree_children(parent_trans_id))
+        return sorted(FinalPaths(self).get_paths(changed_ids))
+
+    def _generate_inventory_delta(self):
+        """Generate an inventory delta for the current transform."""
+        inventory_delta = []
+        new_paths = self._inventory_altered()
+        total_entries = len(new_paths) + len(self._removed_id)
+        with ui.ui_factory.nested_progress_bar() as child_pb:
+            for num, trans_id in enumerate(self._removed_id):
+                if (num % 10) == 0:
+                    child_pb.update(gettext('removing file'),
+                                    num, total_entries)
+                if trans_id == self._new_root:
+                    file_id = self._tree.path2id('')
+                else:
+                    file_id = self.tree_file_id(trans_id)
+                # File-id isn't really being deleted, just moved
+                if file_id in self._r_new_id:
+                    continue
+                path = self._tree_id_paths[trans_id]
+                inventory_delta.append((path, None, file_id, None))
+            new_path_file_ids = dict((t, self.final_file_id(t)) for p, t in
+                                     new_paths)
+            for num, (path, trans_id) in enumerate(new_paths):
+                if (num % 10) == 0:
+                    child_pb.update(gettext('adding file'),
+                                    num + len(self._removed_id), total_entries)
+                file_id = new_path_file_ids[trans_id]
+                if file_id is None:
+                    continue
+                kind = self.final_kind(trans_id)
+                if kind is None:
+                    kind = self._tree.stored_kind(self._tree.id2path(file_id))
+                parent_trans_id = self.final_parent(trans_id)
+                parent_file_id = new_path_file_ids.get(parent_trans_id)
+                if parent_file_id is None:
+                    parent_file_id = self.final_file_id(parent_trans_id)
+                if trans_id in self._new_reference_revision:
+                    new_entry = inventory.TreeReference(
+                        file_id,
+                        self._new_name[trans_id],
+                        self.final_file_id(self._new_parent[trans_id]),
+                        None, self._new_reference_revision[trans_id])
+                else:
+                    new_entry = inventory.make_entry(kind,
+                                                     self.final_name(trans_id),
+                                                     parent_file_id, file_id)
+                try:
+                    old_path = self._tree.id2path(new_entry.file_id)
+                except errors.NoSuchId:
+                    old_path = None
+                new_executability = self._new_executability.get(trans_id)
+                if new_executability is not None:
+                    new_entry.executable = new_executability
+                inventory_delta.append(
+                    (old_path, path, new_entry.file_id, new_entry))
+        return inventory_delta
+
+
+class TransformPreview(InventoryTreeTransform):
+    """A TreeTransform for generating preview trees.
+
+    Unlike TreeTransform, this version works when the input tree is a
+    RevisionTree, rather than a WorkingTree.  As a result, it tends to ignore
+    unversioned files in the input tree.
+    """
+
+    def __init__(self, tree, pb=None, case_sensitive=True):
+        tree.lock_read()
+        limbodir = osutils.mkdtemp(prefix='bzr-limbo-')
+        DiskTreeTransform.__init__(self, tree, limbodir, pb, case_sensitive)
+
+    def canonical_path(self, path):
+        return path
+
+    def tree_kind(self, trans_id):
+        path = self._tree_id_paths.get(trans_id)
+        if path is None:
+            return None
+        kind = self._tree.path_content_summary(path)[0]
+        if kind == 'missing':
+            kind = None
+        return kind
+
+    def _set_mode(self, trans_id, mode_id, typefunc):
+        """Set the mode of new file contents.
+        The mode_id is the existing file to get the mode from (often the same
+        as trans_id).  The operation is only performed if there's a mode match
+        according to typefunc.
+        """
+        # is it ok to ignore this?  probably
+        pass
+
+    def iter_tree_children(self, parent_id):
+        """Iterate through the entry's tree children, if any"""
+        try:
+            path = self._tree_id_paths[parent_id]
+        except KeyError:
+            return
+        try:
+            entry = next(self._tree.iter_entries_by_dir(
+                specific_files=[path]))[1]
+        except StopIteration:
+            return
+        children = getattr(entry, 'children', {})
+        for child in children:
+            childpath = joinpath(path, child)
+            yield self.trans_id_tree_path(childpath)
+
+    def new_orphan(self, trans_id, parent_id):
+        raise NotImplementedError(self.new_orphan)
+
+
+class InventoryPreviewTree(PreviewTree, inventorytree.InventoryTree):
+    """Partial implementation of Tree to support show_diff_trees"""
+
+    def __init__(self, transform):
+        PreviewTree.__init__(self, transform)
+        self._final_paths = FinalPaths(transform)
+        self._iter_changes_cache = {
+            c.file_id: c for c in self._transform.iter_changes()}
+
+    def supports_tree_reference(self):
+        # TODO(jelmer): Support tree references in PreviewTree.
+        # return self._transform._tree.supports_tree_reference()
+        return False
+
+    def _content_change(self, file_id):
+        """Return True if the content of this file changed"""
+        changes = self._iter_changes_cache.get(file_id)
+        return (changes is not None and changes.changed_content)
+
+    def _get_file_revision(self, path, file_id, vf, tree_revision):
+        parent_keys = [
+            (file_id, t.get_file_revision(t.id2path(file_id)))
+            for t in self._iter_parent_trees()]
+        vf.add_lines((file_id, tree_revision), parent_keys,
+                     self.get_file_lines(path))
+        repo = self._get_repository()
+        base_vf = repo.texts
+        if base_vf not in vf.fallback_versionedfiles:
+            vf.fallback_versionedfiles.append(base_vf)
+        return tree_revision
+
+    def _stat_limbo_file(self, trans_id):
+        name = self._transform._limbo_name(trans_id)
+        return os.lstat(name)
+
+    def _comparison_data(self, entry, path):
+        kind, size, executable, link_or_sha1 = self.path_content_summary(path)
+        if kind == 'missing':
+            kind = None
+            executable = False
+        else:
+            file_id = self._transform.final_file_id(self._path2trans_id(path))
+            executable = self.is_executable(path)
+        return kind, executable, None
+
+    @property
+    def root_inventory(self):
+        """This Tree does not use inventory as its backing data."""
+        raise NotImplementedError(PreviewTree.root_inventory)
+
+    def all_file_ids(self):
+        tree_ids = set(self._transform._tree.all_file_ids())
+        tree_ids.difference_update(self._transform.tree_file_id(t)
+                                   for t in self._transform._removed_id)
+        tree_ids.update(viewvalues(self._transform._new_id))
+        return tree_ids
+
+    def all_versioned_paths(self):
+        tree_paths = set(self._transform._tree.all_versioned_paths())
+
+        tree_paths.difference_update(
+            self._transform.trans_id_tree_path(t)
+            for t in self._transform._removed_id)
+
+        tree_paths.update(
+            self._final_paths._determine_path(t)
+            for t in self._transform._new_id)
+
+        return tree_paths
+
+    def path2id(self, path):
+        if isinstance(path, list):
+            if path == []:
+                path = [""]
+            path = osutils.pathjoin(*path)
+        return self._transform.final_file_id(self._path2trans_id(path))
+
+    def id2path(self, file_id, recurse='down'):
+        trans_id = self._transform.trans_id_file_id(file_id)
+        try:
+            return self._final_paths._determine_path(trans_id)
+        except NoFinalPath:
+            raise errors.NoSuchId(self, file_id)
+
+    def extras(self):
+        possible_extras = set(self._transform.trans_id_tree_path(p) for p
+                              in self._transform._tree.extras())
+        possible_extras.update(self._transform._new_contents)
+        possible_extras.update(self._transform._removed_id)
+        for trans_id in possible_extras:
+            if self._transform.final_file_id(trans_id) is None:
+                yield self._final_paths._determine_path(trans_id)
+
+    def _make_inv_entries(self, ordered_entries, specific_files=None):
+        for trans_id, parent_file_id in ordered_entries:
+            file_id = self._transform.final_file_id(trans_id)
+            if file_id is None:
+                continue
+            if (specific_files is not None
+                    and self._final_paths.get_path(trans_id) not in specific_files):
+                continue
+            kind = self._transform.final_kind(trans_id)
+            if kind is None:
+                kind = self._transform._tree.stored_kind(
+                    self._transform._tree.id2path(file_id))
+            new_entry = inventory.make_entry(
+                kind,
+                self._transform.final_name(trans_id),
+                parent_file_id, file_id)
+            yield new_entry, trans_id
+
+    def _list_files_by_dir(self):
+        todo = [ROOT_PARENT]
+        ordered_ids = []
+        while len(todo) > 0:
+            parent = todo.pop()
+            parent_file_id = self._transform.final_file_id(parent)
+            children = list(self._all_children(parent))
+            paths = dict(zip(children, self._final_paths.get_paths(children)))
+            children.sort(key=paths.get)
+            todo.extend(reversed(children))
+            for trans_id in children:
+                ordered_ids.append((trans_id, parent_file_id))
+        return ordered_ids
+
+    def iter_child_entries(self, path):
+        trans_id = self._path2trans_id(path)
+        if trans_id is None:
+            raise errors.NoSuchFile(path)
+        todo = [(child_trans_id, trans_id) for child_trans_id in
+                self._all_children(trans_id)]
+        for entry, trans_id in self._make_inv_entries(todo):
+            yield entry
+
+    def iter_entries_by_dir(self, specific_files=None, recurse_nested=False):
+        if recurse_nested:
+            raise NotImplementedError(
+                'follow tree references not yet supported')
+
+        # This may not be a maximally efficient implementation, but it is
+        # reasonably straightforward.  An implementation that grafts the
+        # TreeTransform changes onto the tree's iter_entries_by_dir results
+        # might be more efficient, but requires tricky inferences about stack
+        # position.
+        ordered_ids = self._list_files_by_dir()
+        for entry, trans_id in self._make_inv_entries(ordered_ids,
+                                                      specific_files):
+            yield self._final_paths.get_path(trans_id), entry
+
+    def _iter_entries_for_dir(self, dir_path):
+        """Return path, entry for items in a directory without recursing down."""
+        ordered_ids = []
+        dir_trans_id = self._path2trans_id(dir_path)
+        dir_id = self._transform.final_file_id(dir_trans_id)
+        for child_trans_id in self._all_children(dir_trans_id):
+            ordered_ids.append((child_trans_id, dir_id))
+        path_entries = []
+        for entry, trans_id in self._make_inv_entries(ordered_ids):
+            path_entries.append((self._final_paths.get_path(trans_id), entry))
+        path_entries.sort()
+        return path_entries
+
+    def list_files(self, include_root=False, from_dir=None, recursive=True,
+                   recurse_nested=False):
+        """See WorkingTree.list_files."""
+        if recurse_nested:
+            raise NotImplementedError(
+                'follow tree references not yet supported')
+
+        # XXX This should behave like WorkingTree.list_files, but is really
+        # more like RevisionTree.list_files.
+        if from_dir == '.':
+            from_dir = None
+        if recursive:
+            prefix = None
+            if from_dir:
+                prefix = from_dir + '/'
+            entries = self.iter_entries_by_dir()
+            for path, entry in entries:
+                if entry.name == '' and not include_root:
+                    continue
+                if prefix:
+                    if not path.startswith(prefix):
+                        continue
+                    path = path[len(prefix):]
+                yield path, 'V', entry.kind, entry
+        else:
+            if from_dir is None and include_root is True:
+                root_entry = inventory.make_entry(
+                    'directory', '', ROOT_PARENT, self.path2id(''))
+                yield '', 'V', 'directory', root_entry
+            entries = self._iter_entries_for_dir(from_dir or '')
+            for path, entry in entries:
+                yield path, 'V', entry.kind, entry
+
+    def get_file_mtime(self, path):
+        """See Tree.get_file_mtime"""
+        file_id = self.path2id(path)
+        if file_id is None:
+            raise errors.NoSuchFile(path)
+        if not self._content_change(file_id):
+            return self._transform._tree.get_file_mtime(
+                self._transform._tree.id2path(file_id))
+        trans_id = self._path2trans_id(path)
+        return self._stat_limbo_file(trans_id).st_mtime
+
+    def path_content_summary(self, path):
+        trans_id = self._path2trans_id(path)
+        tt = self._transform
+        tree_path = tt._tree_id_paths.get(trans_id)
+        kind = tt._new_contents.get(trans_id)
+        if kind is None:
+            if tree_path is None or trans_id in tt._removed_contents:
+                return 'missing', None, None, None
+            summary = tt._tree.path_content_summary(tree_path)
+            kind, size, executable, link_or_sha1 = summary
+        else:
+            link_or_sha1 = None
+            limbo_name = tt._limbo_name(trans_id)
+            if trans_id in tt._new_reference_revision:
+                kind = 'tree-reference'
+            if kind == 'file':
+                statval = os.lstat(limbo_name)
+                size = statval.st_size
+                if not tt._limbo_supports_executable():
+                    executable = False
+                else:
+                    executable = statval.st_mode & S_IEXEC
+            else:
+                size = None
+                executable = None
+            if kind == 'symlink':
+                link_or_sha1 = os.readlink(limbo_name)
+                if not isinstance(link_or_sha1, text_type):
+                    link_or_sha1 = link_or_sha1.decode(osutils._fs_enc)
+        executable = tt._new_executability.get(trans_id, executable)
+        return kind, size, executable, link_or_sha1
+
+    def iter_changes(self, from_tree, include_unchanged=False,
+                     specific_files=None, pb=None, extra_trees=None,
+                     require_versioned=True, want_unversioned=False):
+        """See InterTree.iter_changes.
+
+        This has a fast path that is only used when the from_tree matches
+        the transform tree, and no fancy options are supplied.
+        """
+        if (from_tree is not self._transform._tree or include_unchanged
+                or specific_files or want_unversioned):
+            return tree.InterTree.get(from_tree, self).iter_changes(
+                include_unchanged=include_unchanged,
+                specific_files=specific_files,
+                pb=pb,
+                extra_trees=extra_trees,
+                require_versioned=require_versioned,
+                want_unversioned=want_unversioned)
+        if want_unversioned:
+            raise ValueError('want_unversioned is not supported')
+        return self._transform.iter_changes()
+
+    def annotate_iter(self, path,
+                      default_revision=_mod_revision.CURRENT_REVISION):
+        file_id = self.path2id(path)
+        changes = self._iter_changes_cache.get(file_id)
+        if changes is None:
+            get_old = True
+        else:
+            changed_content, versioned, kind = (
+                changes.changed_content, changes.versioned, changes.kind)
+            if kind[1] is None:
+                return None
+            get_old = (kind[0] == 'file' and versioned[0])
+        if get_old:
+            old_annotation = self._transform._tree.annotate_iter(
+                path, default_revision=default_revision)
+        else:
+            old_annotation = []
+        if changes is None:
+            return old_annotation
+        if not changed_content:
+            return old_annotation
+        # TODO: This is doing something similar to what WT.annotate_iter is
+        #       doing, however it fails slightly because it doesn't know what
+        #       the *other* revision_id is, so it doesn't know how to give the
+        #       other as the origin for some lines, they all get
+        #       'default_revision'
+        #       It would be nice to be able to use the new Annotator based
+        #       approach, as well.
+        return annotate.reannotate([old_annotation],
+                                   self.get_file(path).readlines(),
+                                   default_revision)
+
+    def walkdirs(self, prefix=''):
+        pending = [self._transform.root]
+        while len(pending) > 0:
+            parent_id = pending.pop()
+            children = []
+            subdirs = []
+            prefix = prefix.rstrip('/')
+            parent_path = self._final_paths.get_path(parent_id)
+            parent_file_id = self._transform.final_file_id(parent_id)
+            for child_id in self._all_children(parent_id):
+                path_from_root = self._final_paths.get_path(child_id)
+                basename = self._transform.final_name(child_id)
+                file_id = self._transform.final_file_id(child_id)
+                kind = self._transform.final_kind(child_id)
+                if kind is not None:
+                    versioned_kind = kind
+                else:
+                    kind = 'unknown'
+                    versioned_kind = self._transform._tree.stored_kind(
+                        self._transform._tree.id2path(file_id))
+                if versioned_kind == 'directory':
+                    subdirs.append(child_id)
+                children.append((path_from_root, basename, kind, None,
+                                 file_id, versioned_kind))
+            children.sort()
+            if parent_path.startswith(prefix):
+                yield (parent_path, parent_file_id), children
+            pending.extend(sorted(subdirs, key=self._final_paths.get_path,
+                                  reverse=True))
+
+    def get_symlink_target(self, path):
+        """See Tree.get_symlink_target"""
+        file_id = self.path2id(path)
+        if not self._content_change(file_id):
+            return self._transform._tree.get_symlink_target(path)
+        trans_id = self._path2trans_id(path)
+        name = self._transform._limbo_name(trans_id)
+        return osutils.readlink(name)
+
+    def get_file(self, path):
+        """See Tree.get_file"""
+        file_id = self.path2id(path)
+        if not self._content_change(file_id):
+            return self._transform._tree.get_file(path)
+        trans_id = self._path2trans_id(path)
+        name = self._transform._limbo_name(trans_id)
+        return open(name, 'rb')

=== modified file 'breezy/bzr/versionedfile.py'
--- breezy/bzr/versionedfile.py	2020-06-19 21:26:53 +0000
+++ breezy/bzr/versionedfile.py	2020-07-18 23:14:00 +0000
@@ -64,6 +64,23 @@
                                    'breezy.bzr.knit', 'DeltaAnnotatedToFullText')
 
 
+class UnavailableRepresentation(errors.InternalBzrError):
+
+    _fmt = ("The encoding '%(wanted)s' is not available for key %(key)s which "
+            "is encoded as '%(native)s'.")
+
+    def __init__(self, key, wanted, native):
+        errors.InternalBzrError.__init__(self)
+        self.wanted = wanted
+        self.native = native
+        self.key = key
+
+
+class ExistingContent(errors.BzrError):
+
+    _fmt = "The content being inserted is already present."
+
+
 class ContentFactory(object):
     """Abstract interface for insertion and retrieval from a VersionedFile.
 
@@ -127,8 +144,8 @@
             if self._chunks_are_lines:
                 return self._chunks
             return list(osutils.chunks_to_lines(self._chunks))
-        raise errors.UnavailableRepresentation(self.key, storage_kind,
-                                               self.storage_kind)
+        raise UnavailableRepresentation(self.key, storage_kind,
+                                        self.storage_kind)
 
     def iter_bytes_as(self, storage_kind):
         if storage_kind == 'chunked':
@@ -137,8 +154,8 @@
             if self._chunks_are_lines:
                 return iter(self._chunks)
             return iter(osutils.chunks_to_lines(self._chunks))
-        raise errors.UnavailableRepresentation(self.key, storage_kind,
-                                               self.storage_kind)
+        raise UnavailableRepresentation(self.key, storage_kind,
+                                        self.storage_kind)
 
 class FulltextContentFactory(ContentFactory):
     """Static data content factory.
@@ -174,16 +191,16 @@
             return [self._text]
         elif storage_kind == 'lines':
             return osutils.split_lines(self._text)
-        raise errors.UnavailableRepresentation(self.key, storage_kind,
-                                               self.storage_kind)
+        raise UnavailableRepresentation(self.key, storage_kind,
+                                        self.storage_kind)
 
     def iter_bytes_as(self, storage_kind):
         if storage_kind == 'chunked':
             return iter([self._text])
         elif storage_kind == 'lines':
             return iter(osutils.split_lines(self._text))
-        raise errors.UnavailableRepresentation(self.key, storage_kind,
-                                               self.storage_kind)
+        raise UnavailableRepresentation(self.key, storage_kind,
+                                        self.storage_kind)
 
 
 class FileContentFactory(ContentFactory):
@@ -206,8 +223,8 @@
             return list(osutils.file_iterator(self.file))
         elif storage_kind == 'lines':
             return list(self.file.readlines())
-        raise errors.UnavailableRepresentation(self.key, storage_kind,
-                                               self.storage_kind)
+        raise UnavailableRepresentation(self.key, storage_kind,
+                                        self.storage_kind)
 
     def iter_bytes_as(self, storage_kind):
         self.file.seek(0)
@@ -215,8 +232,8 @@
             return osutils.file_iterator(self.file)
         elif storage_kind == 'lines':
             return self.file
-        raise errors.UnavailableRepresentation(self.key, storage_kind,
-                                               self.storage_kind)
+        raise UnavailableRepresentation(self.key, storage_kind,
+                                        self.storage_kind)
 
 
 class AbsentContentFactory(ContentFactory):

=== modified file 'breezy/bzr/vf_repository.py'
--- breezy/bzr/vf_repository.py	2020-06-01 13:47:29 +0000
+++ breezy/bzr/vf_repository.py	2020-07-18 23:14:00 +0000
@@ -492,7 +492,7 @@
                             file_id, file_obj, heads, nostore_sha,
                             size=(stat_value.st_size if stat_value else None))
                         yield change.path[1], (entry.text_sha1, stat_value)
-                    except errors.ExistingContent:
+                    except versionedfile.ExistingContent:
                         # No content change against a carry_over parent
                         # Perhaps this should also yield a fs hash update?
                         carried_over = True
@@ -1556,61 +1556,6 @@
         for inv in inventories:
             yield inventorytree.InventoryRevisionTree(self, inv, inv.revision_id)
 
-    def get_deltas_for_revisions(self, revisions, specific_fileids=None):
-        """Produce a generator of revision deltas.
-
-        Note that the input is a sequence of REVISIONS, not revision_ids.
-        Trees will be held in memory until the generator exits.
-        Each delta is relative to the revision's lefthand predecessor.
-
-        :param specific_fileids: if not None, the result is filtered
-          so that only those file-ids, their parents and their
-          children are included.
-        """
-        # Get the revision-ids of interest
-        required_trees = set()
-        for revision in revisions:
-            required_trees.add(revision.revision_id)
-            required_trees.update(revision.parent_ids[:1])
-
-        # Get the matching filtered trees. Note that it's more
-        # efficient to pass filtered trees to changes_from() rather
-        # than doing the filtering afterwards. changes_from() could
-        # arguably do the filtering itself but it's path-based, not
-        # file-id based, so filtering before or afterwards is
-        # currently easier.
-        if specific_fileids is None:
-            trees = dict((t.get_revision_id(), t) for
-                         t in self.revision_trees(required_trees))
-        else:
-            trees = dict((t.get_revision_id(), t) for
-                         t in self._filtered_revision_trees(required_trees,
-                                                            specific_fileids))
-
-        # Calculate the deltas
-        for revision in revisions:
-            if not revision.parent_ids:
-                old_tree = self.revision_tree(_mod_revision.NULL_REVISION)
-            else:
-                old_tree = trees[revision.parent_ids[0]]
-            yield trees[revision.revision_id].changes_from(old_tree)
-
-    def _filtered_revision_trees(self, revision_ids, file_ids):
-        """Return Tree for a revision on this branch with only some files.
-
-        :param revision_ids: a sequence of revision-ids;
-          a revision-id may not be None or b'null:'
-        :param file_ids: if not None, the result is filtered
-          so that only those file-ids, their parents and their
-          children are included.
-        """
-        inventories = self.iter_inventories(revision_ids)
-        for inv in inventories:
-            # Should we introduce a FilteredRevisionTree class rather
-            # than pre-filter the inventory here?
-            filtered_inv = inv.filter(file_ids)
-            yield inventorytree.InventoryRevisionTree(self, filtered_inv, filtered_inv.revision_id)
-
     def get_parent_map(self, revision_ids):
         """See graph.StackedParentsProvider.get_parent_map"""
         # revisions index works in keys; this just works in revisions

=== modified file 'breezy/bzr/weave.py'
--- breezy/bzr/weave.py	2020-06-19 21:26:53 +0000
+++ breezy/bzr/weave.py	2020-07-18 23:14:00 +0000
@@ -80,7 +80,6 @@
 from ..errors import (
     RevisionAlreadyPresent,
     RevisionNotPresent,
-    UnavailableRepresentation,
     )
 from ..osutils import dirname, sha, sha_strings, split_lines
 from ..revision import NULL_REVISION
@@ -89,7 +88,9 @@
     AbsentContentFactory,
     adapter_registry,
     ContentFactory,
+    ExistingContent,
     sort_groupcompress,
+    UnavailableRepresentation,
     VersionedFile,
     )
 from .weavefile import _read_weave_v5, write_weave_v5
@@ -484,7 +485,7 @@
         if not sha1:
             sha1 = sha_strings(lines)
         if sha1 == nostore_sha:
-            raise errors.ExistingContent
+            raise ExistingContent
         if version_id is None:
             version_id = b"sha1:" + sha1
         if version_id in self._name_map:

=== modified file 'breezy/bzr/workingtree.py'
--- breezy/bzr/workingtree.py	2020-02-18 01:57:45 +0000
+++ breezy/bzr/workingtree.py	2020-07-18 23:14:00 +0000
@@ -61,6 +61,7 @@
     )
 from breezy.bzr import (
     inventory,
+    serializer,
     xml5,
     xml7,
     )
@@ -96,6 +97,15 @@
 ERROR_PATH_NOT_FOUND = 3    # WindowsError errno code, equivalent to ENOENT
 
 
+class InventoryModified(errors.InternalBzrError):
+
+    _fmt = ("The current inventory for the tree %(tree)r has been modified,"
+            " so a clean inventory cannot be read without data loss.")
+
+    def __init__(self, tree):
+        self.tree = tree
+
+
 class InventoryWorkingTree(WorkingTree, MutableInventoryTree):
     """Base class for working trees that are inventory-oriented.
 
@@ -159,9 +169,9 @@
         else:
             self.case_sensitive = False
 
-    def get_transform(self, pb=None):
-        from ..transform import TreeTransform
-        return TreeTransform(self, pb=pb)
+    def transform(self, pb=None):
+        from .transform import InventoryTreeTransform
+        return InventoryTreeTransform(self, pb=pb)
 
     def _setup_directory_is_tree_reference(self):
         if self._branch.repository._format.supports_tree_reference:
@@ -650,7 +660,7 @@
         # binary.
         with self.lock_read():
             if self._inventory_is_modified:
-                raise errors.InventoryModified(self)
+                raise InventoryModified(self)
             with self._transport.get('inventory') as f:
                 result = self._deserialize(f)
             self._set_inventory(result, dirty=False)
@@ -807,7 +817,7 @@
                     if inv.revision_id == revision_id:
                         return InventoryRevisionTree(
                             self.branch.repository, inv, revision_id)
-                except errors.BadInventoryFormat:
+                except serializer.BadInventoryFormat:
                     pass
         # raise if there was no inventory, or if we read the wrong inventory.
         raise errors.NoSuchRevisionInTree(self, revision_id)

=== modified file 'breezy/bzr/workingtree_4.py'
--- breezy/bzr/workingtree_4.py	2020-06-10 23:47:24 +0000
+++ breezy/bzr/workingtree_4.py	2020-07-18 23:14:00 +0000
@@ -59,6 +59,7 @@
 from ..lockdir import LockDir
 from .inventorytree import (
     InventoryTree,
+    InterInventoryTree,
     InventoryRevisionTree,
     )
 from ..mutabletree import (
@@ -2168,7 +2169,7 @@
                     pending.append((dir[0], dir[4]))
 
 
-class InterDirStateTree(InterTree):
+class InterDirStateTree(InterInventoryTree):
     """Fast path optimiser for changes_from with dirstate trees.
 
     This is used only when both trees are in the dirstate working file, and

=== modified file 'breezy/bzr/xml5.py'
--- breezy/bzr/xml5.py	2020-02-18 01:57:45 +0000
+++ breezy/bzr/xml5.py	2020-07-18 23:14:00 +0000
@@ -73,8 +73,7 @@
                 raise errors.BzrError("parent_id {%s} not in inventory"
                                       % (parent_id,))
             if ie.file_id in byid:
-                raise errors.DuplicateFileId(ie.file_id,
-                                             byid[ie.file_id])
+                raise inventory.DuplicateFileId(ie.file_id, byid[ie.file_id])
             if ie.name in parent.children:
                 raise errors.BzrError(
                     "%s is already versioned" % (

=== modified file 'breezy/bzr/xml_serializer.py'
--- breezy/bzr/xml_serializer.py	2020-02-18 01:57:45 +0000
+++ breezy/bzr/xml_serializer.py	2020-07-18 23:14:00 +0000
@@ -83,7 +83,7 @@
                                           entry_cache=entry_cache,
                                           return_from_cache=return_from_cache)
         except ParseError as e:
-            raise errors.UnexpectedInventoryFormat(str(e))
+            raise serializer.UnexpectedInventoryFormat(str(e))
 
     def read_inventory(self, f, revision_id=None):
         try:
@@ -93,7 +93,7 @@
             finally:
                 f.close()
         except ParseError as e:
-            raise errors.UnexpectedInventoryFormat(str(e))
+            raise serializer.UnexpectedInventoryFormat(str(e))
 
     def write_revision_to_string(self, rev):
         return b''.join(self.write_revision_to_lines(rev))
@@ -317,7 +317,7 @@
         ie = inventory.TreeReference(file_id, name, parent_id, revision,
                                      reference_revision)
     else:
-        raise errors.UnsupportedInventoryKind(kind)
+        raise serializer.UnsupportedInventoryKind(kind)
     ie.revision = revision
     if revision is not None and entry_cache is not None:
         # We cache a copy() because callers like to mutate objects, and
@@ -341,11 +341,11 @@
         encountered
     """
     if elt.tag != 'inventory':
-        raise errors.UnexpectedInventoryFormat('Root tag is %r' % elt.tag)
+        raise serializer.UnexpectedInventoryFormat('Root tag is %r' % elt.tag)
     format = elt.get('format')
     if ((format is None and format_num is not None) or
             format.encode() != format_num):
-        raise errors.UnexpectedInventoryFormat('Invalid format version %r'
+        raise serializer.UnexpectedInventoryFormat('Invalid format version %r'
                                                % format)
     revision_id = elt.get('revision_id')
     if revision_id is not None:
@@ -419,7 +419,7 @@
                     parent_str))
         elif ie.kind == 'tree-reference':
             if ie.kind not in supported_kinds:
-                raise errors.UnsupportedInventoryKind(ie.kind)
+                raise serializer.UnsupportedInventoryKind(ie.kind)
             if not working:
                 append(b'<tree-reference file_id="%s" name="%s"%s '
                        b'revision="%s" reference_revision="%s" />\n' % (
@@ -434,5 +434,5 @@
                     encode_and_escape(ie.name),
                     parent_str))
         else:
-            raise errors.UnsupportedInventoryKind(ie.kind)
+            raise serializer.UnsupportedInventoryKind(ie.kind)
     append(b'</inventory>\n')

=== modified file 'breezy/conflicts.py'
--- breezy/conflicts.py	2020-06-23 01:02:30 +0000
+++ breezy/conflicts.py	2020-07-18 23:14:00 +0000
@@ -361,7 +361,7 @@
         # Stanza is purely a Unicode api.
         if isinstance(file_id, str):
             file_id = cache_utf8.encode(file_id)
-        self.file_id = osutils.safe_file_id(file_id)
+        self.file_id = file_id
 
     def as_stanza(self):
         s = rio.Stanza(type=self.typestring, path=self.path)
@@ -456,7 +456,7 @@
         raise NotImplementedError(self.action_take_other)
 
     def _resolve_with_cleanups(self, tree, *args, **kwargs):
-        with tree.get_transform() as tt:
+        with tree.transform() as tt:
             self._resolve(tt, *args, **kwargs)
 
 
@@ -515,7 +515,7 @@
             tree = self._revision_tree(tt._tree, revid)
             transform.create_from_tree(
                 tt, tid, tree, tree.id2path(file_id))
-            tt.version_file(file_id, tid)
+            tt.version_file(tid, file_id=file_id)
         else:
             tid = tt.trans_id_file_id(file_id)
         # Adjust the path for the retained file id
@@ -669,7 +669,7 @@
                        item_parent_tid, item_tid)
         # Associate the file_id to the right content
         tt.unversion_file(item_tid)
-        tt.version_file(self.file_id, winner_tid)
+        tt.version_file(winner_tid, file_id=self.file_id)
         tt.apply()
 
     def action_auto(self, tree):
@@ -735,7 +735,7 @@
         # so they can be unicode.
         if isinstance(conflict_file_id, str):
             conflict_file_id = cache_utf8.encode(conflict_file_id)
-        self.conflict_file_id = osutils.safe_file_id(conflict_file_id)
+        self.conflict_file_id = conflict_file_id
 
     def _cmp_list(self):
         return HandledConflict._cmp_list(self) + [self.conflict_path,
@@ -793,7 +793,7 @@
         pass
 
     def action_take_other(self, tree):
-        with tree.get_transform() as tt:
+        with tree.transform() as tt:
             p_tid = tt.trans_id_file_id(self.file_id)
             parent_tid = tt.get_tree_parent(p_tid)
             cp_tid = tt.trans_id_file_id(self.conflict_file_id)

=== modified file 'breezy/errors.py'
--- breezy/errors.py	2020-06-23 01:02:30 +0000
+++ breezy/errors.py	2020-07-18 23:14:00 +0000
@@ -230,15 +230,6 @@
     _fmt = "The branch '%(branch)s' is not stacked."
 
 
-class InventoryModified(InternalBzrError):
-
-    _fmt = ("The current inventory for the tree %(tree)r has been modified,"
-            " so a clean inventory cannot be read without data loss.")
-
-    def __init__(self, tree):
-        self.tree = tree
-
-
 class NoWorkingTree(BzrError):
 
     _fmt = 'No WorkingTree exists for "%(base)s".'
@@ -386,18 +377,6 @@
     _fmt = 'Permission denied: "%(path)s"%(extra)s'
 
 
-class UnavailableRepresentation(InternalBzrError):
-
-    _fmt = ("The encoding '%(wanted)s' is not available for key %(key)s which "
-            "is encoded as '%(native)s'.")
-
-    def __init__(self, key, wanted, native):
-        InternalBzrError.__init__(self)
-        self.wanted = wanted
-        self.native = native
-        self.key = key
-
-
 class UnsupportedProtocol(PathError):
 
     _fmt = 'Unsupported protocol for url "%(path)s"%(extra)s'
@@ -1310,6 +1289,31 @@
         TransportError.__init__(self, msg, orig_error=orig_error)
 
 
+class UnexpectedHttpStatus(InvalidHttpResponse):
+
+    _fmt = "Unexpected HTTP status %(code)d for %(path)s"
+
+    def __init__(self, path, code, msg=None):
+        self.path = path
+        self.code = code
+        self.msg = msg
+        full_msg = 'status code %d unexpected' % code
+        if msg is not None:
+            full_msg += ': ' + msg
+        InvalidHttpResponse.__init__(
+            self, path, full_msg)
+
+
+class BadHttpRequest(UnexpectedHttpStatus):
+
+    _fmt = "Bad http request for %(path)s: %(reason)s"
+
+    def __init__(self, path, reason):
+        self.path = path
+        self.reason = reason
+        TransportError.__init__(self, reason)
+
+
 class InvalidHttpRange(InvalidHttpResponse):
 
     _fmt = "Invalid http range %(range)r for %(path)s: %(msg)s"
@@ -1465,16 +1469,6 @@
         self.file_id = file_id
 
 
-class DuplicateFileId(BzrError):
-
-    _fmt = "File id {%(file_id)s} already exists in inventory as %(entry)s"
-
-    def __init__(self, file_id, entry):
-        BzrError.__init__(self)
-        self.file_id = file_id
-        self.entry = entry
-
-
 class DuplicateKey(BzrError):
 
     _fmt = "Key %(key)s is already present in map"
@@ -1488,23 +1482,6 @@
         self.prefix = prefix
 
 
-class MalformedTransform(InternalBzrError):
-
-    _fmt = "Tree transform is malformed %(conflicts)r"
-
-
-class NoFinalPath(BzrError):
-
-    _fmt = ("No final name for trans_id %(trans_id)r\n"
-            "file-id: %(file_id)r\n"
-            "root trans-id: %(root_trans_id)r\n")
-
-    def __init__(self, trans_id, transform):
-        self.trans_id = trans_id
-        self.file_id = transform.final_file_id(trans_id)
-        self.root_trans_id = transform.root
-
-
 class BzrBadParameter(InternalBzrError):
 
     _fmt = "Bad parameter: %(param)r"
@@ -1522,27 +1499,6 @@
     _fmt = "Parameter %(param)s is neither unicode nor utf8."
 
 
-class ReusingTransform(BzrError):
-
-    _fmt = "Attempt to reuse a transform that has already been applied."
-
-
-class CantMoveRoot(BzrError):
-
-    _fmt = "Moving the root directory is not supported at this time"
-
-
-class TransformRenameFailed(BzrError):
-
-    _fmt = "Failed to rename %(from_path)s to %(to_path)s: %(why)s"
-
-    def __init__(self, from_path, to_path, why, errno):
-        self.from_path = from_path
-        self.to_path = to_path
-        self.why = why
-        self.errno = errno
-
-
 class BzrMoveFailedError(BzrError):
 
     _fmt = ("Could not move %(from_path)s%(operator)s %(to_path)s"
@@ -1672,12 +1628,6 @@
     _fmt = "Diff3 is not installed on this machine."
 
 
-class ExistingContent(BzrError):
-    # Added in breezy 0.92, used by VersionedFile.add_lines.
-
-    _fmt = "The content being inserted is already present."
-
-
 class ExistingLimbo(BzrError):
 
     _fmt = """This tree contains left-over files from a failed operation.
@@ -1699,17 +1649,6 @@
         BzrError.__init__(self, pending_deletion=pending_deletion)
 
 
-class ImmortalLimbo(BzrError):
-
-    _fmt = """Unable to delete transform temporary directory %(limbo_dir)s.
-    Please examine %(limbo_dir)s to see if it contains any files you wish to
-    keep, and delete it when you are done."""
-
-    def __init__(self, limbo_dir):
-        BzrError.__init__(self)
-        self.limbo_dir = limbo_dir
-
-
 class ImmortalPendingDeletion(BzrError):
 
     _fmt = ("Unable to delete transform temporary directory "
@@ -1936,19 +1875,6 @@
         self.other = other
 
 
-class BadInventoryFormat(BzrError):
-
-    _fmt = "Root class for inventory serialization errors"
-
-
-class UnexpectedInventoryFormat(BadInventoryFormat):
-
-    _fmt = "The inventory was not in the expected format:\n %(msg)s"
-
-    def __init__(self, msg):
-        BadInventoryFormat.__init__(self, msg=msg)
-
-
 class RootNotRich(BzrError):
 
     _fmt = """This operation requires rich root data storage"""
@@ -2011,16 +1937,6 @@
         " branch location."
 
 
-class IllegalMergeDirectivePayload(BzrError):
-    """A merge directive contained something other than a patch or bundle"""
-
-    _fmt = "Bad merge directive payload %(start)r"
-
-    def __init__(self, start):
-        BzrError(self)
-        self.start = start
-
-
 class PatchVerificationFailed(BzrError):
     """A patch from a merge directive could not be verified"""
 
@@ -2050,14 +1966,6 @@
         self.location = location
 
 
-class UnsupportedInventoryKind(BzrError):
-
-    _fmt = """Unsupported entry kind %(kind)s"""
-
-    def __init__(self, kind):
-        self.kind = kind
-
-
 class BadSubsumeSource(BzrError):
 
     _fmt = "Can't subsume %(other_tree)s into %(tree)s. %(reason)s"

=== modified file 'breezy/git/__init__.py'
--- breezy/git/__init__.py	2020-05-06 02:13:25 +0000
+++ breezy/git/__init__.py	2020-07-18 23:14:00 +0000
@@ -184,9 +184,11 @@
         resp = transport.request('GET', url, headers=headers)
         if resp.status in (404, 405):
             raise brz_errors.NotBranchError(transport.base)
+        elif resp.status == 400 and resp.reason == 'no such method: info':
+            # hgweb :(
+            raise brz_errors.NotBranchError(transport.base)
         elif resp.status != 200:
-            raise brz_errors.InvalidHttpResponse(
-                url, 'Unable to handle http code %d' % resp.status)
+            raise brz_errors.UnexpectedHttpStatus(url, resp.status)
 
         ct = resp.getheader("Content-Type")
         if ct and ct.startswith("application/x-git"):

=== modified file 'breezy/git/annotate.py'
--- breezy/git/annotate.py	2020-02-18 01:57:45 +0000
+++ breezy/git/annotate.py	2020-07-18 23:14:00 +0000
@@ -21,15 +21,20 @@
     )
 
 from .. import osutils
+from ..bzr.versionedfile import UnavailableRepresentation
 from ..errors import (
     NoSuchRevision,
-    UnavailableRepresentation,
     )
 from ..graph import Graph
 from ..revision import (
     NULL_REVISION,
     )
 
+from .mapping import (
+    decode_git_path,
+    encode_git_path,
+    )
+
 
 class GitBlobContentFactory(object):
     """Static data content factory.
@@ -114,7 +119,7 @@
             self.change_scanner.repository.lookup_bzr_revision_id(
                 text_revision))
         text_parents = []
-        path = path.encode('utf-8')
+        path = encode_git_path(path)
         for commit_parent in self.store[commit_id].parents:
             try:
                 (path, text_parent) = (
@@ -125,7 +130,7 @@
             if text_parent not in text_parents:
                 text_parents.append(text_parent)
         return tuple([
-            (path.decode('utf-8'),
+            (decode_git_path(path),
                 self.change_scanner.repository.lookup_foreign_revision_id(p))
             for p in text_parents])
 
@@ -163,7 +168,7 @@
                 continue
             try:
                 (mode, blob_sha) = tree_lookup_path(
-                    store.__getitem__, tree_id, path.encode('utf-8'))
+                    store.__getitem__, tree_id, encode_git_path(path))
             except KeyError:
                 yield GitAbsentContentFactory(store, path, text_revision)
             else:

=== modified file 'breezy/git/branch.py'
--- breezy/git/branch.py	2020-06-23 01:02:30 +0000
+++ breezy/git/branch.py	2020-07-18 23:14:00 +0000
@@ -62,6 +62,10 @@
 from .errors import (
     NoPushSupport,
     )
+from .mapping import (
+    encode_git_path,
+    decode_git_path,
+    )
 from .push import (
     remote_divergence,
     )
@@ -494,7 +498,7 @@
             cs = self.repository._git.get_config_stack()
             try:
                 return cs.get((b"branch", self.name.encode('utf-8')),
-                              b"nick").decode("utf-8")
+                        b"nick").decode("utf-8")
             except KeyError:
                 pass
         return self.name or u"HEAD"
@@ -1039,8 +1043,8 @@
                 for path, url, section in parse_submodules(
                         GitConfigFile.from_file(f)):
                     self.target.set_reference_info(
-                        tree.path2id(path.decode('utf-8')), url.decode('utf-8'),
-                        path.decode('utf-8'))
+                        tree.path2id(decode_git_path(path)), url.decode('utf-8'),
+                        decode_git_path(path))
         except errors.NoSuchFile:
             pass
 
@@ -1171,7 +1175,7 @@
                 isinstance(target, RemoteGitBranch))
 
     def _basic_push(self, overwrite, stop_revision, tag_selector=None):
-        from .remote import RemoteGitError
+        from .remote import parse_git_error
         result = GitBranchPushResult()
         result.source_branch = self.source
         result.target_branch = self.target
@@ -1209,11 +1213,10 @@
         if dw_result is not None and not isinstance(dw_result, dict):
             error = dw_result.ref_status.get(self.target.ref)
             if error:
-                raise RemoteGitError(error)
+                raise parse_git_error(self.target.user_url, error)
             for ref, error in dw_result.ref_status.items():
                 if error:
-                    trace.warning('unable to open ref %s: %s',
-                                  ref, error)
+                    trace.warning('unable to open ref %s: %s', ref, error)
         return result
 
 
@@ -1385,7 +1388,7 @@
                old_refs, new_refs)
         result.tag_updates = {}
         result.tag_conflicts = []
-        ret = dict(old_refs)
+        ret = {}
 
         def ref_equals(refs, ref, git_sha, revid):
             try:

=== modified file 'breezy/git/commands.py'
--- breezy/git/commands.py	2020-06-23 01:02:30 +0000
+++ breezy/git/commands.py	2020-07-18 23:14:00 +0000
@@ -309,6 +309,7 @@
             )
         from ..trace import warning
         from ..repository import Repository
+        from .mapping import encode_git_path
         from .object_store import get_object_store
         from .pristine_tar import (
             revision_pristine_tar_data,
@@ -345,5 +346,5 @@
                     warning(
                         "base git id %s for %s missing in target repository",
                         gitid, filename)
-                store_git_pristine_tar_data(target, filename.encode('utf-8'),
+                store_git_pristine_tar_data(target, encode_git_path(filename),
                                             delta, gitid)

=== modified file 'breezy/git/commit.py'
--- breezy/git/commit.py	2020-06-19 21:26:53 +0000
+++ breezy/git/commit.py	2020-07-18 23:14:00 +0000
@@ -46,6 +46,7 @@
 
 
 from .mapping import (
+    encode_git_path,
     object_mode,
     fix_person_identifier,
     )
@@ -80,7 +81,7 @@
                      entry_factory[change.kind[1]](
                          change.file_id, change.name[1], change.parent_id[1])))
                 if change.kind[0] in ("file", "symlink"):
-                    self._blobs[change.path[0].encode("utf-8")] = None
+                    self._blobs[encode_git_path(change.path[0])] = None
                     self._any_changes = True
                 if change.path[1] == "":
                     seen_root = True
@@ -88,7 +89,7 @@
             self._any_changes = True
             if change.path[1] is None:
                 self._inv_delta.append((change.path[0], change.path[1], change.file_id, None))
-                self._deleted_paths.add(change.path[0].encode("utf-8"))
+                self._deleted_paths.add(encode_git_path(change.path[0]))
                 continue
             try:
                 entry_kls = entry_factory[change.kind[1]]
@@ -110,7 +111,7 @@
             elif change.kind[1] == "symlink":
                 symlink_target = workingtree.get_symlink_target(change.path[1])
                 blob = Blob()
-                blob.data = symlink_target.encode("utf-8")
+                blob.data = encode_git_path(symlink_target)
                 self.store.add_object(blob)
                 sha = blob.id
                 entry.symlink_target = symlink_target
@@ -125,8 +126,8 @@
             mode = object_mode(change.kind[1], change.executable[1])
             self._inv_delta.append((change.path[0], change.path[1], change.file_id, entry))
             if change.path[0] is not None:
-                self._deleted_paths.add(change.path[0].encode("utf-8"))
-            self._blobs[change.path[1].encode("utf-8")] = (mode, sha)
+                self._deleted_paths.add(encode_git_path(change.path[0]))
+            self._blobs[encode_git_path(change.path[1])] = (mode, sha)
             if st is not None:
                 yield change.path[1], (entry.text_sha1, st)
         if not seen_root and len(self.parents) == 0:

=== modified file 'breezy/git/dir.py'
--- breezy/git/dir.py	2020-05-06 02:13:25 +0000
+++ breezy/git/dir.py	2020-07-18 23:14:00 +0000
@@ -39,6 +39,10 @@
     RepositoryAcquisitionPolicy,
     )
 
+from .mapping import (
+    decode_git_path,
+    encode_git_path,
+    )
 from .push import (
     GitPushResult,
     )
@@ -537,7 +541,7 @@
                     target_branch._format, self._format)
             # TODO(jelmer): Do some consistency checking across branches..
             self.control_transport.put_bytes(
-                'commondir', target_path.encode('utf-8'))
+                'commondir', encode_git_path(target_path))
             # TODO(jelmer): Urgh, avoid mucking about with internals.
             self._git._commontransport = (
                 target_branch.repository._git._commontransport.clone())
@@ -577,7 +581,7 @@
                 base_url = self.user_url.rstrip('/')
             else:
                 base_url = urlutils.local_path_to_url(
-                    commondir.decode(osutils._fs_enc)).rstrip('/.git/') + '/'
+                    decode_git_path(commondir)).rstrip('/.git/') + '/'
             return urlutils.join_segment_parameters(base_url, params)
         return None
 

=== modified file 'breezy/git/fetch.py'
--- breezy/git/fetch.py	2020-02-18 01:57:45 +0000
+++ breezy/git/fetch.py	2020-07-18 23:14:00 +0000
@@ -62,6 +62,7 @@
 
 from .mapping import (
     DEFAULT_FILE_MODE,
+    decode_git_path,
     mode_is_executable,
     mode_kind,
     warn_unusual_mode,
@@ -85,7 +86,7 @@
     """
     if not isinstance(path, bytes):
         raise TypeError(path)
-    decoded_path = path.decode('utf-8')
+    decoded_path = decode_git_path(path)
     (base_mode, mode) = modes
     (base_hexsha, hexsha) = hexshas
     if mapping.is_special_file(path):
@@ -98,7 +99,7 @@
         cls = InventoryLink
     else:
         cls = InventoryFile
-    ie = cls(file_id, name.decode("utf-8"), parent_id)
+    ie = cls(file_id, decode_git_path(name), parent_id)
     if ie.kind == "file":
         ie.executable = mode_is_executable(mode)
     if base_hexsha == hexsha and mode_kind(base_mode) == mode_kind(mode):
@@ -116,7 +117,7 @@
         blob = lookup_object(hexsha)
         if ie.kind == "symlink":
             ie.revision = None
-            ie.symlink_target = blob.data.decode("utf-8")
+            ie.symlink_target = decode_git_path(blob.data)
         else:
             ie.text_size = sum(map(len, blob.chunked))
             ie.text_sha1 = osutils.sha_strings(blob.chunked)
@@ -183,10 +184,10 @@
     (base_mode, mode) = modes
     if base_hexsha == hexsha and base_mode == mode:
         return [], {}
-    path = path.decode('utf-8')
+    path = decode_git_path(path)
     file_id = lookup_file_id(path)
     invdelta = []
-    ie = TreeReference(file_id, name.decode("utf-8"), parent_id)
+    ie = TreeReference(file_id, decode_git_path(name), parent_id)
     ie.revision = revision_id
     if base_hexsha is not None:
         old_path = path  # Renames are not supported yet
@@ -221,7 +222,7 @@
     for name, mode, hexsha in base_tree.iteritems():
         if name in existing_children:
             continue
-        c_path = posixpath.join(path, name.decode("utf-8"))
+        c_path = posixpath.join(path, decode_git_path(name))
         file_id = base_bzr_tree.path2id(c_path)
         if file_id is None:
             raise TypeError(file_id)
@@ -258,16 +259,15 @@
         return [], {}
     invdelta = []
     file_id = lookup_file_id(osutils.safe_unicode(path))
-    # We just have to hope this is indeed utf-8:
-    ie = InventoryDirectory(file_id, name.decode("utf-8"), parent_id)
+    ie = InventoryDirectory(file_id, decode_git_path(name), parent_id)
     tree = lookup_object(hexsha)
     if base_hexsha is None:
         base_tree = None
         old_path = None  # Newly appeared here
     else:
         base_tree = lookup_object(base_hexsha)
-        old_path = path.decode("utf-8")  # Renames aren't supported yet
-    new_path = path.decode("utf-8")
+        old_path = decode_git_path(path)  # Renames aren't supported yet
+    new_path = decode_git_path(path)
     if base_tree is None or type(base_tree) is not Tree:
         ie.revision = revision_id
         invdelta.append((old_path, new_path, ie.file_id, ie))

=== modified file 'breezy/git/filegraph.py'
--- breezy/git/filegraph.py	2020-02-18 01:57:45 +0000
+++ breezy/git/filegraph.py	2020-07-18 23:14:00 +0000
@@ -30,6 +30,10 @@
     NULL_REVISION,
     )
 
+from .mapping import (
+    encode_git_path,
+    )
+
 
 class GitFileLastChangeScanner(object):
 
@@ -82,7 +86,7 @@
             self.change_scanner.repository.lookup_bzr_revision_id(
                 text_revision))
         try:
-            path = mapping.parse_file_id(file_id).encode('utf-8')
+            path = encode_git_path(mapping.parse_file_id(file_id))
         except ValueError:
             raise KeyError(file_id)
         text_parents = []

=== modified file 'breezy/git/interrepo.py'
--- breezy/git/interrepo.py	2020-06-23 01:02:30 +0000
+++ breezy/git/interrepo.py	2020-07-18 23:14:00 +0000
@@ -243,7 +243,7 @@
                 # broken symref?
                 continue
             revid = None
-            if not v.startswith(SYMREF):
+            if v and not v.startswith(SYMREF):
                 try:
                     for (kind, type_data) in self.source_store.lookup_git_sha(
                             v):
@@ -309,8 +309,7 @@
                         try:
                             self.mapping.revision_id_bzr_to_foreign(old_revid)
                         except InvalidRevisionId:
-                            refname = self.mapping.revid_as_refname(old_revid)
-                            self.target_refs[refname] = git_sha
+                            pass
                     revidmap[old_revid] = (git_sha, new_revid)
                 self.target_store.add_objects(object_generator)
                 return revidmap

=== modified file 'breezy/git/mapping.py'
--- breezy/git/mapping.py	2020-06-10 23:47:24 +0000
+++ breezy/git/mapping.py	2020-07-18 23:14:00 +0000
@@ -126,6 +126,26 @@
     return b"%s <%s>" % (username, email)
 
 
+def decode_git_path(path):
+    """Take a git path and decode it."""
+    try:
+        return path.decode('utf-8')
+    except UnicodeDecodeError:
+        if PY3:
+            return path.decode('utf-8', 'surrogateescape')
+        raise
+
+
+def encode_git_path(path):
+    """Take a regular path and encode it for git."""
+    try:
+        return path.encode('utf-8')
+    except UnicodeEncodeError:
+        if PY3:
+            return path.encode('utf-8', 'surrogateescape')
+        raise
+
+
 def warn_escaped(commit, num_escaped):
     trace.warning("Escaped %d XML-invalid characters in %s. Will be unable "
                   "to regenerate the SHA map.", num_escaped, commit)
@@ -181,14 +201,7 @@
             return u""
         if not file_id.startswith(FILE_ID_PREFIX):
             raise ValueError
-        return unescape_file_id(file_id[len(FILE_ID_PREFIX):]).decode('utf-8')
-
-    def revid_as_refname(self, revid):
-        if not isinstance(revid, bytes):
-            raise TypeError(revid)
-        revid = revid.decode('utf-8')
-        quoted_revid = urlutils.quote(revid)
-        return b"refs/bzr/" + quoted_revid.encode('utf-8')
+        return decode_git_path(unescape_file_id(file_id[len(FILE_ID_PREFIX):]))
 
     def import_unusual_file_modes(self, rev, unusual_file_modes):
         if unusual_file_modes:
@@ -326,9 +339,7 @@
             commit.author_timezone = commit.commit_timezone
         if u'git-gpg-signature' in rev.properties:
             commit.gpgsig = rev.properties[u'git-gpg-signature'].encode(
-                'utf-8')
-        if u'git-gpg-signature-b64' in rev.properties:
-            commit.gpgsig = base64.b64decode(rev.properties[u'git-gpg-signature-b64'])
+                'utf-8', 'surrogateescape')
         commit.message = self._encode_commit_message(rev, rev.message,
                                                      encoding)
         if not isinstance(commit.message, bytes):
@@ -341,8 +352,7 @@
             mapping_properties = set(
                 [u'author', u'author-timezone', u'author-timezone-neg-utc',
                  u'commit-timezone-neg-utc', u'git-implicit-encoding',
-                 u'git-gpg-signature', u'git-gpg-signature-b64',
-                 u'git-explicit-encoding',
+                 u'git-gpg-signature', u'git-explicit-encoding',
                  u'author-timestamp', u'file-modes'])
             for k, v in rev.properties.items():
                 if k not in mapping_properties:
@@ -424,12 +434,8 @@
         if commit._commit_timezone_neg_utc:
             rev.properties[u'commit-timezone-neg-utc'] = ""
         if commit.gpgsig:
-            try:
-                rev.properties[u'git-gpg-signature'] = commit.gpgsig.decode(
-                    'utf-8')
-            except UnicodeDecodeError:
-                rev.properties[u'git-gpg-signature-b64'] = base64.b64encode(
-                    commit.gpgsig)
+            rev.properties[u'git-gpg-signature'] = commit.gpgsig.decode(
+                'utf-8', 'surrogateescape')
         if commit.mergetag:
             for i, tag in enumerate(commit.mergetag):
                 rev.properties[u'git-mergetag-%d' % i] = tag.as_raw_string()
@@ -576,7 +582,7 @@
     from dulwich.objects import Blob
     blob = Blob()
     if isinstance(symlink_target, str):
-        symlink_target = symlink_target.encode('utf-8')
+        symlink_target = encode_git_path(symlink_target)
     blob.data = symlink_target
     return blob
 

=== modified file 'breezy/git/memorytree.py'
--- breezy/git/memorytree.py	2020-02-18 01:57:45 +0000
+++ breezy/git/memorytree.py	2020-07-18 23:14:00 +0000
@@ -39,6 +39,10 @@
     )
 from breezy.transport.memory import MemoryTransport
 
+from .mapping import (
+    decode_git_path,
+    encode_git_path,
+    )
 from .tree import MutableGitIndexTree
 
 
@@ -94,7 +98,7 @@
         while trees:
             (path, tree) = trees.pop()
             for name, mode, sha in tree.iteritems():
-                subpath = posixpath.join(path, name.decode('utf-8'))
+                subpath = posixpath.join(path, decode_git_path(name))
                 if stat.S_ISDIR(mode):
                     self._file_transport.mkdir(subpath)
                     trees.append((subpath, self.store[sha]))
@@ -181,7 +185,7 @@
             return None
         elif stat.S_ISLNK(stat_val.st_mode):
             blob = Blob.from_string(
-                self._file_transport.readlink(path).encode('utf-8'))
+                encode_git_path(self._file_transport.readlink(path)))
         elif stat.S_ISREG(stat_val.st_mode):
             blob = Blob.from_string(self._file_transport.get_bytes(path))
         else:

=== modified file 'breezy/git/object_store.py'
--- breezy/git/object_store.py	2020-06-01 19:35:12 +0000
+++ breezy/git/object_store.py	2020-07-18 23:14:00 +0000
@@ -54,6 +54,7 @@
     )
 from .mapping import (
     default_mapping,
+    encode_git_path,
     entry_mode,
     extract_unusual_modes,
     mapping_registry,
@@ -199,7 +200,7 @@
             mode = entry_mode(value)
         hexsha = lookup_ie_sha1(child_path, value)
         if hexsha is not None:
-            tree.add(value.name.encode("utf-8"), mode, hexsha)
+            tree.add(encode_git_path(value.name), mode, hexsha)
     if not allow_empty and len(tree) == 0:
         # Only the root can be an empty tree
         if empty_file_name is not None:

=== modified file 'breezy/git/remote.py'
--- breezy/git/remote.py	2020-06-23 01:02:30 +0000
+++ breezy/git/remote.py	2020-07-18 23:14:00 +0000
@@ -76,6 +76,7 @@
     NoSuchRef,
     )
 from .mapping import (
+    encode_git_path,
     mapping_registry,
     )
 from .object_store import (
@@ -203,6 +204,10 @@
     if re.match('(.+) is not a valid repository name',
                 message.splitlines()[0]):
         return NotBranchError(url, message)
+    if message == (
+            'GitLab: You are not allowed to push code to protected branches '
+            'on this project.'):
+        return PermissionDenied(url, message)
     m = re.match(r'Permission to ([^ ]+) denied to ([^ ]+)\.', message)
     if m:
         return PermissionDenied(m.group(1), 'denied to %s' % m.group(2))
@@ -429,7 +434,7 @@
                 write_error,
                 format=(format.encode('ascii') if format else None),
                 subdirs=subdirs,
-                prefix=(prefix.encode('utf-8') if prefix else None))
+                prefix=(encode_git_path(prefix) if prefix else None))
         except HangupException as e:
             raise parse_git_hangup(self.transport.external_url(), e)
         except GitProtocolError as e:
@@ -583,6 +588,9 @@
             # No revision supplied by the user, default to the branch
             # revision
             revision_id = source.last_revision()
+        else:
+            if not source.repository.has_revision(revision_id):
+                raise NoSuchRevision(source, revision_id)
 
         push_result = GitPushResult()
         push_result.workingtree_updated = None
@@ -635,6 +643,9 @@
                             new_sha = repo.lookup_bzr_revision_id(revid)[0]
                         except errors.NoSuchRevision:
                             continue
+                        else:
+                            if not source.repository.has_revision(revid):
+                                continue
                     ret[tag_name_to_ref(tagname)] = new_sha
             return ret
         with source_store.lock_read():

=== modified file 'breezy/git/repository.py'
--- breezy/git/repository.py	2020-06-23 01:02:30 +0000
+++ breezy/git/repository.py	2020-07-18 23:14:00 +0000
@@ -39,6 +39,7 @@
     )
 from .mapping import (
     default_mapping,
+    encode_git_path,
     foreign_vcs_git,
     mapping_registry,
     )
@@ -301,7 +302,7 @@
                 try:
                     obj = tree_lookup_path(
                         self._git.object_store.__getitem__, root_tree,
-                        path.encode('utf-8'))
+                        encode_git_path(path))
                     if isinstance(obj, tuple):
                         (mode, item_id) = obj
                         obj = self._git.object_store[item_id]
@@ -544,41 +545,6 @@
             raise ValueError('invalid revision id %s' % revision_id)
         return GitRevisionTree(self, revision_id)
 
-    def get_deltas_for_revisions(self, revisions, specific_fileids=None):
-        """Produce a generator of revision deltas.
-
-        Note that the input is a sequence of REVISIONS, not revision_ids.
-        Trees will be held in memory until the generator exits.
-        Each delta is relative to the revision's lefthand predecessor.
-
-        :param specific_fileids: if not None, the result is filtered
-          so that only those file-ids, their parents and their
-          children are included.
-        """
-        # Get the revision-ids of interest
-        required_trees = set()
-        for revision in revisions:
-            required_trees.add(revision.revision_id)
-            required_trees.update(revision.parent_ids[:1])
-
-        trees = dict((t.get_revision_id(), t) for
-                     t in self.revision_trees(required_trees))
-
-        # Calculate the deltas
-        for revision in revisions:
-            if not revision.parent_ids:
-                old_tree = self.revision_tree(_mod_revision.NULL_REVISION)
-            else:
-                old_tree = trees[revision.parent_ids[0]]
-            new_tree = trees[revision.revision_id]
-            if specific_fileids is not None:
-                specific_files = [new_tree.id2path(
-                    fid) for fid in specific_fileids]
-            else:
-                specific_files = None
-            yield new_tree.changes_from(
-                old_tree, specific_files=specific_files)
-
     def set_make_working_trees(self, trees):
         raise errors.UnsupportedOperation(self.set_make_working_trees, self)
 

=== modified file 'breezy/git/server.py'
--- breezy/git/server.py	2020-06-23 01:02:30 +0000
+++ breezy/git/server.py	2020-07-18 23:14:00 +0000
@@ -31,6 +31,7 @@
 
 from .mapping import (
     default_mapping,
+    decode_git_path,
     )
 from .object_store import (
     BazaarObjectStore,
@@ -58,7 +59,7 @@
 
     def open_repository(self, path):
         # FIXME: More secure path sanitization
-        transport = self.transport.clone(path.decode('utf-8').lstrip("/"))
+        transport = self.transport.clone(decode_git_path(path).lstrip("/"))
         trace.mutter('client opens %r: %r', path, transport)
         return BzrBackendRepo(transport, self.mapping)
 

=== modified file 'breezy/git/tests/test_remote.py'
--- breezy/git/tests/test_remote.py	2020-06-19 21:26:53 +0000
+++ breezy/git/tests/test_remote.py	2020-07-18 23:14:00 +0000
@@ -143,6 +143,18 @@
 """)
         self.assertIsInstance(e, NotBranchError)
 
+    def test_invalid_git_error(self):
+        self.assertEqual(
+            PermissionDenied(
+                'url',
+                'GitLab: You are not allowed to push code to protected '
+                'branches on this project.'),
+            parse_git_error(
+                'url',
+                RemoteGitError(
+                    'GitLab: You are not allowed to push code to '
+                    'protected branches on this project.')))
+
 
 class ParseHangupTests(TestCase):
 

=== modified file 'breezy/git/tests/test_urls.py'
--- breezy/git/tests/test_urls.py	2020-02-18 01:57:45 +0000
+++ breezy/git/tests/test_urls.py	2020-07-18 23:14:00 +0000
@@ -39,6 +39,12 @@
                 'git+ssh://user@foo/bar/path'),
             ('git+ssh://user@foo/bar/path'))
 
+    def test_just_ssh(self):
+        self.assertEqual(
+            git_url_to_bzr_url(
+                'ssh://user@foo/bar/path'),
+            ('git+ssh://user@foo/bar/path'))
+
     def test_path(self):
         self.assertEqual(git_url_to_bzr_url('/bar/path'), ('/bar/path'))
 

=== added file 'breezy/git/transform.py'
--- breezy/git/transform.py	1970-01-01 00:00:00 +0000
+++ breezy/git/transform.py	2020-07-06 01:43:04 +0000
@@ -0,0 +1,1685 @@
+# Copyright (C) 2006-2011 Canonical Ltd
+# Copyright (C) 2020 Breezy Developers
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+from __future__ import absolute_import
+
+import errno
+import os
+from stat import S_ISREG
+import time
+
+from .. import errors, multiparent, osutils, trace, ui, urlutils
+from ..i18n import gettext
+from ..mutabletree import MutableTree
+from ..sixish import viewitems, viewvalues
+from ..transform import (
+    TreeTransform,
+    _TransformResults,
+    _FileMover,
+    FinalPaths,
+    joinpath,
+    unique_add,
+    TransformRenameFailed,
+    ImmortalLimbo,
+    ROOT_PARENT,
+    ReusingTransform,
+    MalformedTransform,
+    )
+
+from ..bzr import inventory
+from ..bzr.transform import TransformPreview as GitTransformPreview
+
+
+class TreeTransformBase(TreeTransform):
+    """The base class for TreeTransform and its kin."""
+
+    def __init__(self, tree, pb=None, case_sensitive=True):
+        """Constructor.
+
+        :param tree: The tree that will be transformed, but not necessarily
+            the output tree.
+        :param pb: ignored
+        :param case_sensitive: If True, the target of the transform is
+            case sensitive, not just case preserving.
+        """
+        super(TreeTransformBase, self).__init__(tree, pb=pb)
+        # mapping of trans_id => (sha1 of content, stat_value)
+        self._observed_sha1s = {}
+        # Mapping of trans_id -> new file_id
+        self._new_id = {}
+        # Mapping of old file-id -> trans_id
+        self._non_present_ids = {}
+        # Mapping of new file_id -> trans_id
+        self._r_new_id = {}
+        # The trans_id that will be used as the tree root
+        if tree.is_versioned(''):
+            self._new_root = self.trans_id_tree_path('')
+        else:
+            self._new_root = None
+        # Whether the target is case sensitive
+        self._case_sensitive_target = case_sensitive
+
+    def finalize(self):
+        """Release the working tree lock, if held.
+
+        This is required if apply has not been invoked, but can be invoked
+        even after apply.
+        """
+        if self._tree is None:
+            return
+        for hook in MutableTree.hooks['post_transform']:
+            hook(self._tree, self)
+        self._tree.unlock()
+        self._tree = None
+
+    def __get_root(self):
+        return self._new_root
+
+    root = property(__get_root)
+
+    def create_path(self, name, parent):
+        """Assign a transaction id to a new path"""
+        trans_id = self._assign_id()
+        unique_add(self._new_name, trans_id, name)
+        unique_add(self._new_parent, trans_id, parent)
+        return trans_id
+
+    def adjust_root_path(self, name, parent):
+        """Emulate moving the root by moving all children, instead.
+
+        We do this by undoing the association of root's transaction id with the
+        current tree.  This allows us to create a new directory with that
+        transaction id.  We unversion the root directory and version the
+        physically new directory, and hope someone versions the tree root
+        later.
+        """
+        old_root = self._new_root
+        old_root_file_id = self.final_file_id(old_root)
+        # force moving all children of root
+        for child_id in self.iter_tree_children(old_root):
+            if child_id != parent:
+                self.adjust_path(self.final_name(child_id),
+                                 self.final_parent(child_id), child_id)
+            file_id = self.final_file_id(child_id)
+            if file_id is not None:
+                self.unversion_file(child_id)
+            self.version_file(child_id, file_id=file_id)
+
+        # the physical root needs a new transaction id
+        self._tree_path_ids.pop("")
+        self._tree_id_paths.pop(old_root)
+        self._new_root = self.trans_id_tree_path('')
+        if parent == old_root:
+            parent = self._new_root
+        self.adjust_path(name, parent, old_root)
+        self.create_directory(old_root)
+        self.version_file(old_root, file_id=old_root_file_id)
+        self.unversion_file(self._new_root)
+
+    def fixup_new_roots(self):
+        """Reinterpret requests to change the root directory
+
+        Instead of creating a root directory, or moving an existing directory,
+        all the attributes and children of the new root are applied to the
+        existing root directory.
+
+        This means that the old root trans-id becomes obsolete, so it is
+        recommended only to invoke this after the root trans-id has become
+        irrelevant.
+
+        """
+        new_roots = [k for k, v in viewitems(self._new_parent)
+                     if v == ROOT_PARENT]
+        if len(new_roots) < 1:
+            return
+        if len(new_roots) != 1:
+            raise ValueError('A tree cannot have two roots!')
+        if self._new_root is None:
+            self._new_root = new_roots[0]
+            return
+        old_new_root = new_roots[0]
+        # unversion the new root's directory.
+        if self.final_kind(self._new_root) is None:
+            file_id = self.final_file_id(old_new_root)
+        else:
+            file_id = self.final_file_id(self._new_root)
+        if old_new_root in self._new_id:
+            self.cancel_versioning(old_new_root)
+        else:
+            self.unversion_file(old_new_root)
+        # if, at this stage, root still has an old file_id, zap it so we can
+        # stick a new one in.
+        if (self.tree_file_id(self._new_root) is not None
+                and self._new_root not in self._removed_id):
+            self.unversion_file(self._new_root)
+        if file_id is not None:
+            self.version_file(self._new_root, file_id=file_id)
+
+        # Now move children of new root into old root directory.
+        # Ensure all children are registered with the transaction, but don't
+        # use directly-- some tree children have new parents
+        list(self.iter_tree_children(old_new_root))
+        # Move all children of new root into old root directory.
+        for child in self.by_parent().get(old_new_root, []):
+            self.adjust_path(self.final_name(child), self._new_root, child)
+
+        # Ensure old_new_root has no directory.
+        if old_new_root in self._new_contents:
+            self.cancel_creation(old_new_root)
+        else:
+            self.delete_contents(old_new_root)
+
+        # prevent deletion of root directory.
+        if self._new_root in self._removed_contents:
+            self.cancel_deletion(self._new_root)
+
+        # destroy path info for old_new_root.
+        del self._new_parent[old_new_root]
+        del self._new_name[old_new_root]
+
+    def trans_id_file_id(self, file_id):
+        """Determine or set the transaction id associated with a file ID.
+        A new id is only created for file_ids that were never present.  If
+        a transaction has been unversioned, it is deliberately still returned.
+        (this will likely lead to an unversioned parent conflict.)
+        """
+        if file_id is None:
+            raise ValueError('None is not a valid file id')
+        if file_id in self._r_new_id and self._r_new_id[file_id] is not None:
+            return self._r_new_id[file_id]
+        else:
+            try:
+                path = self._tree.id2path(file_id)
+            except errors.NoSuchId:
+                if file_id in self._non_present_ids:
+                    return self._non_present_ids[file_id]
+                else:
+                    trans_id = self._assign_id()
+                    self._non_present_ids[file_id] = trans_id
+                    return trans_id
+            else:
+                return self.trans_id_tree_path(path)
+
+    def version_file(self, trans_id, file_id=None):
+        """Schedule a file to become versioned."""
+        raise NotImplementedError(self.version_file)
+
+    def cancel_versioning(self, trans_id):
+        """Undo a previous versioning of a file"""
+        raise NotImplementedError(self.cancel_versioning)
+
+    def new_paths(self, filesystem_only=False):
+        """Determine the paths of all new and changed files.
+
+        :param filesystem_only: if True, only calculate values for files
+            that require renames or execute bit changes.
+        """
+        new_ids = set()
+        if filesystem_only:
+            stale_ids = self._needs_rename.difference(self._new_name)
+            stale_ids.difference_update(self._new_parent)
+            stale_ids.difference_update(self._new_contents)
+            stale_ids.difference_update(self._new_id)
+            needs_rename = self._needs_rename.difference(stale_ids)
+            id_sets = (needs_rename, self._new_executability)
+        else:
+            id_sets = (self._new_name, self._new_parent, self._new_contents,
+                       self._new_id, self._new_executability)
+        for id_set in id_sets:
+            new_ids.update(id_set)
+        return sorted(FinalPaths(self).get_paths(new_ids))
+
+    def tree_file_id(self, trans_id):
+        """Determine the file id associated with the trans_id in the tree"""
+        path = self.tree_path(trans_id)
+        if path is None:
+            return None
+        # the file is old; the old id is still valid
+        if self._new_root == trans_id:
+            return self._tree.path2id('')
+        return self._tree.path2id(path)
+
+    def final_is_versioned(self, trans_id):
+        return self.final_file_id(trans_id) is not None
+
+    def final_file_id(self, trans_id):
+        """Determine the file id after any changes are applied, or None.
+
+        None indicates that the file will not be versioned after changes are
+        applied.
+        """
+        try:
+            return self._new_id[trans_id]
+        except KeyError:
+            if trans_id in self._removed_id:
+                return None
+        return self.tree_file_id(trans_id)
+
+    def inactive_file_id(self, trans_id):
+        """Return the inactive file_id associated with a transaction id.
+        That is, the one in the tree or in non_present_ids.
+        The file_id may actually be active, too.
+        """
+        file_id = self.tree_file_id(trans_id)
+        if file_id is not None:
+            return file_id
+        for key, value in viewitems(self._non_present_ids):
+            if value == trans_id:
+                return key
+
+    def find_conflicts(self):
+        """Find any violations of inventory or filesystem invariants"""
+        if self._done is True:
+            raise ReusingTransform()
+        conflicts = []
+        # ensure all children of all existent parents are known
+        # all children of non-existent parents are known, by definition.
+        self._add_tree_children()
+        by_parent = self.by_parent()
+        conflicts.extend(self._unversioned_parents(by_parent))
+        conflicts.extend(self._parent_loops())
+        conflicts.extend(self._duplicate_entries(by_parent))
+        conflicts.extend(self._parent_type_conflicts(by_parent))
+        conflicts.extend(self._improper_versioning())
+        conflicts.extend(self._executability_conflicts())
+        conflicts.extend(self._overwrite_conflicts())
+        return conflicts
+
+    def _check_malformed(self):
+        conflicts = self.find_conflicts()
+        if len(conflicts) != 0:
+            raise MalformedTransform(conflicts=conflicts)
+
+    def _add_tree_children(self):
+        """Add all the children of all active parents to the known paths.
+
+        Active parents are those which gain children, and those which are
+        removed.  This is a necessary first step in detecting conflicts.
+        """
+        parents = list(self.by_parent())
+        parents.extend([t for t in self._removed_contents if
+                        self.tree_kind(t) == 'directory'])
+        for trans_id in self._removed_id:
+            path = self.tree_path(trans_id)
+            if path is not None:
+                if self._tree.stored_kind(path) == 'directory':
+                    parents.append(trans_id)
+            elif self.tree_kind(trans_id) == 'directory':
+                parents.append(trans_id)
+
+        for parent_id in parents:
+            # ensure that all children are registered with the transaction
+            list(self.iter_tree_children(parent_id))
+
+    def _has_named_child(self, name, parent_id, known_children):
+        """Does a parent already have a name child.
+
+        :param name: The searched for name.
+
+        :param parent_id: The parent for which the check is made.
+
+        :param known_children: The already known children. This should have
+            been recently obtained from `self.by_parent.get(parent_id)`
+            (or will be if None is passed).
+        """
+        if known_children is None:
+            known_children = self.by_parent().get(parent_id, [])
+        for child in known_children:
+            if self.final_name(child) == name:
+                return True
+        parent_path = self._tree_id_paths.get(parent_id, None)
+        if parent_path is None:
+            # No parent... no children
+            return False
+        child_path = joinpath(parent_path, name)
+        child_id = self._tree_path_ids.get(child_path, None)
+        if child_id is None:
+            # Not known by the tree transform yet, check the filesystem
+            return osutils.lexists(self._tree.abspath(child_path))
+        else:
+            raise AssertionError('child_id is missing: %s, %s, %s'
+                                 % (name, parent_id, child_id))
+
+    def _available_backup_name(self, name, target_id):
+        """Find an available backup name.
+
+        :param name: The basename of the file.
+
+        :param target_id: The directory trans_id where the backup should
+            be placed.
+        """
+        known_children = self.by_parent().get(target_id, [])
+        return osutils.available_backup_name(
+            name,
+            lambda base: self._has_named_child(
+                base, target_id, known_children))
+
+    def _parent_loops(self):
+        """No entry should be its own ancestor"""
+        conflicts = []
+        for trans_id in self._new_parent:
+            seen = set()
+            parent_id = trans_id
+            while parent_id != ROOT_PARENT:
+                seen.add(parent_id)
+                try:
+                    parent_id = self.final_parent(parent_id)
+                except KeyError:
+                    break
+                if parent_id == trans_id:
+                    conflicts.append(('parent loop', trans_id))
+                if parent_id in seen:
+                    break
+        return conflicts
+
+    def _unversioned_parents(self, by_parent):
+        """If parent directories are versioned, children must be versioned."""
+        conflicts = []
+        for parent_id, children in viewitems(by_parent):
+            if parent_id == ROOT_PARENT:
+                continue
+            if self.final_is_versioned(parent_id):
+                continue
+            for child_id in children:
+                if self.final_is_versioned(child_id):
+                    conflicts.append(('unversioned parent', parent_id))
+                    break
+        return conflicts
+
+    def _improper_versioning(self):
+        """Cannot version a file with no contents, or a bad type.
+
+        However, existing entries with no contents are okay.
+        """
+        conflicts = []
+        for trans_id in self._new_id:
+            kind = self.final_kind(trans_id)
+            if kind == 'symlink' and not self._tree.supports_symlinks():
+                # Ignore symlinks as they are not supported on this platform
+                continue
+            if kind is None:
+                conflicts.append(('versioning no contents', trans_id))
+                continue
+            if not self._tree.versionable_kind(kind):
+                conflicts.append(('versioning bad kind', trans_id, kind))
+        return conflicts
+
+    def _executability_conflicts(self):
+        """Check for bad executability changes.
+
+        Only versioned files may have their executability set, because
+        1. only versioned entries can have executability under windows
+        2. only files can be executable.  (The execute bit on a directory
+           does not indicate searchability)
+        """
+        conflicts = []
+        for trans_id in self._new_executability:
+            if not self.final_is_versioned(trans_id):
+                conflicts.append(('unversioned executability', trans_id))
+            else:
+                if self.final_kind(trans_id) != "file":
+                    conflicts.append(('non-file executability', trans_id))
+        return conflicts
+
+    def _overwrite_conflicts(self):
+        """Check for overwrites (not permitted on Win32)"""
+        conflicts = []
+        for trans_id in self._new_contents:
+            if self.tree_kind(trans_id) is None:
+                continue
+            if trans_id not in self._removed_contents:
+                conflicts.append(('overwrite', trans_id,
+                                  self.final_name(trans_id)))
+        return conflicts
+
+    def _duplicate_entries(self, by_parent):
+        """No directory may have two entries with the same name."""
+        conflicts = []
+        if (self._new_name, self._new_parent) == ({}, {}):
+            return conflicts
+        for children in viewvalues(by_parent):
+            name_ids = []
+            for child_tid in children:
+                name = self.final_name(child_tid)
+                if name is not None:
+                    # Keep children only if they still exist in the end
+                    if not self._case_sensitive_target:
+                        name = name.lower()
+                    name_ids.append((name, child_tid))
+            name_ids.sort()
+            last_name = None
+            last_trans_id = None
+            for name, trans_id in name_ids:
+                kind = self.final_kind(trans_id)
+                if kind is None and not self.final_is_versioned(trans_id):
+                    continue
+                if name == last_name:
+                    conflicts.append(('duplicate', last_trans_id, trans_id,
+                                      name))
+                last_name = name
+                last_trans_id = trans_id
+        return conflicts
+
+    def _parent_type_conflicts(self, by_parent):
+        """Children must have a directory parent"""
+        conflicts = []
+        for parent_id, children in viewitems(by_parent):
+            if parent_id == ROOT_PARENT:
+                continue
+            no_children = True
+            for child_id in children:
+                if self.final_kind(child_id) is not None:
+                    no_children = False
+                    break
+            if no_children:
+                continue
+            # There is at least a child, so we need an existing directory to
+            # contain it.
+            kind = self.final_kind(parent_id)
+            if kind is None:
+                # The directory will be deleted
+                conflicts.append(('missing parent', parent_id))
+            elif kind != "directory":
+                # Meh, we need a *directory* to put something in it
+                conflicts.append(('non-directory parent', parent_id))
+        return conflicts
+
+    def _set_executability(self, path, trans_id):
+        """Set the executability of versioned files """
+        if self._tree._supports_executable():
+            new_executability = self._new_executability[trans_id]
+            abspath = self._tree.abspath(path)
+            current_mode = os.stat(abspath).st_mode
+            if new_executability:
+                umask = os.umask(0)
+                os.umask(umask)
+                to_mode = current_mode | (0o100 & ~umask)
+                # Enable x-bit for others only if they can read it.
+                if current_mode & 0o004:
+                    to_mode |= 0o001 & ~umask
+                if current_mode & 0o040:
+                    to_mode |= 0o010 & ~umask
+            else:
+                to_mode = current_mode & ~0o111
+            osutils.chmod_if_possible(abspath, to_mode)
+
+    def _new_entry(self, name, parent_id, file_id):
+        """Helper function to create a new filesystem entry."""
+        trans_id = self.create_path(name, parent_id)
+        if file_id is not None:
+            self.version_file(trans_id, file_id=file_id)
+        return trans_id
+
+    def new_file(self, name, parent_id, contents, file_id=None,
+                 executable=None, sha1=None):
+        """Convenience method to create files.
+
+        name is the name of the file to create.
+        parent_id is the transaction id of the parent directory of the file.
+        contents is an iterator of bytestrings, which will be used to produce
+        the file.
+        :param file_id: The inventory ID of the file, if it is to be versioned.
+        :param executable: Only valid when a file_id has been supplied.
+        """
+        trans_id = self._new_entry(name, parent_id, file_id)
+        # TODO: rather than scheduling a set_executable call,
+        # have create_file create the file with the right mode.
+        self.create_file(contents, trans_id, sha1=sha1)
+        if executable is not None:
+            self.set_executability(executable, trans_id)
+        return trans_id
+
+    def new_directory(self, name, parent_id, file_id=None):
+        """Convenience method to create directories.
+
+        name is the name of the directory to create.
+        parent_id is the transaction id of the parent directory of the
+        directory.
+        file_id is the inventory ID of the directory, if it is to be versioned.
+        """
+        trans_id = self._new_entry(name, parent_id, file_id)
+        self.create_directory(trans_id)
+        return trans_id
+
+    def new_symlink(self, name, parent_id, target, file_id=None):
+        """Convenience method to create symbolic link.
+
+        name is the name of the symlink to create.
+        parent_id is the transaction id of the parent directory of the symlink.
+        target is a bytestring of the target of the symlink.
+        file_id is the inventory ID of the file, if it is to be versioned.
+        """
+        trans_id = self._new_entry(name, parent_id, file_id)
+        self.create_symlink(target, trans_id)
+        return trans_id
+
+    def new_orphan(self, trans_id, parent_id):
+        """Schedule an item to be orphaned.
+
+        When a directory is about to be removed, its children, if they are not
+        versioned are moved out of the way: they don't have a parent anymore.
+
+        :param trans_id: The trans_id of the existing item.
+        :param parent_id: The parent trans_id of the item.
+        """
+        raise NotImplementedError(self.new_orphan)
+
+    def _get_potential_orphans(self, dir_id):
+        """Find the potential orphans in a directory.
+
+        A directory can't be safely deleted if there are versioned files in it.
+        If all the contained files are unversioned then they can be orphaned.
+
+        The 'None' return value means that the directory contains at least one
+        versioned file and should not be deleted.
+
+        :param dir_id: The directory trans id.
+
+        :return: A list of the orphan trans ids or None if at least one
+             versioned file is present.
+        """
+        orphans = []
+        # Find the potential orphans, stop if one item should be kept
+        for child_tid in self.by_parent()[dir_id]:
+            if child_tid in self._removed_contents:
+                # The child is removed as part of the transform. Since it was
+                # versioned before, it's not an orphan
+                continue
+            if not self.final_is_versioned(child_tid):
+                # The child is not versioned
+                orphans.append(child_tid)
+            else:
+                # We have a versioned file here, searching for orphans is
+                # meaningless.
+                orphans = None
+                break
+        return orphans
+
+    def _affected_ids(self):
+        """Return the set of transform ids affected by the transform"""
+        trans_ids = set(self._removed_id)
+        trans_ids.update(self._new_id)
+        trans_ids.update(self._removed_contents)
+        trans_ids.update(self._new_contents)
+        trans_ids.update(self._new_executability)
+        trans_ids.update(self._new_name)
+        trans_ids.update(self._new_parent)
+        return trans_ids
+
+    def _get_file_id_maps(self):
+        """Return mapping of file_ids to trans_ids in the to and from states"""
+        trans_ids = self._affected_ids()
+        from_trans_ids = {}
+        to_trans_ids = {}
+        # Build up two dicts: trans_ids associated with file ids in the
+        # FROM state, vs the TO state.
+        for trans_id in trans_ids:
+            from_file_id = self.tree_file_id(trans_id)
+            if from_file_id is not None:
+                from_trans_ids[from_file_id] = trans_id
+            to_file_id = self.final_file_id(trans_id)
+            if to_file_id is not None:
+                to_trans_ids[to_file_id] = trans_id
+        return from_trans_ids, to_trans_ids
+
+    def _from_file_data(self, from_trans_id, from_versioned, from_path):
+        """Get data about a file in the from (tree) state
+
+        Return a (name, parent, kind, executable) tuple
+        """
+        from_path = self._tree_id_paths.get(from_trans_id)
+        if from_versioned:
+            # get data from working tree if versioned
+            from_entry = next(self._tree.iter_entries_by_dir(
+                specific_files=[from_path]))[1]
+            from_name = from_entry.name
+            from_parent = from_entry.parent_id
+        else:
+            from_entry = None
+            if from_path is None:
+                # File does not exist in FROM state
+                from_name = None
+                from_parent = None
+            else:
+                # File exists, but is not versioned.  Have to use path-
+                # splitting stuff
+                from_name = os.path.basename(from_path)
+                tree_parent = self.get_tree_parent(from_trans_id)
+                from_parent = self.tree_file_id(tree_parent)
+        if from_path is not None:
+            from_kind, from_executable, from_stats = \
+                self._tree._comparison_data(from_entry, from_path)
+        else:
+            from_kind = None
+            from_executable = False
+        return from_name, from_parent, from_kind, from_executable
+
+    def _to_file_data(self, to_trans_id, from_trans_id, from_executable):
+        """Get data about a file in the to (target) state
+
+        Return a (name, parent, kind, executable) tuple
+        """
+        to_name = self.final_name(to_trans_id)
+        to_kind = self.final_kind(to_trans_id)
+        to_parent = self.final_file_id(self.final_parent(to_trans_id))
+        if to_trans_id in self._new_executability:
+            to_executable = self._new_executability[to_trans_id]
+        elif to_trans_id == from_trans_id:
+            to_executable = from_executable
+        else:
+            to_executable = False
+        return to_name, to_parent, to_kind, to_executable
+
+    def iter_changes(self):
+        """Produce output in the same format as Tree.iter_changes.
+
+        Will produce nonsensical results if invoked while inventory/filesystem
+        conflicts (as reported by TreeTransform.find_conflicts()) are present.
+
+        This reads the Transform, but only reproduces changes involving a
+        file_id.  Files that are not versioned in either of the FROM or TO
+        states are not reflected.
+        """
+        final_paths = FinalPaths(self)
+        from_trans_ids, to_trans_ids = self._get_file_id_maps()
+        results = []
+        # Now iterate through all active file_ids
+        for file_id in set(from_trans_ids).union(to_trans_ids):
+            modified = False
+            from_trans_id = from_trans_ids.get(file_id)
+            # find file ids, and determine versioning state
+            if from_trans_id is None:
+                from_versioned = False
+                from_trans_id = to_trans_ids[file_id]
+            else:
+                from_versioned = True
+            to_trans_id = to_trans_ids.get(file_id)
+            if to_trans_id is None:
+                to_versioned = False
+                to_trans_id = from_trans_id
+            else:
+                to_versioned = True
+
+            if not from_versioned:
+                from_path = None
+            else:
+                from_path = self._tree_id_paths.get(from_trans_id)
+            if not to_versioned:
+                to_path = None
+            else:
+                to_path = final_paths.get_path(to_trans_id)
+
+            from_name, from_parent, from_kind, from_executable = \
+                self._from_file_data(from_trans_id, from_versioned, from_path)
+
+            to_name, to_parent, to_kind, to_executable = \
+                self._to_file_data(to_trans_id, from_trans_id, from_executable)
+
+            if from_kind != to_kind:
+                modified = True
+            elif to_kind in ('file', 'symlink') and (
+                    to_trans_id != from_trans_id
+                    or to_trans_id in self._new_contents):
+                modified = True
+            if (not modified and from_versioned == to_versioned
+                and from_parent == to_parent and from_name == to_name
+                    and from_executable == to_executable):
+                continue
+            results.append(
+                TreeChange(
+                    file_id, (from_path, to_path), modified,
+                    (from_versioned, to_versioned),
+                    (from_parent, to_parent),
+                    (from_name, to_name),
+                    (from_kind, to_kind),
+                    (from_executable, to_executable)))
+
+        def path_key(c):
+            return (c.path[0] or '', c.path[1] or '')
+        return iter(sorted(results, key=path_key))
+
+    def get_preview_tree(self):
+        """Return a tree representing the result of the transform.
+
+        The tree is a snapshot, and altering the TreeTransform will invalidate
+        it.
+        """
+        raise NotImplementedError(self.get_preview_tree)
+
+    def commit(self, branch, message, merge_parents=None, strict=False,
+               timestamp=None, timezone=None, committer=None, authors=None,
+               revprops=None, revision_id=None):
+        """Commit the result of this TreeTransform to a branch.
+
+        :param branch: The branch to commit to.
+        :param message: The message to attach to the commit.
+        :param merge_parents: Additional parent revision-ids specified by
+            pending merges.
+        :param strict: If True, abort the commit if there are unversioned
+            files.
+        :param timestamp: if not None, seconds-since-epoch for the time and
+            date.  (May be a float.)
+        :param timezone: Optional timezone for timestamp, as an offset in
+            seconds.
+        :param committer: Optional committer in email-id format.
+            (e.g. "J Random Hacker <jrandom@example.com>")
+        :param authors: Optional list of authors in email-id format.
+        :param revprops: Optional dictionary of revision properties.
+        :param revision_id: Optional revision id.  (Specifying a revision-id
+            may reduce performance for some non-native formats.)
+        :return: The revision_id of the revision committed.
+        """
+        self._check_malformed()
+        if strict:
+            unversioned = set(self._new_contents).difference(set(self._new_id))
+            for trans_id in unversioned:
+                if not self.final_is_versioned(trans_id):
+                    raise errors.StrictCommitFailed()
+
+        revno, last_rev_id = branch.last_revision_info()
+        if last_rev_id == _mod_revision.NULL_REVISION:
+            if merge_parents is not None:
+                raise ValueError('Cannot supply merge parents for first'
+                                 ' commit.')
+            parent_ids = []
+        else:
+            parent_ids = [last_rev_id]
+            if merge_parents is not None:
+                parent_ids.extend(merge_parents)
+        if self._tree.get_revision_id() != last_rev_id:
+            raise ValueError('TreeTransform not based on branch basis: %s' %
+                             self._tree.get_revision_id().decode('utf-8'))
+        from .. import commit
+        revprops = commit.Commit.update_revprops(revprops, branch, authors)
+        builder = branch.get_commit_builder(parent_ids,
+                                            timestamp=timestamp,
+                                            timezone=timezone,
+                                            committer=committer,
+                                            revprops=revprops,
+                                            revision_id=revision_id)
+        preview = self.get_preview_tree()
+        list(builder.record_iter_changes(preview, last_rev_id,
+                                         self.iter_changes()))
+        builder.finish_inventory()
+        revision_id = builder.commit(message)
+        branch.set_last_revision_info(revno + 1, revision_id)
+        return revision_id
+
+    def _text_parent(self, trans_id):
+        path = self.tree_path(trans_id)
+        try:
+            if path is None or self._tree.kind(path) != 'file':
+                return None
+        except errors.NoSuchFile:
+            return None
+        return path
+
+    def _get_parents_texts(self, trans_id):
+        """Get texts for compression parents of this file."""
+        path = self._text_parent(trans_id)
+        if path is None:
+            return ()
+        return (self._tree.get_file_text(path),)
+
+    def _get_parents_lines(self, trans_id):
+        """Get lines for compression parents of this file."""
+        path = self._text_parent(trans_id)
+        if path is None:
+            return ()
+        return (self._tree.get_file_lines(path),)
+
+    def serialize(self, serializer):
+        """Serialize this TreeTransform.
+
+        :param serializer: A Serialiser like pack.ContainerSerializer.
+        """
+        from .. import bencode
+        new_name = {k.encode('utf-8'): v.encode('utf-8')
+                    for k, v in viewitems(self._new_name)}
+        new_parent = {k.encode('utf-8'): v.encode('utf-8')
+                      for k, v in viewitems(self._new_parent)}
+        new_id = {k.encode('utf-8'): v
+                  for k, v in viewitems(self._new_id)}
+        new_executability = {k.encode('utf-8'): int(v)
+                             for k, v in viewitems(self._new_executability)}
+        tree_path_ids = {k.encode('utf-8'): v.encode('utf-8')
+                         for k, v in viewitems(self._tree_path_ids)}
+        non_present_ids = {k: v.encode('utf-8')
+                           for k, v in viewitems(self._non_present_ids)}
+        removed_contents = [trans_id.encode('utf-8')
+                            for trans_id in self._removed_contents]
+        removed_id = [trans_id.encode('utf-8')
+                      for trans_id in self._removed_id]
+        attribs = {
+            b'_id_number': self._id_number,
+            b'_new_name': new_name,
+            b'_new_parent': new_parent,
+            b'_new_executability': new_executability,
+            b'_new_id': new_id,
+            b'_tree_path_ids': tree_path_ids,
+            b'_removed_id': removed_id,
+            b'_removed_contents': removed_contents,
+            b'_non_present_ids': non_present_ids,
+            }
+        yield serializer.bytes_record(bencode.bencode(attribs),
+                                      ((b'attribs',),))
+        for trans_id, kind in sorted(viewitems(self._new_contents)):
+            if kind == 'file':
+                with open(self._limbo_name(trans_id), 'rb') as cur_file:
+                    lines = cur_file.readlines()
+                parents = self._get_parents_lines(trans_id)
+                mpdiff = multiparent.MultiParent.from_lines(lines, parents)
+                content = b''.join(mpdiff.to_patch())
+            if kind == 'directory':
+                content = b''
+            if kind == 'symlink':
+                content = self._read_symlink_target(trans_id)
+                if not isinstance(content, bytes):
+                    content = content.encode('utf-8')
+            yield serializer.bytes_record(
+                content, ((trans_id.encode('utf-8'), kind.encode('ascii')),))
+
+    def deserialize(self, records):
+        """Deserialize a stored TreeTransform.
+
+        :param records: An iterable of (names, content) tuples, as per
+            pack.ContainerPushParser.
+        """
+        from .. import bencode
+        names, content = next(records)
+        attribs = bencode.bdecode(content)
+        self._id_number = attribs[b'_id_number']
+        self._new_name = {k.decode('utf-8'): v.decode('utf-8')
+                          for k, v in viewitems(attribs[b'_new_name'])}
+        self._new_parent = {k.decode('utf-8'): v.decode('utf-8')
+                            for k, v in viewitems(attribs[b'_new_parent'])}
+        self._new_executability = {
+            k.decode('utf-8'): bool(v)
+            for k, v in viewitems(attribs[b'_new_executability'])}
+        self._new_id = {k.decode('utf-8'): v
+                        for k, v in viewitems(attribs[b'_new_id'])}
+        self._r_new_id = {v: k for k, v in viewitems(self._new_id)}
+        self._tree_path_ids = {}
+        self._tree_id_paths = {}
+        for bytepath, trans_id in viewitems(attribs[b'_tree_path_ids']):
+            path = bytepath.decode('utf-8')
+            trans_id = trans_id.decode('utf-8')
+            self._tree_path_ids[path] = trans_id
+            self._tree_id_paths[trans_id] = path
+        self._removed_id = {trans_id.decode('utf-8')
+                            for trans_id in attribs[b'_removed_id']}
+        self._removed_contents = set(
+            trans_id.decode('utf-8')
+            for trans_id in attribs[b'_removed_contents'])
+        self._non_present_ids = {
+            k: v.decode('utf-8')
+            for k, v in viewitems(attribs[b'_non_present_ids'])}
+        for ((trans_id, kind),), content in records:
+            trans_id = trans_id.decode('utf-8')
+            kind = kind.decode('ascii')
+            if kind == 'file':
+                mpdiff = multiparent.MultiParent.from_patch(content)
+                lines = mpdiff.to_lines(self._get_parents_texts(trans_id))
+                self.create_file(lines, trans_id)
+            if kind == 'directory':
+                self.create_directory(trans_id)
+            if kind == 'symlink':
+                self.create_symlink(content.decode('utf-8'), trans_id)
+
+    def create_file(self, contents, trans_id, mode_id=None, sha1=None):
+        """Schedule creation of a new file.
+
+        :seealso: new_file.
+
+        :param contents: an iterator of strings, all of which will be written
+            to the target destination.
+        :param trans_id: TreeTransform handle
+        :param mode_id: If not None, force the mode of the target file to match
+            the mode of the object referenced by mode_id.
+            Otherwise, we will try to preserve mode bits of an existing file.
+        :param sha1: If the sha1 of this content is already known, pass it in.
+            We can use it to prevent future sha1 computations.
+        """
+        raise NotImplementedError(self.create_file)
+
+    def create_directory(self, trans_id):
+        """Schedule creation of a new directory.
+
+        See also new_directory.
+        """
+        raise NotImplementedError(self.create_directory)
+
+    def create_symlink(self, target, trans_id):
+        """Schedule creation of a new symbolic link.
+
+        target is a bytestring.
+        See also new_symlink.
+        """
+        raise NotImplementedError(self.create_symlink)
+
+    def create_hardlink(self, path, trans_id):
+        """Schedule creation of a hard link"""
+        raise NotImplementedError(self.create_hardlink)
+
+    def cancel_creation(self, trans_id):
+        """Cancel the creation of new file contents."""
+        raise NotImplementedError(self.cancel_creation)
+
+    def apply(self, no_conflicts=False, precomputed_delta=None, _mover=None):
+        """Apply all changes to the inventory and filesystem.
+
+        If filesystem or inventory conflicts are present, MalformedTransform
+        will be thrown.
+
+        If apply succeeds, finalize is not necessary.
+
+        :param no_conflicts: if True, the caller guarantees there are no
+            conflicts, so no check is made.
+        :param precomputed_delta: An inventory delta to use instead of
+            calculating one.
+        :param _mover: Supply an alternate FileMover, for testing
+        """
+        raise NotImplementedError(self.apply)
+
+
+class DiskTreeTransform(TreeTransformBase):
+    """Tree transform storing its contents on disk."""
+
+    def __init__(self, tree, limbodir, pb=None, case_sensitive=True):
+        """Constructor.
+        :param tree: The tree that will be transformed, but not necessarily
+            the output tree.
+        :param limbodir: A directory where new files can be stored until
+            they are installed in their proper places
+        :param pb: ignored
+        :param case_sensitive: If True, the target of the transform is
+            case sensitive, not just case preserving.
+        """
+        TreeTransformBase.__init__(self, tree, pb, case_sensitive)
+        self._limbodir = limbodir
+        self._deletiondir = None
+        # A mapping of transform ids to their limbo filename
+        self._limbo_files = {}
+        self._possibly_stale_limbo_files = set()
+        # A mapping of transform ids to a set of the transform ids of children
+        # that their limbo directory has
+        self._limbo_children = {}
+        # Map transform ids to maps of child filename to child transform id
+        self._limbo_children_names = {}
+        # List of transform ids that need to be renamed from limbo into place
+        self._needs_rename = set()
+        self._creation_mtime = None
+        self._create_symlinks = osutils.supports_symlinks(self._limbodir)
+
+    def finalize(self):
+        """Release the working tree lock, if held, clean up limbo dir.
+
+        This is required if apply has not been invoked, but can be invoked
+        even after apply.
+        """
+        if self._tree is None:
+            return
+        try:
+            limbo_paths = list(viewvalues(self._limbo_files))
+            limbo_paths.extend(self._possibly_stale_limbo_files)
+            limbo_paths.sort(reverse=True)
+            for path in limbo_paths:
+                try:
+                    osutils.delete_any(path)
+                except OSError as e:
+                    if e.errno != errno.ENOENT:
+                        raise
+                    # XXX: warn? perhaps we just got interrupted at an
+                    # inconvenient moment, but perhaps files are disappearing
+                    # from under us?
+            try:
+                osutils.delete_any(self._limbodir)
+            except OSError:
+                # We don't especially care *why* the dir is immortal.
+                raise ImmortalLimbo(self._limbodir)
+            try:
+                if self._deletiondir is not None:
+                    osutils.delete_any(self._deletiondir)
+            except OSError:
+                raise errors.ImmortalPendingDeletion(self._deletiondir)
+        finally:
+            TreeTransformBase.finalize(self)
+
+    def _limbo_supports_executable(self):
+        """Check if the limbo path supports the executable bit."""
+        return osutils.supports_executable(self._limbodir)
+
+    def _limbo_name(self, trans_id):
+        """Generate the limbo name of a file"""
+        limbo_name = self._limbo_files.get(trans_id)
+        if limbo_name is None:
+            limbo_name = self._generate_limbo_path(trans_id)
+            self._limbo_files[trans_id] = limbo_name
+        return limbo_name
+
+    def _generate_limbo_path(self, trans_id):
+        """Generate a limbo path using the trans_id as the relative path.
+
+        This is suitable as a fallback, and when the transform should not be
+        sensitive to the path encoding of the limbo directory.
+        """
+        self._needs_rename.add(trans_id)
+        return osutils.pathjoin(self._limbodir, trans_id)
+
+    def adjust_path(self, name, parent, trans_id):
+        previous_parent = self._new_parent.get(trans_id)
+        previous_name = self._new_name.get(trans_id)
+        super(DiskTreeTransform, self).adjust_path(name, parent, trans_id)
+        if (trans_id in self._limbo_files
+                and trans_id not in self._needs_rename):
+            self._rename_in_limbo([trans_id])
+            if previous_parent != parent:
+                self._limbo_children[previous_parent].remove(trans_id)
+            if previous_parent != parent or previous_name != name:
+                del self._limbo_children_names[previous_parent][previous_name]
+
+    def _rename_in_limbo(self, trans_ids):
+        """Fix limbo names so that the right final path is produced.
+
+        This means we outsmarted ourselves-- we tried to avoid renaming
+        these files later by creating them with their final names in their
+        final parents.  But now the previous name or parent is no longer
+        suitable, so we have to rename them.
+
+        Even for trans_ids that have no new contents, we must remove their
+        entries from _limbo_files, because they are now stale.
+        """
+        for trans_id in trans_ids:
+            old_path = self._limbo_files[trans_id]
+            self._possibly_stale_limbo_files.add(old_path)
+            del self._limbo_files[trans_id]
+            if trans_id not in self._new_contents:
+                continue
+            new_path = self._limbo_name(trans_id)
+            os.rename(old_path, new_path)
+            self._possibly_stale_limbo_files.remove(old_path)
+            for descendant in self._limbo_descendants(trans_id):
+                desc_path = self._limbo_files[descendant]
+                desc_path = new_path + desc_path[len(old_path):]
+                self._limbo_files[descendant] = desc_path
+
+    def _limbo_descendants(self, trans_id):
+        """Return the set of trans_ids whose limbo paths descend from this."""
+        descendants = set(self._limbo_children.get(trans_id, []))
+        for descendant in list(descendants):
+            descendants.update(self._limbo_descendants(descendant))
+        return descendants
+
+    def _set_mode(self, trans_id, mode_id, typefunc):
+        raise NotImplementedError(self._set_mode)
+
+    def create_file(self, contents, trans_id, mode_id=None, sha1=None):
+        """Schedule creation of a new file.
+
+        :seealso: new_file.
+
+        :param contents: an iterator of strings, all of which will be written
+            to the target destination.
+        :param trans_id: TreeTransform handle
+        :param mode_id: If not None, force the mode of the target file to match
+            the mode of the object referenced by mode_id.
+            Otherwise, we will try to preserve mode bits of an existing file.
+        :param sha1: If the sha1 of this content is already known, pass it in.
+            We can use it to prevent future sha1 computations.
+        """
+        name = self._limbo_name(trans_id)
+        with open(name, 'wb') as f:
+            unique_add(self._new_contents, trans_id, 'file')
+            f.writelines(contents)
+        self._set_mtime(name)
+        self._set_mode(trans_id, mode_id, S_ISREG)
+        # It is unfortunate we have to use lstat instead of fstat, but we just
+        # used utime and chmod on the file, so we need the accurate final
+        # details.
+        if sha1 is not None:
+            self._observed_sha1s[trans_id] = (sha1, osutils.lstat(name))
+
+    def _read_symlink_target(self, trans_id):
+        return os.readlink(self._limbo_name(trans_id))
+
+    def _set_mtime(self, path):
+        """All files that are created get the same mtime.
+
+        This time is set by the first object to be created.
+        """
+        if self._creation_mtime is None:
+            self._creation_mtime = time.time()
+        os.utime(path, (self._creation_mtime, self._creation_mtime))
+
+    def create_hardlink(self, path, trans_id):
+        """Schedule creation of a hard link"""
+        name = self._limbo_name(trans_id)
+        try:
+            os.link(path, name)
+        except OSError as e:
+            if e.errno != errno.EPERM:
+                raise
+            raise errors.HardLinkNotSupported(path)
+        try:
+            unique_add(self._new_contents, trans_id, 'file')
+        except BaseException:
+            # Clean up the file, it never got registered so
+            # TreeTransform.finalize() won't clean it up.
+            os.unlink(name)
+            raise
+
+    def create_directory(self, trans_id):
+        """Schedule creation of a new directory.
+
+        See also new_directory.
+        """
+        os.mkdir(self._limbo_name(trans_id))
+        unique_add(self._new_contents, trans_id, 'directory')
+
+    def create_symlink(self, target, trans_id):
+        """Schedule creation of a new symbolic link.
+
+        target is a bytestring.
+        See also new_symlink.
+        """
+        if self._create_symlinks:
+            os.symlink(target, self._limbo_name(trans_id))
+        else:
+            try:
+                path = FinalPaths(self).get_path(trans_id)
+            except KeyError:
+                path = None
+            trace.warning(
+                'Unable to create symlink "%s" on this filesystem.' % (path,))
+        # We add symlink to _new_contents even if they are unsupported
+        # and not created. These entries are subsequently used to avoid
+        # conflicts on platforms that don't support symlink
+        unique_add(self._new_contents, trans_id, 'symlink')
+
+    def cancel_creation(self, trans_id):
+        """Cancel the creation of new file contents."""
+        del self._new_contents[trans_id]
+        if trans_id in self._observed_sha1s:
+            del self._observed_sha1s[trans_id]
+        children = self._limbo_children.get(trans_id)
+        # if this is a limbo directory with children, move them before removing
+        # the directory
+        if children is not None:
+            self._rename_in_limbo(children)
+            del self._limbo_children[trans_id]
+            del self._limbo_children_names[trans_id]
+        osutils.delete_any(self._limbo_name(trans_id))
+
+    def new_orphan(self, trans_id, parent_id):
+        conf = self._tree.get_config_stack()
+        handle_orphan = conf.get('transform.orphan_policy')
+        handle_orphan(self, trans_id, parent_id)
+
+
+class GitTreeTransform(DiskTreeTransform):
+    """Represent a tree transformation.
+
+    This object is designed to support incremental generation of the transform,
+    in any order.
+
+    However, it gives optimum performance when parent directories are created
+    before their contents.  The transform is then able to put child files
+    directly in their parent directory, avoiding later renames.
+
+    It is easy to produce malformed transforms, but they are generally
+    harmless.  Attempting to apply a malformed transform will cause an
+    exception to be raised before any modifications are made to the tree.
+
+    Many kinds of malformed transforms can be corrected with the
+    resolve_conflicts function.  The remaining ones indicate programming error,
+    such as trying to create a file with no path.
+
+    Two sets of file creation methods are supplied.  Convenience methods are:
+     * new_file
+     * new_directory
+     * new_symlink
+
+    These are composed of the low-level methods:
+     * create_path
+     * create_file or create_directory or create_symlink
+     * version_file
+     * set_executability
+
+    Transform/Transaction ids
+    -------------------------
+    trans_ids are temporary ids assigned to all files involved in a transform.
+    It's possible, even common, that not all files in the Tree have trans_ids.
+
+    trans_ids are used because filenames and file_ids are not good enough
+    identifiers; filenames change.
+
+    trans_ids are only valid for the TreeTransform that generated them.
+
+    Limbo
+    -----
+    Limbo is a temporary directory use to hold new versions of files.
+    Files are added to limbo by create_file, create_directory, create_symlink,
+    and their convenience variants (new_*).  Files may be removed from limbo
+    using cancel_creation.  Files are renamed from limbo into their final
+    location as part of TreeTransform.apply
+
+    Limbo must be cleaned up, by either calling TreeTransform.apply or
+    calling TreeTransform.finalize.
+
+    Files are placed into limbo inside their parent directories, where
+    possible.  This reduces subsequent renames, and makes operations involving
+    lots of files faster.  This optimization is only possible if the parent
+    directory is created *before* creating any of its children, so avoid
+    creating children before parents, where possible.
+
+    Pending-deletion
+    ----------------
+    This temporary directory is used by _FileMover for storing files that are
+    about to be deleted.  In case of rollback, the files will be restored.
+    FileMover does not delete files until it is sure that a rollback will not
+    happen.
+    """
+
+    def __init__(self, tree, pb=None):
+        """Note: a tree_write lock is taken on the tree.
+
+        Use TreeTransform.finalize() to release the lock (can be omitted if
+        TreeTransform.apply() called).
+        """
+        tree.lock_tree_write()
+        try:
+            limbodir = urlutils.local_path_from_url(
+                tree._transport.abspath('limbo'))
+            osutils.ensure_empty_directory_exists(
+                limbodir,
+                errors.ExistingLimbo)
+            deletiondir = urlutils.local_path_from_url(
+                tree._transport.abspath('pending-deletion'))
+            osutils.ensure_empty_directory_exists(
+                deletiondir,
+                errors.ExistingPendingDeletion)
+        except BaseException:
+            tree.unlock()
+            raise
+
+        # Cache of realpath results, to speed up canonical_path
+        self._realpaths = {}
+        # Cache of relpath results, to speed up canonical_path
+        self._relpaths = {}
+        DiskTreeTransform.__init__(self, tree, limbodir, pb,
+                                   tree.case_sensitive)
+        self._deletiondir = deletiondir
+
+    def canonical_path(self, path):
+        """Get the canonical tree-relative path"""
+        # don't follow final symlinks
+        abs = self._tree.abspath(path)
+        if abs in self._relpaths:
+            return self._relpaths[abs]
+        dirname, basename = os.path.split(abs)
+        if dirname not in self._realpaths:
+            self._realpaths[dirname] = os.path.realpath(dirname)
+        dirname = self._realpaths[dirname]
+        abs = osutils.pathjoin(dirname, basename)
+        if dirname in self._relpaths:
+            relpath = osutils.pathjoin(self._relpaths[dirname], basename)
+            relpath = relpath.rstrip('/\\')
+        else:
+            relpath = self._tree.relpath(abs)
+        self._relpaths[abs] = relpath
+        return relpath
+
+    def tree_kind(self, trans_id):
+        """Determine the file kind in the working tree.
+
+        :returns: The file kind or None if the file does not exist
+        """
+        path = self._tree_id_paths.get(trans_id)
+        if path is None:
+            return None
+        try:
+            return osutils.file_kind(self._tree.abspath(path))
+        except errors.NoSuchFile:
+            return None
+
+    def _set_mode(self, trans_id, mode_id, typefunc):
+        """Set the mode of new file contents.
+        The mode_id is the existing file to get the mode from (often the same
+        as trans_id).  The operation is only performed if there's a mode match
+        according to typefunc.
+        """
+        if mode_id is None:
+            mode_id = trans_id
+        try:
+            old_path = self._tree_id_paths[mode_id]
+        except KeyError:
+            return
+        try:
+            mode = os.stat(self._tree.abspath(old_path)).st_mode
+        except OSError as e:
+            if e.errno in (errno.ENOENT, errno.ENOTDIR):
+                # Either old_path doesn't exist, or the parent of the
+                # target is not a directory (but will be one eventually)
+                # Either way, we know it doesn't exist *right now*
+                # See also bug #248448
+                return
+            else:
+                raise
+        if typefunc(mode):
+            osutils.chmod_if_possible(self._limbo_name(trans_id), mode)
+
+    def iter_tree_children(self, parent_id):
+        """Iterate through the entry's tree children, if any"""
+        try:
+            path = self._tree_id_paths[parent_id]
+        except KeyError:
+            return
+        try:
+            children = os.listdir(self._tree.abspath(path))
+        except OSError as e:
+            if not (osutils._is_error_enotdir(e) or
+                    e.errno in (errno.ENOENT, errno.ESRCH)):
+                raise
+            return
+
+        for child in children:
+            childpath = joinpath(path, child)
+            if self._tree.is_control_filename(childpath):
+                continue
+            yield self.trans_id_tree_path(childpath)
+
+    def _generate_limbo_path(self, trans_id):
+        """Generate a limbo path using the final path if possible.
+
+        This optimizes the performance of applying the tree transform by
+        avoiding renames.  These renames can be avoided only when the parent
+        directory is already scheduled for creation.
+
+        If the final path cannot be used, falls back to using the trans_id as
+        the relpath.
+        """
+        parent = self._new_parent.get(trans_id)
+        # if the parent directory is already in limbo (e.g. when building a
+        # tree), choose a limbo name inside the parent, to reduce further
+        # renames.
+        use_direct_path = False
+        if self._new_contents.get(parent) == 'directory':
+            filename = self._new_name.get(trans_id)
+            if filename is not None:
+                if parent not in self._limbo_children:
+                    self._limbo_children[parent] = set()
+                    self._limbo_children_names[parent] = {}
+                    use_direct_path = True
+                # the direct path can only be used if no other file has
+                # already taken this pathname, i.e. if the name is unused, or
+                # if it is already associated with this trans_id.
+                elif self._case_sensitive_target:
+                    if (self._limbo_children_names[parent].get(filename)
+                            in (trans_id, None)):
+                        use_direct_path = True
+                else:
+                    for l_filename, l_trans_id in viewitems(
+                            self._limbo_children_names[parent]):
+                        if l_trans_id == trans_id:
+                            continue
+                        if l_filename.lower() == filename.lower():
+                            break
+                    else:
+                        use_direct_path = True
+
+        if not use_direct_path:
+            return DiskTreeTransform._generate_limbo_path(self, trans_id)
+
+        limbo_name = osutils.pathjoin(self._limbo_files[parent], filename)
+        self._limbo_children[parent].add(trans_id)
+        self._limbo_children_names[parent][filename] = trans_id
+        return limbo_name
+
+    def version_file(self, trans_id, file_id=None):
+        """Schedule a file to become versioned."""
+        if file_id is None:
+            raise ValueError()
+        unique_add(self._new_id, trans_id, file_id)
+        unique_add(self._r_new_id, file_id, trans_id)
+
+    def cancel_versioning(self, trans_id):
+        """Undo a previous versioning of a file"""
+        file_id = self._new_id[trans_id]
+        del self._new_id[trans_id]
+        del self._r_new_id[file_id]
+
+    def apply(self, no_conflicts=False, precomputed_delta=None, _mover=None):
+        """Apply all changes to the inventory and filesystem.
+
+        If filesystem or inventory conflicts are present, MalformedTransform
+        will be thrown.
+
+        If apply succeeds, finalize is not necessary.
+
+        :param no_conflicts: if True, the caller guarantees there are no
+            conflicts, so no check is made.
+        :param precomputed_delta: An inventory delta to use instead of
+            calculating one.
+        :param _mover: Supply an alternate FileMover, for testing
+        """
+        for hook in MutableTree.hooks['pre_transform']:
+            hook(self._tree, self)
+        if not no_conflicts:
+            self._check_malformed()
+        self.rename_count = 0
+        with ui.ui_factory.nested_progress_bar() as child_pb:
+            if precomputed_delta is None:
+                child_pb.update(gettext('Apply phase'), 0, 2)
+                changes = self._generate_transform_changes()
+                offset = 1
+            else:
+                changes = [
+                    (op, np, ie) for (op, np, fid, ie) in precomputed_delta]
+                offset = 0
+            if _mover is None:
+                mover = _FileMover()
+            else:
+                mover = _mover
+            try:
+                child_pb.update(gettext('Apply phase'), 0 + offset, 2 + offset)
+                self._apply_removals(mover)
+                child_pb.update(gettext('Apply phase'), 1 + offset, 2 + offset)
+                modified_paths = self._apply_insertions(mover)
+            except BaseException:
+                mover.rollback()
+                raise
+            else:
+                mover.apply_deletions()
+        if self.final_file_id(self.root) is None:
+            changes = [e for e in changes if e[0] != '']
+        self._tree._apply_transform_delta(changes)
+        self._done = True
+        self.finalize()
+        return _TransformResults(modified_paths, self.rename_count)
+
+    def _apply_removals(self, mover):
+        """Perform tree operations that remove directory/inventory names.
+
+        That is, delete files that are to be deleted, and put any files that
+        need renaming into limbo.  This must be done in strict child-to-parent
+        order.
+
+        If inventory_delta is None, no inventory delta generation is performed.
+        """
+        tree_paths = sorted(viewitems(self._tree_path_ids), reverse=True)
+        with ui.ui_factory.nested_progress_bar() as child_pb:
+            for num, (path, trans_id) in enumerate(tree_paths):
+                # do not attempt to move root into a subdirectory of itself.
+                if path == '':
+                    continue
+                child_pb.update(gettext('removing file'), num, len(tree_paths))
+                full_path = self._tree.abspath(path)
+                if trans_id in self._removed_contents:
+                    delete_path = os.path.join(self._deletiondir, trans_id)
+                    mover.pre_delete(full_path, delete_path)
+                elif (trans_id in self._new_name or
+                      trans_id in self._new_parent):
+                    try:
+                        mover.rename(full_path, self._limbo_name(trans_id))
+                    except TransformRenameFailed as e:
+                        if e.errno != errno.ENOENT:
+                            raise
+                    else:
+                        self.rename_count += 1
+
+    def _apply_insertions(self, mover):
+        """Perform tree operations that insert directory/inventory names.
+
+        That is, create any files that need to be created, and restore from
+        limbo any files that needed renaming.  This must be done in strict
+        parent-to-child order.
+
+        If inventory_delta is None, no inventory delta is calculated, and
+        no list of modified paths is returned.
+        """
+        new_paths = self.new_paths(filesystem_only=True)
+        modified_paths = []
+        with ui.ui_factory.nested_progress_bar() as child_pb:
+            for num, (path, trans_id) in enumerate(new_paths):
+                if (num % 10) == 0:
+                    child_pb.update(gettext('adding file'),
+                                    num, len(new_paths))
+                full_path = self._tree.abspath(path)
+                if trans_id in self._needs_rename:
+                    try:
+                        mover.rename(self._limbo_name(trans_id), full_path)
+                    except TransformRenameFailed as e:
+                        # We may be renaming a dangling inventory id
+                        if e.errno != errno.ENOENT:
+                            raise
+                    else:
+                        self.rename_count += 1
+                    # TODO: if trans_id in self._observed_sha1s, we should
+                    #       re-stat the final target, since ctime will be
+                    #       updated by the change.
+                if (trans_id in self._new_contents
+                        or self.path_changed(trans_id)):
+                    if trans_id in self._new_contents:
+                        modified_paths.append(full_path)
+                if trans_id in self._new_executability:
+                    self._set_executability(path, trans_id)
+                if trans_id in self._observed_sha1s:
+                    o_sha1, o_st_val = self._observed_sha1s[trans_id]
+                    st = osutils.lstat(full_path)
+                    self._observed_sha1s[trans_id] = (o_sha1, st)
+        for path, trans_id in new_paths:
+            # new_paths includes stuff like workingtree conflicts. Only the
+            # stuff in new_contents actually comes from limbo.
+            if trans_id in self._limbo_files:
+                del self._limbo_files[trans_id]
+        self._new_contents.clear()
+        return modified_paths
+
+    def _inventory_altered(self):
+        """Determine which trans_ids need new Inventory entries.
+
+        An new entry is needed when anything that would be reflected by an
+        inventory entry changes, including file name, file_id, parent file_id,
+        file kind, and the execute bit.
+
+        Some care is taken to return entries with real changes, not cases
+        where the value is deleted and then restored to its original value,
+        but some actually unchanged values may be returned.
+
+        :returns: A list of (path, trans_id) for all items requiring an
+            inventory change. Ordered by path.
+        """
+        changed_ids = set()
+        # Find entries whose file_ids are new (or changed).
+        new_file_id = set(t for t in self._new_id
+                          if self._new_id[t] != self.tree_file_id(t))
+        for id_set in [self._new_name, self._new_parent, new_file_id,
+                       self._new_executability]:
+            changed_ids.update(id_set)
+        # removing implies a kind change
+        changed_kind = set(self._removed_contents)
+        # so does adding
+        changed_kind.intersection_update(self._new_contents)
+        # Ignore entries that are already known to have changed.
+        changed_kind.difference_update(changed_ids)
+        #  to keep only the truly changed ones
+        changed_kind = (t for t in changed_kind
+                        if self.tree_kind(t) != self.final_kind(t))
+        # all kind changes will alter the inventory
+        changed_ids.update(changed_kind)
+        # To find entries with changed parent_ids, find parents which existed,
+        # but changed file_id.
+        # Now add all their children to the set.
+        for parent_trans_id in new_file_id:
+            changed_ids.update(self.iter_tree_children(parent_trans_id))
+        return sorted(FinalPaths(self).get_paths(changed_ids))
+
+    def _generate_transform_changes(self):
+        """Generate an inventory delta for the current transform."""
+        changes = []
+        new_paths = self._inventory_altered()
+        total_entries = len(new_paths) + len(self._removed_id)
+        with ui.ui_factory.nested_progress_bar() as child_pb:
+            for num, trans_id in enumerate(self._removed_id):
+                if (num % 10) == 0:
+                    child_pb.update(gettext('removing file'),
+                                    num, total_entries)
+                if trans_id == self._new_root:
+                    file_id = self._tree.path2id('')
+                else:
+                    file_id = self.tree_file_id(trans_id)
+                # File-id isn't really being deleted, just moved
+                if file_id in self._r_new_id:
+                    continue
+                path = self._tree_id_paths[trans_id]
+                changes.append((path, None, None))
+            new_path_file_ids = dict((t, self.final_file_id(t)) for p, t in
+                                     new_paths)
+            for num, (path, trans_id) in enumerate(new_paths):
+                if (num % 10) == 0:
+                    child_pb.update(gettext('adding file'),
+                                    num + len(self._removed_id), total_entries)
+                file_id = new_path_file_ids[trans_id]
+                if file_id is None:
+                    continue
+                kind = self.final_kind(trans_id)
+                if kind is None:
+                    kind = self._tree.stored_kind(self._tree.id2path(file_id))
+                parent_trans_id = self.final_parent(trans_id)
+                parent_file_id = new_path_file_ids.get(parent_trans_id)
+                if parent_file_id is None:
+                    parent_file_id = self.final_file_id(parent_trans_id)
+                if trans_id in self._new_reference_revision:
+                    new_entry = inventory.TreeReference(
+                        file_id,
+                        self._new_name[trans_id],
+                        self.final_file_id(self._new_parent[trans_id]),
+                        None, self._new_reference_revision[trans_id])
+                else:
+                    new_entry = inventory.make_entry(kind,
+                                                     self.final_name(trans_id),
+                                                     parent_file_id, file_id)
+                try:
+                    old_path = self._tree.id2path(new_entry.file_id)
+                except errors.NoSuchId:
+                    old_path = None
+                new_executability = self._new_executability.get(trans_id)
+                if new_executability is not None:
+                    new_entry.executable = new_executability
+                changes.append(
+                    (old_path, path, new_entry))
+        return changes

=== modified file 'breezy/git/tree.py'
--- breezy/git/tree.py	2020-06-23 01:02:30 +0000
+++ breezy/git/tree.py	2020-07-18 23:14:00 +0000
@@ -66,6 +66,8 @@
     )
 
 from .mapping import (
+    encode_git_path,
+    decode_git_path,
     mode_is_executable,
     mode_kind,
     default_mapping,
@@ -296,10 +298,10 @@
             info = self._submodule_info()[relpath]
         except KeyError:
             nested_repo_transport = self._repository.controldir.user_transport.clone(
-                relpath.decode('utf-8'))
+                decode_git_path(relpath))
         else:
             nested_repo_transport = self._repository.controldir.control_transport.clone(
-                posixpath.join('modules', info[1].decode('utf-8')))
+                posixpath.join('modules', decode_git_path(info[1])))
         nested_controldir = _mod_controldir.ControlDir.open_from_transport(
             nested_repo_transport)
         return nested_controldir.find_repository()
@@ -308,7 +310,7 @@
         return self._get_submodule_repository(relpath)._git.object_store
 
     def get_nested_tree(self, path):
-        encoded_path = path.encode('utf-8')
+        encoded_path = encode_git_path(path)
         nested_repo = self._get_submodule_repository(encoded_path)
         ref_rev = self.get_reference_revision(path)
         return nested_repo.revision_tree(ref_rev)
@@ -321,7 +323,7 @@
         if self.commit_id == ZERO_SHA:
             return NULL_REVISION
         (unused_path, commit_id) = change_scanner.find_last_change_revision(
-            path.encode('utf-8'), self.commit_id)
+            encode_git_path(path), self.commit_id)
         return self._repository.lookup_foreign_revision_id(
             commit_id, self.mapping)
 
@@ -368,7 +370,7 @@
             tree = store[tree_id]
             for name, mode, hexsha in tree.items():
                 subpath = posixpath.join(path, name)
-                ret.add(subpath.decode('utf-8'))
+                ret.add(decode_git_path(subpath))
                 if stat.S_ISDIR(mode):
                     todo.append((store, subpath, hexsha))
         return ret
@@ -377,7 +379,7 @@
         if self.tree is None:
             raise errors.NoSuchFile(path)
 
-        encoded_path = path.encode('utf-8')
+        encoded_path = encode_git_path(path)
         parts = encoded_path.split(b'/')
         hexsha = self.tree
         store = self.store
@@ -444,16 +446,16 @@
             parent_path = posixpath.dirname(from_dir)
             parent_id = self.mapping.generate_file_id(parent_path)
             if mode_kind(mode) == 'directory':
-                root_ie = self._get_dir_ie(from_dir.encode("utf-8"), parent_id)
+                root_ie = self._get_dir_ie(encode_git_path(from_dir), parent_id)
             else:
                 root_ie = self._get_file_ie(
-                    store, from_dir.encode("utf-8"),
+                    store, encode_git_path(from_dir),
                     posixpath.basename(from_dir), mode, hexsha)
         if include_root:
             yield (from_dir, "V", root_ie.kind, root_ie)
         todo = []
         if root_ie.kind == 'directory':
-            todo.append((store, from_dir.encode("utf-8"),
+            todo.append((store, encode_git_path(from_dir),
                          b"", hexsha, root_ie.file_id))
         while todo:
             (store, path, relpath, hexsha, parent_id) = todo.pop()
@@ -476,7 +478,7 @@
                 else:
                     ie = self._get_file_ie(
                         store, child_path, name, mode, hexsha, parent_id)
-                yield (child_relpath.decode('utf-8'), "V", ie.kind, ie)
+                yield (decode_git_path(child_relpath), "V", ie.kind, ie)
 
     def _get_file_ie(self, store, path, name, mode, hexsha, parent_id):
         if not isinstance(path, bytes):
@@ -484,12 +486,12 @@
         if not isinstance(name, bytes):
             raise TypeError(name)
         kind = mode_kind(mode)
-        path = path.decode('utf-8')
-        name = name.decode("utf-8")
+        path = decode_git_path(path)
+        name = decode_git_path(name)
         file_id = self.mapping.generate_file_id(path)
         ie = entry_factory[kind](file_id, name, parent_id)
         if kind == 'symlink':
-            ie.symlink_target = store[hexsha].data.decode('utf-8')
+            ie.symlink_target = decode_git_path(store[hexsha].data)
         elif kind == 'tree-reference':
             ie.reference_revision = self.mapping.revision_id_foreign_to_bzr(
                 hexsha)
@@ -501,7 +503,7 @@
         return ie
 
     def _get_dir_ie(self, path, parent_id):
-        path = path.decode('utf-8')
+        path = decode_git_path(path)
         file_id = self.mapping.generate_file_id(path)
         return GitTreeDirectory(file_id, posixpath.basename(path), parent_id)
 
@@ -511,7 +513,7 @@
         if mode is not None and not stat.S_ISDIR(mode):
             return
 
-        encoded_path = path.encode('utf-8')
+        encoded_path = encode_git_path(path)
         file_id = self.path2id(path)
         tree = store[tree_sha]
         for name, mode, hexsha in tree.iteritems():
@@ -532,7 +534,7 @@
             if specific_files in ([""], []):
                 specific_files = None
             else:
-                specific_files = set([p.encode('utf-8')
+                specific_files = set([encode_git_path(p)
                                       for p in specific_files])
         todo = deque([(self.store, b"", self.tree, self.path2id(''))])
         if specific_files is None or u"" in specific_files:
@@ -545,7 +547,7 @@
                 if self.mapping.is_special_file(name):
                     continue
                 child_path = posixpath.join(path, name)
-                child_path_decoded = child_path.decode('utf-8')
+                child_path_decoded = decode_git_path(child_path)
                 if recurse_nested and S_ISGITLINK(mode):
                     mode = stat.S_IFDIR
                     store = self._get_submodule_store(child_path)
@@ -604,7 +606,7 @@
         """See RevisionTree.get_symlink_target."""
         (store, mode, hexsha) = self._lookup_path(path)
         if stat.S_ISLNK(mode):
-            return store[hexsha].data.decode('utf-8')
+            return decode_git_path(store[hexsha].data)
         else:
             return None
 
@@ -613,7 +615,7 @@
         (store, mode, hexsha) = self._lookup_path(path)
         if S_ISGITLINK(mode):
             try:
-                nested_repo = self._get_submodule_repository(path.encode('utf-8'))
+                nested_repo = self._get_submodule_repository(encode_git_path(path))
             except errors.NotBranchError:
                 return self.mapping.revision_id_foreign_to_bzr(hexsha)
             else:
@@ -639,9 +641,9 @@
             return (kind, len(contents), executable,
                     osutils.sha_string(contents))
         elif kind == 'symlink':
-            return (kind, None, None, store[hexsha].data.decode('utf-8'))
+            return (kind, None, None, decode_git_path(store[hexsha].data))
         elif kind == 'tree-reference':
-            nested_repo = self._get_submodule_repository(path.encode('utf-8'))
+            nested_repo = self._get_submodule_repository(encode_git_path(path))
             return (kind, None, None,
                     nested_repo.lookup_foreign_revision_id(hexsha))
         else:
@@ -697,25 +699,29 @@
     def walkdirs(self, prefix=u""):
         (store, mode, hexsha) = self._lookup_path(prefix)
         todo = deque(
-            [(store, prefix.encode('utf-8'), hexsha, self.path2id(prefix))])
+            [(store, encode_git_path(prefix), hexsha, self.path2id(prefix))])
         while todo:
             store, path, tree_sha, parent_id = todo.popleft()
-            path_decoded = path.decode('utf-8')
+            path_decoded = decode_git_path(path)
             tree = store[tree_sha]
             children = []
             for name, mode, hexsha in tree.iteritems():
                 if self.mapping.is_special_file(name):
                     continue
                 child_path = posixpath.join(path, name)
-                file_id = self.path2id(child_path.decode('utf-8'))
+                file_id = self.path2id(decode_git_path(child_path))
                 if stat.S_ISDIR(mode):
                     todo.append((store, child_path, hexsha, file_id))
                 children.append(
-                    (child_path.decode('utf-8'), name.decode('utf-8'),
+                    (decode_git_path(child_path), decode_git_path(name),
                         mode_kind(mode), None,
                         file_id, mode_kind(mode)))
             yield (path_decoded, parent_id), children
 
+    def preview_transform(self, pb=None):
+        from .transform import GitTransformPreview
+        return GitTransformPreview(self, pb=pb)
+
 
 def tree_delta_from_git_changes(changes, mappings,
                                 specific_files=None,
@@ -740,11 +746,11 @@
             continue
         copied = (change_type == 'copy')
         if oldpath is not None:
-            oldpath_decoded = oldpath.decode('utf-8')
+            oldpath_decoded = decode_git_path(oldpath)
         else:
             oldpath_decoded = None
         if newpath is not None:
-            newpath_decoded = newpath.decode('utf-8')
+            newpath_decoded = decode_git_path(newpath)
         else:
             newpath_decoded = None
         if not (specific_files is None or
@@ -849,7 +855,7 @@
     for path, kind in added:
         if kind == 'directory' and path not in implicit_dirs:
             continue
-        path_decoded = osutils.normalized_filename(path)[0]
+        path_decoded = decode_git_path(path)
         parent_path, basename = osutils.split(path_decoded)
         parent_id = new_mapping.generate_file_id(parent_path)
         file_id = new_mapping.generate_file_id(path_decoded)
@@ -881,11 +887,11 @@
         (oldpath, oldmode, oldsha) = old
         (newpath, newmode, newsha) = new
         if oldpath is not None:
-            oldpath_decoded = oldpath.decode('utf-8')
+            oldpath_decoded = decode_git_path(oldpath)
         else:
             oldpath_decoded = None
         if newpath is not None:
-            newpath_decoded = newpath.decode('utf-8')
+            newpath_decoded = decode_git_path(newpath)
         else:
             newpath_decoded = None
         if not (specific_files is None or
@@ -1005,7 +1011,7 @@
 
     def _iter_git_changes(self, want_unchanged=False, specific_files=None,
                           require_versioned=False, extra_trees=None,
-                          want_unversioned=False):
+                          want_unversioned=False, include_trees=True):
         raise NotImplementedError(self._iter_git_changes)
 
     def find_target_path(self, path, recurse='none'):
@@ -1019,12 +1025,14 @@
     def find_target_paths(self, paths, recurse='none'):
         paths = set(paths)
         ret = {}
-        changes = self._iter_git_changes(specific_files=paths)[0]
+        changes = self._iter_git_changes(
+            specific_files=paths, include_trees=False)[0]
         for (change_type, old, new) in changes:
-            oldpath = old[0]
-            newpath = new[0]
+            if old[0] is None:
+                continue
+            oldpath = decode_git_path(old[0])
             if oldpath in paths:
-                ret[oldpath] = newpath
+                ret[oldpath] = decode_git_path(new[0]) if new[0] else None
         for path in paths:
             if path not in ret:
                 if self.source.has_filename(path):
@@ -1039,12 +1047,14 @@
     def find_source_paths(self, paths, recurse='none'):
         paths = set(paths)
         ret = {}
-        changes = self._iter_git_changes(specific_files=paths)[0]
+        changes = self._iter_git_changes(
+            specific_files=paths, include_trees=False)[0]
         for (change_type, old, new) in changes:
-            oldpath = old[0]
-            newpath = new[0]
+            if new[0] is None:
+                continue
+            newpath = decode_git_path(new[0])
             if newpath in paths:
-                ret[newpath] = oldpath
+                ret[newpath] = decode_git_path(old[0]) if old[0] else None
         for path in paths:
             if path not in ret:
                 if self.target.has_filename(path):
@@ -1071,7 +1081,7 @@
 
     def _iter_git_changes(self, want_unchanged=False, specific_files=None,
                           require_versioned=True, extra_trees=None,
-                          want_unversioned=False):
+                          want_unversioned=False, include_trees=True):
         trees = [self.source]
         if extra_trees is not None:
             trees.extend(extra_trees)
@@ -1090,7 +1100,7 @@
         rename_detector = RenameDetector(store)
         changes = tree_changes(
             store, self.source.tree, self.target.tree,
-            want_unchanged=want_unchanged, include_trees=True,
+            want_unchanged=want_unchanged, include_trees=include_trees,
             change_type_same=True, rename_detector=rename_detector)
         return changes, set(), set()
 
@@ -1109,7 +1119,7 @@
 
     def is_versioned(self, path):
         with self.lock_read():
-            path = path.rstrip('/').encode('utf-8')
+            path = encode_git_path(path.rstrip('/'))
             (index, subpath) = self._lookup_index(path)
             return (subpath in index or self._has_dir(path))
 
@@ -1251,8 +1261,7 @@
                 # old index
                 stat_val = os.stat_result(
                     (stat.S_IFLNK, 0, 0, 0, 0, 0, 0, 0, 0, 0))
-            blob.set_raw_string(
-                self.get_symlink_target(path).encode("utf-8"))
+            blob.set_raw_string(encode_git_path(self.get_symlink_target(path)))
             # Add object to the repository if it didn't exist yet
             if blob.id not in self.store:
                 self.store.add_object(blob)
@@ -1275,7 +1284,7 @@
             raise AssertionError("unknown kind '%s'" % kind)
         # Add an entry to the index or update the existing entry
         ensure_normalized_path(path)
-        encoded_path = path.encode("utf-8")
+        encoded_path = encode_git_path(path)
         if b'\r' in encoded_path or b'\n' in encoded_path:
             # TODO(jelmer): Why do we need to do this?
             trace.mutter('ignoring path with invalid newline in it: %r', path)
@@ -1320,7 +1329,7 @@
                     recurse_nested=recurse_nested):
                 if self.mapping.is_special_file(path):
                     continue
-                path = path.decode("utf-8")
+                path = decode_git_path(path)
                 if specific_files is not None and path not in specific_files:
                     continue
                 (parent, name) = posixpath.split(path)
@@ -1406,7 +1415,7 @@
     def _unversion_path(self, path):
         if self._lock_mode is None:
             raise errors.ObjectNotLocked(self)
-        encoded_path = path.encode("utf-8")
+        encoded_path = encode_git_path(path)
         count = 0
         (index, subpath) = self._lookup_index(encoded_path)
         try:
@@ -1439,7 +1448,7 @@
         for (old_path, new_path, file_id, ie) in delta:
             if old_path is not None:
                 (index, old_subpath) = self._lookup_index(
-                    old_path.encode('utf-8'))
+                    encode_git_path(old_path))
                 if old_subpath in index:
                     self._index_del_entry(index, old_subpath)
                     self._versioned_dirs = None
@@ -1465,11 +1474,11 @@
             return rename_tuples
 
     def rename_one(self, from_rel, to_rel, after=None):
-        from_path = from_rel.encode("utf-8")
+        from_path = encode_git_path(from_rel)
         to_rel, can_access = osutils.normalized_filename(to_rel)
         if not can_access:
             raise errors.InvalidNormalization(to_rel)
-        to_path = to_rel.encode("utf-8")
+        to_path = encode_git_path(to_rel)
         with self.lock_tree_write():
             if not after:
                 # Perhaps it's already moved?
@@ -1595,7 +1604,7 @@
             return (kind, None, None, None)
 
     def stored_kind(self, relpath):
-        (index, index_path) = self._lookup_index(relpath.encode('utf-8'))
+        (index, index_path) = self._lookup_index(encode_git_path(relpath))
         if index is None:
             return kind
         try:
@@ -1619,18 +1628,20 @@
     def _live_entry(self, relpath):
         raise NotImplementedError(self._live_entry)
 
-    def get_transform(self, pb=None):
-        from ..transform import TreeTransform
-        return TreeTransform(self, pb=pb)
-
-
-
-class InterIndexGitTree(InterGitTrees):
+    def transform(self, pb=None):
+        from .transform import GitTreeTransform
+        return GitTreeTransform(self, pb=pb)
+
+    def preview_transform(self, pb=None):
+        from .transform import GitTransformPreview
+        return GitTransformPreview(self, pb=pb)
+
+
+class InterToIndexGitTree(InterGitTrees):
     """InterTree that works between a Git revision tree and an index."""
 
     def __init__(self, source, target):
-        super(InterIndexGitTree, self).__init__(source, target)
-        self._index = target.index
+        super(InterToIndexGitTree, self).__init__(source, target)
         if self.source.store == self.target.store:
             self.store = self.source.store
         else:
@@ -1645,7 +1656,7 @@
 
     def _iter_git_changes(self, want_unchanged=False, specific_files=None,
                           require_versioned=False, extra_trees=None,
-                          want_unversioned=False):
+                          want_unversioned=False, include_trees=True):
         trees = [self.source]
         if extra_trees is not None:
             trees.extend(extra_trees)
@@ -1659,20 +1670,99 @@
                 self.source.store, self.source.tree,
                 self.target, want_unchanged=want_unchanged,
                 want_unversioned=want_unversioned,
-                rename_detector=self.rename_detector)
+                rename_detector=self.rename_detector,
+                include_trees=include_trees)
             return changes, set(), target_extras
 
 
+_mod_tree.InterTree.register_optimiser(InterToIndexGitTree)
+
+
+class InterFromIndexGitTree(InterGitTrees):
+    """InterTree that works between a Git revision tree and an index."""
+
+    def __init__(self, source, target):
+        super(InterFromIndexGitTree, self).__init__(source, target)
+        if self.source.store == self.target.store:
+            self.store = self.source.store
+        else:
+            self.store = OverlayObjectStore(
+                [self.source.store, self.target.store])
+        self.rename_detector = RenameDetector(self.store)
+
+    @classmethod
+    def is_compatible(cls, source, target):
+        return (isinstance(target, GitRevisionTree) and
+                isinstance(source, MutableGitIndexTree))
+
+    def _iter_git_changes(self, want_unchanged=False, specific_files=None,
+                          require_versioned=False, extra_trees=None,
+                          want_unversioned=False, include_trees=True):
+        trees = [self.source]
+        if extra_trees is not None:
+            trees.extend(extra_trees)
+        if specific_files is not None:
+            specific_files = self.target.find_related_paths_across_trees(
+                specific_files, trees,
+                require_versioned=require_versioned)
+        # TODO(jelmer): Restrict to specific_files, for performance reasons.
+        with self.lock_read():
+            from_tree_sha, extras = snapshot_workingtree(self.source, want_unversioned=want_unversioned)
+            return tree_changes(
+                self.store, from_tree_sha, self.target.tree,
+                include_trees=include_trees,
+                rename_detector=self.rename_detector,
+                want_unchanged=want_unchanged, change_type_same=True), extras
+
+
+_mod_tree.InterTree.register_optimiser(InterFromIndexGitTree)
+
+
+class InterIndexGitTree(InterGitTrees):
+    """InterTree that works between a Git revision tree and an index."""
+
+    def __init__(self, source, target):
+        super(InterIndexGitTree, self).__init__(source, target)
+        if self.source.store == self.target.store:
+            self.store = self.source.store
+        else:
+            self.store = OverlayObjectStore(
+                [self.source.store, self.target.store])
+        self.rename_detector = RenameDetector(self.store)
+
+    @classmethod
+    def is_compatible(cls, source, target):
+        return (isinstance(target, MutableGitIndexTree) and
+                isinstance(source, MutableGitIndexTree))
+
+    def _iter_git_changes(self, want_unchanged=False, specific_files=None,
+                          require_versioned=False, extra_trees=None,
+                          want_unversioned=False, include_trees=True):
+        trees = [self.source]
+        if extra_trees is not None:
+            trees.extend(extra_trees)
+        if specific_files is not None:
+            specific_files = self.target.find_related_paths_across_trees(
+                specific_files, trees,
+                require_versioned=require_versioned)
+        # TODO(jelmer): Restrict to specific_files, for performance reasons.
+        with self.lock_read():
+            from_tree_sha, from_extras = snapshot_workingtree(
+                self.source, want_unversioned=want_unversioned)
+            to_tree_sha, to_extras = snapshot_workingtree(
+                self.target, want_unversioned=want_unversioned)
+            changes = tree_changes(
+                self.store, from_tree_sha, to_tree_sha,
+                include_trees=include_trees,
+                rename_detector=self.rename_detector,
+                want_unchanged=want_unchanged, change_type_same=True)
+            return changes, from_extras, to_extras
+
+
 _mod_tree.InterTree.register_optimiser(InterIndexGitTree)
 
 
-def changes_between_git_tree_and_working_copy(source_store, from_tree_sha, target,
-                                              want_unchanged=False,
-                                              want_unversioned=False,
-                                              rename_detector=None):
-    """Determine the changes between a git tree and a working tree with index.
-
-    """
+def snapshot_workingtree(target, want_unversioned=False):
     extras = set()
     blobs = {}
     # Report dirified directories to commit_tree first, so that they can be
@@ -1705,7 +1795,7 @@
                     else:
                         mode &= ~0o111
                 if live_entry.sha != index_entry.sha:
-                    rp = path.decode('utf-8')
+                    rp = decode_git_path(path)
                     if stat.S_ISREG(live_entry.mode):
                         blob = Blob()
                         with target.get_file(rp) as f:
@@ -1725,7 +1815,7 @@
             except UnicodeDecodeError:
                 raise errors.BadFilenameEncoding(
                     e, osutils._fs_enc)
-            np = e.encode('utf-8')
+            np = encode_git_path(e)
             if np in blobs:
                 continue
             st = target._lstat(e)
@@ -1739,10 +1829,21 @@
             target.store.add_object(blob)
             blobs[np] = (blob.id, cleanup_mode(st.st_mode))
             extras.add(np)
-    to_tree_sha = commit_tree(
-        target.store, dirified + [(p, s, m) for (p, (s, m)) in blobs.items()])
+    return commit_tree(
+        target.store, dirified + [(p, s, m) for (p, (s, m)) in blobs.items()]), extras
+
+
+def changes_between_git_tree_and_working_copy(source_store, from_tree_sha, target,
+                                              want_unchanged=False,
+                                              want_unversioned=False,
+                                              rename_detector=None,
+                                              include_trees=True):
+    """Determine the changes between a git tree and a working tree with index.
+
+    """
+    to_tree_sha, extras = snapshot_workingtree(target, want_unversioned=want_unversioned)
     store = OverlayObjectStore([source_store, target.store])
     return tree_changes(
-        store, from_tree_sha, to_tree_sha, include_trees=True,
+        store, from_tree_sha, to_tree_sha, include_trees=include_trees,
         rename_detector=rename_detector,
         want_unchanged=want_unchanged, change_type_same=True), extras

=== modified file 'breezy/git/urls.py'
--- breezy/git/urls.py	2020-02-18 01:57:45 +0000
+++ breezy/git/urls.py	2020-07-18 23:14:00 +0000
@@ -25,7 +25,10 @@
 from dulwich.client import parse_rsync_url
 
 
-KNOWN_GIT_SCHEMES = ['git+ssh', 'git', 'http', 'https', 'ftp']
+KNOWN_GIT_SCHEMES = ['git+ssh', 'git', 'http', 'https', 'ftp', 'ssh']
+SCHEME_REPLACEMENT = {
+    'ssh': 'git+ssh',
+    }
 
 
 def git_url_to_bzr_url(location, branch=None, ref=None):
@@ -47,6 +50,9 @@
                 port=None,
                 quoted_path=urlutils.quote(path, safe="/~"))
         location = str(url)
+    elif url.scheme in SCHEME_REPLACEMENT:
+        url.scheme = SCHEME_REPLACEMENT[url.scheme]
+        location = str(url)
     if ref == b'HEAD':
         ref = branch = None
     if ref:

=== modified file 'breezy/git/workingtree.py'
--- breezy/git/workingtree.py	2020-06-23 01:02:30 +0000
+++ breezy/git/workingtree.py	2020-07-18 23:14:00 +0000
@@ -54,7 +54,6 @@
     controldir as _mod_controldir,
     globbing,
     lock,
-    merge,
     osutils,
     revision as _mod_revision,
     trace,
@@ -79,6 +78,8 @@
     MutableGitIndexTree,
     )
 from .mapping import (
+    encode_git_path,
+    decode_git_path,
     mode_kind,
     )
 
@@ -120,10 +121,10 @@
         try:
             info = self._submodule_info()[relpath]
         except KeyError:
-            index_path = os.path.join(self.basedir, relpath.decode('utf-8'), '.git', 'index')
+            index_path = os.path.join(self.basedir, decode_git_path(relpath), '.git', 'index')
         else:
             index_path = self.control_transport.local_abspath(
-                posixpath.join('modules', info[1].decode('utf-8'), 'index'))
+                posixpath.join('modules', decode_git_path(info[1]), 'index'))
         return Index(index_path)
 
     def lock_read(self):
@@ -220,10 +221,6 @@
         else:
             self.case_sensitive = False
 
-    def get_transform(self, pb=None):
-        from ..transform import TreeTransform
-        return TreeTransform(self, pb=pb)
-
     def merge_modified(self):
         return {}
 
@@ -461,7 +458,7 @@
                 kind = osutils.file_kind(abspath)
                 if kind in ("file", "symlink"):
                     (index, subpath) = self._lookup_index(
-                        filepath.encode('utf-8'))
+                        encode_git_path(filepath))
                     if subpath in index:
                         # Already present
                         continue
@@ -471,7 +468,7 @@
                     added.append(filepath)
                 elif kind == "directory":
                     (index, subpath) = self._lookup_index(
-                        filepath.encode('utf-8'))
+                        encode_git_path(filepath))
                     if subpath not in index:
                         call_action(filepath, kind)
                     if recurse:
@@ -511,7 +508,7 @@
                         user_dirs.append(subp)
                     else:
                         (index, subpath) = self._lookup_index(
-                            subp.encode('utf-8'))
+                            encode_git_path(subp))
                         if subpath in index:
                             # Already present
                             continue
@@ -572,7 +569,7 @@
         """
         with self.lock_read():
             index_paths = set(
-                [p.decode('utf-8') for p, i in self._recurse_index_entries()])
+                [decode_git_path(p) for p, i in self._recurse_index_entries()])
             all_paths = set(self._iter_files_recursive(include_dirs=False))
             return iter(all_paths - index_paths)
 
@@ -677,7 +674,7 @@
 
     def get_file_verifier(self, path, stat_value=None):
         with self.lock_read():
-            (index, subpath) = self._lookup_index(path.encode('utf-8'))
+            (index, subpath) = self._lookup_index(encode_git_path(path))
             try:
                 return ("GIT", index[subpath].sha)
             except KeyError:
@@ -709,7 +706,7 @@
 
     def stored_kind(self, path):
         with self.lock_read():
-            encoded_path = path.encode('utf-8')
+            encoded_path = encode_git_path(path)
             (index, subpath) = self._lookup_index(encoded_path)
             try:
                 return mode_kind(index[subpath].mode)
@@ -723,7 +720,7 @@
         return os.lstat(self.abspath(path))
 
     def _live_entry(self, path):
-        encoded_path = self.abspath(path.decode('utf-8')).encode(
+        encoded_path = self.abspath(decode_git_path(path)).encode(
             osutils._fs_enc)
         return index_entry_from_path(encoded_path)
 
@@ -732,7 +729,7 @@
             if self._supports_executable():
                 mode = self._lstat(path).st_mode
             else:
-                (index, subpath) = self._lookup_index(path.encode('utf-8'))
+                (index, subpath) = self._lookup_index(encode_git_path(path))
                 try:
                     mode = index[subpath].mode
                 except KeyError:
@@ -777,7 +774,7 @@
                          name.decode(osutils._fs_enc))])
             for path in path_iterator:
                 try:
-                    encoded_path = path.encode("utf-8")
+                    encoded_path = encode_git_path(path)
                 except UnicodeEncodeError:
                     raise errors.BadFilenameEncoding(
                         path, osutils._fs_enc)
@@ -831,7 +828,7 @@
             for path in self.index:
                 if self.mapping.is_special_file(path):
                     continue
-                path = path.decode("utf-8")
+                path = decode_git_path(path)
                 paths.add(path)
                 while path != "":
                     path = posixpath.dirname(path).strip("/")
@@ -841,12 +838,12 @@
             return paths
 
     def iter_child_entries(self, path):
-        encoded_path = path.encode('utf-8')
+        encoded_path = encode_git_path(path)
         with self.lock_read():
             parent_id = self.path2id(path)
             found_any = False
             for item_path, value in self.index.iteritems():
-                decoded_item_path = item_path.decode('utf-8')
+                decoded_item_path = decode_git_path(item_path)
                 if self.mapping.is_special_file(item_path):
                     continue
                 if not osutils.is_inside(path, decoded_item_path):
@@ -871,14 +868,14 @@
             for item_path, value in self.index.iteritems():
                 if value.flags & FLAG_STAGEMASK:
                     conflicts.append(_mod_conflicts.TextConflict(
-                        item_path.decode('utf-8')))
+                        decode_git_path(item_path)))
             return conflicts
 
     def set_conflicts(self, conflicts):
         by_path = set()
         for conflict in conflicts:
             if conflict.typestring in ('text conflict', 'contents conflict'):
-                by_path.add(conflict.path.encode('utf-8'))
+                by_path.add(encode_git_path(conflict.path))
             else:
                 raise errors.UnsupportedOperation(self.set_conflicts, self)
         with self.lock_tree_write():
@@ -901,7 +898,7 @@
                                            'contents conflict'):
                     try:
                         self._set_conflicted(
-                            conflict.path.encode('utf-8'), True)
+                            encode_git_path(conflict.path), True)
                     except KeyError:
                         raise errors.UnsupportedOperation(
                             self.add_conflicts, self)
@@ -1037,7 +1034,7 @@
     def _walkdirs(self, prefix=u""):
         if prefix != u"":
             prefix += u"/"
-        prefix = prefix.encode('utf-8')
+        prefix = encode_git_path(prefix)
         per_dir = defaultdict(set)
         if prefix == b"":
             per_dir[(u'', self.path2id(''))] = set()
@@ -1047,14 +1044,14 @@
                 return
             (dirname, child_name) = posixpath.split(path)
             add_entry(dirname, 'directory')
-            dirname = dirname.decode("utf-8")
+            dirname = decode_git_path(dirname)
             dir_file_id = self.path2id(dirname)
             if not isinstance(value, tuple) or len(value) != 10:
                 raise ValueError(value)
             per_dir[(dirname, dir_file_id)].add(
-                (path.decode("utf-8"), child_name.decode("utf-8"),
+                (decode_git_path(path), decode_git_path(child_name),
                  kind, None,
-                 self.path2id(path.decode("utf-8")),
+                 self.path2id(decode_git_path(path)),
                  kind))
         with self.lock_read():
             for path, value in self.index.iteritems():
@@ -1071,11 +1068,11 @@
     def store_uncommitted(self):
         raise errors.StoringUncommittedNotSupported(self)
 
-    def apply_inventory_delta(self, changes):
-        for (old_path, new_path, file_id, ie) in changes:
+    def _apply_transform_delta(self, changes):
+        for (old_path, new_path, ie) in changes:
             if old_path is not None:
                 (index, old_subpath) = self._lookup_index(
-                    old_path.encode('utf-8'))
+                    encode_git_path(old_path))
                 try:
                     self._index_del_entry(index, old_subpath)
                 except KeyError:
@@ -1187,7 +1184,7 @@
                         # Let's at least try to use the working tree file:
                         try:
                             st = self._lstat(self.abspath(
-                                entry.path.decode('utf-8')))
+                                decode_git_path(entry.path)))
                         except OSError:
                             # But if it doesn't exist, we'll make something up.
                             obj = self.store[entry.sha]
@@ -1198,10 +1195,12 @@
                     (index, subpath) = self._lookup_index(entry.path)
                     index[subpath] = index_entry_from_stat(st, entry.sha, 0)
 
-    def _update_git_tree(self, old_revision, new_revision, change_reporter=None,
-                         show_base=False):
+    def _update_git_tree(
+            self, old_revision, new_revision, change_reporter=None,
+            show_base=False):
         basis_tree = self.revision_tree(old_revision)
         if new_revision != old_revision:
+            from .. import merge
             with basis_tree.lock_read():
                 new_basis_tree = self.branch.basis_tree()
                 merge.merge_inner(
@@ -1310,6 +1309,7 @@
 
     def copy_content_into(self, tree, revision_id=None):
         """Copy the current content and user files of this tree into tree."""
+        from .. import merge
         with self.lock_read():
             if revision_id is None:
                 merge.transform_tree(tree, self)
@@ -1338,10 +1338,10 @@
 
     def get_reference_info(self, path):
         submodule_info = self._submodule_info()
-        info = submodule_info.get(path.encode('utf-8'))
+        info = submodule_info.get(encode_git_path(path))
         if info is None:
             return None
-        return info[0].decode('utf-8')
+        return decode_git_path(info[0])
 
     def set_reference_info(self, tree_path, branch_location):
         path = self.abspath('.gitmodules')
@@ -1352,7 +1352,7 @@
                 config = GitConfigFile()
             else:
                 raise
-        section = (b'submodule', tree_path.encode('utf-8'))
+        section = (b'submodule', encode_git_path(tree_path))
         if branch_location is None:
             try:
                 del config[section]
@@ -1364,7 +1364,7 @@
                 branch_location)
             config.set(
                 section,
-                b'path', tree_path.encode('utf-8'))
+                b'path', encode_git_path(tree_path))
             config.set(
                 section,
                 b'url', branch_location.encode('utf-8'))

=== modified file 'breezy/location.py'
--- breezy/location.py	2020-02-18 01:57:45 +0000
+++ breezy/location.py	2020-07-18 23:14:00 +0000
@@ -39,12 +39,11 @@
 
 hooks = LocationHooks()
 
-
-def rcp_location_to_url(location, scheme='ssh'):
+def parse_rcp_location(location):
     """Convert a rcp-style location to a URL.
 
     :param location: Location to convert, e.g. "foo:bar"
-    :param schenme: URL scheme to return, defaults to "ssh"
+    :param scheme: URL scheme to return, defaults to "ssh"
     :return: A URL, e.g. "ssh://foo/bar"
     :raises ValueError: if this is not a RCP-style URL
     """
@@ -53,36 +52,57 @@
         raise ValueError("Not a RCP URL")
     if m.group('path').startswith('//'):
         raise ValueError("Not a RCP URL: already looks like a URL")
-    quoted_user = urlutils.quote(m.group('user')[:-1]) if m.group('user') else None
+    return (m.group('host'),
+            m.group('user')[:-1] if m.group('user') else None,
+            m.group('path'))
+
+
+def rcp_location_to_url(location, scheme='ssh'):
+    """Convert a rcp-style location to a URL.
+
+    :param location: Location to convert, e.g. "foo:bar"
+    :param scheme: URL scheme to return, defaults to "ssh"
+    :return: A URL, e.g. "ssh://foo/bar"
+    :raises ValueError: if this is not a RCP-style URL
+    """
+    (host, user, path) = parse_rcp_location(location)
+    quoted_user = urlutils.quote(user) if user else None
     url = urlutils.URL(
         scheme=scheme, quoted_user=quoted_user,
         port=None, quoted_password=None,
-        quoted_host=urlutils.quote(m.group('host')),
-        quoted_path=urlutils.quote(m.group('path')))
+        quoted_host=urlutils.quote(host),
+        quoted_path=urlutils.quote(path))
     return str(url)
 
 
-def pserver_to_url(location):
-    """Convert a CVS pserver location string to a URL.
-
-    :param location: pserver URL
-    :return: A cvs+pserver URL
-    """
+def parse_cvs_location(location):
     parts = location.split(':')
-    if parts[0] or parts[1] != 'pserver':
+    if parts[0] or parts[1] not in ('pserver', 'ssh'):
         raise ValueError('not a valid pserver location string')
     try:
         (username, hostname) = parts[2].split('@', 1)
     except IndexError:
         hostname = parts[2]
         username = None
+    scheme = parts[1]
+    path = parts[3]
+    return (scheme, hostname, username, path)
+
+
+def cvs_to_url(location):
+    """Convert a CVS pserver location string to a URL.
+
+    :param location: pserver URL
+    :return: A cvs+pserver URL
+    """
+    (scheme, host, user, path) = parse_cvs_location(location)
     return str(urlutils.URL(
-        scheme='cvs+pserver',
-        quoted_user=urlutils.quote(username) if username else None,
-        quoted_host=urlutils.quote(hostname),
+        scheme='cvs+' + scheme,
+        quoted_user=urlutils.quote(user) if user else None,
+        quoted_host=urlutils.quote(host),
         quoted_password=None,
         port=None,
-        quoted_path=urlutils.quote(parts[3])))
+        quoted_path=urlutils.quote(path)))
 
 
 def location_to_url(location, purpose=None):
@@ -100,7 +120,7 @@
         raise AssertionError("location not a byte or unicode string")
 
     if location.startswith(':pserver:'):
-        return pserver_to_url(location)
+        return cvs_to_url(location)
 
     from .directory_service import directories
     location = directories.dereference(location, purpose)

=== modified file 'breezy/log.py'
--- breezy/log.py	2020-06-23 01:02:30 +0000
+++ breezy/log.py	2020-07-18 23:14:00 +0000
@@ -24,7 +24,7 @@
 * in "verbose" mode with a description of what changed from one
   version to the next
 
-* with file-ids and revision-ids shown
+* with files and revision-ids shown
 
 Logs are actually written out through an abstract LogFormatter
 interface, which allows for different preferred formats.  Plugins can
@@ -81,6 +81,7 @@
     format_date_with_offset_in_original_timezone,
     get_diff_header_encoding,
     get_terminal_encoding,
+    is_inside,
     terminal_width,
     )
 from .tree import (
@@ -90,7 +91,7 @@
 
 
 def find_touching_revisions(repository, last_revision, last_tree, last_path):
-    """Yield a description of revisions which affect the file_id.
+    """Yield a description of revisions which affect the file.
 
     Each returned element is (revno, revision_id, description)
 
@@ -138,7 +139,6 @@
              direction='reverse',
              start_revision=None,
              end_revision=None,
-             search=None,
              limit=None,
              show_diff=False,
              match=None):
@@ -160,9 +160,6 @@
 
     :param end_revision: If not None, only show revisions <= end_revision
 
-    :param search: If not None, only show revisions with matching commit
-        messages
-
     :param limit: If set, shows only 'limit' revisions, all revisions are shown
         if None or 0.
 
@@ -199,8 +196,7 @@
     rqst = make_log_request_dict(
         direction=direction,
         start_revision=start_revision, end_revision=end_revision,
-        limit=limit, message_search=search,
-        delta_type=delta_type, diff_type=diff_type)
+        limit=limit, delta_type=delta_type, diff_type=diff_type)
     Logger(branch, rqst).show(lf)
 
 
@@ -215,7 +211,7 @@
     }
 
 
-def make_log_request_dict(direction='reverse', specific_fileids=None,
+def make_log_request_dict(direction='reverse', specific_files=None,
                           start_revision=None, end_revision=None, limit=None,
                           message_search=None, levels=None, generate_tags=True,
                           delta_type=None,
@@ -232,7 +228,7 @@
     :param direction: 'reverse' (default) is latest to earliest;
       'forward' is earliest to latest.
 
-    :param specific_fileids: If not None, only include revisions
+    :param specific_files: If not None, only include revisions
       affecting the specified files, rather than all revisions.
 
     :param start_revision: If not None, only generate
@@ -255,16 +251,16 @@
 `
     :param delta_type: Either 'full', 'partial' or None.
       'full' means generate the complete delta - adds/deletes/modifies/etc;
-      'partial' means filter the delta using specific_fileids;
+      'partial' means filter the delta using specific_files;
       None means do not generate any delta.
 
     :param diff_type: Either 'full', 'partial' or None.
       'full' means generate the complete diff - adds/deletes/modifies/etc;
-      'partial' means filter the diff using specific_fileids;
+      'partial' means filter the diff using specific_files;
       None means do not generate any diff.
 
     :param _match_using_deltas: a private parameter controlling the
-      algorithm used for matching specific_fileids. This parameter
+      algorithm used for matching specific_files. This parameter
       may be removed in the future so breezy client code should NOT
       use it.
 
@@ -292,7 +288,7 @@
             match = {'message': [message_search]}
     return {
         'direction': direction,
-        'specific_fileids': specific_fileids,
+        'specific_files': specific_files,
         'start_revision': start_revision,
         'end_revision': end_revision,
         'limit': limit,
@@ -413,7 +409,73 @@
 
         Subclasses may wish to override this.
         """
-        return _DefaultLogGenerator(branch, rqst)
+        return _DefaultLogGenerator(branch, **rqst)
+
+
+def _log_revision_iterator_using_per_file_graph(
+        branch, delta_type, match, levels, path, start_rev_id, end_rev_id,
+        direction, exclude_common_ancestry):
+    # Get the base revisions, filtering by the revision range.
+    # Note that we always generate the merge revisions because
+    # filter_revisions_touching_path() requires them ...
+    view_revisions = _calc_view_revisions(
+        branch, start_rev_id, end_rev_id,
+        direction, generate_merge_revisions=True,
+        exclude_common_ancestry=exclude_common_ancestry)
+    if not isinstance(view_revisions, list):
+        view_revisions = list(view_revisions)
+    view_revisions = _filter_revisions_touching_path(
+        branch, path, view_revisions,
+        include_merges=levels != 1)
+    return make_log_rev_iterator(
+        branch, view_revisions, delta_type, match)
+
+
+def _log_revision_iterator_using_delta_matching(
+        branch, delta_type, match, levels, specific_files, start_rev_id, end_rev_id,
+        direction, exclude_common_ancestry, limit):
+    # Get the base revisions, filtering by the revision range
+    generate_merge_revisions = levels != 1
+    delayed_graph_generation = not specific_files and (
+        limit or start_rev_id or end_rev_id)
+    view_revisions = _calc_view_revisions(
+        branch, start_rev_id, end_rev_id,
+        direction,
+        generate_merge_revisions=generate_merge_revisions,
+        delayed_graph_generation=delayed_graph_generation,
+        exclude_common_ancestry=exclude_common_ancestry)
+
+    # Apply the other filters
+    return make_log_rev_iterator(branch, view_revisions,
+                                 delta_type, match,
+                                 files=specific_files,
+                                 direction=direction)
+
+
+def _format_diff(branch, rev, diff_type, files=None):
+    """Format a diff.
+
+    :param branch: Branch object
+    :param rev: Revision object
+    :param diff_type: Type of diff to generate
+    :param files: List of files to generate diff for (or None for all)
+    """
+    repo = branch.repository
+    if len(rev.parent_ids) == 0:
+        ancestor_id = _mod_revision.NULL_REVISION
+    else:
+        ancestor_id = rev.parent_ids[0]
+    tree_1 = repo.revision_tree(ancestor_id)
+    tree_2 = repo.revision_tree(rev.revision_id)
+    if diff_type == 'partial' and files is not None:
+        specific_files = files
+    else:
+        specific_files = None
+    s = BytesIO()
+    path_encoding = get_diff_header_encoding()
+    diff.show_diff_trees(tree_1, tree_2, s, specific_files, old_label='',
+                         new_label='', path_encoding=path_encoding)
+    return s.getvalue()
 
 
 class _StartNotLinearAncestor(Exception):
@@ -423,10 +485,28 @@
 class _DefaultLogGenerator(LogGenerator):
     """The default generator of log revisions."""
 
-    def __init__(self, branch, rqst):
+    def __init__(
+            self, branch, levels=None, limit=None, diff_type=None,
+            delta_type=None, show_signature=None, omit_merges=None,
+            generate_tags=None, specific_files=None, match=None,
+            start_revision=None, end_revision=None, direction=None,
+            exclude_common_ancestry=None, _match_using_deltas=None,
+            signature=None):
         self.branch = branch
-        self.rqst = rqst
-        if rqst.get('generate_tags') and branch.supports_tags():
+        self.levels = levels
+        self.limit = limit
+        self.diff_type = diff_type
+        self.delta_type = delta_type
+        self.show_signature = signature
+        self.omit_merges = omit_merges
+        self.specific_files = specific_files
+        self.match = match
+        self.start_revision = start_revision
+        self.end_revision = end_revision
+        self.direction = direction
+        self.exclude_common_ancestry = exclude_common_ancestry
+        self._match_using_deltas = _match_using_deltas
+        if generate_tags and branch.supports_tags():
             self.rev_tag_dict = branch.tags.get_reverse_tag_dict()
         else:
             self.rev_tag_dict = {}
@@ -436,117 +516,74 @@
 
         :return: An iterator yielding LogRevision objects.
         """
-        rqst = self.rqst
-        levels = rqst.get('levels')
-        limit = rqst.get('limit')
-        diff_type = rqst.get('diff_type')
-        show_signature = rqst.get('signature')
-        omit_merges = rqst.get('omit_merges')
         log_count = 0
         revision_iterator = self._create_log_revision_iterator()
         for revs in revision_iterator:
             for (rev_id, revno, merge_depth), rev, delta in revs:
                 # 0 levels means show everything; merge_depth counts from 0
-                if (levels != 0 and merge_depth is not None and
-                        merge_depth >= levels):
+                if (self.levels != 0 and merge_depth is not None and
+                        merge_depth >= self.levels):
                     continue
-                if omit_merges and len(rev.parent_ids) > 1:
+                if self.omit_merges and len(rev.parent_ids) > 1:
                     continue
                 if rev is None:
                     raise errors.GhostRevisionUnusableHere(rev_id)
-                if diff_type is None:
+                if self.diff_type is None:
                     diff = None
                 else:
-                    diff = self._format_diff(rev, rev_id, diff_type)
-                if show_signature:
+                    diff = _format_diff(
+                        self.branch, rev, self.diff_type,
+                        self.specific_files)
+                if self.show_signature:
                     signature = format_signature_validity(rev_id, self.branch)
                 else:
                     signature = None
                 yield LogRevision(
                     rev, revno, merge_depth, delta,
                     self.rev_tag_dict.get(rev_id), diff, signature)
-                if limit:
+                if self.limit:
                     log_count += 1
-                    if log_count >= limit:
+                    if log_count >= self.limit:
                         return
 
-    def _format_diff(self, rev, rev_id, diff_type):
-        repo = self.branch.repository
-        if len(rev.parent_ids) == 0:
-            ancestor_id = _mod_revision.NULL_REVISION
-        else:
-            ancestor_id = rev.parent_ids[0]
-        tree_1 = repo.revision_tree(ancestor_id)
-        tree_2 = repo.revision_tree(rev_id)
-        file_ids = self.rqst.get('specific_fileids')
-        if diff_type == 'partial' and file_ids is not None:
-            specific_files = [tree_2.id2path(id) for id in file_ids]
-        else:
-            specific_files = None
-        s = BytesIO()
-        path_encoding = get_diff_header_encoding()
-        diff.show_diff_trees(tree_1, tree_2, s, specific_files, old_label='',
-                             new_label='', path_encoding=path_encoding)
-        return s.getvalue()
-
     def _create_log_revision_iterator(self):
         """Create a revision iterator for log.
 
         :return: An iterator over lists of ((rev_id, revno, merge_depth), rev,
             delta).
         """
-        self.start_rev_id, self.end_rev_id = _get_revision_limits(
-            self.branch, self.rqst.get('start_revision'),
-            self.rqst.get('end_revision'))
-        if self.rqst.get('_match_using_deltas'):
-            return self._log_revision_iterator_using_delta_matching()
+        start_rev_id, end_rev_id = _get_revision_limits(
+            self.branch, self.start_revision, self.end_revision)
+        if self._match_using_deltas:
+            return _log_revision_iterator_using_delta_matching(
+                self.branch,
+                delta_type=self.delta_type,
+                match=self.match,
+                levels=self.levels,
+                specific_files=self.specific_files,
+                start_rev_id=start_rev_id, end_rev_id=end_rev_id,
+                direction=self.direction,
+                exclude_common_ancestry=self.exclude_common_ancestry,
+                limit=self.limit)
         else:
             # We're using the per-file-graph algorithm. This scales really
             # well but only makes sense if there is a single file and it's
             # not a directory
-            file_count = len(self.rqst.get('specific_fileids'))
+            file_count = len(self.specific_files)
             if file_count != 1:
                 raise errors.BzrError(
                     "illegal LogRequest: must match-using-deltas "
                     "when logging %d files" % file_count)
-            return self._log_revision_iterator_using_per_file_graph()
-
-    def _log_revision_iterator_using_delta_matching(self):
-        # Get the base revisions, filtering by the revision range
-        rqst = self.rqst
-        generate_merge_revisions = rqst.get('levels') != 1
-        delayed_graph_generation = not rqst.get('specific_fileids') and (
-            rqst.get('limit') or self.start_rev_id or self.end_rev_id)
-        view_revisions = _calc_view_revisions(
-            self.branch, self.start_rev_id, self.end_rev_id,
-            rqst.get('direction'),
-            generate_merge_revisions=generate_merge_revisions,
-            delayed_graph_generation=delayed_graph_generation,
-            exclude_common_ancestry=rqst.get('exclude_common_ancestry'))
-
-        # Apply the other filters
-        return make_log_rev_iterator(self.branch, view_revisions,
-                                     rqst.get('delta_type'), rqst.get('match'),
-                                     file_ids=rqst.get('specific_fileids'),
-                                     direction=rqst.get('direction'))
-
-    def _log_revision_iterator_using_per_file_graph(self):
-        # Get the base revisions, filtering by the revision range.
-        # Note that we always generate the merge revisions because
-        # filter_revisions_touching_file_id() requires them ...
-        rqst = self.rqst
-        view_revisions = _calc_view_revisions(
-            self.branch, self.start_rev_id, self.end_rev_id,
-            rqst.get('direction'), generate_merge_revisions=True,
-            exclude_common_ancestry=rqst.get('exclude_common_ancestry'))
-        if not isinstance(view_revisions, list):
-            view_revisions = list(view_revisions)
-        view_revisions = _filter_revisions_touching_file_id(self.branch,
-                                                            rqst.get('specific_fileids')[
-                                                                0], view_revisions,
-                                                            include_merges=rqst.get('levels') != 1)
-        return make_log_rev_iterator(self.branch, view_revisions,
-                                     rqst.get('delta_type'), rqst.get('match'))
+            return _log_revision_iterator_using_per_file_graph(
+                self.branch,
+                delta_type=self.delta_type,
+                match=self.match,
+                levels=self.levels,
+                path=self.specific_files[0],
+                start_rev_id=start_rev_id, end_rev_id=end_rev_id,
+                direction=self.direction,
+                exclude_common_ancestry=self.exclude_common_ancestry
+                )
 
 
 def _calc_view_revisions(branch, start_rev_id, end_rev_id, direction,
@@ -844,7 +881,7 @@
 
 
 def make_log_rev_iterator(branch, view_revisions, generate_delta, search,
-                          file_ids=None, direction='reverse'):
+                          files=None, direction='reverse'):
     """Create a revision iterator for log.
 
     :param branch: The branch being logged.
@@ -852,8 +889,8 @@
     :param generate_delta: Whether to generate a delta for each revision.
       Permitted values are None, 'full' and 'partial'.
     :param search: A user text search string.
-    :param file_ids: If non empty, only revisions matching one or more of
-      the file-ids are to be kept.
+    :param files: If non empty, only revisions matching one or more of
+      the files are to be kept.
     :param direction: the direction in which view_revisions is sorted
     :return: An iterator over lists of ((rev_id, revno, merge_depth), rev,
         delta).
@@ -875,7 +912,7 @@
         # with custom parameters. This will do for now. IGC 20090127
         if adapter == _make_delta_filter:
             log_rev_iterator = adapter(
-                branch, generate_delta, search, log_rev_iterator, file_ids,
+                branch, generate_delta, search, log_rev_iterator, files,
                 direction)
         else:
             log_rev_iterator = adapter(
@@ -932,7 +969,7 @@
 
 
 def _make_delta_filter(branch, generate_delta, search, log_rev_iterator,
-                       fileids=None, direction='reverse'):
+                       files=None, direction='reverse'):
     """Add revision deltas to a log iterator if needed.
 
     :param branch: The branch being logged.
@@ -941,55 +978,56 @@
     :param search: A user text search string.
     :param log_rev_iterator: An input iterator containing all revisions that
         could be displayed, in lists.
-    :param fileids: If non empty, only revisions matching one or more of
-      the file-ids are to be kept.
+    :param files: If non empty, only revisions matching one or more of
+      the files are to be kept.
     :param direction: the direction in which view_revisions is sorted
     :return: An iterator over lists of ((rev_id, revno, merge_depth), rev,
         delta).
     """
-    if not generate_delta and not fileids:
+    if not generate_delta and not files:
         return log_rev_iterator
     return _generate_deltas(branch.repository, log_rev_iterator,
-                            generate_delta, fileids, direction)
-
-
-def _generate_deltas(repository, log_rev_iterator, delta_type, fileids,
+                            generate_delta, files, direction)
+
+
+def _generate_deltas(repository, log_rev_iterator, delta_type, files,
                      direction):
     """Create deltas for each batch of revisions in log_rev_iterator.
 
     If we're only generating deltas for the sake of filtering against
-    file-ids, we stop generating deltas once all file-ids reach the
+    files, we stop generating deltas once all files reach the
     appropriate life-cycle point. If we're receiving data newest to
     oldest, then that life-cycle point is 'add', otherwise it's 'remove'.
     """
-    check_fileids = fileids is not None and len(fileids) > 0
-    if check_fileids:
-        fileid_set = set(fileids)
+    check_files = files is not None and len(files) > 0
+    if check_files:
+        file_set = set(files)
         if direction == 'reverse':
             stop_on = 'add'
         else:
             stop_on = 'remove'
     else:
-        fileid_set = None
+        file_set = None
     for revs in log_rev_iterator:
-        # If we were matching against fileids and we've run out,
+        # If we were matching against files and we've run out,
         # there's nothing left to do
-        if check_fileids and not fileid_set:
+        if check_files and not file_set:
             return
         revisions = [rev[1] for rev in revs]
         new_revs = []
-        if delta_type == 'full' and not check_fileids:
-            deltas = repository.get_deltas_for_revisions(revisions)
+        if delta_type == 'full' and not check_files:
+            deltas = repository.get_revision_deltas(revisions)
             for rev, delta in zip(revs, deltas):
                 new_revs.append((rev[0], rev[1], delta))
         else:
-            deltas = repository.get_deltas_for_revisions(revisions, fileid_set)
+            deltas = repository.get_revision_deltas(
+                revisions, specific_files=file_set)
             for rev, delta in zip(revs, deltas):
-                if check_fileids:
+                if check_files:
                     if delta is None or not delta.has_changed():
                         continue
                     else:
-                        _update_fileids(delta, fileid_set, stop_on)
+                        _update_files(delta, file_set, stop_on)
                         if delta_type is None:
                             delta = None
                         elif delta_type == 'full':
@@ -1006,21 +1044,39 @@
         yield new_revs
 
 
-def _update_fileids(delta, fileids, stop_on):
-    """Update the set of file-ids to search based on file lifecycle events.
+def _update_files(delta, files, stop_on):
+    """Update the set of files to search based on file lifecycle events.
 
-    :param fileids: a set of fileids to update
-    :param stop_on: either 'add' or 'remove' - take file-ids out of the
-      fileids set once their add or remove entry is detected respectively
+    :param files: a set of files to update
+    :param stop_on: either 'add' or 'remove' - take files out of the
+      files set once their add or remove entry is detected respectively
     """
     if stop_on == 'add':
-        for item in delta.added + delta.copied:
-            if item.file_id in fileids:
-                fileids.remove(item.file_id)
+        for item in delta.added:
+            if item.path[1] in files:
+                files.remove(item.path[1])
+        for item in delta.copied + delta.renamed:
+            if item.path[1] in files:
+                files.remove(item.path[1])
+                files.add(item.path[0])
+            if item.kind[1] == 'directory':
+                for path in list(files):
+                    if is_inside(item.path[1], path):
+                        files.remove(path)
+                        files.add(item.path[0] + path[len(item.path[1]):])
     elif stop_on == 'delete':
         for item in delta.removed:
-            if item.file_id in fileids:
-                fileids.remove(item.file_id)
+            if item.path[0] in files:
+                files.remove(item.path[0])
+        for item in delta.copied + delta.renamed:
+            if item.path[0] in files:
+                files.remove(item.path[0])
+                files.add(item.path[1])
+            if item.kind[0] == 'directory':
+                for path in list(files):
+                    if is_inside(item.path[0], path):
+                        files.remove(path)
+                        files.add(item.path[1] + path[len(item.path[0]):])
 
 
 def _make_revision_objects(branch, generate_delta, search, log_rev_iterator):
@@ -1067,14 +1123,11 @@
 def _get_revision_limits(branch, start_revision, end_revision):
     """Get and check revision limits.
 
-    :param  branch: The branch containing the revisions.
-
-    :param  start_revision: The first revision to be logged.
-            but for merge revision support a RevisionInfo is expected.
-
-    :param  end_revision: The last revision to be logged.
-            For backwards compatibility this may be a mainline integer revno,
-            but for merge revision support a RevisionInfo is expected.
+    :param branch: The branch containing the revisions.
+
+    :param start_revision: The first revision to be logged, as a RevisionInfo.
+
+    :param end_revision: The last revision to be logged, as a RevisionInfo
 
     :return: (start_rev_id, end_rev_id) tuple.
     """
@@ -1190,12 +1243,12 @@
     return mainline_revs, rev_nos, start_rev_id, end_rev_id
 
 
-def _filter_revisions_touching_file_id(branch, file_id, view_revisions,
-                                       include_merges=True):
-    r"""Return the list of revision ids which touch a given file id.
+def _filter_revisions_touching_path(branch, path, view_revisions,
+                                    include_merges=True):
+    r"""Return the list of revision ids which touch a given path.
 
     The function filters view_revisions and returns a subset.
-    This includes the revisions which directly change the file id,
+    This includes the revisions which directly change the path,
     and the revisions which merge these changes. So if the
     revision graph is::
 
@@ -1218,7 +1271,7 @@
 
     :param branch: The branch where we can get text revision information.
 
-    :param file_id: Filter out revisions that do not touch file_id.
+    :param path: Filter out revisions that do not touch path.
 
     :param view_revisions: A list of (revision_id, dotted_revno, merge_depth)
         tuples. This is the list of revisions which will be filtered. It is
@@ -1232,6 +1285,8 @@
     # Lookup all possible text keys to determine which ones actually modified
     # the file.
     graph = branch.repository.get_file_graph()
+    start_tree = branch.repository.revision_tree(view_revisions[0][0])
+    file_id = start_tree.path2id(path)
     get_parent_map = graph.get_parent_map
     text_keys = [(file_id, rev_id) for rev_id, revno, depth in view_revisions]
     next_keys = None
@@ -1939,8 +1994,7 @@
                  verbose=False,
                  direction='forward',
                  start_revision=base_idx + 1,
-                 end_revision=len(new_rh),
-                 search=None)
+                 end_revision=len(new_rh))
 
 
 def get_history_change(old_revision_id, new_revision_id, repository):
@@ -2039,7 +2093,7 @@
 
 
 def _get_info_for_log_files(revisionspec_list, file_list, exit_stack):
-    """Find file-ids and kinds given a list of files and a revision range.
+    """Find files and kinds given a list of files and a revision range.
 
     We search for files at the end of the range. If not found there,
     we try the start of the range.
@@ -2051,7 +2105,7 @@
     :param exit_stack: When the branch returned is read locked,
       an unlock call will be queued to the exit stack.
     :return: (branch, info_list, start_rev_info, end_rev_info) where
-      info_list is a list of (relative_path, file_id, kind) tuples where
+      info_list is a list of (relative_path, found, kind) tuples where
       kind is one of values 'directory', 'file', 'symlink', 'tree-reference'.
       branch will be read-locked.
     """
@@ -2081,31 +2135,27 @@
             tree = b.basis_tree()
         tree1 = None
         for fp in relpaths:
-            file_id = tree.path2id(fp)
-            kind = _get_kind_for_file_id(tree, fp, file_id)
-            if file_id is None:
+            kind = _get_kind_for_file(tree, fp)
+            if not kind:
                 # go back to when time began
                 if tree1 is None:
                     try:
                         rev1 = b.get_rev_id(1)
                     except errors.NoSuchRevision:
                         # No history at all
-                        file_id = None
                         kind = None
                     else:
                         tree1 = b.repository.revision_tree(rev1)
                 if tree1:
-                    file_id = tree1.path2id(fp)
-                    kind = _get_kind_for_file_id(tree1, fp, file_id)
-            info_list.append((fp, file_id, kind))
+                    kind = _get_kind_for_file(tree1, fp)
+            info_list.append((fp, kind))
 
     elif start_rev_info == end_rev_info:
         # One revision given - file must exist in it
         tree = b.repository.revision_tree(end_rev_info.rev_id)
         for fp in relpaths:
-            file_id = tree.path2id(fp)
-            kind = _get_kind_for_file_id(tree, fp, file_id)
-            info_list.append((fp, file_id, kind))
+            kind = _get_kind_for_file(tree, fp)
+            info_list.append((fp, kind))
 
     else:
         # Revision range given. Get the file-id from the end tree.
@@ -2117,9 +2167,8 @@
             tree = b.repository.revision_tree(rev_id)
         tree1 = None
         for fp in relpaths:
-            file_id = tree.path2id(fp)
-            kind = _get_kind_for_file_id(tree, fp, file_id)
-            if file_id is None:
+            kind = _get_kind_for_file(tree, fp)
+            if not kind:
                 if tree1 is None:
                     rev_id = start_rev_info.rev_id
                     if rev_id is None:
@@ -2127,18 +2176,18 @@
                         tree1 = b.repository.revision_tree(rev1)
                     else:
                         tree1 = b.repository.revision_tree(rev_id)
-                file_id = tree1.path2id(fp)
-                kind = _get_kind_for_file_id(tree1, fp, file_id)
-            info_list.append((fp, file_id, kind))
+                kind = _get_kind_for_file(tree1, fp)
+            info_list.append((fp, kind))
     return b, info_list, start_rev_info, end_rev_info
 
 
-def _get_kind_for_file_id(tree, path, file_id):
-    """Return the kind of a file-id or None if it doesn't exist."""
-    if file_id is not None:
-        return tree.kind(path)
-    else:
-        return None
+def _get_kind_for_file(tree, path):
+    """Return the kind of a path or None if it doesn't exist."""
+    with tree.lock_read():
+        try:
+            return tree.stored_kind(path)
+        except errors.NoSuchFile:
+            return None
 
 
 properties_handler_registry = registry.Registry()

=== modified file 'breezy/merge.py'
--- breezy/merge.py	2020-02-18 01:57:45 +0000
+++ breezy/merge.py	2020-07-18 23:14:00 +0000
@@ -30,7 +30,6 @@
     revision as _mod_revision,
     textfile,
     trace,
-    transform,
     tree as _mod_tree,
     tsort,
     ui,
@@ -47,6 +46,7 @@
     errors,
     hooks,
     registry,
+    transform,
     )
 # TODO: Report back as changes are merged in
 
@@ -758,7 +758,7 @@
             stack.enter_context(self.this_tree.lock_read())
             stack.enter_context(self.base_tree.lock_read())
             stack.enter_context(self.other_tree.lock_read())
-            self.tt = self.working_tree.get_transform()
+            self.tt = self.working_tree.transform()
             stack.enter_context(self.tt)
             self._compute_transform()
             results = self.tt.apply(no_conflicts=True)
@@ -770,16 +770,16 @@
 
     def make_preview_transform(self):
         with self.base_tree.lock_read(), self.other_tree.lock_read():
-            self.tt = transform.TransformPreview(self.working_tree)
+            self.tt = self.working_tree.preview_transform()
             self._compute_transform()
             return self.tt
 
     def _compute_transform(self):
         if self._lca_trees is None:
-            entries = self._entries3()
+            entries = list(self._entries3())
             resolver = self._three_way
         else:
-            entries = self._entries_lca()
+            entries = list(self._entries_lca())
             resolver = self._lca_multi_way
         # Prepare merge hooks
         factories = Merger.hooks['merge_file_content']
@@ -790,7 +790,6 @@
             for num, (file_id, changed, paths3, parents3, names3,
                       executable3) in enumerate(entries):
                 trans_id = self.tt.trans_id_file_id(file_id)
-
                 # Try merging each entry
                 child_pb.update(gettext('Preparing file merge'),
                                 num, len(entries))
@@ -831,7 +830,6 @@
         other and this.  names3 is a tuple of names for base, other and this.
         executable3 is a tuple of execute-bit values for base, other and this.
         """
-        result = []
         iterator = self.other_tree.iter_changes(self.base_tree,
                                                 specific_files=self.interesting_files,
                                                 extra_trees=[self.this_tree])
@@ -859,10 +857,9 @@
             names3 = change.name + (this_name,)
             paths3 = change.path + (this_path, )
             executable3 = change.executable + (this_executable,)
-            result.append(
+            yield (
                 (change.file_id, change.changed_content, paths3,
                  parents3, names3, executable3))
-        return result
 
     def _entries_lca(self):
         """Gather data about files modified between multiple trees.
@@ -891,7 +888,6 @@
                 self.interesting_files, lookup_trees)
         else:
             interesting_files = None
-        result = []
         from .multiwalker import MultiWalker
         walker = MultiWalker(self.other_tree, self._lca_trees)
 
@@ -1035,7 +1031,7 @@
                     raise AssertionError('unhandled kind: %s' % other_ie.kind)
 
             # If we have gotten this far, that means something has changed
-            result.append((file_id, content_changed,
+            yield (file_id, content_changed,
                            ((base_path, lca_paths),
                             other_path, this_path),
                            ((base_ie.parent_id, lca_parent_ids),
@@ -1044,8 +1040,7 @@
                             other_ie.name, this_ie.name),
                            ((base_ie.executable, lca_executable),
                             other_ie.executable, this_ie.executable)
-                           ))
-        return result
+                           )
 
     def write_modified(self, results):
         if not self.working_tree.supports_merge_modified():
@@ -1284,7 +1279,7 @@
                     keep_this = True
                     # versioning the merged file will trigger a duplicate
                     # conflict
-                    self.tt.version_file(file_id, trans_id)
+                    self.tt.version_file(trans_id, file_id=file_id)
                     transform.create_from_tree(
                         self.tt, trans_id, self.other_tree,
                         other_path,
@@ -1331,7 +1326,7 @@
         else:
             raise AssertionError('unknown hook_status: %r' % (hook_status,))
         if not this_path and result == "modified":
-            self.tt.version_file(file_id, trans_id)
+            self.tt.version_file(trans_id, file_id=file_id)
         if not keep_this:
             # The merge has been performed and produced a new content, so the
             # old contents should not be retained.
@@ -1461,14 +1456,8 @@
             data.append(('BASE', self.base_tree, base_path, base_lines))
 
         # We need to use the actual path in the working tree of the file here,
-        # ignoring the conflict suffixes
-        wt = self.this_tree
-        if wt.supports_content_filtering():
-            try:
-                filter_tree_path = wt.id2path(file_id)
-            except errors.NoSuchId:
-                # file has been deleted
-                filter_tree_path = None
+        if self.this_tree.supports_content_filtering():
+            filter_tree_path = this_path
         else:
             # Skip the id2path lookup for older formats
             filter_tree_path = None
@@ -1482,7 +1471,7 @@
                     filter_tree_path)
                 file_group.append(trans_id)
                 if set_version and not versioned:
-                    self.tt.version_file(file_id, trans_id)
+                    self.tt.version_file(trans_id, file_id=file_id)
                     versioned = True
         return file_group
 

=== modified file 'breezy/merge_directive.py'
--- breezy/merge_directive.py	2020-02-18 01:57:45 +0000
+++ breezy/merge_directive.py	2020-07-18 23:14:00 +0000
@@ -45,6 +45,16 @@
     )
 
 
+class IllegalMergeDirectivePayload(errors.BzrError):
+    """A merge directive contained something other than a patch or bundle"""
+
+    _fmt = "Bad merge directive payload %(start)r"
+
+    def __init__(self, start):
+        errors.BzrError(self)
+        self.start = start
+
+
 class MergeRequestBodyParams(object):
     """Parameter object for the merge_request_body hook."""
 
@@ -542,7 +552,7 @@
                 if start.startswith(b'# Begin bundle'):
                     bundle = b''.join(line_iter)
                 else:
-                    raise errors.IllegalMergeDirectivePayload(start)
+                    raise IllegalMergeDirectivePayload(start)
         time, timezone = timestamp.parse_patch_date(stanza.get('timestamp'))
         kwargs = {}
         for key in ('revision_id', 'testament_sha1', 'target_branch',

=== modified file 'breezy/mutabletree.py'
--- breezy/mutabletree.py	2020-02-18 01:57:45 +0000
+++ breezy/mutabletree.py	2020-07-18 23:14:00 +0000
@@ -401,9 +401,9 @@
         """
         raise NotImplementedError(self.copy_one)
 
-    def get_transform(self, pb=None):
+    def transform(self, pb=None):
         """Return a transform object for use with this tree."""
-        raise NotImplementedError(self.get_transform)
+        raise NotImplementedError(self.transform)
 
 
 class MutableTreeHooks(hooks.Hooks):

=== modified file 'breezy/osutils.py'
--- breezy/osutils.py	2020-05-24 00:42:36 +0000
+++ breezy/osutils.py	2020-07-18 23:14:00 +0000
@@ -1362,38 +1362,6 @@
     return unicode_or_utf8_string.encode('utf-8')
 
 
-def safe_revision_id(unicode_or_utf8_string):
-    """Revision ids should now be utf8, but at one point they were unicode.
-
-    :param unicode_or_utf8_string: A possibly Unicode revision_id. (can also be
-        utf8 or None).
-    :return: None or a utf8 revision id.
-    """
-    if (unicode_or_utf8_string is None
-            or unicode_or_utf8_string.__class__ == bytes):
-        return unicode_or_utf8_string
-    raise TypeError('Unicode revision ids are no longer supported. '
-                    'Revision id generators should be creating utf8 revision '
-                    'ids.')
-
-
-def safe_file_id(unicode_or_utf8_string):
-    """File ids should now be utf8, but at one point they were unicode.
-
-    This is the same as safe_utf8, except it uses the cached encode functions
-    to save a little bit of performance.
-
-    :param unicode_or_utf8_string: A possibly Unicode file_id. (can also be
-        utf8 or None).
-    :return: None or a utf8 file id.
-    """
-    if (unicode_or_utf8_string is None
-            or unicode_or_utf8_string.__class__ == bytes):
-        return unicode_or_utf8_string
-    raise TypeError('Unicode file ids are no longer supported. '
-                    'File id generators should be creating utf8 file ids.')
-
-
 _platform_normalizes_filenames = False
 if sys.platform == 'darwin':
     _platform_normalizes_filenames = True

=== modified file 'breezy/patches.py'
--- breezy/patches.py	2020-02-18 01:57:45 +0000
+++ breezy/patches.py	2020-07-18 23:14:00 +0000
@@ -610,8 +610,7 @@
         self.prefix = prefix
 
     def __enter__(self):
-        from .transform import TransformPreview
-        self._tt = TransformPreview(self.tree)
+        self._tt = self.tree.preview_transform()
         apply_patches(self._tt, self.patches, prefix=self.prefix)
         return self._tt.get_preview_tree()
 

=== modified file 'breezy/plugins/darcs/__init__.py'
--- breezy/plugins/darcs/__init__.py	2020-05-06 02:13:25 +0000
+++ breezy/plugins/darcs/__init__.py	2020-07-18 23:14:00 +0000
@@ -79,7 +79,7 @@
 
     @classmethod
     def probe_transport(klass, transport):
-        if transport.has('_darcs/format'):
+        if transport.has_any(['_darcs/format', '_darcs/inventory']):
             return DarcsDirFormat()
         raise errors.NotBranchError(path=transport.base)
 

=== modified file 'breezy/plugins/fastimport/exporter.py'
--- breezy/plugins/fastimport/exporter.py	2020-02-18 01:57:45 +0000
+++ breezy/plugins/fastimport/exporter.py	2020-07-18 23:14:00 +0000
@@ -341,7 +341,7 @@
     def emit_baseline(self, revobj, ref):
         # Emit a full source tree of the first commit's parent
         mark = 1
-        self.revid_to_mark[revobj.revision_id] = mark
+        self.revid_to_mark[revobj.revision_id] = b"%d" % mark
         tree_old = self.branch.repository.revision_tree(
             breezy.revision.NULL_REVISION)
         [tree_new] = list(self._get_revision_trees([revobj.revision_id]))
@@ -350,12 +350,12 @@
         self.print_cmd(self._get_commit_command(ref, mark, revobj, file_cmds))
 
     def preprocess_commit(self, revid, revobj, ref):
-        if revid in self.revid_to_mark or revid in self.excluded_revisions:
-            return
+        if self.revid_to_mark.get(revid) or revid in self.excluded_revisions:
+            return []
         if revobj is None:
             # This is a ghost revision. Mark it as not found and next!
-            self.revid_to_mark[revid] = -1
-            return
+            self.revid_to_mark[revid] = None
+            return []
         # Get the primary parent
         # TODO: Consider the excluded revisions when deciding the parents.
         # Currently, a commit with parents that are excluded ought to be
@@ -367,9 +367,8 @@
             parent = revobj.parent_ids[0]
 
         # Print the commit
-        mark = len(self.revid_to_mark) + 1
-        self.revid_to_mark[revobj.revision_id] = mark
-
+        self.revid_to_mark[revobj.revision_id] = b"%d" % (
+            len(self.revid_to_mark) + 1)
         return [parent, revobj.revision_id]
 
     def emit_commit(self, revobj, ref, tree_old, tree_new):
@@ -437,7 +436,7 @@
                 continue
             try:
                 parent_mark = self.revid_to_mark[p]
-                non_ghost_parents.append(b":%d" % parent_mark)
+                non_ghost_parents.append(b":%s" % parent_mark)
             except KeyError:
                 # ghost - ignore
                 continue
@@ -657,4 +656,4 @@
                         self.warning('not creating tag %r as its name would not be '
                                      'valid in git.', git_ref)
                         continue
-                self.print_cmd(commands.ResetCommand(git_ref, b":%d" % mark))
+                self.print_cmd(commands.ResetCommand(git_ref, b":%s" % mark))

=== modified file 'breezy/plugins/fastimport/marks_file.py'
--- breezy/plugins/fastimport/marks_file.py	2020-02-18 01:57:45 +0000
+++ breezy/plugins/fastimport/marks_file.py	2020-07-18 23:14:00 +0000
@@ -27,7 +27,7 @@
     """
     # Check that the file is readable and in the right format
     try:
-        f = open(filename, 'r')
+        f = open(filename, 'rb')
     except IOError:
         warning("Could not import marks file %s - not importing marks",
                 filename)
@@ -38,22 +38,22 @@
         revision_ids = {}
 
         line = f.readline()
-        if line == 'format=1\n':
+        if line == b'format=1\n':
             # Cope with old-style marks files
             # Read the branch info
             branch_names = {}
-            for string in f.readline().rstrip('\n').split('\0'):
+            for string in f.readline().rstrip(b'\n').split(b'\0'):
                 if not string:
                     continue
-                name, integer = string.rsplit('.', 1)
+                name, integer = string.rsplit(b'.', 1)
                 branch_names[name] = int(integer)
             line = f.readline()
 
         while line:
-            line = line.rstrip('\n')
-            mark, revid = line.split(' ', 1)
-            mark = mark.lstrip(':')
-            revision_ids[mark] = revid.encode('utf-8')
+            line = line.rstrip(b'\n')
+            mark, revid = line.split(b' ', 1)
+            mark = mark.lstrip(b':')
+            revision_ids[mark] = revid
             line = f.readline()
     finally:
         f.close()
@@ -67,7 +67,7 @@
     :param revision_ids: dictionary mapping marks -> bzr revision-ids
     """
     try:
-        f = open(filename, 'w')
+        f = open(filename, 'wb')
     except IOError:
         warning("Could not open export-marks file %s - not exporting marks",
                 filename)
@@ -75,8 +75,7 @@
 
     try:
         # Write the revision info
-        for mark in revision_ids:
-            f.write(':%s %s\n' % (mark.lstrip(b':').decode('utf-8'),
-                                  revision_ids[mark].decode('utf-8')))
+        for mark, revid in sorted(revision_ids.items()):
+            f.write(b':%s %s\n' % (mark.lstrip(b':'), revid))
     finally:
         f.close()

=== modified file 'breezy/plugins/fastimport/tests/__init__.py'
--- breezy/plugins/fastimport/tests/__init__.py	2020-02-18 01:57:45 +0000
+++ breezy/plugins/fastimport/tests/__init__.py	2020-07-18 23:14:00 +0000
@@ -44,6 +44,7 @@
         'test_exporter',
         'test_branch_mapper',
         'test_generic_processor',
+        'test_marks_file',
         'test_revision_store',
         ]]
     loader = TestLoader()

=== added file 'breezy/plugins/fastimport/tests/test_marks_file.py'
--- breezy/plugins/fastimport/tests/test_marks_file.py	1970-01-01 00:00:00 +0000
+++ breezy/plugins/fastimport/tests/test_marks_file.py	2020-07-01 21:47:15 +0000
@@ -0,0 +1,45 @@
+# Copyright (C) 2020 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
+"""Test the marks file methods."""
+
+from __future__ import absolute_import
+
+from .... import tests
+
+from .. import marks_file
+
+
+class TestMarksFile(tests.TestCaseWithTransport):
+
+    def test_read(self):
+        self.build_tree_contents([('marks', """\
+:1 jelmer@jelmer-rev1
+:2 joe@example.com-rev2
+""")])
+        self.assertEqual({
+            b'1': b'jelmer@jelmer-rev1',
+            b'2': b'joe@example.com-rev2',
+            }, marks_file.import_marks('marks'))
+
+    def test_write(self):
+        marks_file.export_marks('marks', {
+            b'1': b'jelmer@jelmer-rev1',
+            b'2': b'joe@example.com-rev2',
+            })
+        self.assertFileEqual("""\
+:1 jelmer@jelmer-rev1
+:2 joe@example.com-rev2
+""", 'marks')

=== modified file 'breezy/plugins/github/hoster.py'
--- breezy/plugins/github/hoster.py	2020-06-23 01:02:30 +0000
+++ breezy/plugins/github/hoster.py	2020-07-18 23:14:00 +0000
@@ -42,7 +42,11 @@
     version_string as breezy_version,
     )
 from ...config import AuthenticationConfig, GlobalStack
-from ...errors import InvalidHttpResponse, PermissionDenied
+from ...errors import (
+    InvalidHttpResponse,
+    PermissionDenied,
+    UnexpectedHttpStatus,
+    )
 from ...git.urls import git_url_to_bzr_url
 from ...i18n import gettext
 from ...trace import note
@@ -162,7 +166,7 @@
         if response.status == 422:
             raise ValidationFailed(json.loads(response.text))
         if response.status != 200:
-            raise InvalidHttpResponse(self._pr['url'], response.text)
+            raise UnexpectedHttpStatus(self._pr['url'], response.status)
         self._pr = json.loads(response.text)
 
     def set_description(self, description):
@@ -199,7 +203,7 @@
         if response.status == 422:
             raise ValidationFailed(json.loads(response.text))
         if response.status != 200:
-            raise InvalidHttpResponse(self._pr['url'], response.text)
+            raise UnexpectedHttpStatus(self._pr['url'], response.status)
 
     def get_merged_by(self):
         merged_by = self._pr.get('merged_by')
@@ -221,7 +225,8 @@
         if response.status == 422:
             raise ValidationFailed(json.loads(response.text))
         if response.status != 201:
-            raise InvalidHttpResponse(self._pr['comments_url'], response.text)
+            raise UnexpectedHttpStatus(
+                self._pr['comments_url'], response.status)
         json.loads(response.text)
 
 
@@ -282,7 +287,7 @@
             raise NoSuchProject(path)
         if response.status == 200:
             return json.loads(response.text)
-        raise InvalidHttpResponse(path, response.text)
+        raise UnexpectedHttpStatus(path, response.status)
 
     def _get_repo_pulls(self, path, head=None, state=None):
         path = path + '?'
@@ -298,7 +303,7 @@
             raise NoSuchProject(path)
         if response.status == 200:
             return json.loads(response.text)
-        raise InvalidHttpResponse(path, response.text)
+        raise UnexpectedHttpStatus(path, response.status)
 
     def _create_pull(self, path, title, head, base, body=None, labels=None,
                      assignee=None, draft=False, maintainer_can_modify=False):
@@ -321,14 +326,14 @@
         if response.status == 403:
             raise PermissionDenied(path, response.text)
         if response.status != 201:
-            raise InvalidHttpResponse(path, 'req is invalid %d %r: %r' % (response.status, data, response.text))
+            raise UnexpectedHttpStatus(path, response.status)
         return json.loads(response.text)
 
     def _get_user_by_email(self, email):
         path = 'search/users?q=%s+in:email' % email
         response = self._api_request('GET', path)
         if response.status != 200:
-            raise InvalidHttpResponse(path, response.text)
+            raise UnexpectedHttpStatus(path, response.status)
         ret = json.loads(response.text)
         if ret['total_count'] == 0:
             raise KeyError('no user with email %s' % email)
@@ -343,14 +348,14 @@
             path = 'user'
         response = self._api_request('GET', path)
         if response.status != 200:
-            raise InvalidHttpResponse(path, response.text)
+            raise UnexpectedHttpStatus(path, response.status)
         return json.loads(response.text)
 
     def _get_organization(self, name):
         path = 'orgs/%s' % name
         response = self._api_request('GET', path)
         if response.status != 200:
-            raise InvalidHttpResponse(path, response.text)
+            raise UnexpectedHttpStatus(path, response.status)
         return json.loads(response.text)
 
     def _list_paged(self, path, parameters=None, per_page=None):
@@ -369,7 +374,7 @@
                 ';'.join(['%s=%s' % (k, urlutils.quote(v))
                           for (k, v) in parameters.items()]))
             if response.status != 200:
-                raise InvalidHttpResponse(path, response.text)
+                raise UnexpectedHttpStatus(path, response.status)
             data = json.loads(response.text)
             for entry in data['items']:
                 i += 1
@@ -383,11 +388,11 @@
         return self._list_paged(path, {'q': query}, per_page=DEFAULT_PER_PAGE)
 
     def _create_fork(self, path, owner=None):
-        if owner and owner != self._current_user['login']:
+        if owner and owner != self.current_user['login']:
             path += '?organization=%s' % owner
         response = self._api_request('POST', path)
         if response.status != 202:
-            raise InvalidHttpResponse(path, 'status: %d, %r' % (response.status, response.text))
+            raise UnexpectedHttpStatus(path, response.status)
         return json.loads(response.text)
 
     @property
@@ -397,7 +402,13 @@
     def __init__(self, transport):
         self._token = retrieve_github_token('https', GITHUB_HOST)
         self.transport = transport
-        self._current_user = self._get_user()
+        self._current_user = None
+
+    @property
+    def current_user(self):
+        if self._current_user is None:
+            self._current_user = self._get_user()
+        return self._current_user
 
     def publish_derived(self, local_branch, base_branch, name, project=None,
                         owner=None, revision_id=None, overwrite=False,
@@ -405,7 +416,7 @@
         base_owner, base_project, base_branch_name = parse_github_branch_url(base_branch)
         base_repo = self._get_repo(base_owner, base_project)
         if owner is None:
-            owner = self._current_user['login']
+            owner = self.current_user['login']
         if project is None:
             project = base_repo['name']
         try:
@@ -441,7 +452,7 @@
         base_owner, base_project, base_branch_name = parse_github_branch_url(base_branch)
         base_repo = self._get_repo(base_owner, base_project)
         if owner is None:
-            owner = self._current_user['login']
+            owner = self.current_user['login']
         if project is None:
             project = base_repo['name']
         try:
@@ -516,12 +527,12 @@
             query.append('is:closed')
         elif status == 'merged':
             query.append('is:merged')
-        query.append('author:%s' % self._current_user['login'])
+        query.append('author:%s' % self.current_user['login'])
         for issue in self._search_issues(query=' '.join(query)):
             url = issue['pull_request']['url']
             response = self._api_request('GET', url)
             if response.status != 200:
-                raise InvalidHttpResponse(url, response.text)
+                raise UnexpectedHttpStatus(url, response.status)
             yield GitHubMergeProposal(self, json.loads(response.text))
 
     def get_proposal_by_url(self, url):
@@ -530,7 +541,7 @@
     def iter_my_forks(self):
         response = self._api_request('GET', '/user/repos')
         if response.status != 200:
-            raise InvalidHttpResponse(url, response.text)
+            raise UnexpectedHttpStatus(self.transport.user_url, response.status)
         for project in json.loads(response.text):
             if not project['fork']:
                 continue
@@ -545,7 +556,7 @@
             return
         if response.status == 200:
             return json.loads(response.text)
-        raise InvalidHttpResponse(path, response.text)
+        raise UnexpectedHttpStatus(path, response.status)
 
 
 class GitHubMergeProposalBuilder(MergeProposalBuilder):

=== modified file 'breezy/plugins/gitlab/hoster.py'
--- breezy/plugins/gitlab/hoster.py	2020-06-23 01:02:30 +0000
+++ breezy/plugins/gitlab/hoster.py	2020-07-18 23:14:00 +0000
@@ -97,6 +97,14 @@
         self.error = error
 
 
+class ForkingDisabled(errors.BzrError):
+
+    _fmt = ("Forking on project %(project)s is disabled.")
+
+    def __init__(self, project):
+        self.project = project
+
+
 class MergeRequestExists(Exception):
     """Raised when a merge requests already exists."""
 
@@ -325,7 +333,7 @@
             raise KeyError('no such user %s' % username)
         if response.status == 200:
             return json.loads(response.data)
-        raise errors.InvalidHttpResponse(path, response.text)
+        raise errors.UnexpectedHttpStatus(path, response.status)
 
     def _get_user_by_email(self, email):
         path = 'users?search=%s' % urlutils.quote(str(email), '')
@@ -337,7 +345,7 @@
             if len(ret) != 1:
                 raise ValueError('unexpected number of results; %r' % ret)
             return ret[0]
-        raise errors.InvalidHttpResponse(path, response.text)
+        raise errors.UnexpectedHttpStatus(path, response.status)
 
     def _get_project(self, project_name):
         path = 'projects/%s' % urlutils.quote(str(project_name), '')
@@ -346,13 +354,15 @@
             raise NoSuchProject(project_name)
         if response.status == 200:
             return json.loads(response.data)
-        raise errors.InvalidHttpResponse(path, response.text)
+        raise errors.UnexpectedHttpStatus(path, response.status)
 
     def _fork_project(self, project_name, timeout=50, interval=5):
         path = 'projects/%s/fork' % urlutils.quote(str(project_name), '')
         response = self._api_request('POST', path)
+        if response.status == 404:
+            raise ForkingDisabled(project_name)
         if response.status not in (200, 201):
-            raise errors.InvalidHttpResponse(path, response.text)
+            raise errors.UnexpectedHttpStatus(path, response.status)
         # The response should be valid JSON, but let's ignore it
         project = json.loads(response.data)
         # Spin and wait until import_status for new project
@@ -385,7 +395,7 @@
             if response.status == 403:
                 raise errors.PermissionDenied(response.text)
             if response.status != 200:
-                raise errors.InvalidHttpResponse(path, response.text)
+                raise errors.UnexpectedHttpStatus(path, response.status)
             page = response.getheader("X-Next-Page")
             for entry in json.loads(response.data):
                 yield entry
@@ -408,7 +418,7 @@
         if response.status == 403:
             raise errors.PermissionDenied(response.text)
         if response.status != 200:
-            raise errors.InvalidHttpResponse(path, response.text)
+            raise errors.UnexpectedHttpStatus(path, response.status)
         return json.loads(response.data)
 
     def _list_projects(self, owner):
@@ -422,7 +432,7 @@
         response = self._api_request('PUT', path, fields=mr)
         if response.status == 200:
             return json.loads(response.data)
-        raise errors.InvalidHttpResponse(path, response.text)
+        raise errors.UnexpectedHttpStatus(path, response.status)
 
     def _post_merge_request_note(self, project_id, iid, kwargs):
         path = 'projects/%s/merge_requests/%s/notes' % (
@@ -431,7 +441,7 @@
         if response.status == 201:
             json.loads(response.data)
             return
-        raise errors.InvalidHttpResponse(path, response.text)
+        raise errors.UnexpectedHttpStatus(path, response.status)
 
     def _create_mergerequest(
             self, title, source_project_id, target_project_id,
@@ -454,7 +464,7 @@
         if response.status == 409:
             raise MergeRequestExists()
         if response.status != 201:
-            raise errors.InvalidHttpResponse(path, response.text)
+            raise errors.UnexpectedHttpStatus(path, response.status)
         return json.loads(response.data)
 
     def get_push_url(self, branch):
@@ -543,7 +553,7 @@
                 raise GitLabLoginMissing()
             else:
                 raise GitlabLoginError(response.text)
-        raise UnsupportedHoster(url)
+        raise UnsupportedHoster(self.base_url)
 
     @classmethod
     def probe_from_url(cls, url, possible_transports=None):
@@ -602,7 +612,7 @@
         if response.status == 404:
             raise NoSuchProject(project)
         if response.status != 202:
-            raise errors.InvalidHttpResponse(path, response.text)
+            raise errors.UnexpectedHttpStatus(path, response.status)
 
 
 class GitlabMergeProposalBuilder(MergeProposalBuilder):

=== modified file 'breezy/plugins/hg/__init__.py'
--- breezy/plugins/hg/__init__.py	2020-02-18 01:57:45 +0000
+++ breezy/plugins/hg/__init__.py	2020-07-18 23:14:00 +0000
@@ -80,7 +80,7 @@
     def probe_transport(klass, transport):
         """Our format is present if the transport has a '.hg/' subdir."""
         if klass._has_hg_dumb_repository(transport):
-            return HgDirFormat()
+            return LocalHgDirFormat()
         raise errors.NotBranchError(path=transport.base)
 
     @classmethod
@@ -123,7 +123,11 @@
 
     @classmethod
     def priority(klass, transport):
-        return 90
+        if 'hg' in transport.base:
+            return 90
+        # hgweb repositories are prone to return *a* page for every possible URL,
+        # making probing hard for other formats so use 99 here rather than 100.
+        return 99
 
     @staticmethod
     def _has_hg_http_smart_server(transport, external_url):
@@ -135,8 +139,7 @@
         """
         from breezy.urlutils import urlparse
         parsed_url = urlparse.urlparse(external_url)
-        parsed_url = parsed_url._replace(
-            query='cmd=capabilities', path=parsed_url.path.rstrip('/') + '/hg')
+        parsed_url = parsed_url._replace(query='cmd=capabilities')
         url = urlparse.urlunparse(parsed_url)
         resp = transport.request(
             'GET', url, headers={'Accept': 'application/mercurial-0.1'})

=== modified file 'breezy/plugins/launchpad/hoster.py'
--- breezy/plugins/launchpad/hoster.py	2020-06-23 01:02:30 +0000
+++ breezy/plugins/launchpad/hoster.py	2020-07-18 23:14:00 +0000
@@ -228,12 +228,18 @@
             lp_base_url = uris.STAGING_SERVICE_ROOT
         else:
             lp_base_url = uris.LPNET_SERVICE_ROOT
-        self.launchpad = lp_api.connect_launchpad(lp_base_url, version='devel')
+        self._api_base_url = lp_base_url
+        self._launchpad = None
+
+    @property
+    def launchpad(self):
+        if self._launchpad is None:
+            self._launchpad = lp_api.connect_launchpad(self._api_base_url, version='devel')
+        return self._launchpad
 
     @property
     def base_url(self):
-        return lp_api.uris.web_root_for_service_root(
-            str(self.launchpad._root_uri))
+        return lp_api.uris.web_root_for_service_root(self._api_base_url)
 
     def __repr__(self):
         return "Launchpad(staging=%s)" % self._staging

=== modified file 'breezy/plugins/quilt/__init__.py'
--- breezy/plugins/quilt/__init__.py	2020-02-18 01:57:45 +0000
+++ breezy/plugins/quilt/__init__.py	2020-07-18 23:14:00 +0000
@@ -113,7 +113,12 @@
     config = tree.get_config_stack()
     policy = config.get('quilt.commit_policy')
     from .merge import start_commit_quilt_patches
-    start_commit_quilt_patches(tree, policy)
+    from .wrapper import QuiltNotInstalled
+    try:
+        start_commit_quilt_patches(tree, policy)
+    except QuiltNotInstalled:
+        trace.warning(
+            'quilt not installed; not updating patches')
 
 
 def post_build_tree_quilt(tree):
@@ -122,7 +127,12 @@
     if policy is None:
         return
     from .merge import post_process_quilt_patches
-    post_process_quilt_patches(tree, [], policy)
+    from .wrapper import QuiltNotInstalled
+    try:
+        post_process_quilt_patches(tree, [], policy)
+    except QuiltNotInstalled:
+        trace.warning(
+            'quilt not installed; not touching patches')
 
 
 from ...hooks import install_lazy_named_hook

=== modified file 'breezy/plugins/stats/cmds.py'
--- breezy/plugins/stats/cmds.py	2020-02-18 01:57:45 +0000
+++ breezy/plugins/stats/cmds.py	2020-07-18 23:14:00 +0000
@@ -313,7 +313,7 @@
     with ui.ui_factory.nested_progress_bar() as pb:
         with repository.lock_read():
             i = 0
-            for delta in repository.get_deltas_for_revisions(revs):
+            for delta in repository.get_revision_deltas(revs):
                 pb.update("classifying commits", i, len(revs))
                 for c in classify_delta(delta):
                     if c not in ret:
@@ -362,7 +362,7 @@
                     if ps is not None and r != NULL_REVISION]
         revs = repository.get_revisions(ancestry)
         with ui.ui_factory.nested_progress_bar() as pb:
-            iterator = zip(revs, repository.get_deltas_for_revisions(revs))
+            iterator = zip(revs, repository.get_revision_deltas(revs))
             for i, (rev, delta) in enumerate(iterator):
                 pb.update("analysing revisions", i, len(revs))
                 # Don't count merges

=== modified file 'breezy/repository.py'
--- breezy/repository.py	2020-02-18 01:57:45 +0000
+++ breezy/repository.py	2020-07-18 23:14:00 +0000
@@ -884,19 +884,6 @@
         """
         raise NotImplementedError(self.iter_revisions)
 
-    def get_deltas_for_revisions(self, revisions, specific_fileids=None):
-        """Produce a generator of revision deltas.
-
-        Note that the input is a sequence of REVISIONS, not revision_ids.
-        Trees will be held in memory until the generator exits.
-        Each delta is relative to the revision's lefthand predecessor.
-
-        :param specific_fileids: if not None, the result is filtered
-          so that only those file-ids, their parents and their
-          children are included.
-        """
-        raise NotImplementedError(self.get_deltas_for_revisions)
-
     def get_revision_delta(self, revision_id):
         """Return the delta for one revision.
 
@@ -905,7 +892,45 @@
         """
         with self.lock_read():
             r = self.get_revision(revision_id)
-            return list(self.get_deltas_for_revisions([r]))[0]
+            return list(self.get_revision_deltas([r]))[0]
+
+    def get_revision_deltas(self, revisions, specific_files=None):
+        """Produce a generator of revision deltas.
+
+        Note that the input is a sequence of REVISIONS, not revision ids.
+        Trees will be held in memory until the generator exits.
+        Each delta is relative to the revision's lefthand predecessor.
+
+        specific_files should exist in the first revision.
+
+        :param specific_files: if not None, the result is filtered
+          so that only those files, their parents and their
+          children are included.
+        """
+        from .tree import InterTree
+        # Get the revision-ids of interest
+        required_trees = set()
+        for revision in revisions:
+            required_trees.add(revision.revision_id)
+            required_trees.update(revision.parent_ids[:1])
+
+        trees = {
+            t.get_revision_id(): t
+            for t in self.revision_trees(required_trees)}
+
+        # Calculate the deltas
+        for revision in revisions:
+            if not revision.parent_ids:
+                old_tree = self.revision_tree(_mod_revision.NULL_REVISION)
+            else:
+                old_tree = trees[revision.parent_ids[0]]
+            intertree = InterTree.get(old_tree, trees[revision.revision_id])
+            yield intertree.compare(specific_files=specific_files)
+            if specific_files is not None:
+                specific_files = [
+                    p for p in intertree.find_source_paths(
+                        specific_files).values()
+                    if p is not None]
 
     def store_revision_signature(self, gpg_strategy, plaintext, revision_id):
         raise NotImplementedError(self.store_revision_signature)
@@ -1504,7 +1529,7 @@
             try:
                 self.target.set_make_working_trees(
                     self.source.make_working_trees())
-            except NotImplementedError:
+            except (NotImplementedError, errors.RepositoryUpgradeRequired):
                 pass
             self.target.fetch(self.source, revision_id=revision_id)
 

=== modified file 'breezy/rio.py'
--- breezy/rio.py	2020-02-18 01:57:45 +0000
+++ breezy/rio.py	2020-07-18 23:14:00 +0000
@@ -184,7 +184,7 @@
         result = []
         for text_tag, text_value in self.items:
             tag = text_tag.encode('ascii')
-            value = text_value.encode('utf-8')
+            value = text_value.encode('utf-8', 'surrogateescape')
             if value == b'':
                 result.append(tag + b': \n')
             elif b'\n' in value:

=== modified file 'breezy/shelf.py'
--- breezy/shelf.py	2020-02-18 01:57:45 +0000
+++ breezy/shelf.py	2020-07-18 23:14:00 +0000
@@ -66,10 +66,10 @@
         :param file_list: The files to make more similar to the target.
         """
         self.work_tree = work_tree
-        self.work_transform = work_tree.get_transform()
+        self.work_transform = work_tree.transform()
         try:
             self.target_tree = target_tree
-            self.shelf_transform = transform.TransformPreview(self.target_tree)
+            self.shelf_transform = self.target_tree.preview_transform()
             try:
                 self.renames = {}
                 self.creation = {}
@@ -266,7 +266,7 @@
                         to_transform, s_trans_id, tree,
                         tree.id2path(file_id))
         if version:
-            to_transform.version_file(file_id, s_trans_id)
+            to_transform.version_file(s_trans_id, file_id=file_id)
 
     def _inverse_lines(self, new_lines, file_id):
         """Produce a version with only those changes removed from new_lines."""
@@ -363,7 +363,7 @@
             base_tree = tree.revision_tree(base_revision_id)
         except errors.NoSuchRevisionInTree:
             base_tree = tree.branch.repository.revision_tree(base_revision_id)
-        tt = transform.TransformPreview(base_tree)
+        tt = base_tree.preview_transform()
         tt.deserialize(records)
         return klass(tree, base_tree, tt, metadata.get(b'message'))
 

=== modified file 'breezy/tests/__init__.py'
--- breezy/tests/__init__.py	2020-06-23 01:02:30 +0000
+++ breezy/tests/__init__.py	2020-07-18 23:14:00 +0000
@@ -2430,10 +2430,10 @@
         self.stack = ''.join(traceback.format_list(stack))
 
     def __str__(self):
-        return self.call.method
+        return self.call.method.decode('utf-8')
 
     def __repr__(self):
-        return self.call.method
+        return self.call.method.decode('utf-8')
 
     def stack(self):
         return self.stack

=== modified file 'breezy/tests/blackbox/test_log.py'
--- breezy/tests/blackbox/test_log.py	2020-06-23 01:02:30 +0000
+++ breezy/tests/blackbox/test_log.py	2020-07-18 23:14:00 +0000
@@ -902,7 +902,7 @@
         # Check logging a deleted file is ok if the file existed
         # at the start of the revision range
         self.prepare_tree(complex=True)
-        self.assertLogRevnos(['file1'], ['1'])
+        self.assertLogRevnos(['file1'], [])
 
     def test_log_file_renamed(self):
         """File matched against revision range, not current tree."""

=== modified file 'breezy/tests/blackbox/test_tags.py'
--- breezy/tests/blackbox/test_tags.py	2020-06-19 21:26:53 +0000
+++ breezy/tests/blackbox/test_tags.py	2020-07-18 23:14:00 +0000
@@ -22,7 +22,6 @@
     lockable_files,
     lockdir,
     tag,
-    transform,
     )
 from breezy.bzr import (
     branch as bzrbranch,
@@ -147,9 +146,9 @@
     def make_fork(self, branch):
         fork = branch.create_clone_on_transport(self.get_transport('fork'))
         self.addCleanup(fork.lock_write().unlock)
-        with transform.TransformPreview(fork.basis_tree()) as tt:
+        with fork.basis_tree().preview_transform() as tt:
             tt.commit(fork, message='Commit in fork.', revision_id=b'fork-0')
-        with transform.TransformPreview(fork.basis_tree()) as tt:
+        with fork.basis_tree().preview_transform() as tt:
             tt.commit(fork, message='Commit in fork.', revision_id=b'fork-1')
         return fork
 

=== modified file 'breezy/tests/http_server.py'
--- breezy/tests/http_server.py	2020-02-07 02:14:30 +0000
+++ breezy/tests/http_server.py	2020-07-18 23:14:00 +0000
@@ -187,22 +187,6 @@
         header_line = '%s: %s\r\n' % (keyword, value)
         return len(header_line)
 
-    def send_head(self):
-        """Overrides base implementation to work around a bug in python2.5."""
-        path = self.translate_path(self.path)
-        if os.path.isdir(path) and not self.path.endswith('/'):
-            # redirect browser - doing basically what apache does when
-            # DirectorySlash option is On which is quite common (braindead, but
-            # common)
-            self.send_response(301)
-            self.send_header("Location", self.path + "/")
-            # Indicates that the body is empty for HTTP/1.1 clients
-            self.send_header('Content-Length', '0')
-            self.end_headers()
-            return None
-
-        return http_server.SimpleHTTPRequestHandler.send_head(self)
-
     def send_range_content(self, file, start, length):
         file.seek(start)
         self.wfile.write(file.read(length))

=== modified file 'breezy/tests/per_branch/test_branch.py'
--- breezy/tests/per_branch/test_branch.py	2020-06-11 01:25:02 +0000
+++ breezy/tests/per_branch/test_branch.py	2020-07-18 23:14:00 +0000
@@ -28,7 +28,6 @@
     merge,
     osutils,
     urlutils,
-    transform,
     transport,
     repository,
     revision,
@@ -860,7 +859,7 @@
 
     def write_shelf(self, shelf_file, message=None):
         tree = self.branch.repository.revision_tree(revision.NULL_REVISION)
-        with transform.TransformPreview(tree) as tt:
+        with tree.preview_transform() as tt:
             shelf.ShelfCreator._write_shelf(
                 shelf_file, tt, revision.NULL_REVISION)
 

=== modified file 'breezy/tests/per_branch/test_pull.py'
--- breezy/tests/per_branch/test_pull.py	2020-02-19 23:18:42 +0000
+++ breezy/tests/per_branch/test_pull.py	2020-06-17 02:29:54 +0000
@@ -20,7 +20,6 @@
     branch,
     controldir,
     errors,
-    memorytree,
     revision,
     )
 from breezy.tests import (
@@ -322,8 +321,7 @@
         rev1 = target.commit('rev 1')
         target.unlock()
         sourcedir = target.controldir.clone(self.get_url('source'))
-        source = memorytree.MemoryTree.create_on_branch(
-            sourcedir.open_branch())
+        source = sourcedir.open_branch().create_memorytree()
         rev2 = source.commit('rev 2')
         branch.Branch.hooks.install_named_hook(
             'post_pull', self.capture_post_pull_hook, None)

=== modified file 'breezy/tests/per_branch/test_push.py'
--- breezy/tests/per_branch/test_push.py	2020-02-07 02:14:30 +0000
+++ breezy/tests/per_branch/test_push.py	2020-07-18 23:14:00 +0000
@@ -25,7 +25,6 @@
     controldir,
     check,
     errors,
-    memorytree,
     push,
     revision,
     tests,
@@ -373,8 +372,7 @@
         rev1 = target.commit('rev 1')
         target.unlock()
         sourcedir = target.controldir.clone(self.get_url('source'))
-        source = memorytree.MemoryTree.create_on_branch(
-            sourcedir.open_branch())
+        source = sourcedir.open_branch().create_memorytree()
         rev2 = source.commit('rev 2')
         branch.Branch.hooks.install_named_hook(
             'post_push', self.capture_post_push_hook, None)

=== modified file 'breezy/tests/per_controldir/test_push.py'
--- breezy/tests/per_controldir/test_push.py	2020-02-19 04:50:09 +0000
+++ breezy/tests/per_controldir/test_push.py	2020-06-24 23:59:57 +0000
@@ -19,6 +19,7 @@
 from ...errors import (
     LossyPushToSameVCS,
     TagsNotSupported,
+    NoSuchRevision,
     )
 from ...revision import NULL_REVISION
 from .. import TestNotApplicable
@@ -46,6 +47,13 @@
         self.assertEqual(dir.open_branch().base,
                          tree.branch.get_push_location())
 
+    def test_push_no_such_revision(self):
+        tree, rev_1 = self.create_simple_tree()
+        dir = self.make_repository('dir').controldir
+        self.assertRaises(
+            NoSuchRevision, dir.push_branch,
+            tree.branch, revision_id=b'idonotexist')
+
     def test_push_new_branch_fetch_tags(self):
         builder = self.make_branch_builder('from')
         builder.start_series()

=== modified file 'breezy/tests/per_interbranch/test_pull.py'
--- breezy/tests/per_interbranch/test_pull.py	2020-02-19 23:18:42 +0000
+++ breezy/tests/per_interbranch/test_pull.py	2020-06-17 02:29:54 +0000
@@ -19,7 +19,6 @@
 from breezy.branch import Branch
 from breezy.controldir import ControlDir
 from breezy import errors
-from breezy.memorytree import MemoryTree
 from breezy.revision import NULL_REVISION
 from breezy.tests import TestNotApplicable
 from breezy.tests.per_interbranch import TestCaseWithInterBranch
@@ -276,7 +275,7 @@
         rev1 = target.commit('rev 1')
         target.unlock()
         sourcedir = target.controldir.clone(self.get_url('source'))
-        source = MemoryTree.create_on_branch(sourcedir.open_branch())
+        source = sourcedir.open_branch().create_memorytree()
         rev2 = source.commit('rev 2')
         Branch.hooks.install_named_hook('post_pull',
                                         self.capture_post_pull_hook, None)

=== modified file 'breezy/tests/per_interbranch/test_push.py'
--- breezy/tests/per_interbranch/test_push.py	2020-02-21 03:58:42 +0000
+++ breezy/tests/per_interbranch/test_push.py	2020-07-18 23:14:00 +0000
@@ -37,7 +37,6 @@
     )
 from ...branch import Branch
 from ...controldir import ControlDir
-from ...memorytree import MemoryTree
 from ...revision import NULL_REVISION
 from ...bzr.smart.repository import SmartServerRepositoryGetParentMap
 from . import (
@@ -470,7 +469,7 @@
         rev1 = target.commit('rev 1')
         target.unlock()
         sourcedir = target.branch.controldir.clone(self.get_url('source'))
-        source = MemoryTree.create_on_branch(sourcedir.open_branch())
+        source = sourcedir.open_branch().create_memorytree()
         rev2 = source.commit('rev 2')
         Branch.hooks.install_named_hook('post_push',
                                         self.capture_post_push_hook, None)

=== modified file 'breezy/tests/per_intertree/__init__.py'
--- breezy/tests/per_intertree/__init__.py	2020-06-22 01:07:05 +0000
+++ breezy/tests/per_intertree/__init__.py	2020-07-04 12:29:00 +0000
@@ -30,7 +30,6 @@
 from breezy.bzr import (
     inventorytree,
     )
-from breezy.transform import TransformPreview
 from breezy.tests import (
     default_transport,
     multiply_tests,
@@ -111,7 +110,7 @@
 
 
 def mutable_trees_to_preview_trees(test_case, source, target):
-    preview = TransformPreview(target)
+    preview = target.preview_transform()
     test_case.addCleanup(preview.finalize)
     return source, preview.get_preview_tree()
 
@@ -131,7 +130,8 @@
         ])
     test_intertree_permutations = [
         # test InterTree with two default-format working trees.
-        (InterTree.__name__, InterTree,
+        (inventorytree.InterInventoryTree.__name__,
+         inventorytree.InterInventoryTree,
          default_tree_format, default_tree_format,
          return_provided_trees)]
     for optimiser in InterTree.iter_optimisers():
@@ -142,8 +142,8 @@
             chk_tree_format._get_matchingcontroldir = \
                 lambda: breezy.controldir.format_registry.make_controldir('2a')
             test_intertree_permutations.append(
-                (InterTree.__name__ + "(CHKInventory)",
-                 InterTree,
+                (inventorytree.InterInventoryTree.__name__ + "(CHKInventory)",
+                 inventorytree.InterInventoryTree,
                  chk_tree_format,
                  chk_tree_format,
                  mutable_trees_to_revision_trees))
@@ -174,8 +174,8 @@
                  optimiser._test_mutable_trees_to_test_trees))
     # PreviewTree does not have an InterTree optimiser class.
     test_intertree_permutations.append(
-        (InterTree.__name__ + "(PreviewTree)",
-         InterTree,
+        (inventorytree.InterInventoryTree.__name__ + "(PreviewTree)",
+         inventorytree.InterInventoryTree,
          default_tree_format,
          default_tree_format,
          mutable_trees_to_preview_trees))

=== modified file 'breezy/tests/per_intertree/test_compare.py'
--- breezy/tests/per_intertree/test_compare.py	2019-09-22 22:54:48 +0000
+++ breezy/tests/per_intertree/test_compare.py	2020-07-04 12:29:00 +0000
@@ -434,7 +434,7 @@
         self._make_abc_tree(tree)
         self.build_tree(['d/'], transport=tree.controldir.root_transport)
         tree.add(['d'], [b'd-id'])
-        tt = tree.get_transform()
+        tt = tree.transform()
         trans_id = tt.trans_id_tree_path('b')
         parent_trans_id = tt.trans_id_tree_path('d')
         tt.adjust_path('e', parent_trans_id, trans_id)

=== modified file 'breezy/tests/per_merger.py'
--- breezy/tests/per_merger.py	2019-06-22 12:51:11 +0000
+++ breezy/tests/per_merger.py	2020-07-04 12:29:00 +0000
@@ -169,7 +169,7 @@
                 b'e\n', 'test/foo')
 
     def get_limbodir_deletiondir(self, wt):
-        transform = wt.get_transform()
+        transform = wt.transform()
         limbodir = transform._limbodir
         deletiondir = transform._deletiondir
         transform.finalize()

=== modified file 'breezy/tests/per_repository/test_commit_builder.py'
--- breezy/tests/per_repository/test_commit_builder.py	2020-06-16 23:46:14 +0000
+++ breezy/tests/per_repository/test_commit_builder.py	2020-06-28 23:25:21 +0000
@@ -26,6 +26,7 @@
     revision as _mod_revision,
     tests,
     )
+from breezy.sixish import PY3
 from breezy.tree import TreeChange
 from breezy.bzr import (
     inventorytree,
@@ -821,6 +822,24 @@
                           branch, [], branch.get_config_stack(),
                           revprops={'invalid': u'property\rwith\r\ninvalid chars'})
 
+    def test_get_commit_builder_with_surrogateescape(self):
+        if not PY3:
+            self.skipTest('python 2 does not have surrogateescape')
+        tree = self.make_branch_and_tree(".")
+        with tree.lock_write():
+            builder = tree.branch.get_commit_builder([], revprops={
+                'invalid': u'property' + b'\xc0'.decode('utf-8', 'surrogateescape')})
+            list(builder.record_iter_changes(tree, tree.last_revision(),
+                                             tree.iter_changes(tree.basis_tree())))
+            builder.finish_inventory()
+            try:
+                rev_id = builder.commit('foo bar blah')
+            except NotImplementedError:
+                raise tests.TestNotApplicable(
+                    'Format does not support revision properties')
+        rev = tree.branch.repository.get_revision(rev_id)
+        self.assertEqual('foo bar blah', rev.message)
+
     def test_commit_builder_commit_with_invalid_message(self):
         branch = self.make_branch('.')
         branch.repository.lock_write()

=== modified file 'breezy/tests/per_repository/test_repository.py'
--- breezy/tests/per_repository/test_repository.py	2020-02-07 02:14:30 +0000
+++ breezy/tests/per_repository/test_repository.py	2020-07-18 23:14:00 +0000
@@ -993,27 +993,27 @@
         self.assertEqual(repo.control_url, repo.control_transport.base)
 
 
-class TestDeltaRevisionFiltered(per_repository.TestCaseWithRepository):
+class TestDeltaRevisionFilesFiltered(per_repository.TestCaseWithRepository):
 
     def setUp(self):
-        super(TestDeltaRevisionFiltered, self).setUp()
+        super(TestDeltaRevisionFilesFiltered, self).setUp()
         self.tree_a = self.make_branch_and_tree('a')
-        self.build_tree(['a/foo', 'a/bar/', 'a/bar/b1', 'a/bar/b2', 'a/baz'])
-        self.tree_a.add(['foo', 'bar', 'bar/b1', 'bar/b2', 'baz'])
-        self.bar_id = self.tree_a.path2id('bar')
+        self.build_tree(
+            ['a/foo', 'a/bar/', 'a/bar/b1', 'a/bar/b2', 'a/baz', 'a/oldname'])
+        self.tree_a.add(['foo', 'bar', 'bar/b1', 'bar/b2', 'baz', 'oldname'])
         self.rev1 = self.tree_a.commit('rev1')
         self.build_tree(['a/bar/b3'])
         self.tree_a.add('bar/b3')
+        self.tree_a.rename_one('oldname', 'newname')
         self.rev2 = self.tree_a.commit('rev2')
         self.repository = self.tree_a.branch.repository
         self.addCleanup(self.repository.lock_read().unlock)
 
     def test_multiple_files(self):
         # Test multiple files
-        delta = list(self.repository.get_deltas_for_revisions(
-            [self.repository.get_revision(self.rev1)], specific_fileids=[
-                self.tree_a.path2id('foo'),
-                self.tree_a.path2id('baz')]))[0]
+        delta = list(self.repository.get_revision_deltas(
+            [self.repository.get_revision(self.rev1)], specific_files=[
+                'foo', 'baz']))[0]
         self.assertIsInstance(delta, _mod_delta.TreeDelta)
         self.assertEqual([
             ('baz', 'file'),
@@ -1022,9 +1022,9 @@
 
     def test_directory(self):
         # Test a directory
-        delta = list(self.repository.get_deltas_for_revisions(
+        delta = list(self.repository.get_revision_deltas(
             [self.repository.get_revision(self.rev1)],
-            specific_fileids=[self.bar_id]))[0]
+            specific_files=['bar']))[0]
         self.assertIsInstance(delta, _mod_delta.TreeDelta)
         self.assertEqual([
             ('bar', 'directory'),
@@ -1034,17 +1034,33 @@
 
     def test_unrelated(self):
         # Try another revision
-        delta = list(self.repository.get_deltas_for_revisions(
+        delta = list(self.repository.get_revision_deltas(
             [self.repository.get_revision(self.rev2)],
-            specific_fileids=[self.tree_a.path2id('foo')]))[0]
+            specific_files=['foo']))[0]
         self.assertIsInstance(delta, _mod_delta.TreeDelta)
         self.assertEqual([], delta.added)
 
+    def test_renamed(self):
+        # Try another revision
+        self.assertTrue(
+            self.repository.revision_tree(self.rev2).has_filename('newname'))
+        self.assertTrue(
+            self.repository.revision_tree(self.rev1).has_filename('oldname'))
+        revs = [
+            self.repository.get_revision(self.rev2),
+            self.repository.get_revision(self.rev1)]
+        delta2, delta1 = list(self.repository.get_revision_deltas(
+            revs, specific_files=['newname']))
+        self.assertIsInstance(delta1, _mod_delta.TreeDelta)
+        self.assertEqual([('oldname', 'newname')], [c.path for c in delta2.renamed])
+        self.assertIsInstance(delta2, _mod_delta.TreeDelta)
+        self.assertEqual(['oldname'], [c.path[1] for c in delta1.added])
+
     def test_file_in_directory(self):
         # Test a file in a directory, both of which were added
-        delta = list(self.repository.get_deltas_for_revisions(
+        delta = list(self.repository.get_revision_deltas(
             [self.repository.get_revision(self.rev1)],
-            specific_fileids=[self.tree_a.path2id('bar/b2')]))[0]
+            specific_files=['bar/b2']))[0]
         self.assertIsInstance(delta, _mod_delta.TreeDelta)
         self.assertEqual([
             ('bar', 'directory'),
@@ -1052,9 +1068,9 @@
             ], [(c.path[1], c.kind[1]) for c in delta.added])
 
     def test_file_in_unchanged_directory(self):
-        delta = list(self.repository.get_deltas_for_revisions(
+        delta = list(self.repository.get_revision_deltas(
             [self.repository.get_revision(self.rev2)],
-            specific_fileids=[self.tree_a.path2id('bar/b3')]))[0]
+            specific_files=['bar/b3']))[0]
         self.assertIsInstance(delta, _mod_delta.TreeDelta)
         if [(c.path[1], c.kind[1]) for c in delta.added] == [
                 ('bar', 'directory'), ('bar/b3', 'file')]:

=== modified file 'breezy/tests/per_tree/__init__.py'
--- breezy/tests/per_tree/__init__.py	2019-06-17 23:01:58 +0000
+++ breezy/tests/per_tree/__init__.py	2020-07-05 13:18:03 +0000
@@ -39,7 +39,6 @@
     make_scenario as wt_make_scenario,
     )
 from breezy.revisiontree import RevisionTree
-from breezy.transform import TransformPreview
 from breezy.tests import (
     features,
     )
@@ -70,7 +69,7 @@
 
 
 def preview_tree_pre(testcase, tree):
-    tt = TransformPreview(tree)
+    tt = tree.preview_transform()
     testcase.addCleanup(tt.finalize)
     preview_tree = tt.get_preview_tree()
     preview_tree.set_parent_ids(tree.get_parent_ids())
@@ -79,7 +78,7 @@
 
 def preview_tree_post(testcase, tree):
     basis = tree.basis_tree()
-    tt = TransformPreview(basis)
+    tt = basis.preview_transform()
     testcase.addCleanup(tt.finalize)
     tree.lock_read()
     testcase.addCleanup(tree.unlock)
@@ -174,7 +173,7 @@
         This variation changes the executable flag of b/c to True.
         """
         self._make_abc_tree(tree)
-        tt = tree.get_transform()
+        tt = tree.transform()
         trans_id = tt.trans_id_tree_path('b/c')
         tt.set_executability(True, trans_id)
         tt.apply()
@@ -206,7 +205,7 @@
         This variation renames b/c to e, and makes it executable.
         """
         self._make_abc_tree(tree)
-        tt = tree.get_transform()
+        tt = tree.transform()
         trans_id = tt.trans_id_tree_path('b/c')
         parent_trans_id = tt.trans_id_tree_path('')
         tt.adjust_path('e', parent_trans_id, trans_id)
@@ -222,7 +221,7 @@
         self._make_abc_tree(tree)
         self.build_tree(['d/'], transport=tree.controldir.root_transport)
         tree.add(['d'])
-        tt = tree.get_transform()
+        tt = tree.transform()
         trans_id = tt.trans_id_tree_path('b')
         parent_trans_id = tt.trans_id_tree_path('d')
         tt.adjust_path('e', parent_trans_id, trans_id)
@@ -270,7 +269,7 @@
                  ]
         self.build_tree(paths)
         tree.add(paths)
-        tt = tree.get_transform()
+        tt = tree.transform()
         if symlinks:
             root_transaction_id = tt.trans_id_tree_path('')
             tt.new_symlink('symlink',
@@ -355,6 +354,7 @@
         'revision_tree',
         'symlinks',
         'test_trees',
+        'transform',
         'tree',
         'walkdirs',
         ]

=== modified file 'breezy/tests/per_tree/test_path_content_summary.py'
--- breezy/tests/per_tree/test_path_content_summary.py	2019-06-18 00:20:56 +0000
+++ breezy/tests/per_tree/test_path_content_summary.py	2020-07-07 20:32:36 +0000
@@ -21,9 +21,11 @@
 from breezy import (
     osutils,
     tests,
-    transform,
     )
 
+from breezy.transform import (
+    PreviewTree,
+    )
 from breezy.tests import (
     features,
     per_tree,
@@ -88,7 +90,7 @@
         tree = self.make_branch_and_tree('tree')
         self.build_tree(['tree/path'])
         tree.add(['path'])
-        with tree.get_transform() as tt:
+        with tree.transform() as tt:
             tt.set_executability(True, tt.trans_id_tree_path('path'))
             tt.apply()
         summary = self._convert_tree(tree).path_content_summary('path')
@@ -112,7 +114,7 @@
             self.assertEqual('missing', summary[0])
             self.assertIs(None, summary[2])
             self.assertIs(None, summary[3])
-        elif isinstance(tree, transform._PreviewTree):
+        elif isinstance(tree, PreviewTree):
             self.expectFailure('PreviewTree returns "missing" for unversioned'
                                'files', self.assertEqual, 'file', summary[0])
             self.assertEqual('file', summary[0])

=== modified file 'breezy/tests/per_tree/test_symlinks.py'
--- breezy/tests/per_tree/test_symlinks.py	2019-06-29 21:31:14 +0000
+++ breezy/tests/per_tree/test_symlinks.py	2020-07-07 20:32:36 +0000
@@ -27,7 +27,7 @@
     )
 from breezy.mutabletree import MutableTree
 from breezy.tests import TestSkipped
-from breezy.transform import _PreviewTree
+from breezy.transform import PreviewTree
 from breezy.tests import (
     features,
     )
@@ -55,7 +55,7 @@
         self.addCleanup(self.tree.unlock)
 
     def test_symlink_target(self):
-        if isinstance(self.tree, (MutableTree, _PreviewTree)):
+        if isinstance(self.tree, (MutableTree, PreviewTree)):
             raise TestSkipped(
                 'symlinks not accurately represented in working trees and'
                 ' preview trees')

=== added file 'breezy/tests/per_tree/test_transform.py'
--- breezy/tests/per_tree/test_transform.py	1970-01-01 00:00:00 +0000
+++ breezy/tests/per_tree/test_transform.py	2020-07-06 01:43:04 +0000
@@ -0,0 +1,746 @@
+# Copyright (C) 2006-2012, 2016 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+from io import BytesIO
+import os
+
+from ... import (
+    revision as _mod_revision,
+    tests,
+    trace,
+    )
+from ...diff import show_diff_trees
+from ...merge import Merger, Merge3Merger
+from ...transform import (
+    ROOT_PARENT,
+    resolve_conflicts,
+    )
+from ...tree import (
+    find_previous_path,
+    )
+
+
+from breezy.tests.per_tree import TestCaseWithTree
+
+
+from ..features import (
+    HardlinkFeature,
+    SymlinkFeature,
+    UnicodeFilenameFeature,
+    )
+
+
+
+class TestTransformPreview(TestCaseWithTree):
+
+    def setUp(self):
+        super(TestTransformPreview, self).setUp()
+        if not self.workingtree_format.supports_setting_file_ids:
+            self.skipTest('test not compatible with non-file-id trees yet')
+
+    def create_tree(self):
+        tree = self.make_branch_and_tree('.')
+        self.build_tree_contents([('a', b'content 1')])
+        tree.add('a')
+        revid1 = tree.commit('rev1')
+        return tree.branch.repository.revision_tree(revid1)
+
+    def get_empty_preview(self):
+        repository = self.make_repository('repo')
+        tree = repository.revision_tree(_mod_revision.NULL_REVISION)
+        preview = tree.preview_transform()
+        self.addCleanup(preview.finalize)
+        return preview
+
+    def test_transform_preview(self):
+        revision_tree = self.create_tree()
+        preview = revision_tree.preview_transform()
+        self.addCleanup(preview.finalize)
+
+    def test_transform_preview_tree(self):
+        revision_tree = self.create_tree()
+        preview = revision_tree.preview_transform()
+        self.addCleanup(preview.finalize)
+        preview.get_preview_tree()
+
+    def test_transform_new_file(self):
+        revision_tree = self.create_tree()
+        preview = revision_tree.preview_transform()
+        self.addCleanup(preview.finalize)
+        preview.new_file('file2', preview.root, [b'content B\n'], b'file2-id')
+        preview_tree = preview.get_preview_tree()
+        self.assertEqual(preview_tree.kind('file2'), 'file')
+        with preview_tree.get_file('file2') as f:
+            self.assertEqual(f.read(), b'content B\n')
+
+    def test_diff_preview_tree(self):
+        revision_tree = self.create_tree()
+        preview = revision_tree.preview_transform()
+        self.addCleanup(preview.finalize)
+        preview.new_file('file2', preview.root, [b'content B\n'], b'file2-id')
+        preview_tree = preview.get_preview_tree()
+        out = BytesIO()
+        show_diff_trees(revision_tree, preview_tree, out)
+        lines = out.getvalue().splitlines()
+        self.assertEqual(lines[0], b"=== added file 'file2'")
+        # 3 lines of diff administrivia
+        self.assertEqual(lines[4], b"+content B")
+
+    def test_unsupported_symlink_diff(self):
+        self.requireFeature(SymlinkFeature)
+        tree = self.make_branch_and_tree('.')
+        self.build_tree_contents([('a', 'content 1')])
+        tree.add('a')
+        os.symlink('a', 'foo')
+        tree.add('foo')
+        revid1 = tree.commit('rev1')
+        revision_tree = tree.branch.repository.revision_tree(revid1)
+        preview = revision_tree.preview_transform()
+        self.addCleanup(preview.finalize)
+        preview.delete_versioned(preview.trans_id_tree_path('foo'))
+        preview_tree = preview.get_preview_tree()
+        out = BytesIO()
+        log = BytesIO()
+        trace.push_log_file(log)
+        os_symlink = getattr(os, 'symlink', None)
+        os.symlink = None
+        try:
+            show_diff_trees(revision_tree, preview_tree, out)
+            lines = out.getvalue().splitlines()
+        finally:
+            os.symlink = os_symlink
+        self.assertContainsRe(
+            log.getvalue(),
+            b'Ignoring "foo" as symlinks are not supported on this filesystem')
+
+    def test_transform_conflicts(self):
+        revision_tree = self.create_tree()
+        preview = revision_tree.preview_transform()
+        self.addCleanup(preview.finalize)
+        preview.new_file('a', preview.root, [b'content 2'])
+        resolve_conflicts(preview)
+        trans_id = preview.trans_id_tree_path('a')
+        self.assertEqual('a.moved', preview.final_name(trans_id))
+
+    def get_tree_and_preview_tree(self):
+        revision_tree = self.create_tree()
+        preview = revision_tree.preview_transform()
+        self.addCleanup(preview.finalize)
+        a_trans_id = preview.trans_id_tree_path('a')
+        preview.delete_contents(a_trans_id)
+        preview.create_file([b'b content'], a_trans_id)
+        preview_tree = preview.get_preview_tree()
+        return revision_tree, preview_tree
+
+    def test_iter_changes(self):
+        revision_tree, preview_tree = self.get_tree_and_preview_tree()
+        root = revision_tree.path2id('')
+        self.assertEqual([(revision_tree.path2id('a'), ('a', 'a'), True, (True, True),
+                           (root, root), ('a', 'a'), ('file', 'file'),
+                           (False, False), False)],
+                         list(preview_tree.iter_changes(revision_tree)))
+
+    def test_include_unchanged_succeeds(self):
+        revision_tree, preview_tree = self.get_tree_and_preview_tree()
+        changes = preview_tree.iter_changes(revision_tree,
+                                            include_unchanged=True)
+
+        root_id = revision_tree.path2id('')
+        root_entry = (root_id, ('', ''), False, (True, True), (None, None),
+                      ('', ''), ('directory', 'directory'), (False, False), False)
+        a_entry = (revision_tree.path2id('a'), ('a', 'a'), True, (True, True),
+                   (root_id, root_id), ('a', 'a'), ('file', 'file'),
+                   (False, False), False)
+
+ 
+        self.assertEqual([root_entry, a_entry], list(changes))
+
+    def test_specific_files(self):
+        revision_tree, preview_tree = self.get_tree_and_preview_tree()
+        changes = preview_tree.iter_changes(revision_tree,
+                                            specific_files=[''])
+        root_id = revision_tree.path2id('')
+        a_entry = (revision_tree.path2id('a'), ('a', 'a'), True, (True, True),
+                   (root_id, root_id), ('a', 'a'), ('file', 'file'),
+                   (False, False), False)
+
+ 
+        self.assertEqual([a_entry], list(changes))
+
+    def test_want_unversioned(self):
+        revision_tree, preview_tree = self.get_tree_and_preview_tree()
+        changes = preview_tree.iter_changes(revision_tree,
+                                            want_unversioned=True)
+        root_id = revision_tree.path2id('')
+        a_entry = (revision_tree.path2id('a'), ('a', 'a'), True, (True, True),
+                   (root_id, root_id), ('a', 'a'), ('file', 'file'),
+                   (False, False), False)
+
+        self.assertEqual([a_entry], list(changes))
+
+    def test_ignore_extra_trees_no_specific_files(self):
+        # extra_trees is harmless without specific_files, so we'll silently
+        # accept it, even though we won't use it.
+        revision_tree, preview_tree = self.get_tree_and_preview_tree()
+        preview_tree.iter_changes(revision_tree, extra_trees=[preview_tree])
+
+    def test_ignore_require_versioned_no_specific_files(self):
+        # require_versioned is meaningless without specific_files.
+        revision_tree, preview_tree = self.get_tree_and_preview_tree()
+        preview_tree.iter_changes(revision_tree, require_versioned=False)
+
+    def test_ignore_pb(self):
+        # pb could be supported, but TT.iter_changes doesn't support it.
+        revision_tree, preview_tree = self.get_tree_and_preview_tree()
+        preview_tree.iter_changes(revision_tree)
+
+    def test_kind(self):
+        revision_tree = self.create_tree()
+        preview = revision_tree.preview_transform()
+        self.addCleanup(preview.finalize)
+        preview.new_file('file', preview.root, [b'contents'], b'file-id')
+        preview.new_directory('directory', preview.root, b'dir-id')
+        preview_tree = preview.get_preview_tree()
+        self.assertEqual('file', preview_tree.kind('file'))
+        self.assertEqual('directory', preview_tree.kind('directory'))
+
+    def test_get_file_mtime(self):
+        preview = self.get_empty_preview()
+        file_trans_id = preview.new_file('file', preview.root, [b'contents'],
+                                         b'file-id')
+        limbo_path = preview._limbo_name(file_trans_id)
+        preview_tree = preview.get_preview_tree()
+        self.assertEqual(os.stat(limbo_path).st_mtime,
+                         preview_tree.get_file_mtime('file'))
+
+    def test_get_file_mtime_renamed(self):
+        work_tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/file'])
+        work_tree.add('file')
+        preview = work_tree.preview_transform()
+        self.addCleanup(preview.finalize)
+        file_trans_id = preview.trans_id_tree_path('file')
+        preview.adjust_path('renamed', preview.root, file_trans_id)
+        preview_tree = preview.get_preview_tree()
+        preview_mtime = preview_tree.get_file_mtime('renamed')
+        work_mtime = work_tree.get_file_mtime('file')
+
+    def test_get_file_size(self):
+        work_tree = self.make_branch_and_tree('tree')
+        self.build_tree_contents([('tree/old', b'old')])
+        work_tree.add('old')
+        preview = work_tree.preview_transform()
+        self.addCleanup(preview.finalize)
+        preview.new_file('name', preview.root, [b'contents'], b'new-id',
+                         'executable')
+        tree = preview.get_preview_tree()
+        self.assertEqual(len('old'), tree.get_file_size('old'))
+        self.assertEqual(len('contents'), tree.get_file_size('name'))
+
+    def test_get_file(self):
+        preview = self.get_empty_preview()
+        preview.new_file('file', preview.root, [b'contents'], b'file-id')
+        preview_tree = preview.get_preview_tree()
+        with preview_tree.get_file('file') as tree_file:
+            self.assertEqual(b'contents', tree_file.read())
+
+    def test_get_symlink_target(self):
+        self.requireFeature(SymlinkFeature)
+        preview = self.get_empty_preview()
+        preview.new_symlink('symlink', preview.root, 'target', b'symlink-id')
+        preview_tree = preview.get_preview_tree()
+        self.assertEqual('target',
+                         preview_tree.get_symlink_target('symlink'))
+
+    def test_all_file_ids(self):
+        if not self.workingtree_format.supports_setting_file_ids:
+            raise tests.TestNotApplicable(
+                'format does not support setting file ids')
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/a', 'tree/b', 'tree/c'])
+        tree.add(['a', 'b', 'c'], [b'a-id', b'b-id', b'c-id'])
+        preview = tree.preview_transform()
+        self.addCleanup(preview.finalize)
+        preview.unversion_file(preview.trans_id_file_id(b'b-id'))
+        c_trans_id = preview.trans_id_file_id(b'c-id')
+        preview.unversion_file(c_trans_id)
+        preview.version_file(c_trans_id, file_id=b'c-id')
+        preview_tree = preview.get_preview_tree()
+        self.assertEqual({b'a-id', b'c-id', tree.path2id('')},
+                         preview_tree.all_file_ids())
+
+    def test_path2id_deleted_unchanged(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/unchanged', 'tree/deleted'])
+        tree.add(['unchanged', 'deleted'])
+        preview = tree.preview_transform()
+        self.addCleanup(preview.finalize)
+        preview.unversion_file(preview.trans_id_tree_path('deleted'))
+        preview_tree = preview.get_preview_tree()
+        self.assertEqual(
+            'unchanged',
+            find_previous_path(preview_tree, tree, 'unchanged'))
+        self.assertFalse(preview_tree.is_versioned('deleted'))
+
+    def test_path2id_created(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/unchanged'])
+        tree.add(['unchanged'])
+        preview = tree.preview_transform()
+        self.addCleanup(preview.finalize)
+        preview.new_file('new', preview.trans_id_tree_path('unchanged'),
+                         [b'contents'], b'new-id')
+        preview_tree = preview.get_preview_tree()
+        self.assertEqual(b'new-id', preview_tree.path2id('unchanged/new'))
+
+    def test_path2id_moved(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/old_parent/', 'tree/old_parent/child'])
+        tree.add(['old_parent', 'old_parent/child'])
+        preview = tree.preview_transform()
+        self.addCleanup(preview.finalize)
+        new_parent = preview.new_directory('new_parent', preview.root,
+                                           b'new_parent-id')
+        preview.adjust_path('child', new_parent,
+                            preview.trans_id_tree_path('old_parent/child'))
+        preview_tree = preview.get_preview_tree()
+        self.assertFalse(preview_tree.is_versioned('old_parent/child'))
+        self.assertEqual(
+            'new_parent/child',
+            find_previous_path(tree, preview_tree, 'old_parent/child'))
+        if self.workingtree_format.supports_setting_file_ids:
+            self.assertEqual(
+                tree.path2id('old_parent/child'),
+                preview_tree.path2id('new_parent/child'))
+
+    def test_path2id_renamed_parent(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/old_name/', 'tree/old_name/child'])
+        tree.add(['old_name', 'old_name/child'])
+        preview = tree.preview_transform()
+        self.addCleanup(preview.finalize)
+        preview.adjust_path('new_name', preview.root,
+                            preview.trans_id_tree_path('old_name'))
+        preview_tree = preview.get_preview_tree()
+        self.assertFalse(preview_tree.is_versioned('old_name/child'))
+        self.assertEqual(
+            'new_name/child',
+            find_previous_path(tree, preview_tree, 'old_name/child'))
+        if tree.supports_setting_file_ids:
+            self.assertEqual(
+                tree.path2id('old_name/child'),
+                preview_tree.path2id('new_name/child'))
+
+    def assertMatchingIterEntries(self, tt, specific_files=None):
+        preview_tree = tt.get_preview_tree()
+        preview_result = list(preview_tree.iter_entries_by_dir(
+                              specific_files=specific_files))
+        tree = tt._tree
+        tt.apply()
+        actual_result = list(tree.iter_entries_by_dir(
+            specific_files=specific_files))
+        self.assertEqual(actual_result, preview_result)
+
+    def test_iter_entries_by_dir_new(self):
+        tree = self.make_branch_and_tree('tree')
+        tt = tree.transform()
+        tt.new_file('new', tt.root, [b'contents'], b'new-id')
+        self.assertMatchingIterEntries(tt)
+
+    def test_iter_entries_by_dir_deleted(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/deleted'])
+        tree.add('deleted')
+        tt = tree.transform()
+        tt.delete_contents(tt.trans_id_tree_path('deleted'))
+        self.assertMatchingIterEntries(tt)
+
+    def test_iter_entries_by_dir_unversioned(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/removed'])
+        tree.add('removed')
+        tt = tree.transform()
+        tt.unversion_file(tt.trans_id_tree_path('removed'))
+        self.assertMatchingIterEntries(tt)
+
+    def test_iter_entries_by_dir_moved(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/moved', 'tree/new_parent/'])
+        tree.add(['moved', 'new_parent'])
+        tt = tree.transform()
+        tt.adjust_path(
+            'moved', tt.trans_id_tree_path('new_parent'),
+            tt.trans_id_tree_path('moved'))
+        self.assertMatchingIterEntries(tt)
+
+    def test_iter_entries_by_dir_specific_files(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/parent/', 'tree/parent/child'])
+        tree.add(['parent', 'parent/child'])
+        tt = tree.transform()
+        self.assertMatchingIterEntries(tt, ['', 'parent/child'])
+
+    def test_symlink_content_summary(self):
+        self.requireFeature(SymlinkFeature)
+        preview = self.get_empty_preview()
+        preview.new_symlink('path', preview.root, 'target', b'path-id')
+        summary = preview.get_preview_tree().path_content_summary('path')
+        self.assertEqual(('symlink', None, None, 'target'), summary)
+
+    def test_missing_content_summary(self):
+        preview = self.get_empty_preview()
+        summary = preview.get_preview_tree().path_content_summary('path')
+        self.assertEqual(('missing', None, None, None), summary)
+
+    def test_deleted_content_summary(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/path/'])
+        tree.add('path')
+        preview = tree.preview_transform()
+        self.addCleanup(preview.finalize)
+        preview.delete_contents(preview.trans_id_tree_path('path'))
+        summary = preview.get_preview_tree().path_content_summary('path')
+        self.assertEqual(('missing', None, None, None), summary)
+
+    def test_file_content_summary_executable(self):
+        preview = self.get_empty_preview()
+        path_id = preview.new_file('path', preview.root, [
+                                   b'contents'], b'path-id')
+        preview.set_executability(True, path_id)
+        summary = preview.get_preview_tree().path_content_summary('path')
+        self.assertEqual(4, len(summary))
+        self.assertEqual('file', summary[0])
+        # size must be known
+        self.assertEqual(len('contents'), summary[1])
+        # executable
+        self.assertEqual(True, summary[2])
+        # will not have hash (not cheap to determine)
+        self.assertIs(None, summary[3])
+
+    def test_change_executability(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/path'])
+        tree.add('path')
+        preview = tree.preview_transform()
+        self.addCleanup(preview.finalize)
+        path_id = preview.trans_id_tree_path('path')
+        preview.set_executability(True, path_id)
+        summary = preview.get_preview_tree().path_content_summary('path')
+        self.assertEqual(True, summary[2])
+
+    def test_file_content_summary_non_exec(self):
+        preview = self.get_empty_preview()
+        preview.new_file('path', preview.root, [b'contents'], b'path-id')
+        summary = preview.get_preview_tree().path_content_summary('path')
+        self.assertEqual(4, len(summary))
+        self.assertEqual('file', summary[0])
+        # size must be known
+        self.assertEqual(len('contents'), summary[1])
+        # not executable
+        self.assertEqual(False, summary[2])
+        # will not have hash (not cheap to determine)
+        self.assertIs(None, summary[3])
+
+    def test_dir_content_summary(self):
+        preview = self.get_empty_preview()
+        preview.new_directory('path', preview.root, b'path-id')
+        summary = preview.get_preview_tree().path_content_summary('path')
+        self.assertEqual(('directory', None, None, None), summary)
+
+    def test_tree_content_summary(self):
+        preview = self.get_empty_preview()
+        path = preview.new_directory('path', preview.root, b'path-id')
+        preview.set_tree_reference(b'rev-1', path)
+        summary = preview.get_preview_tree().path_content_summary('path')
+        self.assertEqual(4, len(summary))
+        self.assertEqual('tree-reference', summary[0])
+
+    def test_annotate(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree_contents([('tree/file', b'a\n')])
+        tree.add('file')
+        revid1 = tree.commit('a')
+        self.build_tree_contents([('tree/file', b'a\nb\n')])
+        preview = tree.preview_transform()
+        self.addCleanup(preview.finalize)
+        file_trans_id = preview.trans_id_tree_path('file')
+        preview.delete_contents(file_trans_id)
+        preview.create_file([b'a\nb\nc\n'], file_trans_id)
+        preview_tree = preview.get_preview_tree()
+        expected = [
+            (revid1, b'a\n'),
+            (b'me:', b'b\n'),
+            (b'me:', b'c\n'),
+        ]
+        annotation = preview_tree.annotate_iter(
+            'file', default_revision=b'me:')
+        self.assertEqual(expected, annotation)
+
+    def test_annotate_missing(self):
+        preview = self.get_empty_preview()
+        preview.new_file('file', preview.root, [b'a\nb\nc\n'], b'file-id')
+        preview_tree = preview.get_preview_tree()
+        expected = [
+            (b'me:', b'a\n'),
+            (b'me:', b'b\n'),
+            (b'me:', b'c\n'),
+            ]
+        annotation = preview_tree.annotate_iter(
+            'file', default_revision=b'me:')
+        self.assertEqual(expected, annotation)
+
+    def test_annotate_rename(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree_contents([('tree/file', b'a\n')])
+        tree.add('file')
+        revid1 = tree.commit('a')
+        preview = tree.preview_transform()
+        self.addCleanup(preview.finalize)
+        file_trans_id = preview.trans_id_tree_path('file')
+        preview.adjust_path('newname', preview.root, file_trans_id)
+        preview_tree = preview.get_preview_tree()
+        expected = [
+            (revid1, b'a\n'),
+        ]
+        annotation = preview_tree.annotate_iter(
+            'file', default_revision=b'me:')
+        self.assertEqual(expected, annotation)
+
+    def test_annotate_deleted(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree_contents([('tree/file', b'a\n')])
+        tree.add('file')
+        tree.commit('a')
+        self.build_tree_contents([('tree/file', b'a\nb\n')])
+        preview = tree.preview_transform()
+        self.addCleanup(preview.finalize)
+        file_trans_id = preview.trans_id_tree_path('file')
+        preview.delete_contents(file_trans_id)
+        preview_tree = preview.get_preview_tree()
+        annotation = preview_tree.annotate_iter(
+            'file', default_revision=b'me:')
+        self.assertIs(None, annotation)
+
+    def test_stored_kind(self):
+        preview = self.get_empty_preview()
+        preview.new_file('file', preview.root, [b'a\nb\nc\n'], b'file-id')
+        preview_tree = preview.get_preview_tree()
+        self.assertEqual('file', preview_tree.stored_kind('file'))
+
+    def test_is_executable(self):
+        preview = self.get_empty_preview()
+        preview.new_file('file', preview.root, [b'a\nb\nc\n'], b'file-id')
+        preview.set_executability(True, preview.trans_id_file_id(b'file-id'))
+        preview_tree = preview.get_preview_tree()
+        self.assertEqual(True, preview_tree.is_executable('file'))
+
+    def test_get_set_parent_ids(self):
+        revision_tree, preview_tree = self.get_tree_and_preview_tree()
+        self.assertEqual([], preview_tree.get_parent_ids())
+        preview_tree.set_parent_ids([revision_tree.get_revision_id()])
+        self.assertEqual(
+            [revision_tree.get_revision_id()],
+            preview_tree.get_parent_ids())
+
+    def test_plan_file_merge(self):
+        work_a = self.make_branch_and_tree('wta')
+        self.build_tree_contents([('wta/file', b'a\nb\nc\nd\n')])
+        work_a.add('file')
+        base_id = work_a.commit('base version')
+        tree_b = work_a.controldir.sprout('wtb').open_workingtree()
+        preview = work_a.preview_transform()
+        self.addCleanup(preview.finalize)
+        trans_id = preview.trans_id_tree_path('file')
+        preview.delete_contents(trans_id)
+        preview.create_file([b'b\nc\nd\ne\n'], trans_id)
+        self.build_tree_contents([('wtb/file', b'a\nc\nd\nf\n')])
+        tree_a = preview.get_preview_tree()
+        tree_a.set_parent_ids([base_id])
+        self.addCleanup(tree_b.lock_read().unlock)
+        self.assertEqual([
+            ('killed-a', b'a\n'),
+            ('killed-b', b'b\n'),
+            ('unchanged', b'c\n'),
+            ('unchanged', b'd\n'),
+            ('new-a', b'e\n'),
+            ('new-b', b'f\n'),
+        ], list(tree_a.plan_file_merge('file', tree_b)))
+
+    def test_plan_file_merge_revision_tree(self):
+        work_a = self.make_branch_and_tree('wta')
+        self.build_tree_contents([('wta/file', b'a\nb\nc\nd\n')])
+        work_a.add('file')
+        base_id = work_a.commit('base version')
+        tree_b = work_a.controldir.sprout('wtb').open_workingtree()
+        preview = work_a.basis_tree().preview_transform()
+        self.addCleanup(preview.finalize)
+        trans_id = preview.trans_id_tree_path('file')
+        preview.delete_contents(trans_id)
+        preview.create_file([b'b\nc\nd\ne\n'], trans_id)
+        self.build_tree_contents([('wtb/file', b'a\nc\nd\nf\n')])
+        tree_a = preview.get_preview_tree()
+        tree_a.set_parent_ids([base_id])
+        self.addCleanup(tree_b.lock_read().unlock)
+        self.assertEqual([
+            ('killed-a', b'a\n'),
+            ('killed-b', b'b\n'),
+            ('unchanged', b'c\n'),
+            ('unchanged', b'd\n'),
+            ('new-a', b'e\n'),
+            ('new-b', b'f\n'),
+        ], list(tree_a.plan_file_merge('file', tree_b)))
+
+    def test_walkdirs(self):
+        preview = self.get_empty_preview()
+        preview.new_directory('', ROOT_PARENT, b'tree-root')
+        # FIXME: new_directory should mark root.
+        preview.fixup_new_roots()
+        preview_tree = preview.get_preview_tree()
+        preview.new_file('a', preview.root, [b'contents'], b'a-id')
+        expected = [(('', b'tree-root'),
+                     [('a', 'a', 'file', None, b'a-id', 'file')])]
+        self.assertEqual(expected, list(preview_tree.walkdirs()))
+
+    def test_extras(self):
+        work_tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/removed-file', 'tree/existing-file',
+                         'tree/not-removed-file'])
+        work_tree.add(['removed-file', 'not-removed-file'])
+        preview = work_tree.preview_transform()
+        self.addCleanup(preview.finalize)
+        preview.new_file('new-file', preview.root, [b'contents'])
+        preview.new_file('new-versioned-file', preview.root, [b'contents'],
+                         b'new-versioned-id')
+        tree = preview.get_preview_tree()
+        preview.unversion_file(preview.trans_id_tree_path('removed-file'))
+        self.assertEqual({'new-file', 'removed-file', 'existing-file'},
+                         set(tree.extras()))
+
+    def test_merge_into_preview(self):
+        work_tree = self.make_branch_and_tree('tree')
+        self.build_tree_contents([('tree/file', b'b\n')])
+        work_tree.add('file')
+        work_tree.commit('first commit')
+        child_tree = work_tree.controldir.sprout('child').open_workingtree()
+        self.build_tree_contents([('child/file', b'b\nc\n')])
+        child_tree.commit('child commit')
+        child_tree.lock_write()
+        self.addCleanup(child_tree.unlock)
+        work_tree.lock_write()
+        self.addCleanup(work_tree.unlock)
+        preview = work_tree.preview_transform()
+        self.addCleanup(preview.finalize)
+        file_trans_id = preview.trans_id_tree_path('file')
+        preview.delete_contents(file_trans_id)
+        preview.create_file([b'a\nb\n'], file_trans_id)
+        preview_tree = preview.get_preview_tree()
+        merger = Merger.from_revision_ids(preview_tree,
+                                          child_tree.branch.last_revision(),
+                                          other_branch=child_tree.branch,
+                                          tree_branch=work_tree.branch)
+        merger.merge_type = Merge3Merger
+        tt = merger.make_merger().make_preview_transform()
+        self.addCleanup(tt.finalize)
+        final_tree = tt.get_preview_tree()
+        self.assertEqual(
+            b'a\nb\nc\n',
+            final_tree.get_file_text('file'))
+
+    def test_merge_preview_into_workingtree(self):
+        tree = self.make_branch_and_tree('tree')
+        if tree.supports_setting_file_ids():
+            tree.set_root_id(b'TREE_ROOT')
+        tt = tree.preview_transform()
+        self.addCleanup(tt.finalize)
+        tt.new_file('name', tt.root, [b'content'], b'file-id')
+        tree2 = self.make_branch_and_tree('tree2')
+        if tree.supports_setting_file_ids():
+            tree2.set_root_id(b'TREE_ROOT')
+        merger = Merger.from_uncommitted(tree2, tt.get_preview_tree(),
+                                         tree.basis_tree())
+        merger.merge_type = Merge3Merger
+        merger.do_merge()
+
+    def test_merge_preview_into_workingtree_handles_conflicts(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree_contents([('tree/foo', b'bar')])
+        tree.add('foo')
+        tree.commit('foo')
+        tt = tree.preview_transform()
+        self.addCleanup(tt.finalize)
+        trans_id = tt.trans_id_tree_path('foo')
+        tt.delete_contents(trans_id)
+        tt.create_file([b'baz'], trans_id)
+        tree2 = tree.controldir.sprout('tree2').open_workingtree()
+        self.build_tree_contents([('tree2/foo', b'qux')])
+        merger = Merger.from_uncommitted(tree2, tt.get_preview_tree(),
+                                         tree.basis_tree())
+        merger.merge_type = Merge3Merger
+        merger.do_merge()
+
+    def test_has_filename(self):
+        wt = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/unmodified', 'tree/removed', 'tree/modified'])
+        tt = wt.preview_transform()
+        removed_id = tt.trans_id_tree_path('removed')
+        tt.delete_contents(removed_id)
+        tt.new_file('new', tt.root, [b'contents'])
+        modified_id = tt.trans_id_tree_path('modified')
+        tt.delete_contents(modified_id)
+        tt.create_file([b'modified-contents'], modified_id)
+        self.addCleanup(tt.finalize)
+        tree = tt.get_preview_tree()
+        self.assertTrue(tree.has_filename('unmodified'))
+        self.assertFalse(tree.has_filename('not-present'))
+        self.assertFalse(tree.has_filename('removed'))
+        self.assertTrue(tree.has_filename('new'))
+        self.assertTrue(tree.has_filename('modified'))
+
+    def test_is_executable2(self):
+        tree = self.make_branch_and_tree('tree')
+        preview = tree.preview_transform()
+        self.addCleanup(preview.finalize)
+        preview.new_file('foo', preview.root, [b'bar'], b'baz-id')
+        preview_tree = preview.get_preview_tree()
+        self.assertEqual(False, preview_tree.is_executable('tree/foo'))
+
+    def test_commit_preview_tree(self):
+        tree = self.make_branch_and_tree('tree')
+        rev_id = tree.commit('rev1')
+        tree.branch.lock_write()
+        self.addCleanup(tree.branch.unlock)
+        tt = tree.preview_transform()
+        tt.new_file('file', tt.root, [b'contents'], b'file_id')
+        self.addCleanup(tt.finalize)
+        preview = tt.get_preview_tree()
+        preview.set_parent_ids([rev_id])
+        builder = tree.branch.get_commit_builder([rev_id])
+        list(builder.record_iter_changes(preview, rev_id, tt.iter_changes()))
+        builder.finish_inventory()
+        rev2_id = builder.commit('rev2')
+        rev2_tree = tree.branch.repository.revision_tree(rev2_id)
+        self.assertEqual(b'contents', rev2_tree.get_file_text('file'))
+
+    def test_ascii_limbo_paths(self):
+        self.requireFeature(UnicodeFilenameFeature)
+        branch = self.make_branch('any')
+        tree = branch.repository.revision_tree(_mod_revision.NULL_REVISION)
+        tt = tree.preview_transform()
+        self.addCleanup(tt.finalize)
+        foo_id = tt.new_directory('', ROOT_PARENT)
+        bar_id = tt.new_file(u'\u1234bar', foo_id, [b'contents'])
+        limbo_path = tt._limbo_name(bar_id)
+        self.assertEqual(limbo_path, limbo_path)

=== modified file 'breezy/tests/per_workingtree/__init__.py'
--- breezy/tests/per_workingtree/__init__.py	2018-11-11 04:08:32 +0000
+++ breezy/tests/per_workingtree/__init__.py	2020-07-04 12:18:47 +0000
@@ -155,6 +155,7 @@
         'shelf_manager',
         'smart_add',
         'symlinks',
+        'transform',
         'uncommit',
         'unversion',
         'views',

=== modified file 'breezy/tests/per_workingtree/test_add.py'
--- breezy/tests/per_workingtree/test_add.py	2019-06-29 13:16:26 +0000
+++ breezy/tests/per_workingtree/test_add.py	2020-06-25 21:17:44 +0000
@@ -50,8 +50,8 @@
             self.skipTest("tree does not support setting file ids")
         self.build_tree(['a', 'b'])
         tree.add(['a'])
-        self.assertRaises(errors.DuplicateFileId,
-                          tree.add, ['b'], [tree.path2id('a')])
+        self.assertRaises(
+            inventory.DuplicateFileId, tree.add, ['b'], [tree.path2id('a')])
         # And the entry should not have been added.
         self.assertTreeLayout(['', 'a'], tree)
 

=== modified file 'breezy/tests/per_workingtree/test_executable.py'
--- breezy/tests/per_workingtree/test_executable.py	2019-09-27 02:48:17 +0000
+++ breezy/tests/per_workingtree/test_executable.py	2020-07-04 12:29:00 +0000
@@ -32,7 +32,7 @@
         self.b_id = b"b-20051208024829-849e76f7968d7a86"
         wt = self.make_branch_and_tree('b1')
         b = wt.branch
-        tt = wt.get_transform()
+        tt = wt.transform()
         tt.new_file('a', tt.root, [b'a test\n'], self.a_id, True)
         tt.new_file('b', tt.root, [b'b test\n'], self.b_id, False)
         tt.apply()

=== modified file 'breezy/tests/per_workingtree/test_nested_specifics.py'
--- breezy/tests/per_workingtree/test_nested_specifics.py	2019-06-17 23:08:34 +0000
+++ breezy/tests/per_workingtree/test_nested_specifics.py	2020-07-04 12:29:00 +0000
@@ -37,7 +37,7 @@
         tree = self.make_branch_and_tree('.')
         if not isinstance(tree, inventorytree.InventoryTree):
             raise TestNotApplicable('not an inventory tree')
-        transform = tree.get_transform()
+        transform = tree.transform()
         trans_id = transform.new_directory('reference', transform.root,
                                            b'subtree-id')
         transform.set_tree_reference(b'subtree-revision', trans_id)

=== modified file 'breezy/tests/per_workingtree/test_read_working_inventory.py'
--- breezy/tests/per_workingtree/test_read_working_inventory.py	2018-11-11 04:08:32 +0000
+++ breezy/tests/per_workingtree/test_read_working_inventory.py	2020-06-25 21:13:23 +0000
@@ -20,7 +20,7 @@
 from breezy.bzr import inventory
 from breezy.tests import TestNotApplicable
 from breezy.tests.per_workingtree import TestCaseWithWorkingTree
-from breezy.bzr.workingtree import InventoryWorkingTree
+from breezy.bzr.workingtree import InventoryWorkingTree, InventoryModified
 
 
 class TestReadWorkingInventory(TestCaseWithWorkingTree):
@@ -49,7 +49,7 @@
             # OR, the call can be ignored and the changes preserved
             try:
                 tree.read_working_inventory()
-            except errors.InventoryModified:
+            except InventoryModified:
                 pass
             else:
                 self.assertEqual(b'new-root', tree.path2id(''))

=== added file 'breezy/tests/per_workingtree/test_transform.py'
--- breezy/tests/per_workingtree/test_transform.py	1970-01-01 00:00:00 +0000
+++ breezy/tests/per_workingtree/test_transform.py	2020-07-07 22:24:22 +0000
@@ -0,0 +1,1697 @@
+# Copyright (C) 2006-2012, 2016 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+import errno
+from io import BytesIO
+import os
+import sys
+import time
+
+from ... import (
+    errors,
+    osutils,
+    tests,
+    trace,
+    transform,
+    urlutils,
+    )
+from ...conflicts import (
+    DeletingParent,
+    DuplicateEntry,
+    DuplicateID,
+    MissingParent,
+    NonDirectoryParent,
+    ParentLoop,
+    UnversionedParent,
+)
+from ...errors import (
+    DuplicateKey,
+    ExistingLimbo,
+    ExistingPendingDeletion,
+    ImmortalPendingDeletion,
+    LockError,
+)
+from ...osutils import (
+    file_kind,
+    pathjoin,
+)
+from ...sixish import (
+    text_type,
+    )
+from .. import (
+    features,
+    TestSkipped,
+    )
+from ..features import (
+    HardlinkFeature,
+    SymlinkFeature,
+    )
+from ...transform import (
+    create_from_tree,
+    cook_conflicts,
+    FinalPaths,
+    resolve_conflicts,
+    resolve_checkout,
+    ROOT_PARENT,
+    ImmortalLimbo,
+    MalformedTransform,
+    NoFinalPath,
+    ReusingTransform,
+    TransformRenameFailed,
+)
+
+from breezy.tests.per_workingtree import TestCaseWithWorkingTree
+
+
+
+class TestTreeTransform(TestCaseWithWorkingTree):
+
+    def setUp(self):
+        super(TestTreeTransform, self).setUp()
+        if not self.workingtree_format.supports_setting_file_ids:
+            self.skipTest('test not compatible with non-file-id trees yet')
+        self.wt = self.make_branch_and_tree('wt')
+
+    def transform(self):
+        transform = self.wt.transform()
+        self.addCleanup(transform.finalize)
+        return transform, transform.root
+
+    def transform_for_sha1_test(self):
+        trans, root = self.transform()
+        self.wt.lock_tree_write()
+        self.addCleanup(self.wt.unlock)
+        contents = [b'just some content\n']
+        sha1 = osutils.sha_strings(contents)
+        # Roll back the clock
+        trans._creation_mtime = time.time() - 20.0
+        return trans, root, contents, sha1
+
+    def test_existing_limbo(self):
+        transform, root = self.transform()
+        limbo_name = transform._limbodir
+        deletion_path = transform._deletiondir
+        os.mkdir(pathjoin(limbo_name, 'hehe'))
+        self.assertRaises(ImmortalLimbo, transform.apply)
+        self.assertRaises(LockError, self.wt.unlock)
+        self.assertRaises(ExistingLimbo, self.transform)
+        self.assertRaises(LockError, self.wt.unlock)
+        os.rmdir(pathjoin(limbo_name, 'hehe'))
+        os.rmdir(limbo_name)
+        os.rmdir(deletion_path)
+        transform, root = self.transform()
+        transform.apply()
+
+    def test_existing_pending_deletion(self):
+        transform, root = self.transform()
+        deletion_path = self._limbodir = urlutils.local_path_from_url(
+            transform._tree._transport.abspath('pending-deletion'))
+        os.mkdir(pathjoin(deletion_path, 'blocking-directory'))
+        self.assertRaises(ImmortalPendingDeletion, transform.apply)
+        self.assertRaises(LockError, self.wt.unlock)
+        self.assertRaises(ExistingPendingDeletion, self.transform)
+
+    def test_build(self):
+        transform, root = self.transform()
+        self.wt.lock_tree_write()
+        self.addCleanup(self.wt.unlock)
+        self.assertIs(transform.get_tree_parent(root), ROOT_PARENT)
+        imaginary_id = transform.trans_id_tree_path('imaginary')
+        imaginary_id2 = transform.trans_id_tree_path('imaginary/')
+        self.assertEqual(imaginary_id, imaginary_id2)
+        self.assertEqual(root, transform.get_tree_parent(imaginary_id))
+        self.assertEqual('directory', transform.final_kind(root))
+        self.assertEqual(self.wt.path2id(''), transform.final_file_id(root))
+        trans_id = transform.create_path('name', root)
+        self.assertIs(transform.final_file_id(trans_id), None)
+        self.assertIs(None, transform.final_kind(trans_id))
+        transform.create_file([b'contents'], trans_id)
+        transform.set_executability(True, trans_id)
+        transform.version_file(trans_id, file_id=b'my_pretties')
+        self.assertRaises(DuplicateKey, transform.version_file,
+                          trans_id, file_id=b'my_pretties')
+        self.assertEqual(transform.final_file_id(trans_id), b'my_pretties')
+        self.assertEqual(transform.final_parent(trans_id), root)
+        self.assertIs(transform.final_parent(root), ROOT_PARENT)
+        self.assertIs(transform.get_tree_parent(root), ROOT_PARENT)
+        oz_id = transform.create_path('oz', root)
+        transform.create_directory(oz_id)
+        transform.version_file(oz_id, file_id=b'ozzie')
+        trans_id2 = transform.create_path('name2', root)
+        transform.create_file([b'contents'], trans_id2)
+        transform.set_executability(False, trans_id2)
+        transform.version_file(trans_id2, file_id=b'my_pretties2')
+        modified_paths = transform.apply().modified_paths
+        with self.wt.get_file('name') as f:
+            self.assertEqual(b'contents', f.read())
+        self.assertEqual(self.wt.path2id('name'), b'my_pretties')
+        self.assertIs(self.wt.is_executable('name'), True)
+        self.assertIs(self.wt.is_executable('name2'), False)
+        self.assertEqual('directory', file_kind(self.wt.abspath('oz')))
+        self.assertEqual(len(modified_paths), 3)
+        tree_mod_paths = [self.wt.abspath(self.wt.id2path(f)) for f in
+                          (b'ozzie', b'my_pretties', b'my_pretties2')]
+        self.assertSubset(tree_mod_paths, modified_paths)
+        # is it safe to finalize repeatedly?
+        transform.finalize()
+        transform.finalize()
+
+    def test_apply_informs_tree_of_observed_sha1(self):
+        trans, root, contents, sha1 = self.transform_for_sha1_test()
+        trans_id = trans.new_file('file1', root, contents, file_id=b'file1-id',
+                                  sha1=sha1)
+        calls = []
+        orig = self.wt._observed_sha1
+
+        def _observed_sha1(*args):
+            calls.append(args)
+            orig(*args)
+        self.wt._observed_sha1 = _observed_sha1
+        trans.apply()
+        self.assertEqual([('file1', trans._observed_sha1s[trans_id])],
+                         calls)
+
+    def test_create_file_caches_sha1(self):
+        trans, root, contents, sha1 = self.transform_for_sha1_test()
+        trans_id = trans.create_path('file1', root)
+        trans.create_file(contents, trans_id, sha1=sha1)
+        st_val = osutils.lstat(trans._limbo_name(trans_id))
+        o_sha1, o_st_val = trans._observed_sha1s[trans_id]
+        self.assertEqual(o_sha1, sha1)
+        self.assertEqualStat(o_st_val, st_val)
+
+    def test__apply_insertions_updates_sha1(self):
+        trans, root, contents, sha1 = self.transform_for_sha1_test()
+        trans_id = trans.create_path('file1', root)
+        trans.create_file(contents, trans_id, sha1=sha1)
+        st_val = osutils.lstat(trans._limbo_name(trans_id))
+        o_sha1, o_st_val = trans._observed_sha1s[trans_id]
+        self.assertEqual(o_sha1, sha1)
+        self.assertEqualStat(o_st_val, st_val)
+        creation_mtime = trans._creation_mtime + 10.0
+        # We fake a time difference from when the file was created until now it
+        # is being renamed by using os.utime. Note that the change we actually
+        # want to see is the real ctime change from 'os.rename()', but as long
+        # as we observe a new stat value, we should be fine.
+        os.utime(trans._limbo_name(trans_id), (creation_mtime, creation_mtime))
+        trans.apply()
+        new_st_val = osutils.lstat(self.wt.abspath('file1'))
+        o_sha1, o_st_val = trans._observed_sha1s[trans_id]
+        self.assertEqual(o_sha1, sha1)
+        self.assertEqualStat(o_st_val, new_st_val)
+        self.assertNotEqual(st_val.st_mtime, new_st_val.st_mtime)
+
+    def test_new_file_caches_sha1(self):
+        trans, root, contents, sha1 = self.transform_for_sha1_test()
+        trans_id = trans.new_file('file1', root, contents, file_id=b'file1-id',
+                                  sha1=sha1)
+        st_val = osutils.lstat(trans._limbo_name(trans_id))
+        o_sha1, o_st_val = trans._observed_sha1s[trans_id]
+        self.assertEqual(o_sha1, sha1)
+        self.assertEqualStat(o_st_val, st_val)
+
+    def test_cancel_creation_removes_observed_sha1(self):
+        trans, root, contents, sha1 = self.transform_for_sha1_test()
+        trans_id = trans.new_file('file1', root, contents, file_id=b'file1-id',
+                                  sha1=sha1)
+        self.assertTrue(trans_id in trans._observed_sha1s)
+        trans.cancel_creation(trans_id)
+        self.assertFalse(trans_id in trans._observed_sha1s)
+
+    def test_create_files_same_timestamp(self):
+        transform, root = self.transform()
+        self.wt.lock_tree_write()
+        self.addCleanup(self.wt.unlock)
+        # Roll back the clock, so that we know everything is being set to the
+        # exact time
+        transform._creation_mtime = creation_mtime = time.time() - 20.0
+        transform.create_file([b'content-one'],
+                              transform.create_path('one', root))
+        time.sleep(1)  # *ugly*
+        transform.create_file([b'content-two'],
+                              transform.create_path('two', root))
+        transform.apply()
+        fo, st1 = self.wt.get_file_with_stat('one', filtered=False)
+        fo.close()
+        fo, st2 = self.wt.get_file_with_stat('two', filtered=False)
+        fo.close()
+        # We only guarantee 2s resolution
+        self.assertTrue(
+            abs(creation_mtime - st1.st_mtime) < 2.0,
+            "%s != %s within 2 seconds" % (creation_mtime, st1.st_mtime))
+        # But if we have more than that, all files should get the same result
+        self.assertEqual(st1.st_mtime, st2.st_mtime)
+
+    def test_change_root_id(self):
+        if not self.workingtree_format.supports_setting_file_ids:
+            raise tests.TestNotApplicable(
+                'format does not support setting file ids')
+        transform, root = self.transform()
+        self.assertNotEqual(b'new-root-id', self.wt.path2id(''))
+        transform.new_directory('', ROOT_PARENT, b'new-root-id')
+        transform.delete_contents(root)
+        transform.unversion_file(root)
+        transform.fixup_new_roots()
+        transform.apply()
+        self.assertEqual(b'new-root-id', self.wt.path2id(''))
+
+    def test_replace_root(self):
+        transform, root = self.transform()
+        transform.new_directory('', ROOT_PARENT, b'new-root-id')
+        transform.delete_contents(root)
+        transform.unversion_file(root)
+        transform.fixup_new_roots()
+        transform.apply()
+
+    def test_change_root_id_add_files(self):
+        if not self.workingtree_format.supports_setting_file_ids:
+            raise tests.TestNotApplicable(
+                'format does not support setting file ids')
+        transform, root = self.transform()
+        self.assertNotEqual(b'new-root-id', self.wt.path2id(''))
+        new_trans_id = transform.new_directory('', ROOT_PARENT, b'new-root-id')
+        transform.new_file('file', new_trans_id, [b'new-contents\n'],
+                           b'new-file-id')
+        transform.delete_contents(root)
+        transform.unversion_file(root)
+        transform.fixup_new_roots()
+        transform.apply()
+        self.assertEqual(b'new-root-id', self.wt.path2id(''))
+        self.assertEqual(b'new-file-id', self.wt.path2id('file'))
+        self.assertFileEqual(b'new-contents\n', self.wt.abspath('file'))
+
+    def test_add_two_roots(self):
+        transform, root = self.transform()
+        transform.new_directory('', ROOT_PARENT, b'new-root-id')
+        transform.new_directory('', ROOT_PARENT, b'alt-root-id')
+        self.assertRaises(ValueError, transform.fixup_new_roots)
+
+    def test_retain_existing_root(self):
+        tt, root = self.transform()
+        with tt:
+            tt.new_directory('', ROOT_PARENT, b'new-root-id')
+            tt.fixup_new_roots()
+            self.assertNotEqual(b'new-root-id', tt.final_file_id(tt.root))
+
+    def test_retain_existing_root_added_file(self):
+        tt, root = self.transform()
+        new_trans_id = tt.new_directory('', ROOT_PARENT, b'new-root-id')
+        child = tt.new_directory('child', new_trans_id, b'child-id')
+        tt.fixup_new_roots()
+        self.assertEqual(tt.root, tt.final_parent(child))
+
+    def test_add_unversioned_root(self):
+        transform, root = self.transform()
+        transform.new_directory('', ROOT_PARENT, None)
+        transform.delete_contents(transform.root)
+        transform.fixup_new_roots()
+        self.assertNotIn(transform.root, transform._new_id)
+
+    def test_remove_root_fixup(self):
+        transform, root = self.transform()
+        old_root_id = self.wt.path2id('')
+        self.assertNotEqual(b'new-root-id', old_root_id)
+        transform.delete_contents(root)
+        transform.unversion_file(root)
+        transform.fixup_new_roots()
+        transform.apply()
+        self.assertEqual(old_root_id, self.wt.path2id(''))
+
+        transform, root = self.transform()
+        transform.new_directory('', ROOT_PARENT, b'new-root-id')
+        transform.new_directory('', ROOT_PARENT, b'alt-root-id')
+        self.assertRaises(ValueError, transform.fixup_new_roots)
+
+    def test_fixup_new_roots_permits_empty_tree(self):
+        transform, root = self.transform()
+        transform.delete_contents(root)
+        transform.unversion_file(root)
+        transform.fixup_new_roots()
+        self.assertIs(None, transform.final_kind(root))
+        self.assertIs(None, transform.final_file_id(root))
+
+    def test_apply_retains_root_directory(self):
+        # Do not attempt to delete the physical root directory, because that
+        # is impossible.
+        transform, root = self.transform()
+        with transform:
+            transform.delete_contents(root)
+            e = self.assertRaises(AssertionError, self.assertRaises,
+                                  TransformRenameFailed,
+                                  transform.apply)
+        self.assertContainsRe('TransformRenameFailed not raised', str(e))
+
+    def test_apply_retains_file_id(self):
+        transform, root = self.transform()
+        old_root_id = transform.tree_file_id(root)
+        transform.unversion_file(root)
+        transform.apply()
+        self.assertEqual(old_root_id, self.wt.path2id(''))
+
+    def test_hardlink(self):
+        self.requireFeature(HardlinkFeature)
+        transform, root = self.transform()
+        transform.new_file('file1', root, [b'contents'])
+        transform.apply()
+        target = self.make_branch_and_tree('target')
+        target_transform = target.transform()
+        trans_id = target_transform.create_path('file1', target_transform.root)
+        target_transform.create_hardlink(self.wt.abspath('file1'), trans_id)
+        target_transform.apply()
+        self.assertPathExists('target/file1')
+        source_stat = os.stat(self.wt.abspath('file1'))
+        target_stat = os.stat('target/file1')
+        self.assertEqual(source_stat, target_stat)
+
+    def test_convenience(self):
+        transform, root = self.transform()
+        self.wt.lock_tree_write()
+        self.addCleanup(self.wt.unlock)
+        transform.new_file('name', root, [b'contents'], b'my_pretties', True)
+        oz = transform.new_directory('oz', root, b'oz-id')
+        dorothy = transform.new_directory('dorothy', oz, b'dorothy-id')
+        transform.new_file('toto', dorothy, [b'toto-contents'], b'toto-id',
+                           False)
+
+        self.assertEqual(len(transform.find_conflicts()), 0)
+        transform.apply()
+        self.assertRaises(ReusingTransform, transform.find_conflicts)
+        with open(self.wt.abspath('name'), 'r') as f:
+            self.assertEqual('contents', f.read())
+        self.assertEqual(self.wt.path2id('name'), b'my_pretties')
+        self.assertIs(self.wt.is_executable('name'), True)
+        self.assertEqual(self.wt.path2id('oz'), b'oz-id')
+        self.assertEqual(self.wt.path2id('oz/dorothy'), b'dorothy-id')
+        self.assertEqual(self.wt.path2id('oz/dorothy/toto'), b'toto-id')
+
+        self.assertEqual(b'toto-contents',
+                         self.wt.get_file('oz/dorothy/toto').read())
+        self.assertIs(self.wt.is_executable('oz/dorothy/toto'), False)
+
+    def test_tree_reference(self):
+        transform, root = self.transform()
+        tree = transform._tree
+        if not tree.supports_tree_reference():
+            raise tests.TestNotApplicable(
+                'Tree format does not support references')
+
+        trans_id = transform.new_directory('reference', root, b'subtree-id')
+        transform.set_tree_reference(b'subtree-revision', trans_id)
+        transform.apply()
+        tree.lock_read()
+        self.addCleanup(tree.unlock)
+        self.assertEqual(
+            b'subtree-revision',
+            tree.root_inventory.get_entry(b'subtree-id').reference_revision)
+
+    def test_conflicts(self):
+        transform, root = self.transform()
+        trans_id = transform.new_file('name', root, [b'contents'],
+                                      b'my_pretties')
+        self.assertEqual(len(transform.find_conflicts()), 0)
+        trans_id2 = transform.new_file('name', root, [b'Crontents'], b'toto')
+        self.assertEqual(transform.find_conflicts(),
+                         [('duplicate', trans_id, trans_id2, 'name')])
+        self.assertRaises(MalformedTransform, transform.apply)
+        transform.adjust_path('name', trans_id, trans_id2)
+        self.assertEqual(transform.find_conflicts(),
+                         [('non-directory parent', trans_id)])
+        tinman_id = transform.trans_id_tree_path('tinman')
+        transform.adjust_path('name', tinman_id, trans_id2)
+        self.assertEqual(transform.find_conflicts(),
+                         [('unversioned parent', tinman_id),
+                          ('missing parent', tinman_id)])
+        lion_id = transform.create_path('lion', root)
+        self.assertEqual(transform.find_conflicts(),
+                         [('unversioned parent', tinman_id),
+                          ('missing parent', tinman_id)])
+        transform.adjust_path('name', lion_id, trans_id2)
+        self.assertEqual(transform.find_conflicts(),
+                         [('unversioned parent', lion_id),
+                          ('missing parent', lion_id)])
+        transform.version_file(lion_id, file_id=b"Courage")
+        self.assertEqual(transform.find_conflicts(),
+                         [('missing parent', lion_id),
+                          ('versioning no contents', lion_id)])
+        transform.adjust_path('name2', root, trans_id2)
+        self.assertEqual(transform.find_conflicts(),
+                         [('versioning no contents', lion_id)])
+        transform.create_file([b'Contents, okay?'], lion_id)
+        transform.adjust_path('name2', trans_id2, trans_id2)
+        self.assertEqual(transform.find_conflicts(),
+                         [('parent loop', trans_id2),
+                          ('non-directory parent', trans_id2)])
+        transform.adjust_path('name2', root, trans_id2)
+        oz_id = transform.new_directory('oz', root)
+        transform.set_executability(True, oz_id)
+        self.assertEqual(transform.find_conflicts(),
+                         [('unversioned executability', oz_id)])
+        transform.version_file(oz_id, file_id=b'oz-id')
+        self.assertEqual(transform.find_conflicts(),
+                         [('non-file executability', oz_id)])
+        transform.set_executability(None, oz_id)
+        tip_id = transform.new_file('tip', oz_id, [b'ozma'], b'tip-id')
+        transform.apply()
+        self.assertEqual(self.wt.path2id('name'), b'my_pretties')
+        with open(self.wt.abspath('name'), 'rb') as f:
+            self.assertEqual(b'contents', f.read())
+        transform2, root = self.transform()
+        oz_id = transform2.trans_id_tree_path('oz')
+        newtip = transform2.new_file('tip', oz_id, [b'other'], b'tip-id')
+        result = transform2.find_conflicts()
+        fp = FinalPaths(transform2)
+        self.assertTrue('oz/tip' in transform2._tree_path_ids)
+        self.assertEqual(fp.get_path(newtip), pathjoin('oz', 'tip'))
+        self.assertEqual(len(result), 2)
+        self.assertEqual((result[0][0], result[0][1]),
+                         ('duplicate', newtip))
+        self.assertEqual((result[1][0], result[1][2]),
+                         ('duplicate id', newtip))
+        transform2.finalize()
+        transform3 = self.wt.transform()
+        self.addCleanup(transform3.finalize)
+        oz_id = transform3.trans_id_tree_path('oz')
+        transform3.delete_contents(oz_id)
+        self.assertEqual(transform3.find_conflicts(),
+                         [('missing parent', oz_id)])
+        root_id = transform3.root
+        tip_id = transform3.trans_id_tree_path('oz/tip')
+        transform3.adjust_path('tip', root_id, tip_id)
+        transform3.apply()
+
+    def test_conflict_on_case_insensitive(self):
+        tree = self.make_branch_and_tree('tree')
+        # Don't try this at home, kids!
+        # Force the tree to report that it is case sensitive, for conflict
+        # resolution tests
+        tree.case_sensitive = True
+        transform = tree.transform()
+        self.addCleanup(transform.finalize)
+        transform.new_file('file', transform.root, [b'content'])
+        transform.new_file('FiLe', transform.root, [b'content'])
+        result = transform.find_conflicts()
+        self.assertEqual([], result)
+        transform.finalize()
+        # Force the tree to report that it is case insensitive, for conflict
+        # generation tests
+        tree.case_sensitive = False
+        transform = tree.transform()
+        self.addCleanup(transform.finalize)
+        transform.new_file('file', transform.root, [b'content'])
+        transform.new_file('FiLe', transform.root, [b'content'])
+        result = transform.find_conflicts()
+        self.assertEqual([('duplicate', 'new-1', 'new-2', 'file')], result)
+
+    def test_conflict_on_case_insensitive_existing(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/FiLe'])
+        # Don't try this at home, kids!
+        # Force the tree to report that it is case sensitive, for conflict
+        # resolution tests
+        tree.case_sensitive = True
+        transform = tree.transform()
+        self.addCleanup(transform.finalize)
+        transform.new_file('file', transform.root, [b'content'])
+        result = transform.find_conflicts()
+        self.assertEqual([], result)
+        transform.finalize()
+        # Force the tree to report that it is case insensitive, for conflict
+        # generation tests
+        tree.case_sensitive = False
+        transform = tree.transform()
+        self.addCleanup(transform.finalize)
+        transform.new_file('file', transform.root, [b'content'])
+        result = transform.find_conflicts()
+        self.assertEqual([('duplicate', 'new-1', 'new-2', 'file')], result)
+
+    def test_resolve_case_insensitive_conflict(self):
+        tree = self.make_branch_and_tree('tree')
+        # Don't try this at home, kids!
+        # Force the tree to report that it is case insensitive, for conflict
+        # resolution tests
+        tree.case_sensitive = False
+        transform = tree.transform()
+        self.addCleanup(transform.finalize)
+        transform.new_file('file', transform.root, [b'content'])
+        transform.new_file('FiLe', transform.root, [b'content'])
+        resolve_conflicts(transform)
+        transform.apply()
+        self.assertPathExists('tree/file')
+        self.assertPathExists('tree/FiLe.moved')
+
+    def test_resolve_checkout_case_conflict(self):
+        tree = self.make_branch_and_tree('tree')
+        # Don't try this at home, kids!
+        # Force the tree to report that it is case insensitive, for conflict
+        # resolution tests
+        tree.case_sensitive = False
+        transform = tree.transform()
+        self.addCleanup(transform.finalize)
+        transform.new_file('file', transform.root, [b'content'])
+        transform.new_file('FiLe', transform.root, [b'content'])
+        resolve_conflicts(transform,
+                          pass_func=lambda t, c: resolve_checkout(t, c, []))
+        transform.apply()
+        self.assertPathExists('tree/file')
+        self.assertPathExists('tree/FiLe.moved')
+
+    def test_apply_case_conflict(self):
+        """Ensure that a transform with case conflicts can always be applied"""
+        tree = self.make_branch_and_tree('tree')
+        transform = tree.transform()
+        self.addCleanup(transform.finalize)
+        transform.new_file('file', transform.root, [b'content'])
+        transform.new_file('FiLe', transform.root, [b'content'])
+        dir = transform.new_directory('dir', transform.root)
+        transform.new_file('dirfile', dir, [b'content'])
+        transform.new_file('dirFiLe', dir, [b'content'])
+        resolve_conflicts(transform)
+        transform.apply()
+        self.assertPathExists('tree/file')
+        if not os.path.exists('tree/FiLe.moved'):
+            self.assertPathExists('tree/FiLe')
+        self.assertPathExists('tree/dir/dirfile')
+        if not os.path.exists('tree/dir/dirFiLe.moved'):
+            self.assertPathExists('tree/dir/dirFiLe')
+
+    def test_case_insensitive_limbo(self):
+        tree = self.make_branch_and_tree('tree')
+        # Don't try this at home, kids!
+        # Force the tree to report that it is case insensitive
+        tree.case_sensitive = False
+        transform = tree.transform()
+        self.addCleanup(transform.finalize)
+        dir = transform.new_directory('dir', transform.root)
+        first = transform.new_file('file', dir, [b'content'])
+        second = transform.new_file('FiLe', dir, [b'content'])
+        self.assertContainsRe(transform._limbo_name(first), 'new-1/file')
+        self.assertNotContainsRe(transform._limbo_name(second), 'new-1/FiLe')
+
+    def test_adjust_path_updates_child_limbo_names(self):
+        tree = self.make_branch_and_tree('tree')
+        transform = tree.transform()
+        self.addCleanup(transform.finalize)
+        foo_id = transform.new_directory('foo', transform.root)
+        bar_id = transform.new_directory('bar', foo_id)
+        baz_id = transform.new_directory('baz', bar_id)
+        qux_id = transform.new_directory('qux', baz_id)
+        transform.adjust_path('quxx', foo_id, bar_id)
+        self.assertStartsWith(transform._limbo_name(qux_id),
+                              transform._limbo_name(bar_id))
+
+    def test_add_del(self):
+        start, root = self.transform()
+        start.new_directory('a', root, b'a')
+        start.apply()
+        transform, root = self.transform()
+        transform.delete_versioned(transform.trans_id_tree_path('a'))
+        transform.new_directory('a', root, b'a')
+        transform.apply()
+
+    def test_unversioning(self):
+        create_tree, root = self.transform()
+        parent_id = create_tree.new_directory('parent', root, b'parent-id')
+        create_tree.new_file('child', parent_id, [b'child'], b'child-id')
+        create_tree.apply()
+        unversion = self.wt.transform()
+        self.addCleanup(unversion.finalize)
+        parent = unversion.trans_id_tree_path('parent')
+        unversion.unversion_file(parent)
+        self.assertEqual(unversion.find_conflicts(),
+                         [('unversioned parent', parent_id)])
+        file_id = unversion.trans_id_tree_path('parent/child')
+        unversion.unversion_file(file_id)
+        unversion.apply()
+
+    def test_name_invariants(self):
+        create_tree, root = self.transform()
+        # prepare tree
+        root = create_tree.root
+        create_tree.new_file('name1', root, [b'hello1'], b'name1')
+        create_tree.new_file('name2', root, [b'hello2'], b'name2')
+        ddir = create_tree.new_directory('dying_directory', root, b'ddir')
+        create_tree.new_file('dying_file', ddir, [b'goodbye1'], b'dfile')
+        create_tree.new_file('moving_file', ddir, [b'later1'], b'mfile')
+        create_tree.new_file('moving_file2', root, [b'later2'], b'mfile2')
+        create_tree.apply()
+
+        mangle_tree, root = self.transform()
+        root = mangle_tree.root
+        # swap names
+        name1 = mangle_tree.trans_id_tree_path('name1')
+        name2 = mangle_tree.trans_id_tree_path('name2')
+        mangle_tree.adjust_path('name2', root, name1)
+        mangle_tree.adjust_path('name1', root, name2)
+
+        # tests for deleting parent directories
+        ddir = mangle_tree.trans_id_tree_path('dying_directory')
+        mangle_tree.delete_contents(ddir)
+        dfile = mangle_tree.trans_id_tree_path('dying_directory/dying_file')
+        mangle_tree.delete_versioned(dfile)
+        mangle_tree.unversion_file(dfile)
+        mfile = mangle_tree.trans_id_tree_path('dying_directory/moving_file')
+        mangle_tree.adjust_path('mfile', root, mfile)
+
+        # tests for adding parent directories
+        newdir = mangle_tree.new_directory('new_directory', root, b'newdir')
+        mfile2 = mangle_tree.trans_id_tree_path('moving_file2')
+        mangle_tree.adjust_path('mfile2', newdir, mfile2)
+        mangle_tree.new_file('newfile', newdir, [b'hello3'], b'dfile')
+        self.assertEqual(mangle_tree.final_file_id(mfile2), b'mfile2')
+        self.assertEqual(mangle_tree.final_parent(mfile2), newdir)
+        self.assertEqual(mangle_tree.final_file_id(mfile2), b'mfile2')
+        mangle_tree.apply()
+        with open(self.wt.abspath('name1'), 'r') as f:
+            self.assertEqual(f.read(), 'hello2')
+        with open(self.wt.abspath('name2'), 'r') as f:
+            self.assertEqual(f.read(), 'hello1')
+        mfile2_path = self.wt.abspath(pathjoin('new_directory', 'mfile2'))
+        self.assertEqual(mangle_tree.final_parent(mfile2), newdir)
+        with open(mfile2_path, 'r') as f:
+            self.assertEqual(f.read(), 'later2')
+        self.assertEqual(self.wt.id2path(b'mfile2'), 'new_directory/mfile2')
+        self.assertEqual(self.wt.path2id('new_directory/mfile2'), b'mfile2')
+        newfile_path = self.wt.abspath(pathjoin('new_directory', 'newfile'))
+        with open(newfile_path, 'r') as f:
+            self.assertEqual(f.read(), 'hello3')
+        self.assertEqual(self.wt.path2id('dying_directory'), b'ddir')
+        self.assertIs(self.wt.path2id('dying_directory/dying_file'), None)
+        mfile2_path = self.wt.abspath(pathjoin('new_directory', 'mfile2'))
+
+    def test_both_rename(self):
+        create_tree, root = self.transform()
+        newdir = create_tree.new_directory('selftest', root, b'selftest-id')
+        create_tree.new_file('blackbox.py', newdir, [
+                             b'hello1'], b'blackbox-id')
+        create_tree.apply()
+        mangle_tree, root = self.transform()
+        selftest = mangle_tree.trans_id_tree_path('selftest')
+        blackbox = mangle_tree.trans_id_tree_path('selftest/blackbox.py')
+        mangle_tree.adjust_path('test', root, selftest)
+        mangle_tree.adjust_path('test_too_much', root, selftest)
+        mangle_tree.set_executability(True, blackbox)
+        mangle_tree.apply()
+
+    def test_both_rename2(self):
+        create_tree, root = self.transform()
+        breezy = create_tree.new_directory('breezy', root, b'breezy-id')
+        tests = create_tree.new_directory('tests', breezy, b'tests-id')
+        blackbox = create_tree.new_directory('blackbox', tests, b'blackbox-id')
+        create_tree.new_file('test_too_much.py', blackbox, [b'hello1'],
+                             b'test_too_much-id')
+        create_tree.apply()
+        mangle_tree, root = self.transform()
+        breezy = mangle_tree.trans_id_tree_path('breezy')
+        tests = mangle_tree.trans_id_tree_path('breezy/tests')
+        test_too_much = mangle_tree.trans_id_tree_path(
+            'breezy/tests/blackbox/test_too_much.py')
+        mangle_tree.adjust_path('selftest', breezy, tests)
+        mangle_tree.adjust_path('blackbox.py', tests, test_too_much)
+        mangle_tree.set_executability(True, test_too_much)
+        mangle_tree.apply()
+
+    def test_both_rename3(self):
+        create_tree, root = self.transform()
+        tests = create_tree.new_directory('tests', root, b'tests-id')
+        create_tree.new_file('test_too_much.py', tests, [b'hello1'],
+                             b'test_too_much-id')
+        create_tree.apply()
+        mangle_tree, root = self.transform()
+        tests = mangle_tree.trans_id_tree_path('tests')
+        test_too_much = mangle_tree.trans_id_tree_path(
+            'tests/test_too_much.py')
+        mangle_tree.adjust_path('selftest', root, tests)
+        mangle_tree.adjust_path('blackbox.py', tests, test_too_much)
+        mangle_tree.set_executability(True, test_too_much)
+        mangle_tree.apply()
+
+    def test_move_dangling_ie(self):
+        create_tree, root = self.transform()
+        # prepare tree
+        root = create_tree.root
+        create_tree.new_file('name1', root, [b'hello1'], b'name1')
+        create_tree.apply()
+        delete_contents, root = self.transform()
+        file = delete_contents.trans_id_tree_path('name1')
+        delete_contents.delete_contents(file)
+        delete_contents.apply()
+        move_id, root = self.transform()
+        name1 = move_id.trans_id_tree_path('name1')
+        newdir = move_id.new_directory('dir', root, b'newdir')
+        move_id.adjust_path('name2', newdir, name1)
+        move_id.apply()
+
+    def test_replace_dangling_ie(self):
+        create_tree, root = self.transform()
+        # prepare tree
+        root = create_tree.root
+        create_tree.new_file('name1', root, [b'hello1'], b'name1')
+        create_tree.apply()
+        delete_contents = self.wt.transform()
+        self.addCleanup(delete_contents.finalize)
+        file = delete_contents.trans_id_tree_path('name1')
+        delete_contents.delete_contents(file)
+        delete_contents.apply()
+        delete_contents.finalize()
+        replace = self.wt.transform()
+        self.addCleanup(replace.finalize)
+        name2 = replace.new_file('name2', root, [b'hello2'], b'name1')
+        conflicts = replace.find_conflicts()
+        name1 = replace.trans_id_tree_path('name1')
+        self.assertEqual(conflicts, [('duplicate id', name1, name2)])
+        resolve_conflicts(replace)
+        replace.apply()
+
+    def _test_symlinks(self, link_name1, link_target1,
+                       link_name2, link_target2):
+
+        def ozpath(p):
+            return 'oz/' + p
+
+        self.requireFeature(SymlinkFeature)
+        transform, root = self.transform()
+        oz_id = transform.new_directory('oz', root, b'oz-id')
+        transform.new_symlink(link_name1, oz_id, link_target1, b'wizard-id')
+        wiz_id = transform.create_path(link_name2, oz_id)
+        transform.create_symlink(link_target2, wiz_id)
+        transform.version_file(wiz_id, file_id=b'wiz-id2')
+        transform.set_executability(True, wiz_id)
+        self.assertEqual(transform.find_conflicts(),
+                         [('non-file executability', wiz_id)])
+        transform.set_executability(None, wiz_id)
+        transform.apply()
+        self.assertEqual(self.wt.path2id(ozpath(link_name1)), b'wizard-id')
+        self.assertEqual('symlink',
+                         file_kind(self.wt.abspath(ozpath(link_name1))))
+        self.assertEqual(link_target2,
+                         osutils.readlink(self.wt.abspath(ozpath(link_name2))))
+        self.assertEqual(link_target1,
+                         osutils.readlink(self.wt.abspath(ozpath(link_name1))))
+
+    def test_symlinks(self):
+        self._test_symlinks('wizard', 'wizard-target',
+                            'wizard2', 'behind_curtain')
+
+    def test_symlinks_unicode(self):
+        self.requireFeature(features.UnicodeFilenameFeature)
+        self._test_symlinks(u'\N{Euro Sign}wizard',
+                            u'wizard-targ\N{Euro Sign}t',
+                            u'\N{Euro Sign}wizard2',
+                            u'b\N{Euro Sign}hind_curtain')
+
+    def test_unsupported_symlink_no_conflict(self):
+        def tt_helper():
+            wt = self.make_branch_and_tree('.')
+            tt = wt.transform()
+            self.addCleanup(tt.finalize)
+            tt.new_symlink('foo', tt.root, 'bar')
+            result = tt.find_conflicts()
+            self.assertEqual([], result)
+        os_symlink = getattr(os, 'symlink', None)
+        os.symlink = None
+        try:
+            tt_helper()
+        finally:
+            if os_symlink:
+                os.symlink = os_symlink
+
+    def get_conflicted(self):
+        create, root = self.transform()
+        create.new_file('dorothy', root, [b'dorothy'], b'dorothy-id')
+        oz = create.new_directory('oz', root, b'oz-id')
+        create.new_directory('emeraldcity', oz, b'emerald-id')
+        create.apply()
+        conflicts, root = self.transform()
+        # set up duplicate entry, duplicate id
+        new_dorothy = conflicts.new_file('dorothy', root, [b'dorothy'],
+                                         b'dorothy-id')
+        old_dorothy = conflicts.trans_id_tree_path('dorothy')
+        oz = conflicts.trans_id_tree_path('oz')
+        # set up DeletedParent parent conflict
+        conflicts.delete_versioned(oz)
+        emerald = conflicts.trans_id_tree_path('oz/emeraldcity')
+        # set up MissingParent conflict
+        munchkincity = conflicts.trans_id_file_id(b'munchkincity-id')
+        conflicts.adjust_path('munchkincity', root, munchkincity)
+        conflicts.new_directory('auntem', munchkincity, b'auntem-id')
+        # set up parent loop
+        conflicts.adjust_path('emeraldcity', emerald, emerald)
+        return conflicts, emerald, oz, old_dorothy, new_dorothy
+
+    def test_conflict_resolution(self):
+        conflicts, emerald, oz, old_dorothy, new_dorothy =\
+            self.get_conflicted()
+        resolve_conflicts(conflicts)
+        self.assertEqual(conflicts.final_name(old_dorothy), 'dorothy.moved')
+        self.assertIs(conflicts.final_file_id(old_dorothy), None)
+        self.assertEqual(conflicts.final_name(new_dorothy), 'dorothy')
+        self.assertEqual(conflicts.final_file_id(new_dorothy), b'dorothy-id')
+        self.assertEqual(conflicts.final_parent(emerald), oz)
+        conflicts.apply()
+
+    def test_cook_conflicts(self):
+        tt, emerald, oz, old_dorothy, new_dorothy = self.get_conflicted()
+        raw_conflicts = resolve_conflicts(tt)
+        cooked_conflicts = cook_conflicts(raw_conflicts, tt)
+        duplicate = DuplicateEntry('Moved existing file to', 'dorothy.moved',
+                                   'dorothy', None, b'dorothy-id')
+        self.assertEqual(cooked_conflicts[0], duplicate)
+        duplicate_id = DuplicateID('Unversioned existing file',
+                                   'dorothy.moved', 'dorothy', None,
+                                   b'dorothy-id')
+        self.assertEqual(cooked_conflicts[1], duplicate_id)
+        missing_parent = MissingParent('Created directory', 'munchkincity',
+                                       b'munchkincity-id')
+        deleted_parent = DeletingParent('Not deleting', 'oz', b'oz-id')
+        self.assertEqual(cooked_conflicts[2], missing_parent)
+        unversioned_parent = UnversionedParent('Versioned directory',
+                                               'munchkincity',
+                                               b'munchkincity-id')
+        unversioned_parent2 = UnversionedParent('Versioned directory', 'oz',
+                                                b'oz-id')
+        self.assertEqual(cooked_conflicts[3], unversioned_parent)
+        parent_loop = ParentLoop(
+            'Cancelled move', 'oz/emeraldcity',
+            'oz/emeraldcity', b'emerald-id', b'emerald-id')
+        self.assertEqual(cooked_conflicts[4], deleted_parent)
+        self.assertEqual(cooked_conflicts[5], unversioned_parent2)
+        self.assertEqual(cooked_conflicts[6], parent_loop)
+        self.assertEqual(len(cooked_conflicts), 7)
+        tt.finalize()
+
+    def test_string_conflicts(self):
+        tt, emerald, oz, old_dorothy, new_dorothy = self.get_conflicted()
+        raw_conflicts = resolve_conflicts(tt)
+        cooked_conflicts = cook_conflicts(raw_conflicts, tt)
+        tt.finalize()
+        conflicts_s = [text_type(c) for c in cooked_conflicts]
+        self.assertEqual(len(cooked_conflicts), len(conflicts_s))
+        self.assertEqual(conflicts_s[0], 'Conflict adding file dorothy.  '
+                                         'Moved existing file to '
+                                         'dorothy.moved.')
+        self.assertEqual(conflicts_s[1], 'Conflict adding id to dorothy.  '
+                                         'Unversioned existing file '
+                                         'dorothy.moved.')
+        self.assertEqual(conflicts_s[2], 'Conflict adding files to'
+                                         ' munchkincity.  Created directory.')
+        self.assertEqual(conflicts_s[3], 'Conflict because munchkincity is not'
+                                         ' versioned, but has versioned'
+                                         ' children.  Versioned directory.')
+        self.assertEqualDiff(
+            conflicts_s[4], "Conflict: can't delete oz because it"
+                            " is not empty.  Not deleting.")
+        self.assertEqual(conflicts_s[5], 'Conflict because oz is not'
+                                         ' versioned, but has versioned'
+                                         ' children.  Versioned directory.')
+        self.assertEqual(conflicts_s[6], 'Conflict moving oz/emeraldcity into'
+                                         ' oz/emeraldcity. Cancelled move.')
+
+    def prepare_wrong_parent_kind(self):
+        tt, root = self.transform()
+        tt.new_file('parent', root, [b'contents'], b'parent-id')
+        tt.apply()
+        tt, root = self.transform()
+        parent_id = tt.trans_id_file_id(b'parent-id')
+        tt.new_file('child,', parent_id, [b'contents2'], b'file-id')
+        return tt
+
+    def test_find_conflicts_wrong_parent_kind(self):
+        tt = self.prepare_wrong_parent_kind()
+        tt.find_conflicts()
+
+    def test_resolve_conflicts_wrong_existing_parent_kind(self):
+        tt = self.prepare_wrong_parent_kind()
+        raw_conflicts = resolve_conflicts(tt)
+        self.assertEqual({('non-directory parent', 'Created directory',
+                           'new-3')}, raw_conflicts)
+        cooked_conflicts = cook_conflicts(raw_conflicts, tt)
+        self.assertEqual([NonDirectoryParent('Created directory', 'parent.new',
+                                             b'parent-id')], cooked_conflicts)
+        tt.apply()
+        self.assertFalse(self.wt.is_versioned('parent'))
+        self.assertEqual(b'parent-id', self.wt.path2id('parent.new'))
+
+    def test_resolve_conflicts_wrong_new_parent_kind(self):
+        tt, root = self.transform()
+        parent_id = tt.new_directory('parent', root, b'parent-id')
+        tt.new_file('child,', parent_id, [b'contents2'], b'file-id')
+        tt.apply()
+        tt, root = self.transform()
+        parent_id = tt.trans_id_file_id(b'parent-id')
+        tt.delete_contents(parent_id)
+        tt.create_file([b'contents'], parent_id)
+        raw_conflicts = resolve_conflicts(tt)
+        self.assertEqual({('non-directory parent', 'Created directory',
+                           'new-3')}, raw_conflicts)
+        tt.apply()
+        self.assertFalse(self.wt.is_versioned('parent'))
+        self.assertEqual(b'parent-id', self.wt.path2id('parent.new'))
+
+    def test_resolve_conflicts_wrong_parent_kind_unversioned(self):
+        tt, root = self.transform()
+        parent_id = tt.new_directory('parent', root)
+        tt.new_file('child,', parent_id, [b'contents2'])
+        tt.apply()
+        tt, root = self.transform()
+        parent_id = tt.trans_id_tree_path('parent')
+        tt.delete_contents(parent_id)
+        tt.create_file([b'contents'], parent_id)
+        resolve_conflicts(tt)
+        tt.apply()
+        self.assertFalse(self.wt.is_versioned('parent'))
+        self.assertFalse(self.wt.is_versioned('parent.new'))
+
+    def test_resolve_conflicts_missing_parent(self):
+        wt = self.make_branch_and_tree('.')
+        tt = wt.transform()
+        self.addCleanup(tt.finalize)
+        parent = tt.trans_id_file_id(b'parent-id')
+        tt.new_file('file', parent, [b'Contents'])
+        raw_conflicts = resolve_conflicts(tt)
+        # Since the directory doesn't exist it's seen as 'missing'.  So
+        # 'resolve_conflicts' create a conflict asking for it to be created.
+        self.assertLength(1, raw_conflicts)
+        self.assertEqual(('missing parent', 'Created directory', 'new-1'),
+                         raw_conflicts.pop())
+        # apply fail since the missing directory doesn't exist
+        self.assertRaises(NoFinalPath, tt.apply)
+
+    def test_moving_versioned_directories(self):
+        create, root = self.transform()
+        kansas = create.new_directory('kansas', root, b'kansas-id')
+        create.new_directory('house', kansas, b'house-id')
+        create.new_directory('oz', root, b'oz-id')
+        create.apply()
+        cyclone, root = self.transform()
+        oz = cyclone.trans_id_tree_path('oz')
+        house = cyclone.trans_id_tree_path('house')
+        cyclone.adjust_path('house', oz, house)
+        cyclone.apply()
+
+    def test_moving_root(self):
+        create, root = self.transform()
+        fun = create.new_directory('fun', root, b'fun-id')
+        create.new_directory('sun', root, b'sun-id')
+        create.new_directory('moon', root, b'moon')
+        create.apply()
+        transform, root = self.transform()
+        transform.adjust_root_path('oldroot', fun)
+        new_root = transform.trans_id_tree_path('')
+        transform.version_file(new_root, file_id=b'new-root')
+        transform.apply()
+
+    def test_renames(self):
+        create, root = self.transform()
+        old = create.new_directory('old-parent', root, b'old-id')
+        intermediate = create.new_directory('intermediate', old, b'im-id')
+        myfile = create.new_file('myfile', intermediate, [b'myfile-text'],
+                                 b'myfile-id')
+        create.apply()
+        rename, root = self.transform()
+        old = rename.trans_id_file_id(b'old-id')
+        rename.adjust_path('new', root, old)
+        myfile = rename.trans_id_file_id(b'myfile-id')
+        rename.set_executability(True, myfile)
+        rename.apply()
+
+    def test_rename_fails(self):
+        self.requireFeature(features.not_running_as_root)
+        # see https://bugs.launchpad.net/bzr/+bug/491763
+        create, root_id = self.transform()
+        create.new_directory('first-dir', root_id, b'first-id')
+        create.new_file('myfile', root_id, [b'myfile-text'], b'myfile-id')
+        create.apply()
+        if os.name == "posix" and sys.platform != "cygwin":
+            # posix filesystems fail on renaming if the readonly bit is set
+            osutils.make_readonly(self.wt.abspath('first-dir'))
+        elif os.name == "nt":
+            # windows filesystems fail on renaming open files
+            self.addCleanup(open(self.wt.abspath('myfile')).close)
+        else:
+            self.skipTest("Can't force a permissions error on rename")
+        # now transform to rename
+        rename_transform, root_id = self.transform()
+        file_trans_id = rename_transform.trans_id_file_id(b'myfile-id')
+        dir_id = rename_transform.trans_id_file_id(b'first-id')
+        rename_transform.adjust_path('newname', dir_id, file_trans_id)
+        e = self.assertRaises(TransformRenameFailed,
+                              rename_transform.apply)
+        # On nix looks like:
+        # "Failed to rename .../work/.bzr/checkout/limbo/new-1
+        # to .../first-dir/newname: [Errno 13] Permission denied"
+        # On windows looks like:
+        # "Failed to rename .../work/myfile to
+        # .../work/.bzr/checkout/limbo/new-1: [Errno 13] Permission denied"
+        # This test isn't concerned with exactly what the error looks like,
+        # and the strerror will vary across OS and locales, but the assert
+        # that the exeception attributes are what we expect
+        self.assertEqual(e.errno, errno.EACCES)
+        if os.name == "posix":
+            self.assertEndsWith(e.to_path, "/first-dir/newname")
+        else:
+            self.assertEqual(os.path.basename(e.from_path), "myfile")
+
+    def test_set_executability_order(self):
+        """Ensure that executability behaves the same, no matter what order.
+
+        - create file and set executability simultaneously
+        - create file and set executability afterward
+        - unsetting the executability of a file whose executability has not
+          been
+        declared should throw an exception (this may happen when a
+        merge attempts to create a file with a duplicate ID)
+        """
+        transform, root = self.transform()
+        wt = transform._tree
+        wt.lock_read()
+        self.addCleanup(wt.unlock)
+        transform.new_file('set_on_creation', root, [b'Set on creation'],
+                           b'soc', True)
+        sac = transform.new_file('set_after_creation', root,
+                                 [b'Set after creation'], b'sac')
+        transform.set_executability(True, sac)
+        uws = transform.new_file('unset_without_set', root, [b'Unset badly'],
+                                 b'uws')
+        self.assertRaises(KeyError, transform.set_executability, None, uws)
+        transform.apply()
+        self.assertTrue(wt.is_executable('set_on_creation'))
+        self.assertTrue(wt.is_executable('set_after_creation'))
+
+    def test_preserve_mode(self):
+        """File mode is preserved when replacing content"""
+        if sys.platform == 'win32':
+            raise TestSkipped('chmod has no effect on win32')
+        transform, root = self.transform()
+        transform.new_file('file1', root, [b'contents'], b'file1-id', True)
+        transform.apply()
+        self.wt.lock_write()
+        self.addCleanup(self.wt.unlock)
+        self.assertTrue(self.wt.is_executable('file1'))
+        transform, root = self.transform()
+        file1_id = transform.trans_id_tree_path('file1')
+        transform.delete_contents(file1_id)
+        transform.create_file([b'contents2'], file1_id)
+        transform.apply()
+        self.assertTrue(self.wt.is_executable('file1'))
+
+    def test__set_mode_stats_correctly(self):
+        """_set_mode stats to determine file mode."""
+        if sys.platform == 'win32':
+            raise TestSkipped('chmod has no effect on win32')
+
+        stat_paths = []
+        real_stat = os.stat
+
+        def instrumented_stat(path):
+            stat_paths.append(path)
+            return real_stat(path)
+
+        transform, root = self.transform()
+
+        bar1_id = transform.new_file('bar', root, [b'bar contents 1\n'],
+                                     file_id=b'bar-id-1', executable=False)
+        transform.apply()
+
+        transform, root = self.transform()
+        bar1_id = transform.trans_id_tree_path('bar')
+        bar2_id = transform.trans_id_tree_path('bar2')
+        try:
+            os.stat = instrumented_stat
+            transform.create_file([b'bar2 contents\n'],
+                                  bar2_id, mode_id=bar1_id)
+        finally:
+            os.stat = real_stat
+            transform.finalize()
+
+        bar1_abspath = self.wt.abspath('bar')
+        self.assertEqual([bar1_abspath], stat_paths)
+
+    def test_iter_changes(self):
+        root_id = self.wt.path2id('')
+        transform, root = self.transform()
+        transform.new_file('old', root, [b'blah'], b'id-1', True)
+        transform.apply()
+        transform, root = self.transform()
+        try:
+            self.assertEqual([], list(transform.iter_changes()))
+            old = transform.trans_id_tree_path('old')
+            transform.unversion_file(old)
+            self.assertEqual([(b'id-1', ('old', None), False, (True, False),
+                               (root_id, root_id),
+                               ('old', 'old'), ('file', 'file'),
+                               (True, True), False)],
+                             list(transform.iter_changes()))
+            transform.new_directory('new', root, b'id-1')
+            self.assertEqual([(b'id-1', ('old', 'new'), True, (True, True),
+                               (root_id, root_id), ('old', 'new'),
+                               ('file', 'directory'),
+                               (True, False), False)],
+                             list(transform.iter_changes()))
+        finally:
+            transform.finalize()
+
+    def test_iter_changes_new(self):
+        root_id = self.wt.path2id('')
+        transform, root = self.transform()
+        transform.new_file('old', root, [b'blah'])
+        transform.apply()
+        transform, root = self.transform()
+        try:
+            old = transform.trans_id_tree_path('old')
+            transform.version_file(old, file_id=b'id-1')
+            self.assertEqual([(b'id-1', (None, 'old'), False, (False, True),
+                               (root_id, root_id),
+                               ('old', 'old'), ('file', 'file'),
+                               (False, False), False)],
+                             list(transform.iter_changes()))
+        finally:
+            transform.finalize()
+
+    def test_iter_changes_modifications(self):
+        root_id = self.wt.path2id('')
+        transform, root = self.transform()
+        transform.new_file('old', root, [b'blah'], b'id-1')
+        transform.new_file('new', root, [b'blah'])
+        transform.new_directory('subdir', root, b'subdir-id')
+        transform.apply()
+        transform, root = self.transform()
+        try:
+            old = transform.trans_id_tree_path('old')
+            subdir = transform.trans_id_tree_path('subdir')
+            new = transform.trans_id_tree_path('new')
+            self.assertEqual([], list(transform.iter_changes()))
+
+            # content deletion
+            transform.delete_contents(old)
+            self.assertEqual([(b'id-1', ('old', 'old'), True, (True, True),
+                               (root_id, root_id),
+                               ('old', 'old'), ('file', None),
+                               (False, False), False)],
+                             list(transform.iter_changes()))
+
+            # content change
+            transform.create_file([b'blah'], old)
+            self.assertEqual([(b'id-1', ('old', 'old'), True, (True, True),
+                               (root_id, root_id),
+                               ('old', 'old'), ('file', 'file'),
+                               (False, False), False)],
+                             list(transform.iter_changes()))
+            transform.cancel_deletion(old)
+            self.assertEqual([(b'id-1', ('old', 'old'), True, (True, True),
+                               (root_id, root_id),
+                               ('old', 'old'), ('file', 'file'),
+                               (False, False), False)],
+                             list(transform.iter_changes()))
+            transform.cancel_creation(old)
+
+            # move file_id to a different file
+            self.assertEqual([], list(transform.iter_changes()))
+            transform.unversion_file(old)
+            transform.version_file(new, file_id=b'id-1')
+            transform.adjust_path('old', root, new)
+            self.assertEqual([(b'id-1', ('old', 'old'), True, (True, True),
+                               (root_id, root_id),
+                               ('old', 'old'), ('file', 'file'),
+                               (False, False), False)],
+                             list(transform.iter_changes()))
+            transform.cancel_versioning(new)
+            transform._removed_id = set()
+
+            # execute bit
+            self.assertEqual([], list(transform.iter_changes()))
+            transform.set_executability(True, old)
+            self.assertEqual([(b'id-1', ('old', 'old'), False, (True, True),
+                               (root_id, root_id),
+                               ('old', 'old'), ('file', 'file'),
+                               (False, True), False)],
+                             list(transform.iter_changes()))
+            transform.set_executability(None, old)
+
+            # filename
+            self.assertEqual([], list(transform.iter_changes()))
+            transform.adjust_path('new', root, old)
+            transform._new_parent = {}
+            self.assertEqual([(b'id-1', ('old', 'new'), False, (True, True),
+                               (root_id, root_id),
+                               ('old', 'new'), ('file', 'file'),
+                               (False, False), False)],
+                             list(transform.iter_changes()))
+            transform._new_name = {}
+
+            # parent directory
+            self.assertEqual([], list(transform.iter_changes()))
+            transform.adjust_path('new', subdir, old)
+            transform._new_name = {}
+            self.assertEqual([(b'id-1', ('old', 'subdir/old'), False,
+                               (True, True), (root_id, b'subdir-id'), ('old', 'old'),
+                               ('file', 'file'), (False, False), False)],
+                             list(transform.iter_changes()))
+            transform._new_path = {}
+
+        finally:
+            transform.finalize()
+
+    def test_iter_changes_modified_bleed(self):
+        root_id = self.wt.path2id('')
+        """Modified flag should not bleed from one change to another"""
+        # unfortunately, we have no guarantee that file1 (which is modified)
+        # will be applied before file2.  And if it's applied after file2, it
+        # obviously can't bleed into file2's change output.  But for now, it
+        # works.
+        transform, root = self.transform()
+        transform.new_file('file1', root, [b'blah'], b'id-1')
+        transform.new_file('file2', root, [b'blah'], b'id-2')
+        transform.apply()
+        transform, root = self.transform()
+        try:
+            transform.delete_contents(transform.trans_id_file_id(b'id-1'))
+            transform.set_executability(True,
+                                        transform.trans_id_file_id(b'id-2'))
+            self.assertEqual(
+                [(b'id-1', (u'file1', u'file1'), True, (True, True),
+                 (root_id, root_id), ('file1', u'file1'),
+                 ('file', None), (False, False), False),
+                 (b'id-2', (u'file2', u'file2'), False, (True, True),
+                 (root_id, root_id), ('file2', u'file2'),
+                 ('file', 'file'), (False, True), False)],
+                list(transform.iter_changes()))
+        finally:
+            transform.finalize()
+
+    def test_iter_changes_move_missing(self):
+        """Test moving ids with no files around"""
+        root_id = self.wt.path2id('')
+        # Need two steps because versioning a non-existant file is a conflict.
+        transform, root = self.transform()
+        transform.new_directory('floater', root, b'floater-id')
+        transform.apply()
+        transform, root = self.transform()
+        transform.delete_contents(transform.trans_id_tree_path('floater'))
+        transform.apply()
+        transform, root = self.transform()
+        floater = transform.trans_id_tree_path('floater')
+        try:
+            transform.adjust_path('flitter', root, floater)
+            self.assertEqual([(b'floater-id', ('floater', 'flitter'), False,
+                               (True, True),
+                               (root_id, root_id),
+                               ('floater', 'flitter'),
+                               (None, None), (False, False), False)],
+                             list(transform.iter_changes()))
+        finally:
+            transform.finalize()
+
+    def test_iter_changes_pointless(self):
+        """Ensure that no-ops are not treated as modifications"""
+        transform, root = self.transform()
+        transform.new_file('old', root, [b'blah'], b'id-1')
+        transform.new_directory('subdir', root, b'subdir-id')
+        transform.apply()
+        transform, root = self.transform()
+        try:
+            old = transform.trans_id_tree_path('old')
+            subdir = transform.trans_id_tree_path('subdir')
+            self.assertEqual([], list(transform.iter_changes()))
+            transform.delete_contents(subdir)
+            transform.create_directory(subdir)
+            transform.set_executability(False, old)
+            transform.unversion_file(old)
+            transform.version_file(old, file_id=b'id-1')
+            transform.adjust_path('old', root, old)
+            self.assertEqual([], list(transform.iter_changes()))
+        finally:
+            transform.finalize()
+
+    def test_rename_count(self):
+        transform, root = self.transform()
+        transform.new_file('name1', root, [b'contents'])
+        result = transform.apply()
+        self.assertEqual(result.rename_count, 1)
+        transform2, root = self.transform()
+        transform2.adjust_path('name2', root,
+                               transform2.trans_id_tree_path('name1'))
+        result = transform2.apply()
+        self.assertEqual(result.rename_count, 2)
+
+    def test_change_parent(self):
+        """Ensure that after we change a parent, the results are still right.
+
+        Renames and parent changes on pending transforms can happen as part
+        of conflict resolution, and are explicitly permitted by the
+        TreeTransform API.
+
+        This test ensures they work correctly with the rename-avoidance
+        optimization.
+        """
+        transform, root = self.transform()
+        parent1 = transform.new_directory('parent1', root)
+        child1 = transform.new_file('child1', parent1, [b'contents'])
+        parent2 = transform.new_directory('parent2', root)
+        transform.adjust_path('child1', parent2, child1)
+        transform.apply()
+        self.assertPathDoesNotExist(self.wt.abspath('parent1/child1'))
+        self.assertPathExists(self.wt.abspath('parent2/child1'))
+        # rename limbo/new-1 => parent1, rename limbo/new-3 => parent2
+        # no rename for child1 (counting only renames during apply)
+        self.assertEqual(2, transform.rename_count)
+
+    def test_cancel_parent(self):
+        """Cancelling a parent doesn't cause deletion of a non-empty directory
+
+        This is like the test_change_parent, except that we cancel the parent
+        before adjusting the path.  The transform must detect that the
+        directory is non-empty, and move children to safe locations.
+        """
+        transform, root = self.transform()
+        parent1 = transform.new_directory('parent1', root)
+        child1 = transform.new_file('child1', parent1, [b'contents'])
+        child2 = transform.new_file('child2', parent1, [b'contents'])
+        try:
+            transform.cancel_creation(parent1)
+        except OSError:
+            self.fail('Failed to move child1 before deleting parent1')
+        transform.cancel_creation(child2)
+        transform.create_directory(parent1)
+        try:
+            transform.cancel_creation(parent1)
+        # If the transform incorrectly believes that child2 is still in
+        # parent1's limbo directory, it will try to rename it and fail
+        # because was already moved by the first cancel_creation.
+        except OSError:
+            self.fail('Transform still thinks child2 is a child of parent1')
+        parent2 = transform.new_directory('parent2', root)
+        transform.adjust_path('child1', parent2, child1)
+        transform.apply()
+        self.assertPathDoesNotExist(self.wt.abspath('parent1'))
+        self.assertPathExists(self.wt.abspath('parent2/child1'))
+        # rename limbo/new-3 => parent2, rename limbo/new-2 => child1
+        self.assertEqual(2, transform.rename_count)
+
+    def test_adjust_and_cancel(self):
+        """Make sure adjust_path keeps track of limbo children properly"""
+        transform, root = self.transform()
+        parent1 = transform.new_directory('parent1', root)
+        child1 = transform.new_file('child1', parent1, [b'contents'])
+        parent2 = transform.new_directory('parent2', root)
+        transform.adjust_path('child1', parent2, child1)
+        transform.cancel_creation(child1)
+        try:
+            transform.cancel_creation(parent1)
+        # if the transform thinks child1 is still in parent1's limbo
+        # directory, it will attempt to move it and fail.
+        except OSError:
+            self.fail('Transform still thinks child1 is a child of parent1')
+        transform.finalize()
+
+    def test_noname_contents(self):
+        """TreeTransform should permit deferring naming files."""
+        transform, root = self.transform()
+        parent = transform.trans_id_file_id(b'parent-id')
+        try:
+            transform.create_directory(parent)
+        except KeyError:
+            self.fail("Can't handle contents with no name")
+        transform.finalize()
+
+    def test_noname_contents_nested(self):
+        """TreeTransform should permit deferring naming files."""
+        transform, root = self.transform()
+        parent = transform.trans_id_file_id(b'parent-id')
+        try:
+            transform.create_directory(parent)
+        except KeyError:
+            self.fail("Can't handle contents with no name")
+        transform.new_directory('child', parent)
+        transform.adjust_path('parent', root, parent)
+        transform.apply()
+        self.assertPathExists(self.wt.abspath('parent/child'))
+        self.assertEqual(1, transform.rename_count)
+
+    def test_reuse_name(self):
+        """Avoid reusing the same limbo name for different files"""
+        transform, root = self.transform()
+        parent = transform.new_directory('parent', root)
+        transform.new_directory('child', parent)
+        try:
+            child2 = transform.new_directory('child', parent)
+        except OSError:
+            self.fail('Tranform tried to use the same limbo name twice')
+        transform.adjust_path('child2', parent, child2)
+        transform.apply()
+        # limbo/new-1 => parent, limbo/new-3 => parent/child2
+        # child2 is put into top-level limbo because child1 has already
+        # claimed the direct limbo path when child2 is created.  There is no
+        # advantage in renaming files once they're in top-level limbo, except
+        # as part of apply.
+        self.assertEqual(2, transform.rename_count)
+
+    def test_reuse_when_first_moved(self):
+        """Don't avoid direct paths when it is safe to use them"""
+        transform, root = self.transform()
+        parent = transform.new_directory('parent', root)
+        child1 = transform.new_directory('child', parent)
+        transform.adjust_path('child1', parent, child1)
+        transform.new_directory('child', parent)
+        transform.apply()
+        # limbo/new-1 => parent
+        self.assertEqual(1, transform.rename_count)
+
+    def test_reuse_after_cancel(self):
+        """Don't avoid direct paths when it is safe to use them"""
+        transform, root = self.transform()
+        parent2 = transform.new_directory('parent2', root)
+        child1 = transform.new_directory('child1', parent2)
+        transform.cancel_creation(parent2)
+        transform.create_directory(parent2)
+        transform.new_directory('child1', parent2)
+        transform.adjust_path('child2', parent2, child1)
+        transform.apply()
+        # limbo/new-1 => parent2, limbo/new-2 => parent2/child1
+        self.assertEqual(2, transform.rename_count)
+
+    def test_finalize_order(self):
+        """Finalize must be done in child-to-parent order"""
+        transform, root = self.transform()
+        parent = transform.new_directory('parent', root)
+        transform.new_directory('child', parent)
+        try:
+            transform.finalize()
+        except OSError:
+            self.fail('Tried to remove parent before child1')
+
+    def test_cancel_with_cancelled_child_should_succeed(self):
+        transform, root = self.transform()
+        parent = transform.new_directory('parent', root)
+        child = transform.new_directory('child', parent)
+        transform.cancel_creation(child)
+        transform.cancel_creation(parent)
+        transform.finalize()
+
+    def test_rollback_on_directory_clash(self):
+        def tt_helper():
+            wt = self.make_branch_and_tree('.')
+            tt = wt.transform()  # TreeTransform obtains write lock
+            try:
+                foo = tt.new_directory('foo', tt.root)
+                tt.new_file('bar', foo, [b'foobar'])
+                baz = tt.new_directory('baz', tt.root)
+                tt.new_file('qux', baz, [b'quux'])
+                # Ask for a rename 'foo' -> 'baz'
+                tt.adjust_path('baz', tt.root, foo)
+                # Lie to tt that we've already resolved all conflicts.
+                tt.apply(no_conflicts=True)
+            except BaseException:
+                wt.unlock()
+                raise
+        # The rename will fail because the target directory is not empty (but
+        # raises FileExists anyway).
+        err = self.assertRaises(errors.FileExists, tt_helper)
+        self.assertEndsWith(err.path, "/baz")
+
+    def test_two_directories_clash(self):
+        def tt_helper():
+            wt = self.make_branch_and_tree('.')
+            tt = wt.transform()  # TreeTransform obtains write lock
+            try:
+                foo_1 = tt.new_directory('foo', tt.root)
+                tt.new_directory('bar', foo_1)
+                # Adding the same directory with a different content
+                foo_2 = tt.new_directory('foo', tt.root)
+                tt.new_directory('baz', foo_2)
+                # Lie to tt that we've already resolved all conflicts.
+                tt.apply(no_conflicts=True)
+            except BaseException:
+                wt.unlock()
+                raise
+        err = self.assertRaises(errors.FileExists, tt_helper)
+        self.assertEndsWith(err.path, "/foo")
+
+    def test_two_directories_clash_finalize(self):
+        def tt_helper():
+            wt = self.make_branch_and_tree('.')
+            tt = wt.transform()  # TreeTransform obtains write lock
+            try:
+                foo_1 = tt.new_directory('foo', tt.root)
+                tt.new_directory('bar', foo_1)
+                # Adding the same directory with a different content
+                foo_2 = tt.new_directory('foo', tt.root)
+                tt.new_directory('baz', foo_2)
+                # Lie to tt that we've already resolved all conflicts.
+                tt.apply(no_conflicts=True)
+            except BaseException:
+                tt.finalize()
+                raise
+        err = self.assertRaises(errors.FileExists, tt_helper)
+        self.assertEndsWith(err.path, "/foo")
+
+    def test_file_to_directory(self):
+        wt = self.make_branch_and_tree('.')
+        self.build_tree(['foo'])
+        wt.add(['foo'])
+        wt.commit("one")
+        tt = wt.transform()
+        self.addCleanup(tt.finalize)
+        foo_trans_id = tt.trans_id_tree_path("foo")
+        tt.delete_contents(foo_trans_id)
+        tt.create_directory(foo_trans_id)
+        bar_trans_id = tt.trans_id_tree_path("foo/bar")
+        tt.create_file([b"aa\n"], bar_trans_id)
+        tt.version_file(bar_trans_id, file_id=b"bar-1")
+        tt.apply()
+        self.assertPathExists("foo/bar")
+        with wt.lock_read():
+            self.assertEqual(wt.kind("foo"), "directory")
+        wt.commit("two")
+        changes = wt.changes_from(wt.basis_tree())
+        self.assertFalse(changes.has_changed(), changes)
+
+    def test_file_to_symlink(self):
+        self.requireFeature(SymlinkFeature)
+        wt = self.make_branch_and_tree('.')
+        self.build_tree(['foo'])
+        wt.add(['foo'])
+        wt.commit("one")
+        tt = wt.transform()
+        self.addCleanup(tt.finalize)
+        foo_trans_id = tt.trans_id_tree_path("foo")
+        tt.delete_contents(foo_trans_id)
+        tt.create_symlink("bar", foo_trans_id)
+        tt.apply()
+        self.assertPathExists("foo")
+        wt.lock_read()
+        self.addCleanup(wt.unlock)
+        self.assertEqual(wt.kind("foo"), "symlink")
+
+    def test_file_to_symlink_unsupported(self):
+        wt = self.make_branch_and_tree('.')
+        self.build_tree(['foo'])
+        wt.add(['foo'])
+        wt.commit("one")
+        self.overrideAttr(osutils, 'supports_symlinks', lambda p: False)
+        tt = wt.transform()
+        self.addCleanup(tt.finalize)
+        foo_trans_id = tt.trans_id_tree_path("foo")
+        tt.delete_contents(foo_trans_id)
+        log = BytesIO()
+        trace.push_log_file(log)
+        tt.create_symlink("bar", foo_trans_id)
+        tt.apply()
+        self.assertContainsRe(
+            log.getvalue(),
+            b'Unable to create symlink "foo" on this filesystem')
+
+    def test_dir_to_file(self):
+        wt = self.make_branch_and_tree('.')
+        self.build_tree(['foo/', 'foo/bar'])
+        wt.add(['foo', 'foo/bar'])
+        wt.commit("one")
+        tt = wt.transform()
+        self.addCleanup(tt.finalize)
+        foo_trans_id = tt.trans_id_tree_path("foo")
+        bar_trans_id = tt.trans_id_tree_path("foo/bar")
+        tt.delete_contents(foo_trans_id)
+        tt.delete_versioned(bar_trans_id)
+        tt.create_file([b"aa\n"], foo_trans_id)
+        tt.apply()
+        self.assertPathExists("foo")
+        wt.lock_read()
+        self.addCleanup(wt.unlock)
+        self.assertEqual(wt.kind("foo"), "file")
+
+    def test_dir_to_hardlink(self):
+        self.requireFeature(HardlinkFeature)
+        wt = self.make_branch_and_tree('.')
+        self.build_tree(['foo/', 'foo/bar'])
+        wt.add(['foo', 'foo/bar'])
+        wt.commit("one")
+        tt = wt.transform()
+        self.addCleanup(tt.finalize)
+        foo_trans_id = tt.trans_id_tree_path("foo")
+        bar_trans_id = tt.trans_id_tree_path("foo/bar")
+        tt.delete_contents(foo_trans_id)
+        tt.delete_versioned(bar_trans_id)
+        self.build_tree(['baz'])
+        tt.create_hardlink("baz", foo_trans_id)
+        tt.apply()
+        self.assertPathExists("foo")
+        self.assertPathExists("baz")
+        wt.lock_read()
+        self.addCleanup(wt.unlock)
+        self.assertEqual(wt.kind("foo"), "file")
+
+    def test_no_final_path(self):
+        transform, root = self.transform()
+        trans_id = transform.trans_id_file_id(b'foo')
+        transform.create_file([b'bar'], trans_id)
+        transform.cancel_creation(trans_id)
+        transform.apply()
+
+    def test_create_from_tree(self):
+        tree1 = self.make_branch_and_tree('tree1')
+        self.build_tree_contents([('tree1/foo/',), ('tree1/bar', b'baz')])
+        tree1.add(['foo', 'bar'])
+        tree2 = self.make_branch_and_tree('tree2')
+        tt = tree2.transform()
+        foo_trans_id = tt.create_path('foo', tt.root)
+        create_from_tree(tt, foo_trans_id, tree1, 'foo')
+        bar_trans_id = tt.create_path('bar', tt.root)
+        create_from_tree(tt, bar_trans_id, tree1, 'bar')
+        tt.apply()
+        self.assertEqual('directory', osutils.file_kind('tree2/foo'))
+        self.assertFileEqual(b'baz', 'tree2/bar')
+
+    def test_create_from_tree_bytes(self):
+        """Provided lines are used instead of tree content."""
+        tree1 = self.make_branch_and_tree('tree1')
+        self.build_tree_contents([('tree1/foo', b'bar'), ])
+        tree1.add('foo')
+        tree2 = self.make_branch_and_tree('tree2')
+        tt = tree2.transform()
+        foo_trans_id = tt.create_path('foo', tt.root)
+        create_from_tree(tt, foo_trans_id, tree1, 'foo', chunks=[b'qux'])
+        tt.apply()
+        self.assertFileEqual(b'qux', 'tree2/foo')
+
+    def test_create_from_tree_symlink(self):
+        self.requireFeature(SymlinkFeature)
+        tree1 = self.make_branch_and_tree('tree1')
+        os.symlink('bar', 'tree1/foo')
+        tree1.add('foo')
+        tt = self.make_branch_and_tree('tree2').transform()
+        foo_trans_id = tt.create_path('foo', tt.root)
+        create_from_tree(tt, foo_trans_id, tree1, 'foo')
+        tt.apply()
+        self.assertEqual('bar', os.readlink('tree2/foo'))

=== modified file 'breezy/tests/per_workingtree/test_walkdirs.py'
--- breezy/tests/per_workingtree/test_walkdirs.py	2019-06-17 23:08:34 +0000
+++ breezy/tests/per_workingtree/test_walkdirs.py	2020-07-04 12:29:00 +0000
@@ -176,7 +176,7 @@
         paths = ['file1', 'file2', 'dir1/', 'dir2/']
         self.build_tree(paths)
         tree.add(paths)
-        tt = tree.get_transform()
+        tt = tree.transform()
         root_transaction_id = tt.trans_id_tree_path('')
         tt.new_symlink('link1',
                        root_transaction_id, 'link-target', b'link1')

=== modified file 'breezy/tests/per_workingtree/test_workingtree.py'
--- breezy/tests/per_workingtree/test_workingtree.py	2020-02-07 02:14:30 +0000
+++ breezy/tests/per_workingtree/test_workingtree.py	2020-07-18 23:14:00 +0000
@@ -141,9 +141,9 @@
                     ('zz_dir', 'directory'),
                     ], files)
 
-    def test_get_transform(self):
+    def test_transform(self):
         tree = self.make_branch_and_tree('tree')
-        with tree.get_transform():
+        with tree.transform():
             pass
 
     def test_list_files_kind_change(self):

=== modified file 'breezy/tests/test_conflicts.py'
--- breezy/tests/test_conflicts.py	2020-02-07 02:14:30 +0000
+++ breezy/tests/test_conflicts.py	2020-07-18 23:14:00 +0000
@@ -23,6 +23,7 @@
     option,
     osutils,
     tests,
+    transform,
     )
 from . import (
     script,
@@ -1076,7 +1077,7 @@
         # This is nearly like TestResolveNonDirectoryParent but with branch and
         # trunk switched. As such it should certainly produce the same
         # conflict.
-        self.assertRaises(errors.MalformedTransform,
+        self.assertRaises(transform.MalformedTransform,
                           self.run_script, """
 $ brz init trunk
 ...

=== modified file 'breezy/tests/test_diff.py'
--- breezy/tests/test_diff.py	2020-06-01 19:35:12 +0000
+++ breezy/tests/test_diff.py	2020-07-18 23:14:00 +0000
@@ -559,7 +559,7 @@
     def test_internal_diff_exec_property(self):
         tree = self.make_branch_and_tree('tree')
 
-        tt = tree.get_transform()
+        tt = tree.transform()
         tt.new_file('a', tt.root, [b'contents\n'], b'a-id', True)
         tt.new_file('b', tt.root, [b'contents\n'], b'b-id', False)
         tt.new_file('c', tt.root, [b'contents\n'], b'c-id', True)
@@ -569,7 +569,7 @@
         tt.apply()
         tree.commit('one', rev_id=b'rev-1')
 
-        tt = tree.get_transform()
+        tt = tree.transform()
         tt.set_executability(False, tt.trans_id_file_id(b'a-id'))
         tt.set_executability(True, tt.trans_id_file_id(b'b-id'))
         tt.set_executability(False, tt.trans_id_file_id(b'c-id'))

=== modified file 'breezy/tests/test_errors.py'
--- breezy/tests/test_errors.py	2020-02-07 02:14:30 +0000
+++ breezy/tests/test_errors.py	2020-07-18 23:14:00 +0000
@@ -60,11 +60,6 @@
             "^Filename b?'bad/filen\\\\xe5me' is not valid in your current"
             " filesystem encoding UTF-8$")
 
-    def test_duplicate_file_id(self):
-        error = errors.DuplicateFileId('a_file_id', 'foo')
-        self.assertEqualDiff('File id {a_file_id} already exists in inventory'
-                             ' as foo', str(error))
-
     def test_duplicate_help_prefix(self):
         error = errors.DuplicateHelpPrefix('foo')
         self.assertEqualDiff('The prefix foo is in the help search path twice.',
@@ -118,13 +113,6 @@
         self.assertEqual("Invalid range access in path at 12: bad range",
                          str(error))
 
-    def test_inventory_modified(self):
-        error = errors.InventoryModified("a tree to be repred")
-        self.assertEqualDiff("The current inventory for the tree 'a tree to "
-                             "be repred' has been modified, so a clean inventory cannot be "
-                             "read without data loss.",
-                             str(error))
-
     def test_jail_break(self):
         error = errors.JailBreak("some url")
         self.assertEqualDiff("An attempt to access a url outside the server"
@@ -192,13 +180,6 @@
                              "the currently open request.",
                              str(error))
 
-    def test_unavailable_representation(self):
-        error = errors.UnavailableRepresentation(
-            ('key',), "mpdiff", "fulltext")
-        self.assertEqualDiff("The encoding 'mpdiff' is not available for key "
-                             "('key',) which is encoded as 'fulltext'.",
-                             str(error))
-
     def test_unstackable_location(self):
         error = errors.UnstackableLocationError('foo', 'bar')
         self.assertEqualDiff("The branch 'foo' cannot be stacked on 'bar'.",
@@ -524,12 +505,6 @@
             str(e),
             r'Cannot bind address "example\.com:22":.*Permission denied')
 
-    def test_transform_rename_failed(self):
-        e = errors.TransformRenameFailed(u"from", u"to", "readonly file", 2)
-        self.assertEqual(
-            u"Failed to rename from to to: readonly file",
-            str(e))
-
 
 class TestErrorsUsingTransport(tests.TestCaseWithMemoryTransport):
     """Tests for errors that need to use a branch or repo."""

=== modified file 'breezy/tests/test_import_tariff.py'
--- breezy/tests/test_import_tariff.py	2020-06-23 01:02:30 +0000
+++ breezy/tests/test_import_tariff.py	2020-07-18 23:14:00 +0000
@@ -223,6 +223,7 @@
             'breezy.gpg',
             'breezy.info',
             'breezy.bzr.knit',
+            'breezy.merge',
             'breezy.merge3',
             'breezy.merge_directive',
             'breezy.msgeditor',

=== modified file 'breezy/tests/test_location.py'
--- breezy/tests/test_location.py	2019-11-17 03:58:40 +0000
+++ breezy/tests/test_location.py	2020-06-28 23:13:22 +0000
@@ -83,6 +83,13 @@
                 ':pserver:anonymous@odessa.cvs.sourceforge.net:/cvsroot/odess'))
         self.assertRaises(ValueError, location_to_url, ':pserver:blah')
 
+    def test_missing_scheme(self):
+        self.skipTest('need clever guessing of scheme')
+        self.assertEqual(
+            'cvs+pserver://anonymous@savi.cvs.sourceforge.net:/cvsroot/savi',
+            location_to_url(
+                'anonymous@savi.cvs.sourceforge.net:/cvsroot/savi'))
+
     def test_rcp_url(self):
         self.assertEqual(
             "ssh://example.com/srv/git/bar",

=== modified file 'breezy/tests/test_merge.py'
--- breezy/tests/test_merge.py	2020-02-07 02:14:30 +0000
+++ breezy/tests/test_merge.py	2020-07-18 23:14:00 +0000
@@ -145,7 +145,7 @@
         merger = _mod_merge.Merge3Merger(wt, wt, wt.basis_tree(), other_tree,
                                          this_branch=wt.branch,
                                          do_merge=False)
-        with transform.TransformPreview(wt) as merger.tt:
+        with wt.preview_transform() as merger.tt:
             merger._compute_transform()
             new_root_id = merger.tt.final_file_id(merger.tt.root)
             self.assertEqual(wt.path2id(''), new_root_id)
@@ -2259,7 +2259,7 @@
         builder.build_snapshot([b'C-id', b'B-id'], [], revision_id=b'E-id')
         # Have to use a real WT, because BranchBuilder doesn't support exec bit
         wt = self.get_wt_from_builder(builder)
-        with wt.get_transform() as tt:
+        with wt.transform() as tt:
             tt.set_executability(True, tt.trans_id_tree_path('foo'))
             tt.apply()
         self.assertTrue(wt.is_executable('foo'))

=== modified file 'breezy/tests/test_merge_core.py'
--- breezy/tests/test_merge_core.py	2019-06-17 23:01:58 +0000
+++ breezy/tests/test_merge_core.py	2020-07-04 12:29:00 +0000
@@ -57,7 +57,7 @@
             wt.lock_write()
             wt.set_root_id(self.tree_root)
             wt.flush()
-            tt = wt.get_transform()
+            tt = wt.transform()
             return wt, tt
         self.base, self.base_tt = wt('base')
         self.this, self.this_tt = wt('this')

=== modified file 'breezy/tests/test_osutils.py'
--- breezy/tests/test_osutils.py	2020-05-24 00:42:36 +0000
+++ breezy/tests/test_osutils.py	2020-07-18 23:14:00 +0000
@@ -818,50 +818,6 @@
                           osutils.safe_utf8, b'\xbb\xbb')
 
 
-class TestSafeRevisionId(tests.TestCase):
-
-    def test_from_ascii_string(self):
-        # this shouldn't give a warning because it's getting an ascii string
-        self.assertEqual(b'foobar', osutils.safe_revision_id(b'foobar'))
-
-    def test_from_unicode_string_ascii_contents(self):
-        self.assertRaises(TypeError,
-                          osutils.safe_revision_id, u'bargam')
-
-    def test_from_unicode_string_unicode_contents(self):
-        self.assertRaises(TypeError,
-                          osutils.safe_revision_id, u'bargam\xae')
-
-    def test_from_utf8_string(self):
-        self.assertEqual(b'foo\xc2\xae',
-                         osutils.safe_revision_id(b'foo\xc2\xae'))
-
-    def test_none(self):
-        """Currently, None is a valid revision_id"""
-        self.assertEqual(None, osutils.safe_revision_id(None))
-
-
-class TestSafeFileId(tests.TestCase):
-
-    def test_from_ascii_string(self):
-        self.assertEqual(b'foobar', osutils.safe_file_id(b'foobar'))
-
-    def test_from_unicode_string_ascii_contents(self):
-        self.assertRaises(TypeError, osutils.safe_file_id, u'bargam')
-
-    def test_from_unicode_string_unicode_contents(self):
-        self.assertRaises(TypeError,
-                          osutils.safe_file_id, u'bargam\xae')
-
-    def test_from_utf8_string(self):
-        self.assertEqual(b'foo\xc2\xae',
-                         osutils.safe_file_id(b'foo\xc2\xae'))
-
-    def test_none(self):
-        """Currently, None is a valid revision_id"""
-        self.assertEqual(None, osutils.safe_file_id(None))
-
-
 class TestSendAll(tests.TestCase):
 
     def test_send_with_disconnected_socket(self):

=== modified file 'breezy/tests/test_revert.py'
--- breezy/tests/test_revert.py	2019-06-29 13:16:26 +0000
+++ breezy/tests/test_revert.py	2020-07-04 12:29:00 +0000
@@ -85,7 +85,7 @@
 
     def tree_with_executable(self):
         tree = self.make_branch_and_tree('tree')
-        tt = tree.get_transform()
+        tt = tree.transform()
         tt.new_file('newfile', tt.root, [b'helooo!'], b'newfile-id', True)
         tt.apply()
         with tree.lock_write():
@@ -95,7 +95,7 @@
 
     def test_preserve_execute(self):
         tree = self.tree_with_executable()
-        tt = tree.get_transform()
+        tt = tree.transform()
         newfile = tt.trans_id_tree_path('newfile')
         tt.delete_contents(newfile)
         tt.create_file([b'Woooorld!'], newfile)
@@ -111,7 +111,7 @@
 
     def test_revert_executable(self):
         tree = self.tree_with_executable()
-        tt = tree.get_transform()
+        tt = tree.transform()
         newfile = tt.trans_id_tree_path('newfile')
         tt.set_executability(False, newfile)
         tt.apply()

=== modified file 'breezy/tests/test_rio.py'
--- breezy/tests/test_rio.py	2018-11-11 04:08:32 +0000
+++ breezy/tests/test_rio.py	2020-06-28 21:33:00 +0000
@@ -328,6 +328,21 @@
         s = Stanza()
         self.assertRaises(TypeError, s.add, 10, {})
 
+    def test_rio_surrogateescape(self):
+        raw_bytes = b'\xcb'
+        self.assertRaises(UnicodeDecodeError, raw_bytes.decode, 'utf-8')
+        try:
+            uni_data = raw_bytes.decode('utf-8', 'surrogateescape')
+        except LookupError:
+            self.skipTest('surrogateescape is not available on Python < 3')
+        s = Stanza(foo=uni_data)
+        self.assertEqual(s.get('foo'), uni_data)
+        raw_lines = s.to_lines()
+        self.assertEqual(raw_lines,
+                         [b'foo: ' + uni_data.encode('utf-8', 'surrogateescape') + b'\n'])
+        new_s = read_stanza(raw_lines)
+        self.assertEqual(new_s.get('foo'), uni_data)
+
     def test_rio_unicode(self):
         uni_data = u'\N{KATAKANA LETTER O}'
         s = Stanza(foo=uni_data)

=== modified file 'breezy/tests/test_shelf.py'
--- breezy/tests/test_shelf.py	2019-09-27 02:48:17 +0000
+++ breezy/tests/test_shelf.py	2020-07-04 12:29:00 +0000
@@ -32,9 +32,6 @@
     features,
     KnownFailure,
     )
-from ..errors import (
-    MalformedTransform,
-    )
 
 
 EMPTY_SHELF = (b"Bazaar pack format 1 (introduced in 0.18)\n"
@@ -254,7 +251,7 @@
         try:
             creator.transform()
             self.check_shelve_creation(creator, tree)
-        except MalformedTransform:
+        except transform.MalformedTransform:
             raise KnownFailure(
                 'shelving directory with ignored file: see bug #611739')
 
@@ -503,7 +500,7 @@
         parser = pack.ContainerPushParser()
         with open('shelf', 'rb') as shelf_file:
             parser.accept_bytes(shelf_file.read())
-        tt = transform.TransformPreview(tree)
+        tt = tree.preview_transform()
         self.addCleanup(tt.finalize)
         records = iter(parser.read_pending_records())
         # skip revision-id

=== modified file 'breezy/tests/test_transform.py'
--- breezy/tests/test_transform.py	2020-02-07 02:14:30 +0000
+++ breezy/tests/test_transform.py	2020-07-18 23:14:00 +0000
@@ -51,11 +51,8 @@
     DuplicateKey,
     ExistingLimbo,
     ExistingPendingDeletion,
-    ImmortalLimbo,
     ImmortalPendingDeletion,
     LockError,
-    MalformedTransform,
-    ReusingTransform,
 )
 from ..osutils import (
     file_kind,
@@ -81,1619 +78,14 @@
     resolve_conflicts,
     resolve_checkout,
     ROOT_PARENT,
-    TransformPreview,
-    TreeTransform,
+    ImmortalLimbo,
+    MalformedTransform,
+    NoFinalPath,
+    ReusingTransform,
+    TransformRenameFailed,
 )
 
 
-class TestTreeTransform(tests.TestCaseWithTransport):
-
-    def setUp(self):
-        super(TestTreeTransform, self).setUp()
-        self.wt = self.make_branch_and_tree('.', format='development-subtree')
-        os.chdir('..')
-
-    def get_transform(self):
-        transform = TreeTransform(self.wt)
-        self.addCleanup(transform.finalize)
-        return transform, transform.root
-
-    def get_transform_for_sha1_test(self):
-        trans, root = self.get_transform()
-        self.wt.lock_tree_write()
-        self.addCleanup(self.wt.unlock)
-        contents = [b'just some content\n']
-        sha1 = osutils.sha_strings(contents)
-        # Roll back the clock
-        trans._creation_mtime = time.time() - 20.0
-        return trans, root, contents, sha1
-
-    def test_existing_limbo(self):
-        transform, root = self.get_transform()
-        limbo_name = transform._limbodir
-        deletion_path = transform._deletiondir
-        os.mkdir(pathjoin(limbo_name, 'hehe'))
-        self.assertRaises(ImmortalLimbo, transform.apply)
-        self.assertRaises(LockError, self.wt.unlock)
-        self.assertRaises(ExistingLimbo, self.get_transform)
-        self.assertRaises(LockError, self.wt.unlock)
-        os.rmdir(pathjoin(limbo_name, 'hehe'))
-        os.rmdir(limbo_name)
-        os.rmdir(deletion_path)
-        transform, root = self.get_transform()
-        transform.apply()
-
-    def test_existing_pending_deletion(self):
-        transform, root = self.get_transform()
-        deletion_path = self._limbodir = urlutils.local_path_from_url(
-            transform._tree._transport.abspath('pending-deletion'))
-        os.mkdir(pathjoin(deletion_path, 'blocking-directory'))
-        self.assertRaises(ImmortalPendingDeletion, transform.apply)
-        self.assertRaises(LockError, self.wt.unlock)
-        self.assertRaises(ExistingPendingDeletion, self.get_transform)
-
-    def test_build(self):
-        transform, root = self.get_transform()
-        self.wt.lock_tree_write()
-        self.addCleanup(self.wt.unlock)
-        self.assertIs(transform.get_tree_parent(root), ROOT_PARENT)
-        imaginary_id = transform.trans_id_tree_path('imaginary')
-        imaginary_id2 = transform.trans_id_tree_path('imaginary/')
-        self.assertEqual(imaginary_id, imaginary_id2)
-        self.assertEqual(root, transform.get_tree_parent(imaginary_id))
-        self.assertEqual('directory', transform.final_kind(root))
-        self.assertEqual(self.wt.path2id(''), transform.final_file_id(root))
-        trans_id = transform.create_path('name', root)
-        self.assertIs(transform.final_file_id(trans_id), None)
-        self.assertIs(None, transform.final_kind(trans_id))
-        transform.create_file([b'contents'], trans_id)
-        transform.set_executability(True, trans_id)
-        transform.version_file(b'my_pretties', trans_id)
-        self.assertRaises(DuplicateKey, transform.version_file,
-                          b'my_pretties', trans_id)
-        self.assertEqual(transform.final_file_id(trans_id), b'my_pretties')
-        self.assertEqual(transform.final_parent(trans_id), root)
-        self.assertIs(transform.final_parent(root), ROOT_PARENT)
-        self.assertIs(transform.get_tree_parent(root), ROOT_PARENT)
-        oz_id = transform.create_path('oz', root)
-        transform.create_directory(oz_id)
-        transform.version_file(b'ozzie', oz_id)
-        trans_id2 = transform.create_path('name2', root)
-        transform.create_file([b'contents'], trans_id2)
-        transform.set_executability(False, trans_id2)
-        transform.version_file(b'my_pretties2', trans_id2)
-        modified_paths = transform.apply().modified_paths
-        with self.wt.get_file('name') as f:
-            self.assertEqual(b'contents', f.read())
-        self.assertEqual(self.wt.path2id('name'), b'my_pretties')
-        self.assertIs(self.wt.is_executable('name'), True)
-        self.assertIs(self.wt.is_executable('name2'), False)
-        self.assertEqual('directory', file_kind(self.wt.abspath('oz')))
-        self.assertEqual(len(modified_paths), 3)
-        tree_mod_paths = [self.wt.abspath(self.wt.id2path(f)) for f in
-                          (b'ozzie', b'my_pretties', b'my_pretties2')]
-        self.assertSubset(tree_mod_paths, modified_paths)
-        # is it safe to finalize repeatedly?
-        transform.finalize()
-        transform.finalize()
-
-    def test_apply_informs_tree_of_observed_sha1(self):
-        trans, root, contents, sha1 = self.get_transform_for_sha1_test()
-        trans_id = trans.new_file('file1', root, contents, file_id=b'file1-id',
-                                  sha1=sha1)
-        calls = []
-        orig = self.wt._observed_sha1
-
-        def _observed_sha1(*args):
-            calls.append(args)
-            orig(*args)
-        self.wt._observed_sha1 = _observed_sha1
-        trans.apply()
-        self.assertEqual([('file1', trans._observed_sha1s[trans_id])],
-                         calls)
-
-    def test_create_file_caches_sha1(self):
-        trans, root, contents, sha1 = self.get_transform_for_sha1_test()
-        trans_id = trans.create_path('file1', root)
-        trans.create_file(contents, trans_id, sha1=sha1)
-        st_val = osutils.lstat(trans._limbo_name(trans_id))
-        o_sha1, o_st_val = trans._observed_sha1s[trans_id]
-        self.assertEqual(o_sha1, sha1)
-        self.assertEqualStat(o_st_val, st_val)
-
-    def test__apply_insertions_updates_sha1(self):
-        trans, root, contents, sha1 = self.get_transform_for_sha1_test()
-        trans_id = trans.create_path('file1', root)
-        trans.create_file(contents, trans_id, sha1=sha1)
-        st_val = osutils.lstat(trans._limbo_name(trans_id))
-        o_sha1, o_st_val = trans._observed_sha1s[trans_id]
-        self.assertEqual(o_sha1, sha1)
-        self.assertEqualStat(o_st_val, st_val)
-        creation_mtime = trans._creation_mtime + 10.0
-        # We fake a time difference from when the file was created until now it
-        # is being renamed by using os.utime. Note that the change we actually
-        # want to see is the real ctime change from 'os.rename()', but as long
-        # as we observe a new stat value, we should be fine.
-        os.utime(trans._limbo_name(trans_id), (creation_mtime, creation_mtime))
-        trans.apply()
-        new_st_val = osutils.lstat(self.wt.abspath('file1'))
-        o_sha1, o_st_val = trans._observed_sha1s[trans_id]
-        self.assertEqual(o_sha1, sha1)
-        self.assertEqualStat(o_st_val, new_st_val)
-        self.assertNotEqual(st_val.st_mtime, new_st_val.st_mtime)
-
-    def test_new_file_caches_sha1(self):
-        trans, root, contents, sha1 = self.get_transform_for_sha1_test()
-        trans_id = trans.new_file('file1', root, contents, file_id=b'file1-id',
-                                  sha1=sha1)
-        st_val = osutils.lstat(trans._limbo_name(trans_id))
-        o_sha1, o_st_val = trans._observed_sha1s[trans_id]
-        self.assertEqual(o_sha1, sha1)
-        self.assertEqualStat(o_st_val, st_val)
-
-    def test_cancel_creation_removes_observed_sha1(self):
-        trans, root, contents, sha1 = self.get_transform_for_sha1_test()
-        trans_id = trans.new_file('file1', root, contents, file_id=b'file1-id',
-                                  sha1=sha1)
-        self.assertTrue(trans_id in trans._observed_sha1s)
-        trans.cancel_creation(trans_id)
-        self.assertFalse(trans_id in trans._observed_sha1s)
-
-    def test_create_files_same_timestamp(self):
-        transform, root = self.get_transform()
-        self.wt.lock_tree_write()
-        self.addCleanup(self.wt.unlock)
-        # Roll back the clock, so that we know everything is being set to the
-        # exact time
-        transform._creation_mtime = creation_mtime = time.time() - 20.0
-        transform.create_file([b'content-one'],
-                              transform.create_path('one', root))
-        time.sleep(1)  # *ugly*
-        transform.create_file([b'content-two'],
-                              transform.create_path('two', root))
-        transform.apply()
-        fo, st1 = self.wt.get_file_with_stat('one', filtered=False)
-        fo.close()
-        fo, st2 = self.wt.get_file_with_stat('two', filtered=False)
-        fo.close()
-        # We only guarantee 2s resolution
-        self.assertTrue(
-            abs(creation_mtime - st1.st_mtime) < 2.0,
-            "%s != %s within 2 seconds" % (creation_mtime, st1.st_mtime))
-        # But if we have more than that, all files should get the same result
-        self.assertEqual(st1.st_mtime, st2.st_mtime)
-
-    def test_change_root_id(self):
-        transform, root = self.get_transform()
-        self.assertNotEqual(b'new-root-id', self.wt.path2id(''))
-        transform.new_directory('', ROOT_PARENT, b'new-root-id')
-        transform.delete_contents(root)
-        transform.unversion_file(root)
-        transform.fixup_new_roots()
-        transform.apply()
-        self.assertEqual(b'new-root-id', self.wt.path2id(''))
-
-    def test_change_root_id_add_files(self):
-        transform, root = self.get_transform()
-        self.assertNotEqual(b'new-root-id', self.wt.path2id(''))
-        new_trans_id = transform.new_directory('', ROOT_PARENT, b'new-root-id')
-        transform.new_file('file', new_trans_id, [b'new-contents\n'],
-                           b'new-file-id')
-        transform.delete_contents(root)
-        transform.unversion_file(root)
-        transform.fixup_new_roots()
-        transform.apply()
-        self.assertEqual(b'new-root-id', self.wt.path2id(''))
-        self.assertEqual(b'new-file-id', self.wt.path2id('file'))
-        self.assertFileEqual(b'new-contents\n', self.wt.abspath('file'))
-
-    def test_add_two_roots(self):
-        transform, root = self.get_transform()
-        transform.new_directory('', ROOT_PARENT, b'new-root-id')
-        transform.new_directory('', ROOT_PARENT, b'alt-root-id')
-        self.assertRaises(ValueError, transform.fixup_new_roots)
-
-    def test_retain_existing_root(self):
-        tt, root = self.get_transform()
-        with tt:
-            tt.new_directory('', ROOT_PARENT, b'new-root-id')
-            tt.fixup_new_roots()
-            self.assertNotEqual(b'new-root-id', tt.final_file_id(tt.root))
-
-    def test_retain_existing_root_added_file(self):
-        tt, root = self.get_transform()
-        new_trans_id = tt.new_directory('', ROOT_PARENT, b'new-root-id')
-        child = tt.new_directory('child', new_trans_id, b'child-id')
-        tt.fixup_new_roots()
-        self.assertEqual(tt.root, tt.final_parent(child))
-
-    def test_add_unversioned_root(self):
-        transform, root = self.get_transform()
-        transform.new_directory('', ROOT_PARENT, None)
-        transform.delete_contents(transform.root)
-        transform.fixup_new_roots()
-        self.assertNotIn(transform.root, transform._new_id)
-
-    def test_remove_root_fixup(self):
-        transform, root = self.get_transform()
-        old_root_id = self.wt.path2id('')
-        self.assertNotEqual(b'new-root-id', old_root_id)
-        transform.delete_contents(root)
-        transform.unversion_file(root)
-        transform.fixup_new_roots()
-        transform.apply()
-        self.assertEqual(old_root_id, self.wt.path2id(''))
-
-        transform, root = self.get_transform()
-        transform.new_directory('', ROOT_PARENT, b'new-root-id')
-        transform.new_directory('', ROOT_PARENT, b'alt-root-id')
-        self.assertRaises(ValueError, transform.fixup_new_roots)
-
-    def test_fixup_new_roots_permits_empty_tree(self):
-        transform, root = self.get_transform()
-        transform.delete_contents(root)
-        transform.unversion_file(root)
-        transform.fixup_new_roots()
-        self.assertIs(None, transform.final_kind(root))
-        self.assertIs(None, transform.final_file_id(root))
-
-    def test_apply_retains_root_directory(self):
-        # Do not attempt to delete the physical root directory, because that
-        # is impossible.
-        transform, root = self.get_transform()
-        with transform:
-            transform.delete_contents(root)
-            e = self.assertRaises(AssertionError, self.assertRaises,
-                                  errors.TransformRenameFailed,
-                                  transform.apply)
-        self.assertContainsRe('TransformRenameFailed not raised', str(e))
-
-    def test_apply_retains_file_id(self):
-        transform, root = self.get_transform()
-        old_root_id = transform.tree_file_id(root)
-        transform.unversion_file(root)
-        transform.apply()
-        self.assertEqual(old_root_id, self.wt.path2id(''))
-
-    def test_hardlink(self):
-        self.requireFeature(HardlinkFeature)
-        transform, root = self.get_transform()
-        transform.new_file('file1', root, [b'contents'])
-        transform.apply()
-        target = self.make_branch_and_tree('target')
-        target_transform = TreeTransform(target)
-        trans_id = target_transform.create_path('file1', target_transform.root)
-        target_transform.create_hardlink(self.wt.abspath('file1'), trans_id)
-        target_transform.apply()
-        self.assertPathExists('target/file1')
-        source_stat = os.stat(self.wt.abspath('file1'))
-        target_stat = os.stat('target/file1')
-        self.assertEqual(source_stat, target_stat)
-
-    def test_convenience(self):
-        transform, root = self.get_transform()
-        self.wt.lock_tree_write()
-        self.addCleanup(self.wt.unlock)
-        transform.new_file('name', root, [b'contents'], b'my_pretties', True)
-        oz = transform.new_directory('oz', root, b'oz-id')
-        dorothy = transform.new_directory('dorothy', oz, b'dorothy-id')
-        transform.new_file('toto', dorothy, [b'toto-contents'], b'toto-id',
-                           False)
-
-        self.assertEqual(len(transform.find_conflicts()), 0)
-        transform.apply()
-        self.assertRaises(ReusingTransform, transform.find_conflicts)
-        with open(self.wt.abspath('name'), 'r') as f:
-            self.assertEqual('contents', f.read())
-        self.assertEqual(self.wt.path2id('name'), b'my_pretties')
-        self.assertIs(self.wt.is_executable('name'), True)
-        self.assertEqual(self.wt.path2id('oz'), b'oz-id')
-        self.assertEqual(self.wt.path2id('oz/dorothy'), b'dorothy-id')
-        self.assertEqual(self.wt.path2id('oz/dorothy/toto'), b'toto-id')
-
-        self.assertEqual(b'toto-contents',
-                         self.wt.get_file('oz/dorothy/toto').read())
-        self.assertIs(self.wt.is_executable('oz/dorothy/toto'), False)
-
-    def test_tree_reference(self):
-        transform, root = self.get_transform()
-        tree = transform._tree
-        trans_id = transform.new_directory('reference', root, b'subtree-id')
-        transform.set_tree_reference(b'subtree-revision', trans_id)
-        transform.apply()
-        tree.lock_read()
-        self.addCleanup(tree.unlock)
-        self.assertEqual(
-            b'subtree-revision',
-            tree.root_inventory.get_entry(b'subtree-id').reference_revision)
-
-    def test_conflicts(self):
-        transform, root = self.get_transform()
-        trans_id = transform.new_file('name', root, [b'contents'],
-                                      b'my_pretties')
-        self.assertEqual(len(transform.find_conflicts()), 0)
-        trans_id2 = transform.new_file('name', root, [b'Crontents'], b'toto')
-        self.assertEqual(transform.find_conflicts(),
-                         [('duplicate', trans_id, trans_id2, 'name')])
-        self.assertRaises(MalformedTransform, transform.apply)
-        transform.adjust_path('name', trans_id, trans_id2)
-        self.assertEqual(transform.find_conflicts(),
-                         [('non-directory parent', trans_id)])
-        tinman_id = transform.trans_id_tree_path('tinman')
-        transform.adjust_path('name', tinman_id, trans_id2)
-        self.assertEqual(transform.find_conflicts(),
-                         [('unversioned parent', tinman_id),
-                          ('missing parent', tinman_id)])
-        lion_id = transform.create_path('lion', root)
-        self.assertEqual(transform.find_conflicts(),
-                         [('unversioned parent', tinman_id),
-                          ('missing parent', tinman_id)])
-        transform.adjust_path('name', lion_id, trans_id2)
-        self.assertEqual(transform.find_conflicts(),
-                         [('unversioned parent', lion_id),
-                          ('missing parent', lion_id)])
-        transform.version_file(b"Courage", lion_id)
-        self.assertEqual(transform.find_conflicts(),
-                         [('missing parent', lion_id),
-                          ('versioning no contents', lion_id)])
-        transform.adjust_path('name2', root, trans_id2)
-        self.assertEqual(transform.find_conflicts(),
-                         [('versioning no contents', lion_id)])
-        transform.create_file([b'Contents, okay?'], lion_id)
-        transform.adjust_path('name2', trans_id2, trans_id2)
-        self.assertEqual(transform.find_conflicts(),
-                         [('parent loop', trans_id2),
-                          ('non-directory parent', trans_id2)])
-        transform.adjust_path('name2', root, trans_id2)
-        oz_id = transform.new_directory('oz', root)
-        transform.set_executability(True, oz_id)
-        self.assertEqual(transform.find_conflicts(),
-                         [('unversioned executability', oz_id)])
-        transform.version_file(b'oz-id', oz_id)
-        self.assertEqual(transform.find_conflicts(),
-                         [('non-file executability', oz_id)])
-        transform.set_executability(None, oz_id)
-        tip_id = transform.new_file('tip', oz_id, [b'ozma'], b'tip-id')
-        transform.apply()
-        self.assertEqual(self.wt.path2id('name'), b'my_pretties')
-        with open(self.wt.abspath('name'), 'rb') as f:
-            self.assertEqual(b'contents', f.read())
-        transform2, root = self.get_transform()
-        oz_id = transform2.trans_id_tree_path('oz')
-        newtip = transform2.new_file('tip', oz_id, [b'other'], b'tip-id')
-        result = transform2.find_conflicts()
-        fp = FinalPaths(transform2)
-        self.assertTrue('oz/tip' in transform2._tree_path_ids)
-        self.assertEqual(fp.get_path(newtip), pathjoin('oz', 'tip'))
-        self.assertEqual(len(result), 2)
-        self.assertEqual((result[0][0], result[0][1]),
-                         ('duplicate', newtip))
-        self.assertEqual((result[1][0], result[1][2]),
-                         ('duplicate id', newtip))
-        transform2.finalize()
-        transform3 = TreeTransform(self.wt)
-        self.addCleanup(transform3.finalize)
-        oz_id = transform3.trans_id_tree_path('oz')
-        transform3.delete_contents(oz_id)
-        self.assertEqual(transform3.find_conflicts(),
-                         [('missing parent', oz_id)])
-        root_id = transform3.root
-        tip_id = transform3.trans_id_tree_path('oz/tip')
-        transform3.adjust_path('tip', root_id, tip_id)
-        transform3.apply()
-
-    def test_conflict_on_case_insensitive(self):
-        tree = self.make_branch_and_tree('tree')
-        # Don't try this at home, kids!
-        # Force the tree to report that it is case sensitive, for conflict
-        # resolution tests
-        tree.case_sensitive = True
-        transform = TreeTransform(tree)
-        self.addCleanup(transform.finalize)
-        transform.new_file('file', transform.root, [b'content'])
-        transform.new_file('FiLe', transform.root, [b'content'])
-        result = transform.find_conflicts()
-        self.assertEqual([], result)
-        transform.finalize()
-        # Force the tree to report that it is case insensitive, for conflict
-        # generation tests
-        tree.case_sensitive = False
-        transform = TreeTransform(tree)
-        self.addCleanup(transform.finalize)
-        transform.new_file('file', transform.root, [b'content'])
-        transform.new_file('FiLe', transform.root, [b'content'])
-        result = transform.find_conflicts()
-        self.assertEqual([('duplicate', 'new-1', 'new-2', 'file')], result)
-
-    def test_conflict_on_case_insensitive_existing(self):
-        tree = self.make_branch_and_tree('tree')
-        self.build_tree(['tree/FiLe'])
-        # Don't try this at home, kids!
-        # Force the tree to report that it is case sensitive, for conflict
-        # resolution tests
-        tree.case_sensitive = True
-        transform = TreeTransform(tree)
-        self.addCleanup(transform.finalize)
-        transform.new_file('file', transform.root, [b'content'])
-        result = transform.find_conflicts()
-        self.assertEqual([], result)
-        transform.finalize()
-        # Force the tree to report that it is case insensitive, for conflict
-        # generation tests
-        tree.case_sensitive = False
-        transform = TreeTransform(tree)
-        self.addCleanup(transform.finalize)
-        transform.new_file('file', transform.root, [b'content'])
-        result = transform.find_conflicts()
-        self.assertEqual([('duplicate', 'new-1', 'new-2', 'file')], result)
-
-    def test_resolve_case_insensitive_conflict(self):
-        tree = self.make_branch_and_tree('tree')
-        # Don't try this at home, kids!
-        # Force the tree to report that it is case insensitive, for conflict
-        # resolution tests
-        tree.case_sensitive = False
-        transform = TreeTransform(tree)
-        self.addCleanup(transform.finalize)
-        transform.new_file('file', transform.root, [b'content'])
-        transform.new_file('FiLe', transform.root, [b'content'])
-        resolve_conflicts(transform)
-        transform.apply()
-        self.assertPathExists('tree/file')
-        self.assertPathExists('tree/FiLe.moved')
-
-    def test_resolve_checkout_case_conflict(self):
-        tree = self.make_branch_and_tree('tree')
-        # Don't try this at home, kids!
-        # Force the tree to report that it is case insensitive, for conflict
-        # resolution tests
-        tree.case_sensitive = False
-        transform = TreeTransform(tree)
-        self.addCleanup(transform.finalize)
-        transform.new_file('file', transform.root, [b'content'])
-        transform.new_file('FiLe', transform.root, [b'content'])
-        resolve_conflicts(transform,
-                          pass_func=lambda t, c: resolve_checkout(t, c, []))
-        transform.apply()
-        self.assertPathExists('tree/file')
-        self.assertPathExists('tree/FiLe.moved')
-
-    def test_apply_case_conflict(self):
-        """Ensure that a transform with case conflicts can always be applied"""
-        tree = self.make_branch_and_tree('tree')
-        transform = TreeTransform(tree)
-        self.addCleanup(transform.finalize)
-        transform.new_file('file', transform.root, [b'content'])
-        transform.new_file('FiLe', transform.root, [b'content'])
-        dir = transform.new_directory('dir', transform.root)
-        transform.new_file('dirfile', dir, [b'content'])
-        transform.new_file('dirFiLe', dir, [b'content'])
-        resolve_conflicts(transform)
-        transform.apply()
-        self.assertPathExists('tree/file')
-        if not os.path.exists('tree/FiLe.moved'):
-            self.assertPathExists('tree/FiLe')
-        self.assertPathExists('tree/dir/dirfile')
-        if not os.path.exists('tree/dir/dirFiLe.moved'):
-            self.assertPathExists('tree/dir/dirFiLe')
-
-    def test_case_insensitive_limbo(self):
-        tree = self.make_branch_and_tree('tree')
-        # Don't try this at home, kids!
-        # Force the tree to report that it is case insensitive
-        tree.case_sensitive = False
-        transform = TreeTransform(tree)
-        self.addCleanup(transform.finalize)
-        dir = transform.new_directory('dir', transform.root)
-        first = transform.new_file('file', dir, [b'content'])
-        second = transform.new_file('FiLe', dir, [b'content'])
-        self.assertContainsRe(transform._limbo_name(first), 'new-1/file')
-        self.assertNotContainsRe(transform._limbo_name(second), 'new-1/FiLe')
-
-    def test_adjust_path_updates_child_limbo_names(self):
-        tree = self.make_branch_and_tree('tree')
-        transform = TreeTransform(tree)
-        self.addCleanup(transform.finalize)
-        foo_id = transform.new_directory('foo', transform.root)
-        bar_id = transform.new_directory('bar', foo_id)
-        baz_id = transform.new_directory('baz', bar_id)
-        qux_id = transform.new_directory('qux', baz_id)
-        transform.adjust_path('quxx', foo_id, bar_id)
-        self.assertStartsWith(transform._limbo_name(qux_id),
-                              transform._limbo_name(bar_id))
-
-    def test_add_del(self):
-        start, root = self.get_transform()
-        start.new_directory('a', root, b'a')
-        start.apply()
-        transform, root = self.get_transform()
-        transform.delete_versioned(transform.trans_id_tree_path('a'))
-        transform.new_directory('a', root, b'a')
-        transform.apply()
-
-    def test_unversioning(self):
-        create_tree, root = self.get_transform()
-        parent_id = create_tree.new_directory('parent', root, b'parent-id')
-        create_tree.new_file('child', parent_id, [b'child'], b'child-id')
-        create_tree.apply()
-        unversion = TreeTransform(self.wt)
-        self.addCleanup(unversion.finalize)
-        parent = unversion.trans_id_tree_path('parent')
-        unversion.unversion_file(parent)
-        self.assertEqual(unversion.find_conflicts(),
-                         [('unversioned parent', parent_id)])
-        file_id = unversion.trans_id_tree_path('parent/child')
-        unversion.unversion_file(file_id)
-        unversion.apply()
-
-    def test_name_invariants(self):
-        create_tree, root = self.get_transform()
-        # prepare tree
-        root = create_tree.root
-        create_tree.new_file('name1', root, [b'hello1'], b'name1')
-        create_tree.new_file('name2', root, [b'hello2'], b'name2')
-        ddir = create_tree.new_directory('dying_directory', root, b'ddir')
-        create_tree.new_file('dying_file', ddir, [b'goodbye1'], b'dfile')
-        create_tree.new_file('moving_file', ddir, [b'later1'], b'mfile')
-        create_tree.new_file('moving_file2', root, [b'later2'], b'mfile2')
-        create_tree.apply()
-
-        mangle_tree, root = self.get_transform()
-        root = mangle_tree.root
-        # swap names
-        name1 = mangle_tree.trans_id_tree_path('name1')
-        name2 = mangle_tree.trans_id_tree_path('name2')
-        mangle_tree.adjust_path('name2', root, name1)
-        mangle_tree.adjust_path('name1', root, name2)
-
-        # tests for deleting parent directories
-        ddir = mangle_tree.trans_id_tree_path('dying_directory')
-        mangle_tree.delete_contents(ddir)
-        dfile = mangle_tree.trans_id_tree_path('dying_directory/dying_file')
-        mangle_tree.delete_versioned(dfile)
-        mangle_tree.unversion_file(dfile)
-        mfile = mangle_tree.trans_id_tree_path('dying_directory/moving_file')
-        mangle_tree.adjust_path('mfile', root, mfile)
-
-        # tests for adding parent directories
-        newdir = mangle_tree.new_directory('new_directory', root, b'newdir')
-        mfile2 = mangle_tree.trans_id_tree_path('moving_file2')
-        mangle_tree.adjust_path('mfile2', newdir, mfile2)
-        mangle_tree.new_file('newfile', newdir, [b'hello3'], b'dfile')
-        self.assertEqual(mangle_tree.final_file_id(mfile2), b'mfile2')
-        self.assertEqual(mangle_tree.final_parent(mfile2), newdir)
-        self.assertEqual(mangle_tree.final_file_id(mfile2), b'mfile2')
-        mangle_tree.apply()
-        with open(self.wt.abspath('name1'), 'r') as f:
-            self.assertEqual(f.read(), 'hello2')
-        with open(self.wt.abspath('name2'), 'r') as f:
-            self.assertEqual(f.read(), 'hello1')
-        mfile2_path = self.wt.abspath(pathjoin('new_directory', 'mfile2'))
-        self.assertEqual(mangle_tree.final_parent(mfile2), newdir)
-        with open(mfile2_path, 'r') as f:
-            self.assertEqual(f.read(), 'later2')
-        self.assertEqual(self.wt.id2path(b'mfile2'), 'new_directory/mfile2')
-        self.assertEqual(self.wt.path2id('new_directory/mfile2'), b'mfile2')
-        newfile_path = self.wt.abspath(pathjoin('new_directory', 'newfile'))
-        with open(newfile_path, 'r') as f:
-            self.assertEqual(f.read(), 'hello3')
-        self.assertEqual(self.wt.path2id('dying_directory'), b'ddir')
-        self.assertIs(self.wt.path2id('dying_directory/dying_file'), None)
-        mfile2_path = self.wt.abspath(pathjoin('new_directory', 'mfile2'))
-
-    def test_both_rename(self):
-        create_tree, root = self.get_transform()
-        newdir = create_tree.new_directory('selftest', root, b'selftest-id')
-        create_tree.new_file('blackbox.py', newdir, [
-                             b'hello1'], b'blackbox-id')
-        create_tree.apply()
-        mangle_tree, root = self.get_transform()
-        selftest = mangle_tree.trans_id_tree_path('selftest')
-        blackbox = mangle_tree.trans_id_tree_path('selftest/blackbox.py')
-        mangle_tree.adjust_path('test', root, selftest)
-        mangle_tree.adjust_path('test_too_much', root, selftest)
-        mangle_tree.set_executability(True, blackbox)
-        mangle_tree.apply()
-
-    def test_both_rename2(self):
-        create_tree, root = self.get_transform()
-        breezy = create_tree.new_directory('breezy', root, b'breezy-id')
-        tests = create_tree.new_directory('tests', breezy, b'tests-id')
-        blackbox = create_tree.new_directory('blackbox', tests, b'blackbox-id')
-        create_tree.new_file('test_too_much.py', blackbox, [b'hello1'],
-                             b'test_too_much-id')
-        create_tree.apply()
-        mangle_tree, root = self.get_transform()
-        breezy = mangle_tree.trans_id_tree_path('breezy')
-        tests = mangle_tree.trans_id_tree_path('breezy/tests')
-        test_too_much = mangle_tree.trans_id_tree_path(
-            'breezy/tests/blackbox/test_too_much.py')
-        mangle_tree.adjust_path('selftest', breezy, tests)
-        mangle_tree.adjust_path('blackbox.py', tests, test_too_much)
-        mangle_tree.set_executability(True, test_too_much)
-        mangle_tree.apply()
-
-    def test_both_rename3(self):
-        create_tree, root = self.get_transform()
-        tests = create_tree.new_directory('tests', root, b'tests-id')
-        create_tree.new_file('test_too_much.py', tests, [b'hello1'],
-                             b'test_too_much-id')
-        create_tree.apply()
-        mangle_tree, root = self.get_transform()
-        tests = mangle_tree.trans_id_tree_path('tests')
-        test_too_much = mangle_tree.trans_id_tree_path(
-            'tests/test_too_much.py')
-        mangle_tree.adjust_path('selftest', root, tests)
-        mangle_tree.adjust_path('blackbox.py', tests, test_too_much)
-        mangle_tree.set_executability(True, test_too_much)
-        mangle_tree.apply()
-
-    def test_move_dangling_ie(self):
-        create_tree, root = self.get_transform()
-        # prepare tree
-        root = create_tree.root
-        create_tree.new_file('name1', root, [b'hello1'], b'name1')
-        create_tree.apply()
-        delete_contents, root = self.get_transform()
-        file = delete_contents.trans_id_tree_path('name1')
-        delete_contents.delete_contents(file)
-        delete_contents.apply()
-        move_id, root = self.get_transform()
-        name1 = move_id.trans_id_tree_path('name1')
-        newdir = move_id.new_directory('dir', root, b'newdir')
-        move_id.adjust_path('name2', newdir, name1)
-        move_id.apply()
-
-    def test_replace_dangling_ie(self):
-        create_tree, root = self.get_transform()
-        # prepare tree
-        root = create_tree.root
-        create_tree.new_file('name1', root, [b'hello1'], b'name1')
-        create_tree.apply()
-        delete_contents = TreeTransform(self.wt)
-        self.addCleanup(delete_contents.finalize)
-        file = delete_contents.trans_id_tree_path('name1')
-        delete_contents.delete_contents(file)
-        delete_contents.apply()
-        delete_contents.finalize()
-        replace = TreeTransform(self.wt)
-        self.addCleanup(replace.finalize)
-        name2 = replace.new_file('name2', root, [b'hello2'], b'name1')
-        conflicts = replace.find_conflicts()
-        name1 = replace.trans_id_tree_path('name1')
-        self.assertEqual(conflicts, [('duplicate id', name1, name2)])
-        resolve_conflicts(replace)
-        replace.apply()
-
-    def _test_symlinks(self, link_name1, link_target1,
-                       link_name2, link_target2):
-
-        def ozpath(p):
-            return 'oz/' + p
-
-        self.requireFeature(SymlinkFeature)
-        transform, root = self.get_transform()
-        oz_id = transform.new_directory('oz', root, b'oz-id')
-        transform.new_symlink(link_name1, oz_id, link_target1, b'wizard-id')
-        wiz_id = transform.create_path(link_name2, oz_id)
-        transform.create_symlink(link_target2, wiz_id)
-        transform.version_file(b'wiz-id2', wiz_id)
-        transform.set_executability(True, wiz_id)
-        self.assertEqual(transform.find_conflicts(),
-                         [('non-file executability', wiz_id)])
-        transform.set_executability(None, wiz_id)
-        transform.apply()
-        self.assertEqual(self.wt.path2id(ozpath(link_name1)), b'wizard-id')
-        self.assertEqual('symlink',
-                         file_kind(self.wt.abspath(ozpath(link_name1))))
-        self.assertEqual(link_target2,
-                         osutils.readlink(self.wt.abspath(ozpath(link_name2))))
-        self.assertEqual(link_target1,
-                         osutils.readlink(self.wt.abspath(ozpath(link_name1))))
-
-    def test_symlinks(self):
-        self._test_symlinks('wizard', 'wizard-target',
-                            'wizard2', 'behind_curtain')
-
-    def test_symlinks_unicode(self):
-        self.requireFeature(features.UnicodeFilenameFeature)
-        self._test_symlinks(u'\N{Euro Sign}wizard',
-                            u'wizard-targ\N{Euro Sign}t',
-                            u'\N{Euro Sign}wizard2',
-                            u'b\N{Euro Sign}hind_curtain')
-
-    def test_unsupported_symlink_no_conflict(self):
-        def tt_helper():
-            wt = self.make_branch_and_tree('.')
-            tt = TreeTransform(wt)
-            self.addCleanup(tt.finalize)
-            tt.new_symlink('foo', tt.root, 'bar')
-            result = tt.find_conflicts()
-            self.assertEqual([], result)
-        os_symlink = getattr(os, 'symlink', None)
-        os.symlink = None
-        try:
-            tt_helper()
-        finally:
-            if os_symlink:
-                os.symlink = os_symlink
-
-    def get_conflicted(self):
-        create, root = self.get_transform()
-        create.new_file('dorothy', root, [b'dorothy'], b'dorothy-id')
-        oz = create.new_directory('oz', root, b'oz-id')
-        create.new_directory('emeraldcity', oz, b'emerald-id')
-        create.apply()
-        conflicts, root = self.get_transform()
-        # set up duplicate entry, duplicate id
-        new_dorothy = conflicts.new_file('dorothy', root, [b'dorothy'],
-                                         b'dorothy-id')
-        old_dorothy = conflicts.trans_id_tree_path('dorothy')
-        oz = conflicts.trans_id_tree_path('oz')
-        # set up DeletedParent parent conflict
-        conflicts.delete_versioned(oz)
-        emerald = conflicts.trans_id_tree_path('oz/emeraldcity')
-        # set up MissingParent conflict
-        munchkincity = conflicts.trans_id_file_id(b'munchkincity-id')
-        conflicts.adjust_path('munchkincity', root, munchkincity)
-        conflicts.new_directory('auntem', munchkincity, b'auntem-id')
-        # set up parent loop
-        conflicts.adjust_path('emeraldcity', emerald, emerald)
-        return conflicts, emerald, oz, old_dorothy, new_dorothy
-
-    def test_conflict_resolution(self):
-        conflicts, emerald, oz, old_dorothy, new_dorothy =\
-            self.get_conflicted()
-        resolve_conflicts(conflicts)
-        self.assertEqual(conflicts.final_name(old_dorothy), 'dorothy.moved')
-        self.assertIs(conflicts.final_file_id(old_dorothy), None)
-        self.assertEqual(conflicts.final_name(new_dorothy), 'dorothy')
-        self.assertEqual(conflicts.final_file_id(new_dorothy), b'dorothy-id')
-        self.assertEqual(conflicts.final_parent(emerald), oz)
-        conflicts.apply()
-
-    def test_cook_conflicts(self):
-        tt, emerald, oz, old_dorothy, new_dorothy = self.get_conflicted()
-        raw_conflicts = resolve_conflicts(tt)
-        cooked_conflicts = cook_conflicts(raw_conflicts, tt)
-        duplicate = DuplicateEntry('Moved existing file to', 'dorothy.moved',
-                                   'dorothy', None, b'dorothy-id')
-        self.assertEqual(cooked_conflicts[0], duplicate)
-        duplicate_id = DuplicateID('Unversioned existing file',
-                                   'dorothy.moved', 'dorothy', None,
-                                   b'dorothy-id')
-        self.assertEqual(cooked_conflicts[1], duplicate_id)
-        missing_parent = MissingParent('Created directory', 'munchkincity',
-                                       b'munchkincity-id')
-        deleted_parent = DeletingParent('Not deleting', 'oz', b'oz-id')
-        self.assertEqual(cooked_conflicts[2], missing_parent)
-        unversioned_parent = UnversionedParent('Versioned directory',
-                                               'munchkincity',
-                                               b'munchkincity-id')
-        unversioned_parent2 = UnversionedParent('Versioned directory', 'oz',
-                                                b'oz-id')
-        self.assertEqual(cooked_conflicts[3], unversioned_parent)
-        parent_loop = ParentLoop(
-            'Cancelled move', 'oz/emeraldcity',
-            'oz/emeraldcity', b'emerald-id', b'emerald-id')
-        self.assertEqual(cooked_conflicts[4], deleted_parent)
-        self.assertEqual(cooked_conflicts[5], unversioned_parent2)
-        self.assertEqual(cooked_conflicts[6], parent_loop)
-        self.assertEqual(len(cooked_conflicts), 7)
-        tt.finalize()
-
-    def test_string_conflicts(self):
-        tt, emerald, oz, old_dorothy, new_dorothy = self.get_conflicted()
-        raw_conflicts = resolve_conflicts(tt)
-        cooked_conflicts = cook_conflicts(raw_conflicts, tt)
-        tt.finalize()
-        conflicts_s = [str(c) for c in cooked_conflicts]
-        self.assertEqual(len(cooked_conflicts), len(conflicts_s))
-        self.assertEqual(conflicts_s[0], 'Conflict adding file dorothy.  '
-                                         'Moved existing file to '
-                                         'dorothy.moved.')
-        self.assertEqual(conflicts_s[1], 'Conflict adding id to dorothy.  '
-                                         'Unversioned existing file '
-                                         'dorothy.moved.')
-        self.assertEqual(conflicts_s[2], 'Conflict adding files to'
-                                         ' munchkincity.  Created directory.')
-        self.assertEqual(conflicts_s[3], 'Conflict because munchkincity is not'
-                                         ' versioned, but has versioned'
-                                         ' children.  Versioned directory.')
-        self.assertEqualDiff(
-            conflicts_s[4], "Conflict: can't delete oz because it"
-                            " is not empty.  Not deleting.")
-        self.assertEqual(conflicts_s[5], 'Conflict because oz is not'
-                                         ' versioned, but has versioned'
-                                         ' children.  Versioned directory.')
-        self.assertEqual(conflicts_s[6], 'Conflict moving oz/emeraldcity into'
-                                         ' oz/emeraldcity. Cancelled move.')
-
-    def prepare_wrong_parent_kind(self):
-        tt, root = self.get_transform()
-        tt.new_file('parent', root, [b'contents'], b'parent-id')
-        tt.apply()
-        tt, root = self.get_transform()
-        parent_id = tt.trans_id_file_id(b'parent-id')
-        tt.new_file('child,', parent_id, [b'contents2'], b'file-id')
-        return tt
-
-    def test_find_conflicts_wrong_parent_kind(self):
-        tt = self.prepare_wrong_parent_kind()
-        tt.find_conflicts()
-
-    def test_resolve_conflicts_wrong_existing_parent_kind(self):
-        tt = self.prepare_wrong_parent_kind()
-        raw_conflicts = resolve_conflicts(tt)
-        self.assertEqual({('non-directory parent', 'Created directory',
-                           'new-3')}, raw_conflicts)
-        cooked_conflicts = cook_conflicts(raw_conflicts, tt)
-        self.assertEqual([NonDirectoryParent('Created directory', 'parent.new',
-                                             b'parent-id')], cooked_conflicts)
-        tt.apply()
-        self.assertFalse(self.wt.is_versioned('parent'))
-        self.assertEqual(b'parent-id', self.wt.path2id('parent.new'))
-
-    def test_resolve_conflicts_wrong_new_parent_kind(self):
-        tt, root = self.get_transform()
-        parent_id = tt.new_directory('parent', root, b'parent-id')
-        tt.new_file('child,', parent_id, [b'contents2'], b'file-id')
-        tt.apply()
-        tt, root = self.get_transform()
-        parent_id = tt.trans_id_file_id(b'parent-id')
-        tt.delete_contents(parent_id)
-        tt.create_file([b'contents'], parent_id)
-        raw_conflicts = resolve_conflicts(tt)
-        self.assertEqual({('non-directory parent', 'Created directory',
-                           'new-3')}, raw_conflicts)
-        tt.apply()
-        self.assertFalse(self.wt.is_versioned('parent'))
-        self.assertEqual(b'parent-id', self.wt.path2id('parent.new'))
-
-    def test_resolve_conflicts_wrong_parent_kind_unversioned(self):
-        tt, root = self.get_transform()
-        parent_id = tt.new_directory('parent', root)
-        tt.new_file('child,', parent_id, [b'contents2'])
-        tt.apply()
-        tt, root = self.get_transform()
-        parent_id = tt.trans_id_tree_path('parent')
-        tt.delete_contents(parent_id)
-        tt.create_file([b'contents'], parent_id)
-        resolve_conflicts(tt)
-        tt.apply()
-        self.assertFalse(self.wt.is_versioned('parent'))
-        self.assertFalse(self.wt.is_versioned('parent.new'))
-
-    def test_resolve_conflicts_missing_parent(self):
-        wt = self.make_branch_and_tree('.')
-        tt = TreeTransform(wt)
-        self.addCleanup(tt.finalize)
-        parent = tt.trans_id_file_id(b'parent-id')
-        tt.new_file('file', parent, [b'Contents'])
-        raw_conflicts = resolve_conflicts(tt)
-        # Since the directory doesn't exist it's seen as 'missing'.  So
-        # 'resolve_conflicts' create a conflict asking for it to be created.
-        self.assertLength(1, raw_conflicts)
-        self.assertEqual(('missing parent', 'Created directory', 'new-1'),
-                         raw_conflicts.pop())
-        # apply fail since the missing directory doesn't exist
-        self.assertRaises(errors.NoFinalPath, tt.apply)
-
-    def test_moving_versioned_directories(self):
-        create, root = self.get_transform()
-        kansas = create.new_directory('kansas', root, b'kansas-id')
-        create.new_directory('house', kansas, b'house-id')
-        create.new_directory('oz', root, b'oz-id')
-        create.apply()
-        cyclone, root = self.get_transform()
-        oz = cyclone.trans_id_tree_path('oz')
-        house = cyclone.trans_id_tree_path('house')
-        cyclone.adjust_path('house', oz, house)
-        cyclone.apply()
-
-    def test_moving_root(self):
-        create, root = self.get_transform()
-        fun = create.new_directory('fun', root, b'fun-id')
-        create.new_directory('sun', root, b'sun-id')
-        create.new_directory('moon', root, b'moon')
-        create.apply()
-        transform, root = self.get_transform()
-        transform.adjust_root_path('oldroot', fun)
-        new_root = transform.trans_id_tree_path('')
-        transform.version_file(b'new-root', new_root)
-        transform.apply()
-
-    def test_renames(self):
-        create, root = self.get_transform()
-        old = create.new_directory('old-parent', root, b'old-id')
-        intermediate = create.new_directory('intermediate', old, b'im-id')
-        myfile = create.new_file('myfile', intermediate, [b'myfile-text'],
-                                 b'myfile-id')
-        create.apply()
-        rename, root = self.get_transform()
-        old = rename.trans_id_file_id(b'old-id')
-        rename.adjust_path('new', root, old)
-        myfile = rename.trans_id_file_id(b'myfile-id')
-        rename.set_executability(True, myfile)
-        rename.apply()
-
-    def test_rename_fails(self):
-        self.requireFeature(features.not_running_as_root)
-        # see https://bugs.launchpad.net/bzr/+bug/491763
-        create, root_id = self.get_transform()
-        create.new_directory('first-dir', root_id, b'first-id')
-        create.new_file('myfile', root_id, [b'myfile-text'], b'myfile-id')
-        create.apply()
-        if os.name == "posix" and sys.platform != "cygwin":
-            # posix filesystems fail on renaming if the readonly bit is set
-            osutils.make_readonly(self.wt.abspath('first-dir'))
-        elif os.name == "nt":
-            # windows filesystems fail on renaming open files
-            self.addCleanup(open(self.wt.abspath('myfile')).close)
-        else:
-            self.skipTest("Can't force a permissions error on rename")
-        # now transform to rename
-        rename_transform, root_id = self.get_transform()
-        file_trans_id = rename_transform.trans_id_file_id(b'myfile-id')
-        dir_id = rename_transform.trans_id_file_id(b'first-id')
-        rename_transform.adjust_path('newname', dir_id, file_trans_id)
-        e = self.assertRaises(errors.TransformRenameFailed,
-                              rename_transform.apply)
-        # On nix looks like:
-        # "Failed to rename .../work/.bzr/checkout/limbo/new-1
-        # to .../first-dir/newname: [Errno 13] Permission denied"
-        # On windows looks like:
-        # "Failed to rename .../work/myfile to
-        # .../work/.bzr/checkout/limbo/new-1: [Errno 13] Permission denied"
-        # This test isn't concerned with exactly what the error looks like,
-        # and the strerror will vary across OS and locales, but the assert
-        # that the exeception attributes are what we expect
-        self.assertEqual(e.errno, errno.EACCES)
-        if os.name == "posix":
-            self.assertEndsWith(e.to_path, "/first-dir/newname")
-        else:
-            self.assertEqual(os.path.basename(e.from_path), "myfile")
-
-    def test_set_executability_order(self):
-        """Ensure that executability behaves the same, no matter what order.
-
-        - create file and set executability simultaneously
-        - create file and set executability afterward
-        - unsetting the executability of a file whose executability has not
-          been
-        declared should throw an exception (this may happen when a
-        merge attempts to create a file with a duplicate ID)
-        """
-        transform, root = self.get_transform()
-        wt = transform._tree
-        wt.lock_read()
-        self.addCleanup(wt.unlock)
-        transform.new_file('set_on_creation', root, [b'Set on creation'],
-                           b'soc', True)
-        sac = transform.new_file('set_after_creation', root,
-                                 [b'Set after creation'], b'sac')
-        transform.set_executability(True, sac)
-        uws = transform.new_file('unset_without_set', root, [b'Unset badly'],
-                                 b'uws')
-        self.assertRaises(KeyError, transform.set_executability, None, uws)
-        transform.apply()
-        self.assertTrue(wt.is_executable('set_on_creation'))
-        self.assertTrue(wt.is_executable('set_after_creation'))
-
-    def test_preserve_mode(self):
-        """File mode is preserved when replacing content"""
-        if sys.platform == 'win32':
-            raise TestSkipped('chmod has no effect on win32')
-        transform, root = self.get_transform()
-        transform.new_file('file1', root, [b'contents'], b'file1-id', True)
-        transform.apply()
-        self.wt.lock_write()
-        self.addCleanup(self.wt.unlock)
-        self.assertTrue(self.wt.is_executable('file1'))
-        transform, root = self.get_transform()
-        file1_id = transform.trans_id_tree_path('file1')
-        transform.delete_contents(file1_id)
-        transform.create_file([b'contents2'], file1_id)
-        transform.apply()
-        self.assertTrue(self.wt.is_executable('file1'))
-
-    def test__set_mode_stats_correctly(self):
-        """_set_mode stats to determine file mode."""
-        if sys.platform == 'win32':
-            raise TestSkipped('chmod has no effect on win32')
-
-        stat_paths = []
-        real_stat = os.stat
-
-        def instrumented_stat(path):
-            stat_paths.append(path)
-            return real_stat(path)
-
-        transform, root = self.get_transform()
-
-        bar1_id = transform.new_file('bar', root, [b'bar contents 1\n'],
-                                     file_id=b'bar-id-1', executable=False)
-        transform.apply()
-
-        transform, root = self.get_transform()
-        bar1_id = transform.trans_id_tree_path('bar')
-        bar2_id = transform.trans_id_tree_path('bar2')
-        try:
-            os.stat = instrumented_stat
-            transform.create_file([b'bar2 contents\n'],
-                                  bar2_id, mode_id=bar1_id)
-        finally:
-            os.stat = real_stat
-            transform.finalize()
-
-        bar1_abspath = self.wt.abspath('bar')
-        self.assertEqual([bar1_abspath], stat_paths)
-
-    def test_iter_changes(self):
-        self.wt.set_root_id(b'eert_toor')
-        transform, root = self.get_transform()
-        transform.new_file('old', root, [b'blah'], b'id-1', True)
-        transform.apply()
-        transform, root = self.get_transform()
-        try:
-            self.assertEqual([], list(transform.iter_changes()))
-            old = transform.trans_id_tree_path('old')
-            transform.unversion_file(old)
-            self.assertEqual([(b'id-1', ('old', None), False, (True, False),
-                               (b'eert_toor', b'eert_toor'),
-                               ('old', 'old'), ('file', 'file'),
-                               (True, True), False)],
-                             list(transform.iter_changes()))
-            transform.new_directory('new', root, b'id-1')
-            self.assertEqual([(b'id-1', ('old', 'new'), True, (True, True),
-                               (b'eert_toor', b'eert_toor'), ('old', 'new'),
-                               ('file', 'directory'),
-                               (True, False), False)],
-                             list(transform.iter_changes()))
-        finally:
-            transform.finalize()
-
-    def test_iter_changes_new(self):
-        self.wt.set_root_id(b'eert_toor')
-        transform, root = self.get_transform()
-        transform.new_file('old', root, [b'blah'])
-        transform.apply()
-        transform, root = self.get_transform()
-        try:
-            old = transform.trans_id_tree_path('old')
-            transform.version_file(b'id-1', old)
-            self.assertEqual([(b'id-1', (None, 'old'), False, (False, True),
-                               (b'eert_toor', b'eert_toor'),
-                               ('old', 'old'), ('file', 'file'),
-                               (False, False), False)],
-                             list(transform.iter_changes()))
-        finally:
-            transform.finalize()
-
-    def test_iter_changes_modifications(self):
-        self.wt.set_root_id(b'eert_toor')
-        transform, root = self.get_transform()
-        transform.new_file('old', root, [b'blah'], b'id-1')
-        transform.new_file('new', root, [b'blah'])
-        transform.new_directory('subdir', root, b'subdir-id')
-        transform.apply()
-        transform, root = self.get_transform()
-        try:
-            old = transform.trans_id_tree_path('old')
-            subdir = transform.trans_id_tree_path('subdir')
-            new = transform.trans_id_tree_path('new')
-            self.assertEqual([], list(transform.iter_changes()))
-
-            # content deletion
-            transform.delete_contents(old)
-            self.assertEqual([(b'id-1', ('old', 'old'), True, (True, True),
-                               (b'eert_toor', b'eert_toor'),
-                               ('old', 'old'), ('file', None),
-                               (False, False), False)],
-                             list(transform.iter_changes()))
-
-            # content change
-            transform.create_file([b'blah'], old)
-            self.assertEqual([(b'id-1', ('old', 'old'), True, (True, True),
-                               (b'eert_toor', b'eert_toor'),
-                               ('old', 'old'), ('file', 'file'),
-                               (False, False), False)],
-                             list(transform.iter_changes()))
-            transform.cancel_deletion(old)
-            self.assertEqual([(b'id-1', ('old', 'old'), True, (True, True),
-                               (b'eert_toor', b'eert_toor'),
-                               ('old', 'old'), ('file', 'file'),
-                               (False, False), False)],
-                             list(transform.iter_changes()))
-            transform.cancel_creation(old)
-
-            # move file_id to a different file
-            self.assertEqual([], list(transform.iter_changes()))
-            transform.unversion_file(old)
-            transform.version_file(b'id-1', new)
-            transform.adjust_path('old', root, new)
-            self.assertEqual([(b'id-1', ('old', 'old'), True, (True, True),
-                               (b'eert_toor', b'eert_toor'),
-                               ('old', 'old'), ('file', 'file'),
-                               (False, False), False)],
-                             list(transform.iter_changes()))
-            transform.cancel_versioning(new)
-            transform._removed_id = set()
-
-            # execute bit
-            self.assertEqual([], list(transform.iter_changes()))
-            transform.set_executability(True, old)
-            self.assertEqual([(b'id-1', ('old', 'old'), False, (True, True),
-                               (b'eert_toor', b'eert_toor'),
-                               ('old', 'old'), ('file', 'file'),
-                               (False, True), False)],
-                             list(transform.iter_changes()))
-            transform.set_executability(None, old)
-
-            # filename
-            self.assertEqual([], list(transform.iter_changes()))
-            transform.adjust_path('new', root, old)
-            transform._new_parent = {}
-            self.assertEqual([(b'id-1', ('old', 'new'), False, (True, True),
-                               (b'eert_toor', b'eert_toor'),
-                               ('old', 'new'), ('file', 'file'),
-                               (False, False), False)],
-                             list(transform.iter_changes()))
-            transform._new_name = {}
-
-            # parent directory
-            self.assertEqual([], list(transform.iter_changes()))
-            transform.adjust_path('new', subdir, old)
-            transform._new_name = {}
-            self.assertEqual([(b'id-1', ('old', 'subdir/old'), False,
-                               (True, True), (b'eert_toor',
-                                              b'subdir-id'), ('old', 'old'),
-                               ('file', 'file'), (False, False), False)],
-                             list(transform.iter_changes()))
-            transform._new_path = {}
-
-        finally:
-            transform.finalize()
-
-    def test_iter_changes_modified_bleed(self):
-        self.wt.set_root_id(b'eert_toor')
-        """Modified flag should not bleed from one change to another"""
-        # unfortunately, we have no guarantee that file1 (which is modified)
-        # will be applied before file2.  And if it's applied after file2, it
-        # obviously can't bleed into file2's change output.  But for now, it
-        # works.
-        transform, root = self.get_transform()
-        transform.new_file('file1', root, [b'blah'], b'id-1')
-        transform.new_file('file2', root, [b'blah'], b'id-2')
-        transform.apply()
-        transform, root = self.get_transform()
-        try:
-            transform.delete_contents(transform.trans_id_file_id(b'id-1'))
-            transform.set_executability(True,
-                                        transform.trans_id_file_id(b'id-2'))
-            self.assertEqual(
-                [(b'id-1', (u'file1', u'file1'), True, (True, True),
-                 (b'eert_toor', b'eert_toor'), ('file1', u'file1'),
-                 ('file', None), (False, False), False),
-                 (b'id-2', (u'file2', u'file2'), False, (True, True),
-                 (b'eert_toor', b'eert_toor'), ('file2', u'file2'),
-                 ('file', 'file'), (False, True), False)],
-                list(transform.iter_changes()))
-        finally:
-            transform.finalize()
-
-    def test_iter_changes_move_missing(self):
-        """Test moving ids with no files around"""
-        self.wt.set_root_id(b'toor_eert')
-        # Need two steps because versioning a non-existant file is a conflict.
-        transform, root = self.get_transform()
-        transform.new_directory('floater', root, b'floater-id')
-        transform.apply()
-        transform, root = self.get_transform()
-        transform.delete_contents(transform.trans_id_tree_path('floater'))
-        transform.apply()
-        transform, root = self.get_transform()
-        floater = transform.trans_id_tree_path('floater')
-        try:
-            transform.adjust_path('flitter', root, floater)
-            self.assertEqual([(b'floater-id', ('floater', 'flitter'), False,
-                               (True, True),
-                               (b'toor_eert', b'toor_eert'),
-                               ('floater', 'flitter'),
-                               (None, None), (False, False), False)],
-                             list(transform.iter_changes()))
-        finally:
-            transform.finalize()
-
-    def test_iter_changes_pointless(self):
-        """Ensure that no-ops are not treated as modifications"""
-        self.wt.set_root_id(b'eert_toor')
-        transform, root = self.get_transform()
-        transform.new_file('old', root, [b'blah'], b'id-1')
-        transform.new_directory('subdir', root, b'subdir-id')
-        transform.apply()
-        transform, root = self.get_transform()
-        try:
-            old = transform.trans_id_tree_path('old')
-            subdir = transform.trans_id_tree_path('subdir')
-            self.assertEqual([], list(transform.iter_changes()))
-            transform.delete_contents(subdir)
-            transform.create_directory(subdir)
-            transform.set_executability(False, old)
-            transform.unversion_file(old)
-            transform.version_file(b'id-1', old)
-            transform.adjust_path('old', root, old)
-            self.assertEqual([], list(transform.iter_changes()))
-        finally:
-            transform.finalize()
-
-    def test_rename_count(self):
-        transform, root = self.get_transform()
-        transform.new_file('name1', root, [b'contents'])
-        self.assertEqual(transform.rename_count, 0)
-        transform.apply()
-        self.assertEqual(transform.rename_count, 1)
-        transform2, root = self.get_transform()
-        transform2.adjust_path('name2', root,
-                               transform2.trans_id_tree_path('name1'))
-        self.assertEqual(transform2.rename_count, 0)
-        transform2.apply()
-        self.assertEqual(transform2.rename_count, 2)
-
-    def test_change_parent(self):
-        """Ensure that after we change a parent, the results are still right.
-
-        Renames and parent changes on pending transforms can happen as part
-        of conflict resolution, and are explicitly permitted by the
-        TreeTransform API.
-
-        This test ensures they work correctly with the rename-avoidance
-        optimization.
-        """
-        transform, root = self.get_transform()
-        parent1 = transform.new_directory('parent1', root)
-        child1 = transform.new_file('child1', parent1, [b'contents'])
-        parent2 = transform.new_directory('parent2', root)
-        transform.adjust_path('child1', parent2, child1)
-        transform.apply()
-        self.assertPathDoesNotExist(self.wt.abspath('parent1/child1'))
-        self.assertPathExists(self.wt.abspath('parent2/child1'))
-        # rename limbo/new-1 => parent1, rename limbo/new-3 => parent2
-        # no rename for child1 (counting only renames during apply)
-        self.assertEqual(2, transform.rename_count)
-
-    def test_cancel_parent(self):
-        """Cancelling a parent doesn't cause deletion of a non-empty directory
-
-        This is like the test_change_parent, except that we cancel the parent
-        before adjusting the path.  The transform must detect that the
-        directory is non-empty, and move children to safe locations.
-        """
-        transform, root = self.get_transform()
-        parent1 = transform.new_directory('parent1', root)
-        child1 = transform.new_file('child1', parent1, [b'contents'])
-        child2 = transform.new_file('child2', parent1, [b'contents'])
-        try:
-            transform.cancel_creation(parent1)
-        except OSError:
-            self.fail('Failed to move child1 before deleting parent1')
-        transform.cancel_creation(child2)
-        transform.create_directory(parent1)
-        try:
-            transform.cancel_creation(parent1)
-        # If the transform incorrectly believes that child2 is still in
-        # parent1's limbo directory, it will try to rename it and fail
-        # because was already moved by the first cancel_creation.
-        except OSError:
-            self.fail('Transform still thinks child2 is a child of parent1')
-        parent2 = transform.new_directory('parent2', root)
-        transform.adjust_path('child1', parent2, child1)
-        transform.apply()
-        self.assertPathDoesNotExist(self.wt.abspath('parent1'))
-        self.assertPathExists(self.wt.abspath('parent2/child1'))
-        # rename limbo/new-3 => parent2, rename limbo/new-2 => child1
-        self.assertEqual(2, transform.rename_count)
-
-    def test_adjust_and_cancel(self):
-        """Make sure adjust_path keeps track of limbo children properly"""
-        transform, root = self.get_transform()
-        parent1 = transform.new_directory('parent1', root)
-        child1 = transform.new_file('child1', parent1, [b'contents'])
-        parent2 = transform.new_directory('parent2', root)
-        transform.adjust_path('child1', parent2, child1)
-        transform.cancel_creation(child1)
-        try:
-            transform.cancel_creation(parent1)
-        # if the transform thinks child1 is still in parent1's limbo
-        # directory, it will attempt to move it and fail.
-        except OSError:
-            self.fail('Transform still thinks child1 is a child of parent1')
-        transform.finalize()
-
-    def test_noname_contents(self):
-        """TreeTransform should permit deferring naming files."""
-        transform, root = self.get_transform()
-        parent = transform.trans_id_file_id(b'parent-id')
-        try:
-            transform.create_directory(parent)
-        except KeyError:
-            self.fail("Can't handle contents with no name")
-        transform.finalize()
-
-    def test_noname_contents_nested(self):
-        """TreeTransform should permit deferring naming files."""
-        transform, root = self.get_transform()
-        parent = transform.trans_id_file_id(b'parent-id')
-        try:
-            transform.create_directory(parent)
-        except KeyError:
-            self.fail("Can't handle contents with no name")
-        transform.new_directory('child', parent)
-        transform.adjust_path('parent', root, parent)
-        transform.apply()
-        self.assertPathExists(self.wt.abspath('parent/child'))
-        self.assertEqual(1, transform.rename_count)
-
-    def test_reuse_name(self):
-        """Avoid reusing the same limbo name for different files"""
-        transform, root = self.get_transform()
-        parent = transform.new_directory('parent', root)
-        transform.new_directory('child', parent)
-        try:
-            child2 = transform.new_directory('child', parent)
-        except OSError:
-            self.fail('Tranform tried to use the same limbo name twice')
-        transform.adjust_path('child2', parent, child2)
-        transform.apply()
-        # limbo/new-1 => parent, limbo/new-3 => parent/child2
-        # child2 is put into top-level limbo because child1 has already
-        # claimed the direct limbo path when child2 is created.  There is no
-        # advantage in renaming files once they're in top-level limbo, except
-        # as part of apply.
-        self.assertEqual(2, transform.rename_count)
-
-    def test_reuse_when_first_moved(self):
-        """Don't avoid direct paths when it is safe to use them"""
-        transform, root = self.get_transform()
-        parent = transform.new_directory('parent', root)
-        child1 = transform.new_directory('child', parent)
-        transform.adjust_path('child1', parent, child1)
-        transform.new_directory('child', parent)
-        transform.apply()
-        # limbo/new-1 => parent
-        self.assertEqual(1, transform.rename_count)
-
-    def test_reuse_after_cancel(self):
-        """Don't avoid direct paths when it is safe to use them"""
-        transform, root = self.get_transform()
-        parent2 = transform.new_directory('parent2', root)
-        child1 = transform.new_directory('child1', parent2)
-        transform.cancel_creation(parent2)
-        transform.create_directory(parent2)
-        transform.new_directory('child1', parent2)
-        transform.adjust_path('child2', parent2, child1)
-        transform.apply()
-        # limbo/new-1 => parent2, limbo/new-2 => parent2/child1
-        self.assertEqual(2, transform.rename_count)
-
-    def test_finalize_order(self):
-        """Finalize must be done in child-to-parent order"""
-        transform, root = self.get_transform()
-        parent = transform.new_directory('parent', root)
-        transform.new_directory('child', parent)
-        try:
-            transform.finalize()
-        except OSError:
-            self.fail('Tried to remove parent before child1')
-
-    def test_cancel_with_cancelled_child_should_succeed(self):
-        transform, root = self.get_transform()
-        parent = transform.new_directory('parent', root)
-        child = transform.new_directory('child', parent)
-        transform.cancel_creation(child)
-        transform.cancel_creation(parent)
-        transform.finalize()
-
-    def test_rollback_on_directory_clash(self):
-        def tt_helper():
-            wt = self.make_branch_and_tree('.')
-            tt = TreeTransform(wt)  # TreeTransform obtains write lock
-            try:
-                foo = tt.new_directory('foo', tt.root)
-                tt.new_file('bar', foo, [b'foobar'])
-                baz = tt.new_directory('baz', tt.root)
-                tt.new_file('qux', baz, [b'quux'])
-                # Ask for a rename 'foo' -> 'baz'
-                tt.adjust_path('baz', tt.root, foo)
-                # Lie to tt that we've already resolved all conflicts.
-                tt.apply(no_conflicts=True)
-            except BaseException:
-                wt.unlock()
-                raise
-        # The rename will fail because the target directory is not empty (but
-        # raises FileExists anyway).
-        err = self.assertRaises(errors.FileExists, tt_helper)
-        self.assertEndsWith(err.path, "/baz")
-
-    def test_two_directories_clash(self):
-        def tt_helper():
-            wt = self.make_branch_and_tree('.')
-            tt = TreeTransform(wt)  # TreeTransform obtains write lock
-            try:
-                foo_1 = tt.new_directory('foo', tt.root)
-                tt.new_directory('bar', foo_1)
-                # Adding the same directory with a different content
-                foo_2 = tt.new_directory('foo', tt.root)
-                tt.new_directory('baz', foo_2)
-                # Lie to tt that we've already resolved all conflicts.
-                tt.apply(no_conflicts=True)
-            except BaseException:
-                wt.unlock()
-                raise
-        err = self.assertRaises(errors.FileExists, tt_helper)
-        self.assertEndsWith(err.path, "/foo")
-
-    def test_two_directories_clash_finalize(self):
-        def tt_helper():
-            wt = self.make_branch_and_tree('.')
-            tt = TreeTransform(wt)  # TreeTransform obtains write lock
-            try:
-                foo_1 = tt.new_directory('foo', tt.root)
-                tt.new_directory('bar', foo_1)
-                # Adding the same directory with a different content
-                foo_2 = tt.new_directory('foo', tt.root)
-                tt.new_directory('baz', foo_2)
-                # Lie to tt that we've already resolved all conflicts.
-                tt.apply(no_conflicts=True)
-            except BaseException:
-                tt.finalize()
-                raise
-        err = self.assertRaises(errors.FileExists, tt_helper)
-        self.assertEndsWith(err.path, "/foo")
-
-    def test_file_to_directory(self):
-        wt = self.make_branch_and_tree('.')
-        self.build_tree(['foo'])
-        wt.add(['foo'])
-        wt.commit("one")
-        tt = TreeTransform(wt)
-        self.addCleanup(tt.finalize)
-        foo_trans_id = tt.trans_id_tree_path("foo")
-        tt.delete_contents(foo_trans_id)
-        tt.create_directory(foo_trans_id)
-        bar_trans_id = tt.trans_id_tree_path("foo/bar")
-        tt.create_file([b"aa\n"], bar_trans_id)
-        tt.version_file(b"bar-1", bar_trans_id)
-        tt.apply()
-        self.assertPathExists("foo/bar")
-        wt.lock_read()
-        try:
-            self.assertEqual(wt.kind("foo"), "directory")
-        finally:
-            wt.unlock()
-        wt.commit("two")
-        changes = wt.changes_from(wt.basis_tree())
-        self.assertFalse(changes.has_changed(), changes)
-
-    def test_file_to_symlink(self):
-        self.requireFeature(SymlinkFeature)
-        wt = self.make_branch_and_tree('.')
-        self.build_tree(['foo'])
-        wt.add(['foo'])
-        wt.commit("one")
-        tt = TreeTransform(wt)
-        self.addCleanup(tt.finalize)
-        foo_trans_id = tt.trans_id_tree_path("foo")
-        tt.delete_contents(foo_trans_id)
-        tt.create_symlink("bar", foo_trans_id)
-        tt.apply()
-        self.assertPathExists("foo")
-        wt.lock_read()
-        self.addCleanup(wt.unlock)
-        self.assertEqual(wt.kind("foo"), "symlink")
-
-    def test_file_to_symlink_unsupported(self):
-        wt = self.make_branch_and_tree('.')
-        self.build_tree(['foo'])
-        wt.add(['foo'])
-        wt.commit("one")
-        self.overrideAttr(osutils, 'supports_symlinks', lambda p: False)
-        tt = TreeTransform(wt)
-        self.addCleanup(tt.finalize)
-        foo_trans_id = tt.trans_id_tree_path("foo")
-        tt.delete_contents(foo_trans_id)
-        log = BytesIO()
-        trace.push_log_file(log)
-        tt.create_symlink("bar", foo_trans_id)
-        tt.apply()
-        self.assertContainsRe(
-            log.getvalue(),
-            b'Unable to create symlink "foo" on this filesystem')
-
-    def test_dir_to_file(self):
-        wt = self.make_branch_and_tree('.')
-        self.build_tree(['foo/', 'foo/bar'])
-        wt.add(['foo', 'foo/bar'])
-        wt.commit("one")
-        tt = TreeTransform(wt)
-        self.addCleanup(tt.finalize)
-        foo_trans_id = tt.trans_id_tree_path("foo")
-        bar_trans_id = tt.trans_id_tree_path("foo/bar")
-        tt.delete_contents(foo_trans_id)
-        tt.delete_versioned(bar_trans_id)
-        tt.create_file([b"aa\n"], foo_trans_id)
-        tt.apply()
-        self.assertPathExists("foo")
-        wt.lock_read()
-        self.addCleanup(wt.unlock)
-        self.assertEqual(wt.kind("foo"), "file")
-
-    def test_dir_to_hardlink(self):
-        self.requireFeature(HardlinkFeature)
-        wt = self.make_branch_and_tree('.')
-        self.build_tree(['foo/', 'foo/bar'])
-        wt.add(['foo', 'foo/bar'])
-        wt.commit("one")
-        tt = TreeTransform(wt)
-        self.addCleanup(tt.finalize)
-        foo_trans_id = tt.trans_id_tree_path("foo")
-        bar_trans_id = tt.trans_id_tree_path("foo/bar")
-        tt.delete_contents(foo_trans_id)
-        tt.delete_versioned(bar_trans_id)
-        self.build_tree(['baz'])
-        tt.create_hardlink("baz", foo_trans_id)
-        tt.apply()
-        self.assertPathExists("foo")
-        self.assertPathExists("baz")
-        wt.lock_read()
-        self.addCleanup(wt.unlock)
-        self.assertEqual(wt.kind("foo"), "file")
-
-    def test_no_final_path(self):
-        transform, root = self.get_transform()
-        trans_id = transform.trans_id_file_id(b'foo')
-        transform.create_file([b'bar'], trans_id)
-        transform.cancel_creation(trans_id)
-        transform.apply()
-
-    def test_create_from_tree(self):
-        tree1 = self.make_branch_and_tree('tree1')
-        self.build_tree_contents([('tree1/foo/',), ('tree1/bar', b'baz')])
-        tree1.add(['foo', 'bar'], [b'foo-id', b'bar-id'])
-        tree2 = self.make_branch_and_tree('tree2')
-        tt = TreeTransform(tree2)
-        foo_trans_id = tt.create_path('foo', tt.root)
-        create_from_tree(tt, foo_trans_id, tree1, 'foo')
-        bar_trans_id = tt.create_path('bar', tt.root)
-        create_from_tree(tt, bar_trans_id, tree1, 'bar')
-        tt.apply()
-        self.assertEqual('directory', osutils.file_kind('tree2/foo'))
-        self.assertFileEqual(b'baz', 'tree2/bar')
-
-    def test_create_from_tree_bytes(self):
-        """Provided lines are used instead of tree content."""
-        tree1 = self.make_branch_and_tree('tree1')
-        self.build_tree_contents([('tree1/foo', b'bar'), ])
-        tree1.add('foo', b'foo-id')
-        tree2 = self.make_branch_and_tree('tree2')
-        tt = TreeTransform(tree2)
-        foo_trans_id = tt.create_path('foo', tt.root)
-        create_from_tree(tt, foo_trans_id, tree1, 'foo', chunks=[b'qux'])
-        tt.apply()
-        self.assertFileEqual(b'qux', 'tree2/foo')
-
-    def test_create_from_tree_symlink(self):
-        self.requireFeature(SymlinkFeature)
-        tree1 = self.make_branch_and_tree('tree1')
-        os.symlink('bar', 'tree1/foo')
-        tree1.add('foo', b'foo-id')
-        tt = TreeTransform(self.make_branch_and_tree('tree2'))
-        foo_trans_id = tt.create_path('foo', tt.root)
-        create_from_tree(tt, foo_trans_id, tree1, 'foo')
-        tt.apply()
-        self.assertEqual('bar', os.readlink('tree2/foo'))
-
-
 class TransformGroup(object):
 
     def __init__(self, dirname, root_id):
@@ -1702,7 +94,7 @@
         self.wt = ControlDir.create_standalone_workingtree(dirname)
         self.wt.set_root_id(root_id)
         self.b = self.wt.branch
-        self.tt = TreeTransform(self.wt)
+        self.tt = self.wt.transform()
         self.root = self.tt.trans_id_tree_path('')
 
 
@@ -1711,38 +103,6 @@
     return template % (b'<' * 7, tree, b'=' * 7, merge, b'>' * 7)
 
 
-class TestInventoryAltered(tests.TestCaseWithTransport):
-
-    def test_inventory_altered_unchanged(self):
-        tree = self.make_branch_and_tree('tree')
-        self.build_tree(['tree/foo'])
-        tree.add('foo', b'foo-id')
-        with TransformPreview(tree) as tt:
-            self.assertEqual([], tt._inventory_altered())
-
-    def test_inventory_altered_changed_parent_id(self):
-        tree = self.make_branch_and_tree('tree')
-        self.build_tree(['tree/foo'])
-        tree.add('foo', b'foo-id')
-        with TransformPreview(tree) as tt:
-            tt.unversion_file(tt.root)
-            tt.version_file(b'new-id', tt.root)
-            foo_trans_id = tt.trans_id_tree_path('foo')
-            foo_tuple = ('foo', foo_trans_id)
-            root_tuple = ('', tt.root)
-            self.assertEqual([root_tuple, foo_tuple], tt._inventory_altered())
-
-    def test_inventory_altered_noop_changed_parent_id(self):
-        tree = self.make_branch_and_tree('tree')
-        self.build_tree(['tree/foo'])
-        tree.add('foo', b'foo-id')
-        with TransformPreview(tree) as tt:
-            tt.unversion_file(tt.root)
-            tt.version_file(tree.path2id(''), tt.root)
-            tt.trans_id_tree_path('foo')
-            self.assertEqual([], tt._inventory_altered())
-
-
 class TestTransformMerge(TestCaseInTempDir):
 
     def test_text_merge(self):
@@ -2219,7 +579,7 @@
     def test_build_tree_hardlinks_preserve_execute(self):
         self.requireFeature(HardlinkFeature)
         source = self.create_ab_tree()
-        tt = TreeTransform(source)
+        tt = source.transform()
         trans_id = tt.trans_id_tree_path('file1')
         tt.set_executability(True, trans_id)
         tt.apply()
@@ -2349,7 +709,7 @@
 
     def get_branch_and_transform(self):
         branch = self.get_branch()
-        tt = TransformPreview(branch.basis_tree())
+        tt = branch.basis_tree().preview_transform()
         self.addCleanup(tt.finalize)
         return branch, tt
 
@@ -2357,7 +717,7 @@
         branch = self.get_branch()
         basis = branch.repository.revision_tree(
             _mod_revision.NULL_REVISION)
-        tt = TransformPreview(basis)
+        tt = basis.preview_transform()
         self.addCleanup(tt.finalize)
         e = self.assertRaises(ValueError, tt.commit, branch, '')
         self.assertEqual('TreeTransform not based on branch basis: null:',
@@ -2380,7 +740,7 @@
         branch = self.make_branch('branch')
         branch.lock_write()
         self.addCleanup(branch.unlock)
-        tt = TransformPreview(branch.basis_tree())
+        tt = branch.basis_tree().preview_transform()
         self.addCleanup(tt.finalize)
         tt.new_directory('', ROOT_PARENT, b'TREE_ROOT')
         tt.commit(branch, 'my message')
@@ -2392,7 +752,7 @@
         branch = self.make_branch('branch')
         branch.lock_write()
         self.addCleanup(branch.unlock)
-        tt = TransformPreview(branch.basis_tree())
+        tt = branch.basis_tree().preview_transform()
         self.addCleanup(tt.finalize)
         e = self.assertRaises(ValueError, tt.commit, branch,
                               'my message', [b'rev1b-id'])
@@ -2425,7 +785,7 @@
         branch, tt = self.get_branch_and_transform()
         tt.new_file('file', tt.root, [b'contents'], b'file-id')
         tt.commit(branch, 'message', strict=True)
-        tt = TransformPreview(branch.basis_tree())
+        tt = branch.basis_tree().preview_transform()
         self.addCleanup(tt.finalize)
         trans_id = tt.trans_id_file_id(b'file-id')
         tt.delete_contents(trans_id)
@@ -2440,7 +800,7 @@
         branch, tt = self.get_branch_and_transform()
         parent_id = tt.trans_id_file_id(b'parent-id')
         tt.new_file('file', parent_id, [b'contents'], b'file-id')
-        self.assertRaises(errors.MalformedTransform, tt.commit, branch,
+        self.assertRaises(MalformedTransform, tt.commit, branch,
                           'message')
 
     def test_commit_rich_revision_data(self):
@@ -2552,7 +912,7 @@
     def test_rollback_rename(self):
         tree = self.make_branch_and_tree('.')
         self.build_tree(['a/', 'a/b'])
-        tt = TreeTransform(tree)
+        tt = tree.transform()
         self.addCleanup(tt.finalize)
         a_id = tt.trans_id_tree_path('a')
         tt.adjust_path('c', tt.root, a_id)
@@ -2568,7 +928,7 @@
     def test_rollback_rename_into_place(self):
         tree = self.make_branch_and_tree('.')
         self.build_tree(['a/', 'a/b'])
-        tt = TreeTransform(tree)
+        tt = tree.transform()
         self.addCleanup(tt.finalize)
         a_id = tt.trans_id_tree_path('a')
         tt.adjust_path('c', tt.root, a_id)
@@ -2584,7 +944,7 @@
     def test_rollback_deletion(self):
         tree = self.make_branch_and_tree('.')
         self.build_tree(['a/', 'a/b'])
-        tt = TreeTransform(tree)
+        tt = tree.transform()
         self.addCleanup(tt.finalize)
         a_id = tt.trans_id_tree_path('a')
         tt.delete_contents(a_id)
@@ -2624,13 +984,13 @@
     def create_transform_and_root_trans_id(self):
         """Setup a transform creating a file in limbo"""
         tree = self.make_branch_and_tree('.')
-        tt = TreeTransform(tree)
+        tt = tree.transform()
         return tt, tt.create_path("a", tt.root)
 
     def create_transform_and_subdir_trans_id(self):
         """Setup a transform creating a directory containing a file in limbo"""
         tree = self.make_branch_and_tree('.')
-        tt = TreeTransform(tree)
+        tt = tree.transform()
         d_trans_id = tt.create_path("d", tt.root)
         tt.create_directory(d_trans_id)
         f_trans_id = tt.create_path("a", d_trans_id)
@@ -2730,7 +1090,7 @@
         self.build_tree(['dir/', ])
         wt.add(['dir'], [b'dir-id'])
         wt.commit('Create dir')
-        tt = TreeTransform(wt)
+        tt = wt.transform()
         self.addCleanup(tt.finalize)
         return wt, tt
 
@@ -2758,671 +1118,6 @@
                          conflicts.pop())
 
 
-A_ENTRY = (b'a-id', ('a', 'a'), True, (True, True),
-           (b'TREE_ROOT', b'TREE_ROOT'), ('a', 'a'), ('file', 'file'),
-           (False, False), False)
-ROOT_ENTRY = (b'TREE_ROOT', ('', ''), False, (True, True), (None, None),
-              ('', ''), ('directory', 'directory'), (False, False), False)
-
-
-class TestTransformPreview(tests.TestCaseWithTransport):
-
-    def create_tree(self):
-        tree = self.make_branch_and_tree('.')
-        self.build_tree_contents([('a', b'content 1')])
-        tree.set_root_id(b'TREE_ROOT')
-        tree.add('a', b'a-id')
-        tree.commit('rev1', rev_id=b'rev1')
-        return tree.branch.repository.revision_tree(b'rev1')
-
-    def get_empty_preview(self):
-        repository = self.make_repository('repo')
-        tree = repository.revision_tree(_mod_revision.NULL_REVISION)
-        preview = TransformPreview(tree)
-        self.addCleanup(preview.finalize)
-        return preview
-
-    def test_transform_preview(self):
-        revision_tree = self.create_tree()
-        preview = TransformPreview(revision_tree)
-        self.addCleanup(preview.finalize)
-
-    def test_transform_preview_tree(self):
-        revision_tree = self.create_tree()
-        preview = TransformPreview(revision_tree)
-        self.addCleanup(preview.finalize)
-        preview.get_preview_tree()
-
-    def test_transform_new_file(self):
-        revision_tree = self.create_tree()
-        preview = TransformPreview(revision_tree)
-        self.addCleanup(preview.finalize)
-        preview.new_file('file2', preview.root, [b'content B\n'], b'file2-id')
-        preview_tree = preview.get_preview_tree()
-        self.assertEqual(preview_tree.kind('file2'), 'file')
-        with preview_tree.get_file('file2') as f:
-            self.assertEqual(f.read(), b'content B\n')
-
-    def test_diff_preview_tree(self):
-        revision_tree = self.create_tree()
-        preview = TransformPreview(revision_tree)
-        self.addCleanup(preview.finalize)
-        preview.new_file('file2', preview.root, [b'content B\n'], b'file2-id')
-        preview_tree = preview.get_preview_tree()
-        out = BytesIO()
-        show_diff_trees(revision_tree, preview_tree, out)
-        lines = out.getvalue().splitlines()
-        self.assertEqual(lines[0], b"=== added file 'file2'")
-        # 3 lines of diff administrivia
-        self.assertEqual(lines[4], b"+content B")
-
-    def test_unsupported_symlink_diff(self):
-        self.requireFeature(SymlinkFeature)
-        tree = self.make_branch_and_tree('.')
-        self.build_tree_contents([('a', 'content 1')])
-        tree.set_root_id(b'TREE_ROOT')
-        tree.add('a', b'a-id')
-        os.symlink('a', 'foo')
-        tree.add('foo', b'foo-id')
-        tree.commit('rev1', rev_id=b'rev1')
-        revision_tree = tree.branch.repository.revision_tree(b'rev1')
-        preview = TransformPreview(revision_tree)
-        self.addCleanup(preview.finalize)
-        preview.delete_versioned(preview.trans_id_tree_path('foo'))
-        preview_tree = preview.get_preview_tree()
-        out = BytesIO()
-        log = BytesIO()
-        trace.push_log_file(log)
-        os_symlink = getattr(os, 'symlink', None)
-        os.symlink = None
-        try:
-            show_diff_trees(revision_tree, preview_tree, out)
-            lines = out.getvalue().splitlines()
-        finally:
-            os.symlink = os_symlink
-        self.assertContainsRe(
-            log.getvalue(),
-            b'Ignoring "foo" as symlinks are not supported on this filesystem')
-
-    def test_transform_conflicts(self):
-        revision_tree = self.create_tree()
-        preview = TransformPreview(revision_tree)
-        self.addCleanup(preview.finalize)
-        preview.new_file('a', preview.root, [b'content 2'])
-        resolve_conflicts(preview)
-        trans_id = preview.trans_id_file_id(b'a-id')
-        self.assertEqual('a.moved', preview.final_name(trans_id))
-
-    def get_tree_and_preview_tree(self):
-        revision_tree = self.create_tree()
-        preview = TransformPreview(revision_tree)
-        self.addCleanup(preview.finalize)
-        a_trans_id = preview.trans_id_file_id(b'a-id')
-        preview.delete_contents(a_trans_id)
-        preview.create_file([b'b content'], a_trans_id)
-        preview_tree = preview.get_preview_tree()
-        return revision_tree, preview_tree
-
-    def test_iter_changes(self):
-        revision_tree, preview_tree = self.get_tree_and_preview_tree()
-        root = revision_tree.path2id('')
-        self.assertEqual([(b'a-id', ('a', 'a'), True, (True, True),
-                           (root, root), ('a', 'a'), ('file', 'file'),
-                           (False, False), False)],
-                         list(preview_tree.iter_changes(revision_tree)))
-
-    def test_include_unchanged_succeeds(self):
-        revision_tree, preview_tree = self.get_tree_and_preview_tree()
-        changes = preview_tree.iter_changes(revision_tree,
-                                            include_unchanged=True)
-        self.assertEqual([ROOT_ENTRY, A_ENTRY], list(changes))
-
-    def test_specific_files(self):
-        revision_tree, preview_tree = self.get_tree_and_preview_tree()
-        changes = preview_tree.iter_changes(revision_tree,
-                                            specific_files=[''])
-        self.assertEqual([A_ENTRY], list(changes))
-
-    def test_want_unversioned(self):
-        revision_tree, preview_tree = self.get_tree_and_preview_tree()
-        changes = preview_tree.iter_changes(revision_tree,
-                                            want_unversioned=True)
-        self.assertEqual([A_ENTRY], list(changes))
-
-    def test_ignore_extra_trees_no_specific_files(self):
-        # extra_trees is harmless without specific_files, so we'll silently
-        # accept it, even though we won't use it.
-        revision_tree, preview_tree = self.get_tree_and_preview_tree()
-        preview_tree.iter_changes(revision_tree, extra_trees=[preview_tree])
-
-    def test_ignore_require_versioned_no_specific_files(self):
-        # require_versioned is meaningless without specific_files.
-        revision_tree, preview_tree = self.get_tree_and_preview_tree()
-        preview_tree.iter_changes(revision_tree, require_versioned=False)
-
-    def test_ignore_pb(self):
-        # pb could be supported, but TT.iter_changes doesn't support it.
-        revision_tree, preview_tree = self.get_tree_and_preview_tree()
-        preview_tree.iter_changes(revision_tree)
-
-    def test_kind(self):
-        revision_tree = self.create_tree()
-        preview = TransformPreview(revision_tree)
-        self.addCleanup(preview.finalize)
-        preview.new_file('file', preview.root, [b'contents'], b'file-id')
-        preview.new_directory('directory', preview.root, b'dir-id')
-        preview_tree = preview.get_preview_tree()
-        self.assertEqual('file', preview_tree.kind('file'))
-        self.assertEqual('directory', preview_tree.kind('directory'))
-
-    def test_get_file_mtime(self):
-        preview = self.get_empty_preview()
-        file_trans_id = preview.new_file('file', preview.root, [b'contents'],
-                                         b'file-id')
-        limbo_path = preview._limbo_name(file_trans_id)
-        preview_tree = preview.get_preview_tree()
-        self.assertEqual(os.stat(limbo_path).st_mtime,
-                         preview_tree.get_file_mtime('file'))
-
-    def test_get_file_mtime_renamed(self):
-        work_tree = self.make_branch_and_tree('tree')
-        self.build_tree(['tree/file'])
-        work_tree.add('file', b'file-id')
-        preview = TransformPreview(work_tree)
-        self.addCleanup(preview.finalize)
-        file_trans_id = preview.trans_id_tree_path('file')
-        preview.adjust_path('renamed', preview.root, file_trans_id)
-        preview_tree = preview.get_preview_tree()
-        preview_mtime = preview_tree.get_file_mtime('renamed')
-        work_mtime = work_tree.get_file_mtime('file')
-
-    def test_get_file_size(self):
-        work_tree = self.make_branch_and_tree('tree')
-        self.build_tree_contents([('tree/old', b'old')])
-        work_tree.add('old', b'old-id')
-        preview = TransformPreview(work_tree)
-        self.addCleanup(preview.finalize)
-        preview.new_file('name', preview.root, [b'contents'], b'new-id',
-                         'executable')
-        tree = preview.get_preview_tree()
-        self.assertEqual(len('old'), tree.get_file_size('old'))
-        self.assertEqual(len('contents'), tree.get_file_size('name'))
-
-    def test_get_file(self):
-        preview = self.get_empty_preview()
-        preview.new_file('file', preview.root, [b'contents'], b'file-id')
-        preview_tree = preview.get_preview_tree()
-        with preview_tree.get_file('file') as tree_file:
-            self.assertEqual(b'contents', tree_file.read())
-
-    def test_get_symlink_target(self):
-        self.requireFeature(SymlinkFeature)
-        preview = self.get_empty_preview()
-        preview.new_symlink('symlink', preview.root, 'target', b'symlink-id')
-        preview_tree = preview.get_preview_tree()
-        self.assertEqual('target',
-                         preview_tree.get_symlink_target('symlink'))
-
-    def test_all_file_ids(self):
-        tree = self.make_branch_and_tree('tree')
-        self.build_tree(['tree/a', 'tree/b', 'tree/c'])
-        tree.add(['a', 'b', 'c'], [b'a-id', b'b-id', b'c-id'])
-        preview = TransformPreview(tree)
-        self.addCleanup(preview.finalize)
-        preview.unversion_file(preview.trans_id_file_id(b'b-id'))
-        c_trans_id = preview.trans_id_file_id(b'c-id')
-        preview.unversion_file(c_trans_id)
-        preview.version_file(b'c-id', c_trans_id)
-        preview_tree = preview.get_preview_tree()
-        self.assertEqual({b'a-id', b'c-id', tree.path2id('')},
-                         preview_tree.all_file_ids())
-
-    def test_path2id_deleted_unchanged(self):
-        tree = self.make_branch_and_tree('tree')
-        self.build_tree(['tree/unchanged', 'tree/deleted'])
-        tree.add(['unchanged', 'deleted'], [b'unchanged-id', b'deleted-id'])
-        preview = TransformPreview(tree)
-        self.addCleanup(preview.finalize)
-        preview.unversion_file(preview.trans_id_file_id(b'deleted-id'))
-        preview_tree = preview.get_preview_tree()
-        self.assertEqual(b'unchanged-id', preview_tree.path2id('unchanged'))
-        self.assertFalse(preview_tree.is_versioned('deleted'))
-
-    def test_path2id_created(self):
-        tree = self.make_branch_and_tree('tree')
-        self.build_tree(['tree/unchanged'])
-        tree.add(['unchanged'], [b'unchanged-id'])
-        preview = TransformPreview(tree)
-        self.addCleanup(preview.finalize)
-        preview.new_file('new', preview.trans_id_file_id(b'unchanged-id'),
-                         [b'contents'], b'new-id')
-        preview_tree = preview.get_preview_tree()
-        self.assertEqual(b'new-id', preview_tree.path2id('unchanged/new'))
-
-    def test_path2id_moved(self):
-        tree = self.make_branch_and_tree('tree')
-        self.build_tree(['tree/old_parent/', 'tree/old_parent/child'])
-        tree.add(['old_parent', 'old_parent/child'],
-                 [b'old_parent-id', b'child-id'])
-        preview = TransformPreview(tree)
-        self.addCleanup(preview.finalize)
-        new_parent = preview.new_directory('new_parent', preview.root,
-                                           b'new_parent-id')
-        preview.adjust_path('child', new_parent,
-                            preview.trans_id_file_id(b'child-id'))
-        preview_tree = preview.get_preview_tree()
-        self.assertFalse(preview_tree.is_versioned('old_parent/child'))
-        self.assertEqual(b'child-id', preview_tree.path2id('new_parent/child'))
-
-    def test_path2id_renamed_parent(self):
-        tree = self.make_branch_and_tree('tree')
-        self.build_tree(['tree/old_name/', 'tree/old_name/child'])
-        tree.add(['old_name', 'old_name/child'],
-                 [b'parent-id', b'child-id'])
-        preview = TransformPreview(tree)
-        self.addCleanup(preview.finalize)
-        preview.adjust_path('new_name', preview.root,
-                            preview.trans_id_file_id(b'parent-id'))
-        preview_tree = preview.get_preview_tree()
-        self.assertFalse(preview_tree.is_versioned('old_name/child'))
-        self.assertEqual(b'child-id', preview_tree.path2id('new_name/child'))
-
-    def assertMatchingIterEntries(self, tt, specific_files=None):
-        preview_tree = tt.get_preview_tree()
-        preview_result = list(preview_tree.iter_entries_by_dir(
-                              specific_files=specific_files))
-        tree = tt._tree
-        tt.apply()
-        actual_result = list(tree.iter_entries_by_dir(
-            specific_files=specific_files))
-        self.assertEqual(actual_result, preview_result)
-
-    def test_iter_entries_by_dir_new(self):
-        tree = self.make_branch_and_tree('tree')
-        tt = TreeTransform(tree)
-        tt.new_file('new', tt.root, [b'contents'], b'new-id')
-        self.assertMatchingIterEntries(tt)
-
-    def test_iter_entries_by_dir_deleted(self):
-        tree = self.make_branch_and_tree('tree')
-        self.build_tree(['tree/deleted'])
-        tree.add('deleted', b'deleted-id')
-        tt = TreeTransform(tree)
-        tt.delete_contents(tt.trans_id_file_id(b'deleted-id'))
-        self.assertMatchingIterEntries(tt)
-
-    def test_iter_entries_by_dir_unversioned(self):
-        tree = self.make_branch_and_tree('tree')
-        self.build_tree(['tree/removed'])
-        tree.add('removed', b'removed-id')
-        tt = TreeTransform(tree)
-        tt.unversion_file(tt.trans_id_file_id(b'removed-id'))
-        self.assertMatchingIterEntries(tt)
-
-    def test_iter_entries_by_dir_moved(self):
-        tree = self.make_branch_and_tree('tree')
-        self.build_tree(['tree/moved', 'tree/new_parent/'])
-        tree.add(['moved', 'new_parent'], [b'moved-id', b'new_parent-id'])
-        tt = TreeTransform(tree)
-        tt.adjust_path('moved', tt.trans_id_file_id(b'new_parent-id'),
-                       tt.trans_id_file_id(b'moved-id'))
-        self.assertMatchingIterEntries(tt)
-
-    def test_iter_entries_by_dir_specific_files(self):
-        tree = self.make_branch_and_tree('tree')
-        tree.set_root_id(b'tree-root-id')
-        self.build_tree(['tree/parent/', 'tree/parent/child'])
-        tree.add(['parent', 'parent/child'], [b'parent-id', b'child-id'])
-        tt = TreeTransform(tree)
-        self.assertMatchingIterEntries(tt, ['', 'parent/child'])
-
-    def test_symlink_content_summary(self):
-        self.requireFeature(SymlinkFeature)
-        preview = self.get_empty_preview()
-        preview.new_symlink('path', preview.root, 'target', b'path-id')
-        summary = preview.get_preview_tree().path_content_summary('path')
-        self.assertEqual(('symlink', None, None, 'target'), summary)
-
-    def test_missing_content_summary(self):
-        preview = self.get_empty_preview()
-        summary = preview.get_preview_tree().path_content_summary('path')
-        self.assertEqual(('missing', None, None, None), summary)
-
-    def test_deleted_content_summary(self):
-        tree = self.make_branch_and_tree('tree')
-        self.build_tree(['tree/path/'])
-        tree.add('path')
-        preview = TransformPreview(tree)
-        self.addCleanup(preview.finalize)
-        preview.delete_contents(preview.trans_id_tree_path('path'))
-        summary = preview.get_preview_tree().path_content_summary('path')
-        self.assertEqual(('missing', None, None, None), summary)
-
-    def test_file_content_summary_executable(self):
-        preview = self.get_empty_preview()
-        path_id = preview.new_file('path', preview.root, [
-                                   b'contents'], b'path-id')
-        preview.set_executability(True, path_id)
-        summary = preview.get_preview_tree().path_content_summary('path')
-        self.assertEqual(4, len(summary))
-        self.assertEqual('file', summary[0])
-        # size must be known
-        self.assertEqual(len('contents'), summary[1])
-        # executable
-        self.assertEqual(True, summary[2])
-        # will not have hash (not cheap to determine)
-        self.assertIs(None, summary[3])
-
-    def test_change_executability(self):
-        tree = self.make_branch_and_tree('tree')
-        self.build_tree(['tree/path'])
-        tree.add('path')
-        preview = TransformPreview(tree)
-        self.addCleanup(preview.finalize)
-        path_id = preview.trans_id_tree_path('path')
-        preview.set_executability(True, path_id)
-        summary = preview.get_preview_tree().path_content_summary('path')
-        self.assertEqual(True, summary[2])
-
-    def test_file_content_summary_non_exec(self):
-        preview = self.get_empty_preview()
-        preview.new_file('path', preview.root, [b'contents'], b'path-id')
-        summary = preview.get_preview_tree().path_content_summary('path')
-        self.assertEqual(4, len(summary))
-        self.assertEqual('file', summary[0])
-        # size must be known
-        self.assertEqual(len('contents'), summary[1])
-        # not executable
-        self.assertEqual(False, summary[2])
-        # will not have hash (not cheap to determine)
-        self.assertIs(None, summary[3])
-
-    def test_dir_content_summary(self):
-        preview = self.get_empty_preview()
-        preview.new_directory('path', preview.root, b'path-id')
-        summary = preview.get_preview_tree().path_content_summary('path')
-        self.assertEqual(('directory', None, None, None), summary)
-
-    def test_tree_content_summary(self):
-        preview = self.get_empty_preview()
-        path = preview.new_directory('path', preview.root, b'path-id')
-        preview.set_tree_reference(b'rev-1', path)
-        summary = preview.get_preview_tree().path_content_summary('path')
-        self.assertEqual(4, len(summary))
-        self.assertEqual('tree-reference', summary[0])
-
-    def test_annotate(self):
-        tree = self.make_branch_and_tree('tree')
-        self.build_tree_contents([('tree/file', b'a\n')])
-        tree.add('file', b'file-id')
-        tree.commit('a', rev_id=b'one')
-        self.build_tree_contents([('tree/file', b'a\nb\n')])
-        preview = TransformPreview(tree)
-        self.addCleanup(preview.finalize)
-        file_trans_id = preview.trans_id_file_id(b'file-id')
-        preview.delete_contents(file_trans_id)
-        preview.create_file([b'a\nb\nc\n'], file_trans_id)
-        preview_tree = preview.get_preview_tree()
-        expected = [
-            (b'one', b'a\n'),
-            (b'me:', b'b\n'),
-            (b'me:', b'c\n'),
-        ]
-        annotation = preview_tree.annotate_iter(
-            'file', default_revision=b'me:')
-        self.assertEqual(expected, annotation)
-
-    def test_annotate_missing(self):
-        preview = self.get_empty_preview()
-        preview.new_file('file', preview.root, [b'a\nb\nc\n'], b'file-id')
-        preview_tree = preview.get_preview_tree()
-        expected = [
-            (b'me:', b'a\n'),
-            (b'me:', b'b\n'),
-            (b'me:', b'c\n'),
-            ]
-        annotation = preview_tree.annotate_iter(
-            'file', default_revision=b'me:')
-        self.assertEqual(expected, annotation)
-
-    def test_annotate_rename(self):
-        tree = self.make_branch_and_tree('tree')
-        self.build_tree_contents([('tree/file', b'a\n')])
-        tree.add('file', b'file-id')
-        tree.commit('a', rev_id=b'one')
-        preview = TransformPreview(tree)
-        self.addCleanup(preview.finalize)
-        file_trans_id = preview.trans_id_file_id(b'file-id')
-        preview.adjust_path('newname', preview.root, file_trans_id)
-        preview_tree = preview.get_preview_tree()
-        expected = [
-            (b'one', b'a\n'),
-        ]
-        annotation = preview_tree.annotate_iter(
-            'file', default_revision=b'me:')
-        self.assertEqual(expected, annotation)
-
-    def test_annotate_deleted(self):
-        tree = self.make_branch_and_tree('tree')
-        self.build_tree_contents([('tree/file', b'a\n')])
-        tree.add('file', b'file-id')
-        tree.commit('a', rev_id=b'one')
-        self.build_tree_contents([('tree/file', b'a\nb\n')])
-        preview = TransformPreview(tree)
-        self.addCleanup(preview.finalize)
-        file_trans_id = preview.trans_id_file_id(b'file-id')
-        preview.delete_contents(file_trans_id)
-        preview_tree = preview.get_preview_tree()
-        annotation = preview_tree.annotate_iter(
-            'file', default_revision=b'me:')
-        self.assertIs(None, annotation)
-
-    def test_stored_kind(self):
-        preview = self.get_empty_preview()
-        preview.new_file('file', preview.root, [b'a\nb\nc\n'], b'file-id')
-        preview_tree = preview.get_preview_tree()
-        self.assertEqual('file', preview_tree.stored_kind('file'))
-
-    def test_is_executable(self):
-        preview = self.get_empty_preview()
-        preview.new_file('file', preview.root, [b'a\nb\nc\n'], b'file-id')
-        preview.set_executability(True, preview.trans_id_file_id(b'file-id'))
-        preview_tree = preview.get_preview_tree()
-        self.assertEqual(True, preview_tree.is_executable('file'))
-
-    def test_get_set_parent_ids(self):
-        revision_tree, preview_tree = self.get_tree_and_preview_tree()
-        self.assertEqual([], preview_tree.get_parent_ids())
-        preview_tree.set_parent_ids([b'rev-1'])
-        self.assertEqual([b'rev-1'], preview_tree.get_parent_ids())
-
-    def test_plan_file_merge(self):
-        work_a = self.make_branch_and_tree('wta')
-        self.build_tree_contents([('wta/file', b'a\nb\nc\nd\n')])
-        work_a.add('file', b'file-id')
-        base_id = work_a.commit('base version')
-        tree_b = work_a.controldir.sprout('wtb').open_workingtree()
-        preview = TransformPreview(work_a)
-        self.addCleanup(preview.finalize)
-        trans_id = preview.trans_id_file_id(b'file-id')
-        preview.delete_contents(trans_id)
-        preview.create_file([b'b\nc\nd\ne\n'], trans_id)
-        self.build_tree_contents([('wtb/file', b'a\nc\nd\nf\n')])
-        tree_a = preview.get_preview_tree()
-        tree_a.set_parent_ids([base_id])
-        self.assertEqual([
-            ('killed-a', b'a\n'),
-            ('killed-b', b'b\n'),
-            ('unchanged', b'c\n'),
-            ('unchanged', b'd\n'),
-            ('new-a', b'e\n'),
-            ('new-b', b'f\n'),
-        ], list(tree_a.plan_file_merge('file', tree_b)))
-
-    def test_plan_file_merge_revision_tree(self):
-        work_a = self.make_branch_and_tree('wta')
-        self.build_tree_contents([('wta/file', b'a\nb\nc\nd\n')])
-        work_a.add('file', b'file-id')
-        base_id = work_a.commit('base version')
-        tree_b = work_a.controldir.sprout('wtb').open_workingtree()
-        preview = TransformPreview(work_a.basis_tree())
-        self.addCleanup(preview.finalize)
-        trans_id = preview.trans_id_file_id(b'file-id')
-        preview.delete_contents(trans_id)
-        preview.create_file([b'b\nc\nd\ne\n'], trans_id)
-        self.build_tree_contents([('wtb/file', b'a\nc\nd\nf\n')])
-        tree_a = preview.get_preview_tree()
-        tree_a.set_parent_ids([base_id])
-        self.assertEqual([
-            ('killed-a', b'a\n'),
-            ('killed-b', b'b\n'),
-            ('unchanged', b'c\n'),
-            ('unchanged', b'd\n'),
-            ('new-a', b'e\n'),
-            ('new-b', b'f\n'),
-        ], list(tree_a.plan_file_merge('file', tree_b)))
-
-    def test_walkdirs(self):
-        preview = self.get_empty_preview()
-        preview.new_directory('', ROOT_PARENT, b'tree-root')
-        # FIXME: new_directory should mark root.
-        preview.fixup_new_roots()
-        preview_tree = preview.get_preview_tree()
-        preview.new_file('a', preview.root, [b'contents'], b'a-id')
-        expected = [(('', b'tree-root'),
-                     [('a', 'a', 'file', None, b'a-id', 'file')])]
-        self.assertEqual(expected, list(preview_tree.walkdirs()))
-
-    def test_extras(self):
-        work_tree = self.make_branch_and_tree('tree')
-        self.build_tree(['tree/removed-file', 'tree/existing-file',
-                         'tree/not-removed-file'])
-        work_tree.add(['removed-file', 'not-removed-file'])
-        preview = TransformPreview(work_tree)
-        self.addCleanup(preview.finalize)
-        preview.new_file('new-file', preview.root, [b'contents'])
-        preview.new_file('new-versioned-file', preview.root, [b'contents'],
-                         b'new-versioned-id')
-        tree = preview.get_preview_tree()
-        preview.unversion_file(preview.trans_id_tree_path('removed-file'))
-        self.assertEqual({'new-file', 'removed-file', 'existing-file'},
-                         set(tree.extras()))
-
-    def test_merge_into_preview(self):
-        work_tree = self.make_branch_and_tree('tree')
-        self.build_tree_contents([('tree/file', b'b\n')])
-        work_tree.add('file', b'file-id')
-        work_tree.commit('first commit')
-        child_tree = work_tree.controldir.sprout('child').open_workingtree()
-        self.build_tree_contents([('child/file', b'b\nc\n')])
-        child_tree.commit('child commit')
-        child_tree.lock_write()
-        self.addCleanup(child_tree.unlock)
-        work_tree.lock_write()
-        self.addCleanup(work_tree.unlock)
-        preview = TransformPreview(work_tree)
-        self.addCleanup(preview.finalize)
-        file_trans_id = preview.trans_id_file_id(b'file-id')
-        preview.delete_contents(file_trans_id)
-        preview.create_file([b'a\nb\n'], file_trans_id)
-        preview_tree = preview.get_preview_tree()
-        merger = Merger.from_revision_ids(preview_tree,
-                                          child_tree.branch.last_revision(),
-                                          other_branch=child_tree.branch,
-                                          tree_branch=work_tree.branch)
-        merger.merge_type = Merge3Merger
-        tt = merger.make_merger().make_preview_transform()
-        self.addCleanup(tt.finalize)
-        final_tree = tt.get_preview_tree()
-        self.assertEqual(
-            b'a\nb\nc\n',
-            final_tree.get_file_text(final_tree.id2path(b'file-id')))
-
-    def test_merge_preview_into_workingtree(self):
-        tree = self.make_branch_and_tree('tree')
-        tree.set_root_id(b'TREE_ROOT')
-        tt = TransformPreview(tree)
-        self.addCleanup(tt.finalize)
-        tt.new_file('name', tt.root, [b'content'], b'file-id')
-        tree2 = self.make_branch_and_tree('tree2')
-        tree2.set_root_id(b'TREE_ROOT')
-        merger = Merger.from_uncommitted(tree2, tt.get_preview_tree(),
-                                         tree.basis_tree())
-        merger.merge_type = Merge3Merger
-        merger.do_merge()
-
-    def test_merge_preview_into_workingtree_handles_conflicts(self):
-        tree = self.make_branch_and_tree('tree')
-        self.build_tree_contents([('tree/foo', b'bar')])
-        tree.add('foo', b'foo-id')
-        tree.commit('foo')
-        tt = TransformPreview(tree)
-        self.addCleanup(tt.finalize)
-        trans_id = tt.trans_id_file_id(b'foo-id')
-        tt.delete_contents(trans_id)
-        tt.create_file([b'baz'], trans_id)
-        tree2 = tree.controldir.sprout('tree2').open_workingtree()
-        self.build_tree_contents([('tree2/foo', b'qux')])
-        merger = Merger.from_uncommitted(tree2, tt.get_preview_tree(),
-                                         tree.basis_tree())
-        merger.merge_type = Merge3Merger
-        merger.do_merge()
-
-    def test_has_filename(self):
-        wt = self.make_branch_and_tree('tree')
-        self.build_tree(['tree/unmodified', 'tree/removed', 'tree/modified'])
-        tt = TransformPreview(wt)
-        removed_id = tt.trans_id_tree_path('removed')
-        tt.delete_contents(removed_id)
-        tt.new_file('new', tt.root, [b'contents'])
-        modified_id = tt.trans_id_tree_path('modified')
-        tt.delete_contents(modified_id)
-        tt.create_file([b'modified-contents'], modified_id)
-        self.addCleanup(tt.finalize)
-        tree = tt.get_preview_tree()
-        self.assertTrue(tree.has_filename('unmodified'))
-        self.assertFalse(tree.has_filename('not-present'))
-        self.assertFalse(tree.has_filename('removed'))
-        self.assertTrue(tree.has_filename('new'))
-        self.assertTrue(tree.has_filename('modified'))
-
-    def test_is_executable(self):
-        tree = self.make_branch_and_tree('tree')
-        preview = TransformPreview(tree)
-        self.addCleanup(preview.finalize)
-        preview.new_file('foo', preview.root, [b'bar'], b'baz-id')
-        preview_tree = preview.get_preview_tree()
-        self.assertEqual(False, preview_tree.is_executable('tree/foo'))
-
-    def test_commit_preview_tree(self):
-        tree = self.make_branch_and_tree('tree')
-        rev_id = tree.commit('rev1')
-        tree.branch.lock_write()
-        self.addCleanup(tree.branch.unlock)
-        tt = TransformPreview(tree)
-        tt.new_file('file', tt.root, [b'contents'], b'file_id')
-        self.addCleanup(tt.finalize)
-        preview = tt.get_preview_tree()
-        preview.set_parent_ids([rev_id])
-        builder = tree.branch.get_commit_builder([rev_id])
-        list(builder.record_iter_changes(preview, rev_id, tt.iter_changes()))
-        builder.finish_inventory()
-        rev2_id = builder.commit('rev2')
-        rev2_tree = tree.branch.repository.revision_tree(rev2_id)
-        self.assertEqual(b'contents', rev2_tree.get_file_text('file'))
-
-    def test_ascii_limbo_paths(self):
-        self.requireFeature(features.UnicodeFilenameFeature)
-        branch = self.make_branch('any')
-        tree = branch.repository.revision_tree(_mod_revision.NULL_REVISION)
-        tt = TransformPreview(tree)
-        self.addCleanup(tt.finalize)
-        foo_id = tt.new_directory('', ROOT_PARENT)
-        bar_id = tt.new_file(u'\u1234bar', foo_id, [b'contents'])
-        limbo_path = tt._limbo_name(bar_id)
-        self.assertEqual(limbo_path, limbo_path)
-
-
 class FakeSerializer(object):
     """Serializer implementation that simply returns the input.
 
@@ -3440,7 +1135,7 @@
     def get_preview(self, tree=None):
         if tree is None:
             tree = self.make_branch_and_tree('tree')
-        tt = TransformPreview(tree)
+        tt = tree.preview_transform()
         self.addCleanup(tt.finalize)
         return tt
 
@@ -3500,11 +1195,8 @@
         self.assertEqual({'new-1': True}, tt._new_executability)
         self.assertEqual({'new-1': 'file',
                           'new-2': 'directory'}, tt._new_contents)
-        foo_limbo = open(tt._limbo_name('new-1'), 'rb')
-        try:
+        with open(tt._limbo_name('new-1'), 'rb') as foo_limbo:
             foo_content = foo_limbo.read()
-        finally:
-            foo_limbo.close()
         self.assertEqual(b'bar', foo_content)
 
     def symlink_creation_records(self):
@@ -3699,7 +1391,7 @@
 
     def test_no_orphan_for_transform_preview(self):
         tree = self.make_branch_and_tree('tree')
-        tt = transform.TransformPreview(tree)
+        tt = tree.preview_transform()
         self.addCleanup(tt.finalize)
         self.assertRaises(NotImplementedError, tt.new_orphan, 'foo', 'bar')
 
@@ -3711,7 +1403,7 @@
         self.build_tree(['dir/', 'dir/file', 'dir/foo'])
         wt.add(['dir', 'dir/file'], [b'dir-id', b'file-id'])
         wt.commit('add dir and file ignoring foo')
-        tt = transform.TreeTransform(wt)
+        tt = wt.transform()
         self.addCleanup(tt.finalize)
         # dir and bar are deleted
         dir_tid = tt.trans_id_tree_path('dir')
@@ -3795,8 +1487,8 @@
         self.wt = self.make_branch_and_tree('.')
         os.chdir('..')
 
-    def get_transform(self):
-        transform = TreeTransform(self.wt)
+    def transform(self):
+        transform = self.wt.transform()
         self.addCleanup(transform.finalize)
         return transform, transform.root
 
@@ -3807,7 +1499,7 @@
             calls.append((tree, tt))
         MutableTree.hooks.install_named_hook(
             'pre_transform', record_pre_transform, "Pre transform")
-        transform, root = self.get_transform()
+        transform, root = self.transform()
         old_root_id = transform.tree_file_id(root)
         transform.apply()
         self.assertEqual(old_root_id, self.wt.path2id(''))
@@ -3820,7 +1512,7 @@
             calls.append((tree, tt))
         MutableTree.hooks.install_named_hook(
             'post_transform', record_post_transform, "Post transform")
-        transform, root = self.get_transform()
+        transform, root = self.transform()
         old_root_id = transform.tree_file_id(root)
         transform.apply()
         self.assertEqual(old_root_id, self.wt.path2id(''))
@@ -3855,7 +1547,7 @@
 
     def test_link_fails_if_execute_bit_changed(self):
         """If the file to be linked has modified execute bit, don't link."""
-        tt = TreeTransform(self.child_tree)
+        tt = self.child_tree.transform()
         try:
             trans_id = tt.trans_id_tree_path('foo')
             tt.set_executability(True, trans_id)
@@ -3869,3 +1561,12 @@
         """If the file to be linked is unmodified, link"""
         transform.link_tree(self.child_tree, self.parent_tree)
         self.assertTrue(self.hardlinked())
+
+
+class ErrorTests(tests.TestCase):
+
+    def test_transform_rename_failed(self):
+        e = TransformRenameFailed(u"from", u"to", "readonly file", 2)
+        self.assertEqual(
+            u"Failed to rename from to to: readonly file",
+            str(e))

=== modified file 'breezy/tests/test_tree.py'
--- breezy/tests/test_tree.py	2019-06-29 19:50:18 +0000
+++ breezy/tests/test_tree.py	2020-07-05 00:45:13 +0000
@@ -91,6 +91,10 @@
              include_root, want_unversioned)
             )
 
+    def find_source_path(self, target_path, recurse='none'):
+        self.calls.append(
+            ('find_source_path', self.source, self.target, target_path, recurse))
+
     @classmethod
     def is_compatible(klass, source, target):
         return True
@@ -106,6 +110,7 @@
             RecordingOptimiser.calls = []
             InterTree.register_optimiser(RecordingOptimiser)
             tree = self.make_branch_and_tree('1')
+            null_tree = tree.basis_tree()
             tree2 = self.make_branch_and_tree('2')
             # do a series of calls:
             # trivial usage
@@ -126,6 +131,8 @@
             InterTree._optimisers = old_optimisers
         self.assertEqual(
             [
+                ('find_source_path', null_tree, tree, '', 'none'),
+                ('find_source_path', null_tree, tree2, '', 'none'),
                 ('compare', tree2, tree, False, None, None, False, False,
                     False),
                 ('compare', tree2, tree, 'unchanged', 'specific', 'extra',

=== modified file 'breezy/tests/test_upstream_import.py'
--- breezy/tests/test_upstream_import.py	2020-06-10 21:13:00 +0000
+++ breezy/tests/test_upstream_import.py	2020-07-04 12:29:00 +0000
@@ -284,7 +284,7 @@
     def get_empty_tt(self):
         b = self.make_repository('foo')
         null_tree = b.revision_tree(_mod_revision.NULL_REVISION)
-        tt = transform.TransformPreview(null_tree)
+        tt = null_tree.preview_transform()
         tt.new_directory('', transform.ROOT_PARENT, b'tree-root')
         tt.fixup_new_roots()
         self.addCleanup(tt.finalize)

=== modified file 'breezy/transform.py'
--- breezy/transform.py	2020-02-18 01:57:45 +0000
+++ breezy/transform.py	2020-07-18 23:14:00 +0000
@@ -22,36 +22,27 @@
 
 from . import (
     config as _mod_config,
+    controldir,
     errors,
     lazy_import,
+    lock,
+    osutils,
     registry,
     trace,
-    tree,
     )
 lazy_import.lazy_import(globals(), """
 from breezy import (
-    annotate,
-    bencode,
-    controldir,
-    commit,
     conflicts,
-    delta,
-    lock,
     multiparent,
-    osutils,
     revision as _mod_revision,
     ui,
     urlutils,
     )
-from breezy.bzr import (
-    inventory,
-    inventorytree,
-    )
 from breezy.i18n import gettext
 """)
-from .errors import (DuplicateKey, MalformedTransform,
-                     ReusingTransform, CantMoveRoot,
-                     ImmortalLimbo, NoFinalPath)
+
+from .errors import (DuplicateKey,
+                     BzrError, InternalBzrError)
 from .filters import filtered_output_bytes, ContentFilterContext
 from .mutabletree import MutableTree
 from .osutils import (
@@ -66,12 +57,62 @@
 from .tree import (
     InterTree,
     TreeChange,
+    find_previous_path,
     )
 
 
 ROOT_PARENT = "root-parent"
 
 
+class NoFinalPath(BzrError):
+
+    _fmt = ("No final name for trans_id %(trans_id)r\n"
+            "file-id: %(file_id)r\n"
+            "root trans-id: %(root_trans_id)r\n")
+
+    def __init__(self, trans_id, transform):
+        self.trans_id = trans_id
+        self.file_id = transform.final_file_id(trans_id)
+        self.root_trans_id = transform.root
+
+
+class ReusingTransform(BzrError):
+
+    _fmt = "Attempt to reuse a transform that has already been applied."
+
+
+class MalformedTransform(InternalBzrError):
+
+    _fmt = "Tree transform is malformed %(conflicts)r"
+
+
+class CantMoveRoot(BzrError):
+
+    _fmt = "Moving the root directory is not supported at this time"
+
+
+class ImmortalLimbo(BzrError):
+
+    _fmt = """Unable to delete transform temporary directory %(limbo_dir)s.
+    Please examine %(limbo_dir)s to see if it contains any files you wish to
+    keep, and delete it when you are done."""
+
+    def __init__(self, limbo_dir):
+        BzrError.__init__(self)
+        self.limbo_dir = limbo_dir
+
+
+class TransformRenameFailed(BzrError):
+
+    _fmt = "Failed to rename %(from_path)s to %(to_path)s: %(why)s"
+
+    def __init__(self, from_path, to_path, why, errno):
+        self.from_path = from_path
+        self.to_path = to_path
+        self.why = why
+        self.errno = errno
+
+
 def unique_add(map, key, value):
     if key in map:
         raise DuplicateKey(key=key)
@@ -79,66 +120,75 @@
 
 
 class _TransformResults(object):
+
     def __init__(self, modified_paths, rename_count):
         object.__init__(self)
         self.modified_paths = modified_paths
         self.rename_count = rename_count
 
 
-class TreeTransformBase(object):
-    """The base class for TreeTransform and its kin."""
-
-    def __init__(self, tree, pb=None, case_sensitive=True):
-        """Constructor.
-
-        :param tree: The tree that will be transformed, but not necessarily
-            the output tree.
-        :param pb: ignored
-        :param case_sensitive: If True, the target of the transform is
-            case sensitive, not just case preserving.
-        """
-        object.__init__(self)
+class TreeTransform(object):
+    """Represent a tree transformation.
+
+    This object is designed to support incremental generation of the transform,
+    in any order.
+
+    However, it gives optimum performance when parent directories are created
+    before their contents.  The transform is then able to put child files
+    directly in their parent directory, avoiding later renames.
+
+    It is easy to produce malformed transforms, but they are generally
+    harmless.  Attempting to apply a malformed transform will cause an
+    exception to be raised before any modifications are made to the tree.
+
+    Many kinds of malformed transforms can be corrected with the
+    resolve_conflicts function.  The remaining ones indicate programming error,
+    such as trying to create a file with no path.
+
+    Two sets of file creation methods are supplied.  Convenience methods are:
+     * new_file
+     * new_directory
+     * new_symlink
+
+    These are composed of the low-level methods:
+     * create_path
+     * create_file or create_directory or create_symlink
+     * version_file
+     * set_executability
+
+    Transform/Transaction ids
+    -------------------------
+    trans_ids are temporary ids assigned to all files involved in a transform.
+    It's possible, even common, that not all files in the Tree have trans_ids.
+
+    trans_ids are only valid for the TreeTransform that generated them.
+    """
+
+    def __init__(self, tree, pb=None):
         self._tree = tree
+        # A progress bar
+        self._pb = pb
         self._id_number = 0
+        # Mapping of path in old tree -> trans_id
+        self._tree_path_ids = {}
+        # Mapping trans_id -> path in old tree
+        self._tree_id_paths = {}
         # mapping of trans_id -> new basename
         self._new_name = {}
         # mapping of trans_id -> new parent trans_id
         self._new_parent = {}
         # mapping of trans_id with new contents -> new file_kind
         self._new_contents = {}
-        # mapping of trans_id => (sha1 of content, stat_value)
-        self._observed_sha1s = {}
         # Set of trans_ids whose contents will be removed
         self._removed_contents = set()
         # Mapping of trans_id -> new execute-bit value
         self._new_executability = {}
         # Mapping of trans_id -> new tree-reference value
         self._new_reference_revision = {}
-        # Mapping of trans_id -> new file_id
-        self._new_id = {}
-        # Mapping of old file-id -> trans_id
-        self._non_present_ids = {}
-        # Mapping of new file_id -> trans_id
-        self._r_new_id = {}
         # Set of trans_ids that will be removed
         self._removed_id = set()
-        # Mapping of path in old tree -> trans_id
-        self._tree_path_ids = {}
-        # Mapping trans_id -> path in old tree
-        self._tree_id_paths = {}
-        # The trans_id that will be used as the tree root
-        if tree.is_versioned(''):
-            self._new_root = self.trans_id_tree_path('')
-        else:
-            self._new_root = None
         # Indicator of whether the transform has been applied
         self._done = False
-        # A progress bar
-        self._pb = pb
-        # Whether the target is case sensitive
-        self._case_sensitive_target = case_sensitive
-        # A counter of how many files have been renamed
-        self.rename_count = 0
 
     def __enter__(self):
         """Support Context Manager API."""
@@ -148,29 +198,42 @@
         """Support Context Manager API."""
         self.finalize()
 
+    def iter_tree_children(self, trans_id):
+        """Iterate through the entry's tree children, if any.
+
+        :param trans_id: trans id to iterate
+        :returns: Iterator over paths
+        """
+        raise NotImplementedError(self.iter_tree_children)
+
+    def canonical_path(self, path):
+        return path
+
+    def tree_kind(self, trans_id):
+        raise NotImplementedError(self.tree_kind)
+
+    def by_parent(self):
+        """Return a map of parent: children for known parents.
+
+        Only new paths and parents of tree files with assigned ids are used.
+        """
+        by_parent = {}
+        items = list(viewitems(self._new_parent))
+        items.extend((t, self.final_parent(t))
+                     for t in list(self._tree_id_paths))
+        for trans_id, parent_id in items:
+            if parent_id not in by_parent:
+                by_parent[parent_id] = set()
+            by_parent[parent_id].add(trans_id)
+        return by_parent
+
     def finalize(self):
         """Release the working tree lock, if held.
 
         This is required if apply has not been invoked, but can be invoked
         even after apply.
         """
-        if self._tree is None:
-            return
-        for hook in MutableTree.hooks['post_transform']:
-            hook(self._tree, self)
-        self._tree.unlock()
-        self._tree = None
-
-    def __get_root(self):
-        return self._new_root
-
-    root = property(__get_root)
-
-    def _assign_id(self):
-        """Produce a new tranform id"""
-        new_id = "new-%s" % self._id_number
-        self._id_number += 1
-        return new_id
+        raise NotImplementedError(self.finalize)
 
     def create_path(self, name, parent):
         """Assign a transaction id to a new path"""
@@ -197,28 +260,7 @@
         physically new directory, and hope someone versions the tree root
         later.
         """
-        old_root = self._new_root
-        old_root_file_id = self.final_file_id(old_root)
-        # force moving all children of root
-        for child_id in self.iter_tree_children(old_root):
-            if child_id != parent:
-                self.adjust_path(self.final_name(child_id),
-                                 self.final_parent(child_id), child_id)
-            file_id = self.final_file_id(child_id)
-            if file_id is not None:
-                self.unversion_file(child_id)
-            self.version_file(file_id, child_id)
-
-        # the physical root needs a new transaction id
-        self._tree_path_ids.pop("")
-        self._tree_id_paths.pop(old_root)
-        self._new_root = self.trans_id_tree_path('')
-        if parent == old_root:
-            parent = self._new_root
-        self.adjust_path(name, parent, old_root)
-        self.create_directory(old_root)
-        self.version_file(old_root_file_id, old_root)
-        self.unversion_file(self._new_root)
+        raise NotImplementedError(self.adjust_root_path)
 
     def fixup_new_roots(self):
         """Reinterpret requests to change the root directory
@@ -230,79 +272,14 @@
         This means that the old root trans-id becomes obsolete, so it is
         recommended only to invoke this after the root trans-id has become
         irrelevant.
-
-        """
-        new_roots = [k for k, v in self._new_parent.items()
-                     if v == ROOT_PARENT]
-        if len(new_roots) < 1:
-            return
-        if len(new_roots) != 1:
-            raise ValueError('A tree cannot have two roots!')
-        if self._new_root is None:
-            self._new_root = new_roots[0]
-            return
-        old_new_root = new_roots[0]
-        # unversion the new root's directory.
-        if self.final_kind(self._new_root) is None:
-            file_id = self.final_file_id(old_new_root)
-        else:
-            file_id = self.final_file_id(self._new_root)
-        if old_new_root in self._new_id:
-            self.cancel_versioning(old_new_root)
-        else:
-            self.unversion_file(old_new_root)
-        # if, at this stage, root still has an old file_id, zap it so we can
-        # stick a new one in.
-        if (self.tree_file_id(self._new_root) is not None
-                and self._new_root not in self._removed_id):
-            self.unversion_file(self._new_root)
-        if file_id is not None:
-            self.version_file(file_id, self._new_root)
-
-        # Now move children of new root into old root directory.
-        # Ensure all children are registered with the transaction, but don't
-        # use directly-- some tree children have new parents
-        list(self.iter_tree_children(old_new_root))
-        # Move all children of new root into old root directory.
-        for child in self.by_parent().get(old_new_root, []):
-            self.adjust_path(self.final_name(child), self._new_root, child)
-
-        # Ensure old_new_root has no directory.
-        if old_new_root in self._new_contents:
-            self.cancel_creation(old_new_root)
-        else:
-            self.delete_contents(old_new_root)
-
-        # prevent deletion of root directory.
-        if self._new_root in self._removed_contents:
-            self.cancel_deletion(self._new_root)
-
-        # destroy path info for old_new_root.
-        del self._new_parent[old_new_root]
-        del self._new_name[old_new_root]
-
-    def trans_id_file_id(self, file_id):
-        """Determine or set the transaction id associated with a file ID.
-        A new id is only created for file_ids that were never present.  If
-        a transaction has been unversioned, it is deliberately still returned.
-        (this will likely lead to an unversioned parent conflict.)
-        """
-        if file_id is None:
-            raise ValueError('None is not a valid file id')
-        if file_id in self._r_new_id and self._r_new_id[file_id] is not None:
-            return self._r_new_id[file_id]
-        else:
-            try:
-                path = self._tree.id2path(file_id)
-            except errors.NoSuchId:
-                if file_id in self._non_present_ids:
-                    return self._non_present_ids[file_id]
-                else:
-                    trans_id = self._assign_id()
-                    self._non_present_ids[file_id] = trans_id
-                    return trans_id
-            else:
-                return self.trans_id_tree_path(path)
+        """
+        raise NotImplementedError(self.fixup_new_roots)
+
+    def _assign_id(self):
+        """Produce a new tranform id"""
+        new_id = "new-%s" % self._id_number
+        self._id_number += 1
+        return new_id
 
     def trans_id_tree_path(self, path):
         """Determine (and maybe set) the transaction ID for a tree path."""
@@ -329,10 +306,6 @@
         """Cancel a scheduled deletion"""
         self._removed_contents.remove(trans_id)
 
-    def unversion_file(self, trans_id):
-        """Schedule a path entry to become unversioned"""
-        self._removed_id.add(trans_id)
-
     def delete_versioned(self, trans_id):
         """Delete and unversion a versioned file"""
         self.delete_contents(trans_id)
@@ -351,18 +324,17 @@
         """Set the reference associated with a directory"""
         unique_add(self._new_reference_revision, trans_id, revision_id)
 
-    def version_file(self, file_id, trans_id):
+    def version_file(self, trans_id, file_id=None):
         """Schedule a file to become versioned."""
-        if file_id is None:
-            raise ValueError()
-        unique_add(self._new_id, trans_id, file_id)
-        unique_add(self._r_new_id, file_id, trans_id)
+        raise NotImplementedError(self.version_file)
 
     def cancel_versioning(self, trans_id):
         """Undo a previous versioning of a file"""
-        file_id = self._new_id[trans_id]
-        del self._new_id[trans_id]
-        del self._r_new_id[file_id]
+        raise NotImplementedError(self.cancel_versioning)
+
+    def unversion_file(self, trans_id):
+        """Schedule a path entry to become unversioned"""
+        self._removed_id.add(trans_id)
 
     def new_paths(self, filesystem_only=False):
         """Determine the paths of all new and changed files.
@@ -370,59 +342,7 @@
         :param filesystem_only: if True, only calculate values for files
             that require renames or execute bit changes.
         """
-        new_ids = set()
-        if filesystem_only:
-            stale_ids = self._needs_rename.difference(self._new_name)
-            stale_ids.difference_update(self._new_parent)
-            stale_ids.difference_update(self._new_contents)
-            stale_ids.difference_update(self._new_id)
-            needs_rename = self._needs_rename.difference(stale_ids)
-            id_sets = (needs_rename, self._new_executability)
-        else:
-            id_sets = (self._new_name, self._new_parent, self._new_contents,
-                       self._new_id, self._new_executability)
-        for id_set in id_sets:
-            new_ids.update(id_set)
-        return sorted(FinalPaths(self).get_paths(new_ids))
-
-    def _inventory_altered(self):
-        """Determine which trans_ids need new Inventory entries.
-
-        An new entry is needed when anything that would be reflected by an
-        inventory entry changes, including file name, file_id, parent file_id,
-        file kind, and the execute bit.
-
-        Some care is taken to return entries with real changes, not cases
-        where the value is deleted and then restored to its original value,
-        but some actually unchanged values may be returned.
-
-        :returns: A list of (path, trans_id) for all items requiring an
-            inventory change. Ordered by path.
-        """
-        changed_ids = set()
-        # Find entries whose file_ids are new (or changed).
-        new_file_id = set(t for t in self._new_id
-                          if self._new_id[t] != self.tree_file_id(t))
-        for id_set in [self._new_name, self._new_parent, new_file_id,
-                       self._new_executability]:
-            changed_ids.update(id_set)
-        # removing implies a kind change
-        changed_kind = set(self._removed_contents)
-        # so does adding
-        changed_kind.intersection_update(self._new_contents)
-        # Ignore entries that are already known to have changed.
-        changed_kind.difference_update(changed_ids)
-        #  to keep only the truly changed ones
-        changed_kind = (t for t in changed_kind
-                        if self.tree_kind(t) != self.final_kind(t))
-        # all kind changes will alter the inventory
-        changed_ids.update(changed_kind)
-        # To find entries with changed parent_ids, find parents which existed,
-        # but changed file_id.
-        # Now add all their children to the set.
-        for parent_trans_id in new_file_id:
-            changed_ids.update(self.iter_tree_children(parent_trans_id))
-        return sorted(FinalPaths(self).get_paths(changed_ids))
+        raise NotImplementedError(self.new_paths)
 
     def final_kind(self, trans_id):
         """Determine the final file kind, after any changes applied.
@@ -442,40 +362,8 @@
         """Determine the tree path associated with the trans_id."""
         return self._tree_id_paths.get(trans_id)
 
-    def tree_file_id(self, trans_id):
-        """Determine the file id associated with the trans_id in the tree"""
-        path = self.tree_path(trans_id)
-        if path is None:
-            return None
-        # the file is old; the old id is still valid
-        if self._new_root == trans_id:
-            return self._tree.path2id('')
-        return self._tree.path2id(path)
-
-    def final_file_id(self, trans_id):
-        """Determine the file id after any changes are applied, or None.
-
-        None indicates that the file will not be versioned after changes are
-        applied.
-        """
-        try:
-            return self._new_id[trans_id]
-        except KeyError:
-            if trans_id in self._removed_id:
-                return None
-        return self.tree_file_id(trans_id)
-
-    def inactive_file_id(self, trans_id):
-        """Return the inactive file_id associated with a transaction id.
-        That is, the one in the tree or in non_present_ids.
-        The file_id may actually be active, too.
-        """
-        file_id = self.tree_file_id(trans_id)
-        if file_id is not None:
-            return file_id
-        for key, value in self._non_present_ids.items():
-            if value == trans_id:
-                return key
+    def final_is_versioned(self, trans_id):
+        raise NotImplementedError(self.final_is_versioned)
 
     def final_parent(self, trans_id):
         """Determine the parent file_id, after any changes are applied.
@@ -497,21 +385,6 @@
             except KeyError:
                 raise NoFinalPath(trans_id, self)
 
-    def by_parent(self):
-        """Return a map of parent: children for known parents.
-
-        Only new paths and parents of tree files with assigned ids are used.
-        """
-        by_parent = {}
-        items = list(self._new_parent.items())
-        items.extend((t, self.final_parent(t))
-                     for t in list(self._tree_id_paths))
-        for trans_id, parent_id in items:
-            if parent_id not in by_parent:
-                by_parent[parent_id] = set()
-            by_parent[parent_id].add(trans_id)
-        return by_parent
-
     def path_changed(self, trans_id):
         """Return True if a trans_id's path has changed."""
         return (trans_id in self._new_name) or (trans_id in self._new_parent)
@@ -521,266 +394,7 @@
 
     def find_conflicts(self):
         """Find any violations of inventory or filesystem invariants"""
-        if self._done is True:
-            raise ReusingTransform()
-        conflicts = []
-        # ensure all children of all existent parents are known
-        # all children of non-existent parents are known, by definition.
-        self._add_tree_children()
-        by_parent = self.by_parent()
-        conflicts.extend(self._unversioned_parents(by_parent))
-        conflicts.extend(self._parent_loops())
-        conflicts.extend(self._duplicate_entries(by_parent))
-        conflicts.extend(self._duplicate_ids())
-        conflicts.extend(self._parent_type_conflicts(by_parent))
-        conflicts.extend(self._improper_versioning())
-        conflicts.extend(self._executability_conflicts())
-        conflicts.extend(self._overwrite_conflicts())
-        return conflicts
-
-    def _check_malformed(self):
-        conflicts = self.find_conflicts()
-        if len(conflicts) != 0:
-            raise MalformedTransform(conflicts=conflicts)
-
-    def _add_tree_children(self):
-        """Add all the children of all active parents to the known paths.
-
-        Active parents are those which gain children, and those which are
-        removed.  This is a necessary first step in detecting conflicts.
-        """
-        parents = list(self.by_parent())
-        parents.extend([t for t in self._removed_contents if
-                        self.tree_kind(t) == 'directory'])
-        for trans_id in self._removed_id:
-            path = self.tree_path(trans_id)
-            if path is not None:
-                if self._tree.stored_kind(path) == 'directory':
-                    parents.append(trans_id)
-            elif self.tree_kind(trans_id) == 'directory':
-                parents.append(trans_id)
-
-        for parent_id in parents:
-            # ensure that all children are registered with the transaction
-            list(self.iter_tree_children(parent_id))
-
-    def _has_named_child(self, name, parent_id, known_children):
-        """Does a parent already have a name child.
-
-        :param name: The searched for name.
-
-        :param parent_id: The parent for which the check is made.
-
-        :param known_children: The already known children. This should have
-            been recently obtained from `self.by_parent.get(parent_id)`
-            (or will be if None is passed).
-        """
-        if known_children is None:
-            known_children = self.by_parent().get(parent_id, [])
-        for child in known_children:
-            if self.final_name(child) == name:
-                return True
-        parent_path = self._tree_id_paths.get(parent_id, None)
-        if parent_path is None:
-            # No parent... no children
-            return False
-        child_path = joinpath(parent_path, name)
-        child_id = self._tree_path_ids.get(child_path, None)
-        if child_id is None:
-            # Not known by the tree transform yet, check the filesystem
-            return osutils.lexists(self._tree.abspath(child_path))
-        else:
-            raise AssertionError('child_id is missing: %s, %s, %s'
-                                 % (name, parent_id, child_id))
-
-    def _available_backup_name(self, name, target_id):
-        """Find an available backup name.
-
-        :param name: The basename of the file.
-
-        :param target_id: The directory trans_id where the backup should
-            be placed.
-        """
-        known_children = self.by_parent().get(target_id, [])
-        return osutils.available_backup_name(
-            name,
-            lambda base: self._has_named_child(
-                base, target_id, known_children))
-
-    def _parent_loops(self):
-        """No entry should be its own ancestor"""
-        conflicts = []
-        for trans_id in self._new_parent:
-            seen = set()
-            parent_id = trans_id
-            while parent_id != ROOT_PARENT:
-                seen.add(parent_id)
-                try:
-                    parent_id = self.final_parent(parent_id)
-                except KeyError:
-                    break
-                if parent_id == trans_id:
-                    conflicts.append(('parent loop', trans_id))
-                if parent_id in seen:
-                    break
-        return conflicts
-
-    def _unversioned_parents(self, by_parent):
-        """If parent directories are versioned, children must be versioned."""
-        conflicts = []
-        for parent_id, children in by_parent.items():
-            if parent_id == ROOT_PARENT:
-                continue
-            if self.final_file_id(parent_id) is not None:
-                continue
-            for child_id in children:
-                if self.final_file_id(child_id) is not None:
-                    conflicts.append(('unversioned parent', parent_id))
-                    break
-        return conflicts
-
-    def _improper_versioning(self):
-        """Cannot version a file with no contents, or a bad type.
-
-        However, existing entries with no contents are okay.
-        """
-        conflicts = []
-        for trans_id in self._new_id:
-            kind = self.final_kind(trans_id)
-            if kind == 'symlink' and not self._tree.supports_symlinks():
-                # Ignore symlinks as they are not supported on this platform
-                continue
-            if kind is None:
-                conflicts.append(('versioning no contents', trans_id))
-                continue
-            if not self._tree.versionable_kind(kind):
-                conflicts.append(('versioning bad kind', trans_id, kind))
-        return conflicts
-
-    def _executability_conflicts(self):
-        """Check for bad executability changes.
-
-        Only versioned files may have their executability set, because
-        1. only versioned entries can have executability under windows
-        2. only files can be executable.  (The execute bit on a directory
-           does not indicate searchability)
-        """
-        conflicts = []
-        for trans_id in self._new_executability:
-            if self.final_file_id(trans_id) is None:
-                conflicts.append(('unversioned executability', trans_id))
-            else:
-                if self.final_kind(trans_id) != "file":
-                    conflicts.append(('non-file executability', trans_id))
-        return conflicts
-
-    def _overwrite_conflicts(self):
-        """Check for overwrites (not permitted on Win32)"""
-        conflicts = []
-        for trans_id in self._new_contents:
-            if self.tree_kind(trans_id) is None:
-                continue
-            if trans_id not in self._removed_contents:
-                conflicts.append(('overwrite', trans_id,
-                                  self.final_name(trans_id)))
-        return conflicts
-
-    def _duplicate_entries(self, by_parent):
-        """No directory may have two entries with the same name."""
-        conflicts = []
-        if (self._new_name, self._new_parent) == ({}, {}):
-            return conflicts
-        for children in by_parent.values():
-            name_ids = []
-            for child_tid in children:
-                name = self.final_name(child_tid)
-                if name is not None:
-                    # Keep children only if they still exist in the end
-                    if not self._case_sensitive_target:
-                        name = name.lower()
-                    name_ids.append((name, child_tid))
-            name_ids.sort()
-            last_name = None
-            last_trans_id = None
-            for name, trans_id in name_ids:
-                kind = self.final_kind(trans_id)
-                file_id = self.final_file_id(trans_id)
-                if kind is None and file_id is None:
-                    continue
-                if name == last_name:
-                    conflicts.append(('duplicate', last_trans_id, trans_id,
-                                      name))
-                last_name = name
-                last_trans_id = trans_id
-        return conflicts
-
-    def _duplicate_ids(self):
-        """Each inventory id may only be used once"""
-        conflicts = []
-        try:
-            all_ids = self._tree.all_file_ids()
-        except errors.UnsupportedOperation:
-            # it's okay for non-file-id trees to raise UnsupportedOperation.
-            return []
-        removed_tree_ids = set((self.tree_file_id(trans_id) for trans_id in
-                                self._removed_id))
-        active_tree_ids = all_ids.difference(removed_tree_ids)
-        for trans_id, file_id in self._new_id.items():
-            if file_id in active_tree_ids:
-                path = self._tree.id2path(file_id)
-                old_trans_id = self.trans_id_tree_path(path)
-                conflicts.append(('duplicate id', old_trans_id, trans_id))
-        return conflicts
-
-    def _parent_type_conflicts(self, by_parent):
-        """Children must have a directory parent"""
-        conflicts = []
-        for parent_id, children in by_parent.items():
-            if parent_id == ROOT_PARENT:
-                continue
-            no_children = True
-            for child_id in children:
-                if self.final_kind(child_id) is not None:
-                    no_children = False
-                    break
-            if no_children:
-                continue
-            # There is at least a child, so we need an existing directory to
-            # contain it.
-            kind = self.final_kind(parent_id)
-            if kind is None:
-                # The directory will be deleted
-                conflicts.append(('missing parent', parent_id))
-            elif kind != "directory":
-                # Meh, we need a *directory* to put something in it
-                conflicts.append(('non-directory parent', parent_id))
-        return conflicts
-
-    def _set_executability(self, path, trans_id):
-        """Set the executability of versioned files """
-        if self._tree._supports_executable():
-            new_executability = self._new_executability[trans_id]
-            abspath = self._tree.abspath(path)
-            current_mode = os.stat(abspath).st_mode
-            if new_executability:
-                umask = os.umask(0)
-                os.umask(umask)
-                to_mode = current_mode | (0o100 & ~umask)
-                # Enable x-bit for others only if they can read it.
-                if current_mode & 0o004:
-                    to_mode |= 0o001 & ~umask
-                if current_mode & 0o040:
-                    to_mode |= 0o010 & ~umask
-            else:
-                to_mode = current_mode & ~0o111
-            osutils.chmod_if_possible(abspath, to_mode)
-
-    def _new_entry(self, name, parent_id, file_id):
-        """Helper function to create a new filesystem entry."""
-        trans_id = self.create_path(name, parent_id)
-        if file_id is not None:
-            self.version_file(file_id, trans_id)
-        return trans_id
+        raise NotImplementedError(self.find_conflicts)
 
     def new_file(self, name, parent_id, contents, file_id=None,
                  executable=None, sha1=None):
@@ -793,13 +407,7 @@
         :param file_id: The inventory ID of the file, if it is to be versioned.
         :param executable: Only valid when a file_id has been supplied.
         """
-        trans_id = self._new_entry(name, parent_id, file_id)
-        # TODO: rather than scheduling a set_executable call,
-        # have create_file create the file with the right mode.
-        self.create_file(contents, trans_id, sha1=sha1)
-        if executable is not None:
-            self.set_executability(executable, trans_id)
-        return trans_id
+        raise NotImplementedError(self.new_file)
 
     def new_directory(self, name, parent_id, file_id=None):
         """Convenience method to create directories.
@@ -809,9 +417,7 @@
         directory.
         file_id is the inventory ID of the directory, if it is to be versioned.
         """
-        trans_id = self._new_entry(name, parent_id, file_id)
-        self.create_directory(trans_id)
-        return trans_id
+        raise NotImplementedError(self.new_directory)
 
     def new_symlink(self, name, parent_id, target, file_id=None):
         """Convenience method to create symbolic link.
@@ -821,9 +427,7 @@
         target is a bytestring of the target of the symlink.
         file_id is the inventory ID of the file, if it is to be versioned.
         """
-        trans_id = self._new_entry(name, parent_id, file_id)
-        self.create_symlink(target, trans_id)
-        return trans_id
+        raise NotImplementedError(self.new_symlink)
 
     def new_orphan(self, trans_id, parent_id):
         """Schedule an item to be orphaned.
@@ -836,112 +440,6 @@
         """
         raise NotImplementedError(self.new_orphan)
 
-    def _get_potential_orphans(self, dir_id):
-        """Find the potential orphans in a directory.
-
-        A directory can't be safely deleted if there are versioned files in it.
-        If all the contained files are unversioned then they can be orphaned.
-
-        The 'None' return value means that the directory contains at least one
-        versioned file and should not be deleted.
-
-        :param dir_id: The directory trans id.
-
-        :return: A list of the orphan trans ids or None if at least one
-             versioned file is present.
-        """
-        orphans = []
-        # Find the potential orphans, stop if one item should be kept
-        for child_tid in self.by_parent()[dir_id]:
-            if child_tid in self._removed_contents:
-                # The child is removed as part of the transform. Since it was
-                # versioned before, it's not an orphan
-                continue
-            elif self.final_file_id(child_tid) is None:
-                # The child is not versioned
-                orphans.append(child_tid)
-            else:
-                # We have a versioned file here, searching for orphans is
-                # meaningless.
-                orphans = None
-                break
-        return orphans
-
-    def _affected_ids(self):
-        """Return the set of transform ids affected by the transform"""
-        trans_ids = set(self._removed_id)
-        trans_ids.update(self._new_id)
-        trans_ids.update(self._removed_contents)
-        trans_ids.update(self._new_contents)
-        trans_ids.update(self._new_executability)
-        trans_ids.update(self._new_name)
-        trans_ids.update(self._new_parent)
-        return trans_ids
-
-    def _get_file_id_maps(self):
-        """Return mapping of file_ids to trans_ids in the to and from states"""
-        trans_ids = self._affected_ids()
-        from_trans_ids = {}
-        to_trans_ids = {}
-        # Build up two dicts: trans_ids associated with file ids in the
-        # FROM state, vs the TO state.
-        for trans_id in trans_ids:
-            from_file_id = self.tree_file_id(trans_id)
-            if from_file_id is not None:
-                from_trans_ids[from_file_id] = trans_id
-            to_file_id = self.final_file_id(trans_id)
-            if to_file_id is not None:
-                to_trans_ids[to_file_id] = trans_id
-        return from_trans_ids, to_trans_ids
-
-    def _from_file_data(self, from_trans_id, from_versioned, from_path):
-        """Get data about a file in the from (tree) state
-
-        Return a (name, parent, kind, executable) tuple
-        """
-        from_path = self._tree_id_paths.get(from_trans_id)
-        if from_versioned:
-            # get data from working tree if versioned
-            from_entry = next(self._tree.iter_entries_by_dir(
-                specific_files=[from_path]))[1]
-            from_name = from_entry.name
-            from_parent = from_entry.parent_id
-        else:
-            from_entry = None
-            if from_path is None:
-                # File does not exist in FROM state
-                from_name = None
-                from_parent = None
-            else:
-                # File exists, but is not versioned.  Have to use path-
-                # splitting stuff
-                from_name = os.path.basename(from_path)
-                tree_parent = self.get_tree_parent(from_trans_id)
-                from_parent = self.tree_file_id(tree_parent)
-        if from_path is not None:
-            from_kind, from_executable, from_stats = \
-                self._tree._comparison_data(from_entry, from_path)
-        else:
-            from_kind = None
-            from_executable = False
-        return from_name, from_parent, from_kind, from_executable
-
-    def _to_file_data(self, to_trans_id, from_trans_id, from_executable):
-        """Get data about a file in the to (target) state
-
-        Return a (name, parent, kind, executable) tuple
-        """
-        to_name = self.final_name(to_trans_id)
-        to_kind = self.final_kind(to_trans_id)
-        to_parent = self.final_file_id(self.final_parent(to_trans_id))
-        if to_trans_id in self._new_executability:
-            to_executable = self._new_executability[to_trans_id]
-        elif to_trans_id == from_trans_id:
-            to_executable = from_executable
-        else:
-            to_executable = False
-        return to_name, to_parent, to_kind, to_executable
-
     def iter_changes(self):
         """Produce output in the same format as Tree.iter_changes.
 
@@ -952,63 +450,7 @@
         file_id.  Files that are not versioned in either of the FROM or TO
         states are not reflected.
         """
-        final_paths = FinalPaths(self)
-        from_trans_ids, to_trans_ids = self._get_file_id_maps()
-        results = []
-        # Now iterate through all active file_ids
-        for file_id in set(from_trans_ids).union(to_trans_ids):
-            modified = False
-            from_trans_id = from_trans_ids.get(file_id)
-            # find file ids, and determine versioning state
-            if from_trans_id is None:
-                from_versioned = False
-                from_trans_id = to_trans_ids[file_id]
-            else:
-                from_versioned = True
-            to_trans_id = to_trans_ids.get(file_id)
-            if to_trans_id is None:
-                to_versioned = False
-                to_trans_id = from_trans_id
-            else:
-                to_versioned = True
-
-            if not from_versioned:
-                from_path = None
-            else:
-                from_path = self._tree_id_paths.get(from_trans_id)
-            if not to_versioned:
-                to_path = None
-            else:
-                to_path = final_paths.get_path(to_trans_id)
-
-            from_name, from_parent, from_kind, from_executable = \
-                self._from_file_data(from_trans_id, from_versioned, from_path)
-
-            to_name, to_parent, to_kind, to_executable = \
-                self._to_file_data(to_trans_id, from_trans_id, from_executable)
-
-            if from_kind != to_kind:
-                modified = True
-            elif to_kind in ('file', 'symlink') and (
-                    to_trans_id != from_trans_id
-                    or to_trans_id in self._new_contents):
-                modified = True
-            if (not modified and from_versioned == to_versioned
-                and from_parent == to_parent and from_name == to_name
-                    and from_executable == to_executable):
-                continue
-            results.append(
-                TreeChange(
-                    file_id, (from_path, to_path), modified,
-                    (from_versioned, to_versioned),
-                    (from_parent, to_parent),
-                    (from_name, to_name),
-                    (from_kind, to_kind),
-                    (from_executable, to_executable)))
-
-        def path_key(c):
-            return (c.path[0] or '', c.path[1] or '')
-        return iter(sorted(results, key=path_key))
+        raise NotImplementedError(self.iter_changes)
 
     def get_preview_tree(self):
         """Return a tree representing the result of the transform.
@@ -1016,7 +458,7 @@
         The tree is a snapshot, and altering the TreeTransform will invalidate
         it.
         """
-        return _PreviewTree(self)
+        raise NotImplementedError(self.get_preview_tree)
 
     def commit(self, branch, message, merge_parents=None, strict=False,
                timestamp=None, timezone=None, committer=None, authors=None,
@@ -1041,159 +483,7 @@
             may reduce performance for some non-native formats.)
         :return: The revision_id of the revision committed.
         """
-        self._check_malformed()
-        if strict:
-            unversioned = set(self._new_contents).difference(set(self._new_id))
-            for trans_id in unversioned:
-                if self.final_file_id(trans_id) is None:
-                    raise errors.StrictCommitFailed()
-
-        revno, last_rev_id = branch.last_revision_info()
-        if last_rev_id == _mod_revision.NULL_REVISION:
-            if merge_parents is not None:
-                raise ValueError('Cannot supply merge parents for first'
-                                 ' commit.')
-            parent_ids = []
-        else:
-            parent_ids = [last_rev_id]
-            if merge_parents is not None:
-                parent_ids.extend(merge_parents)
-        if self._tree.get_revision_id() != last_rev_id:
-            raise ValueError('TreeTransform not based on branch basis: %s' %
-                             self._tree.get_revision_id().decode('utf-8'))
-        revprops = commit.Commit.update_revprops(revprops, branch, authors)
-        builder = branch.get_commit_builder(parent_ids,
-                                            timestamp=timestamp,
-                                            timezone=timezone,
-                                            committer=committer,
-                                            revprops=revprops,
-                                            revision_id=revision_id)
-        preview = self.get_preview_tree()
-        list(builder.record_iter_changes(preview, last_rev_id,
-                                         self.iter_changes()))
-        builder.finish_inventory()
-        revision_id = builder.commit(message)
-        branch.set_last_revision_info(revno + 1, revision_id)
-        return revision_id
-
-    def _text_parent(self, trans_id):
-        path = self.tree_path(trans_id)
-        try:
-            if path is None or self._tree.kind(path) != 'file':
-                return None
-        except errors.NoSuchFile:
-            return None
-        return path
-
-    def _get_parents_texts(self, trans_id):
-        """Get texts for compression parents of this file."""
-        path = self._text_parent(trans_id)
-        if path is None:
-            return ()
-        return (self._tree.get_file_text(path),)
-
-    def _get_parents_lines(self, trans_id):
-        """Get lines for compression parents of this file."""
-        path = self._text_parent(trans_id)
-        if path is None:
-            return ()
-        return (self._tree.get_file_lines(path),)
-
-    def serialize(self, serializer):
-        """Serialize this TreeTransform.
-
-        :param serializer: A Serialiser like pack.ContainerSerializer.
-        """
-        new_name = {k.encode('utf-8'): v.encode('utf-8')
-                    for k, v in self._new_name.items()}
-        new_parent = {k.encode('utf-8'): v.encode('utf-8')
-                      for k, v in self._new_parent.items()}
-        new_id = {k.encode('utf-8'): v
-                  for k, v in self._new_id.items()}
-        new_executability = {k.encode('utf-8'): int(v)
-                             for k, v in self._new_executability.items()}
-        tree_path_ids = {k.encode('utf-8'): v.encode('utf-8')
-                         for k, v in self._tree_path_ids.items()}
-        non_present_ids = {k: v.encode('utf-8')
-                           for k, v in self._non_present_ids.items()}
-        removed_contents = [trans_id.encode('utf-8')
-                            for trans_id in self._removed_contents]
-        removed_id = [trans_id.encode('utf-8')
-                      for trans_id in self._removed_id]
-        attribs = {
-            b'_id_number': self._id_number,
-            b'_new_name': new_name,
-            b'_new_parent': new_parent,
-            b'_new_executability': new_executability,
-            b'_new_id': new_id,
-            b'_tree_path_ids': tree_path_ids,
-            b'_removed_id': removed_id,
-            b'_removed_contents': removed_contents,
-            b'_non_present_ids': non_present_ids,
-            }
-        yield serializer.bytes_record(bencode.bencode(attribs),
-                                      ((b'attribs',),))
-        for trans_id, kind in sorted(self._new_contents.items()):
-            if kind == 'file':
-                with open(self._limbo_name(trans_id), 'rb') as cur_file:
-                    lines = cur_file.readlines()
-                parents = self._get_parents_lines(trans_id)
-                mpdiff = multiparent.MultiParent.from_lines(lines, parents)
-                content = b''.join(mpdiff.to_patch())
-            if kind == 'directory':
-                content = b''
-            if kind == 'symlink':
-                content = self._read_symlink_target(trans_id)
-                if not isinstance(content, bytes):
-                    content = content.encode('utf-8')
-            yield serializer.bytes_record(
-                content, ((trans_id.encode('utf-8'), kind.encode('ascii')),))
-
-    def deserialize(self, records):
-        """Deserialize a stored TreeTransform.
-
-        :param records: An iterable of (names, content) tuples, as per
-            pack.ContainerPushParser.
-        """
-        names, content = next(records)
-        attribs = bencode.bdecode(content)
-        self._id_number = attribs[b'_id_number']
-        self._new_name = {k.decode('utf-8'): v.decode('utf-8')
-                          for k, v in attribs[b'_new_name'].items()}
-        self._new_parent = {k.decode('utf-8'): v.decode('utf-8')
-                            for k, v in attribs[b'_new_parent'].items()}
-        self._new_executability = {
-            k.decode('utf-8'): bool(v)
-            for k, v in attribs[b'_new_executability'].items()}
-        self._new_id = {k.decode('utf-8'): v
-                        for k, v in attribs[b'_new_id'].items()}
-        self._r_new_id = {v: k for k, v in self._new_id.items()}
-        self._tree_path_ids = {}
-        self._tree_id_paths = {}
-        for bytepath, trans_id in attribs[b'_tree_path_ids'].items():
-            path = bytepath.decode('utf-8')
-            trans_id = trans_id.decode('utf-8')
-            self._tree_path_ids[path] = trans_id
-            self._tree_id_paths[trans_id] = path
-        self._removed_id = {trans_id.decode('utf-8')
-                            for trans_id in attribs[b'_removed_id']}
-        self._removed_contents = set(
-            trans_id.decode('utf-8')
-            for trans_id in attribs[b'_removed_contents'])
-        self._non_present_ids = {
-            k: v.decode('utf-8')
-            for k, v in attribs[b'_non_present_ids'].items()}
-        for ((trans_id, kind),), content in records:
-            trans_id = trans_id.decode('utf-8')
-            kind = kind.decode('ascii')
-            if kind == 'file':
-                mpdiff = multiparent.MultiParent.from_patch(content)
-                lines = mpdiff.to_lines(self._get_parents_texts(trans_id))
-                self.create_file(lines, trans_id)
-            if kind == 'directory':
-                self.create_directory(trans_id)
-            if kind == 'symlink':
-                self.create_symlink(content.decode('utf-8'), trans_id)
+        raise NotImplementedError(self.commit)
 
     def create_file(self, contents, trans_id, mode_id=None, sha1=None):
         """Schedule creation of a new file.
@@ -1235,237 +525,6 @@
         raise NotImplementedError(self.cancel_creation)
 
 
-class DiskTreeTransform(TreeTransformBase):
-    """Tree transform storing its contents on disk."""
-
-    def __init__(self, tree, limbodir, pb=None, case_sensitive=True):
-        """Constructor.
-        :param tree: The tree that will be transformed, but not necessarily
-            the output tree.
-        :param limbodir: A directory where new files can be stored until
-            they are installed in their proper places
-        :param pb: ignored
-        :param case_sensitive: If True, the target of the transform is
-            case sensitive, not just case preserving.
-        """
-        TreeTransformBase.__init__(self, tree, pb, case_sensitive)
-        self._limbodir = limbodir
-        self._deletiondir = None
-        # A mapping of transform ids to their limbo filename
-        self._limbo_files = {}
-        self._possibly_stale_limbo_files = set()
-        # A mapping of transform ids to a set of the transform ids of children
-        # that their limbo directory has
-        self._limbo_children = {}
-        # Map transform ids to maps of child filename to child transform id
-        self._limbo_children_names = {}
-        # List of transform ids that need to be renamed from limbo into place
-        self._needs_rename = set()
-        self._creation_mtime = None
-        self._create_symlinks = osutils.supports_symlinks(self._limbodir)
-
-    def finalize(self):
-        """Release the working tree lock, if held, clean up limbo dir.
-
-        This is required if apply has not been invoked, but can be invoked
-        even after apply.
-        """
-        if self._tree is None:
-            return
-        try:
-            limbo_paths = list(self._limbo_files.values())
-            limbo_paths.extend(self._possibly_stale_limbo_files)
-            limbo_paths.sort(reverse=True)
-            for path in limbo_paths:
-                try:
-                    delete_any(path)
-                except OSError as e:
-                    if e.errno != errno.ENOENT:
-                        raise
-                    # XXX: warn? perhaps we just got interrupted at an
-                    # inconvenient moment, but perhaps files are disappearing
-                    # from under us?
-            try:
-                delete_any(self._limbodir)
-            except OSError:
-                # We don't especially care *why* the dir is immortal.
-                raise ImmortalLimbo(self._limbodir)
-            try:
-                if self._deletiondir is not None:
-                    delete_any(self._deletiondir)
-            except OSError:
-                raise errors.ImmortalPendingDeletion(self._deletiondir)
-        finally:
-            TreeTransformBase.finalize(self)
-
-    def _limbo_supports_executable(self):
-        """Check if the limbo path supports the executable bit."""
-        return osutils.supports_executable(self._limbodir)
-
-    def _limbo_name(self, trans_id):
-        """Generate the limbo name of a file"""
-        limbo_name = self._limbo_files.get(trans_id)
-        if limbo_name is None:
-            limbo_name = self._generate_limbo_path(trans_id)
-            self._limbo_files[trans_id] = limbo_name
-        return limbo_name
-
-    def _generate_limbo_path(self, trans_id):
-        """Generate a limbo path using the trans_id as the relative path.
-
-        This is suitable as a fallback, and when the transform should not be
-        sensitive to the path encoding of the limbo directory.
-        """
-        self._needs_rename.add(trans_id)
-        return pathjoin(self._limbodir, trans_id)
-
-    def adjust_path(self, name, parent, trans_id):
-        previous_parent = self._new_parent.get(trans_id)
-        previous_name = self._new_name.get(trans_id)
-        TreeTransformBase.adjust_path(self, name, parent, trans_id)
-        if (trans_id in self._limbo_files
-                and trans_id not in self._needs_rename):
-            self._rename_in_limbo([trans_id])
-            if previous_parent != parent:
-                self._limbo_children[previous_parent].remove(trans_id)
-            if previous_parent != parent or previous_name != name:
-                del self._limbo_children_names[previous_parent][previous_name]
-
-    def _rename_in_limbo(self, trans_ids):
-        """Fix limbo names so that the right final path is produced.
-
-        This means we outsmarted ourselves-- we tried to avoid renaming
-        these files later by creating them with their final names in their
-        final parents.  But now the previous name or parent is no longer
-        suitable, so we have to rename them.
-
-        Even for trans_ids that have no new contents, we must remove their
-        entries from _limbo_files, because they are now stale.
-        """
-        for trans_id in trans_ids:
-            old_path = self._limbo_files[trans_id]
-            self._possibly_stale_limbo_files.add(old_path)
-            del self._limbo_files[trans_id]
-            if trans_id not in self._new_contents:
-                continue
-            new_path = self._limbo_name(trans_id)
-            os.rename(old_path, new_path)
-            self._possibly_stale_limbo_files.remove(old_path)
-            for descendant in self._limbo_descendants(trans_id):
-                desc_path = self._limbo_files[descendant]
-                desc_path = new_path + desc_path[len(old_path):]
-                self._limbo_files[descendant] = desc_path
-
-    def _limbo_descendants(self, trans_id):
-        """Return the set of trans_ids whose limbo paths descend from this."""
-        descendants = set(self._limbo_children.get(trans_id, []))
-        for descendant in list(descendants):
-            descendants.update(self._limbo_descendants(descendant))
-        return descendants
-
-    def create_file(self, contents, trans_id, mode_id=None, sha1=None):
-        """Schedule creation of a new file.
-
-        :seealso: new_file.
-
-        :param contents: an iterator of strings, all of which will be written
-            to the target destination.
-        :param trans_id: TreeTransform handle
-        :param mode_id: If not None, force the mode of the target file to match
-            the mode of the object referenced by mode_id.
-            Otherwise, we will try to preserve mode bits of an existing file.
-        :param sha1: If the sha1 of this content is already known, pass it in.
-            We can use it to prevent future sha1 computations.
-        """
-        name = self._limbo_name(trans_id)
-        with open(name, 'wb') as f:
-            unique_add(self._new_contents, trans_id, 'file')
-            f.writelines(contents)
-        self._set_mtime(name)
-        self._set_mode(trans_id, mode_id, S_ISREG)
-        # It is unfortunate we have to use lstat instead of fstat, but we just
-        # used utime and chmod on the file, so we need the accurate final
-        # details.
-        if sha1 is not None:
-            self._observed_sha1s[trans_id] = (sha1, osutils.lstat(name))
-
-    def _read_symlink_target(self, trans_id):
-        return os.readlink(self._limbo_name(trans_id))
-
-    def _set_mtime(self, path):
-        """All files that are created get the same mtime.
-
-        This time is set by the first object to be created.
-        """
-        if self._creation_mtime is None:
-            self._creation_mtime = time.time()
-        os.utime(path, (self._creation_mtime, self._creation_mtime))
-
-    def create_hardlink(self, path, trans_id):
-        """Schedule creation of a hard link"""
-        name = self._limbo_name(trans_id)
-        try:
-            os.link(path, name)
-        except OSError as e:
-            if e.errno != errno.EPERM:
-                raise
-            raise errors.HardLinkNotSupported(path)
-        try:
-            unique_add(self._new_contents, trans_id, 'file')
-        except BaseException:
-            # Clean up the file, it never got registered so
-            # TreeTransform.finalize() won't clean it up.
-            os.unlink(name)
-            raise
-
-    def create_directory(self, trans_id):
-        """Schedule creation of a new directory.
-
-        See also new_directory.
-        """
-        os.mkdir(self._limbo_name(trans_id))
-        unique_add(self._new_contents, trans_id, 'directory')
-
-    def create_symlink(self, target, trans_id):
-        """Schedule creation of a new symbolic link.
-
-        target is a bytestring.
-        See also new_symlink.
-        """
-        if self._create_symlinks:
-            os.symlink(target, self._limbo_name(trans_id))
-        else:
-            try:
-                path = FinalPaths(self).get_path(trans_id)
-            except KeyError:
-                path = None
-            trace.warning(
-                'Unable to create symlink "%s" on this filesystem.' % (path,))
-        # We add symlink to _new_contents even if they are unsupported
-        # and not created. These entries are subsequently used to avoid
-        # conflicts on platforms that don't support symlink
-        unique_add(self._new_contents, trans_id, 'symlink')
-
-    def cancel_creation(self, trans_id):
-        """Cancel the creation of new file contents."""
-        del self._new_contents[trans_id]
-        if trans_id in self._observed_sha1s:
-            del self._observed_sha1s[trans_id]
-        children = self._limbo_children.get(trans_id)
-        # if this is a limbo directory with children, move them before removing
-        # the directory
-        if children is not None:
-            self._rename_in_limbo(children)
-            del self._limbo_children[trans_id]
-            del self._limbo_children_names[trans_id]
-        delete_any(self._limbo_name(trans_id))
-
-    def new_orphan(self, trans_id, parent_id):
-        conf = self._tree.get_config_stack()
-        handle_orphan = conf.get('transform.orphan_policy')
-        handle_orphan(self, trans_id, parent_id)
-
-
 class OrphaningError(errors.BzrError):
 
     # Only bugs could lead to such exception being seen by the user
@@ -1537,989 +596,6 @@
     invalid='warning')
 
 
-class TreeTransform(DiskTreeTransform):
-    """Represent a tree transformation.
-
-    This object is designed to support incremental generation of the transform,
-    in any order.
-
-    However, it gives optimum performance when parent directories are created
-    before their contents.  The transform is then able to put child files
-    directly in their parent directory, avoiding later renames.
-
-    It is easy to produce malformed transforms, but they are generally
-    harmless.  Attempting to apply a malformed transform will cause an
-    exception to be raised before any modifications are made to the tree.
-
-    Many kinds of malformed transforms can be corrected with the
-    resolve_conflicts function.  The remaining ones indicate programming error,
-    such as trying to create a file with no path.
-
-    Two sets of file creation methods are supplied.  Convenience methods are:
-     * new_file
-     * new_directory
-     * new_symlink
-
-    These are composed of the low-level methods:
-     * create_path
-     * create_file or create_directory or create_symlink
-     * version_file
-     * set_executability
-
-    Transform/Transaction ids
-    -------------------------
-    trans_ids are temporary ids assigned to all files involved in a transform.
-    It's possible, even common, that not all files in the Tree have trans_ids.
-
-    trans_ids are used because filenames and file_ids are not good enough
-    identifiers; filenames change, and not all files have file_ids.  File-ids
-    are also associated with trans-ids, so that moving a file moves its
-    file-id.
-
-    trans_ids are only valid for the TreeTransform that generated them.
-
-    Limbo
-    -----
-    Limbo is a temporary directory use to hold new versions of files.
-    Files are added to limbo by create_file, create_directory, create_symlink,
-    and their convenience variants (new_*).  Files may be removed from limbo
-    using cancel_creation.  Files are renamed from limbo into their final
-    location as part of TreeTransform.apply
-
-    Limbo must be cleaned up, by either calling TreeTransform.apply or
-    calling TreeTransform.finalize.
-
-    Files are placed into limbo inside their parent directories, where
-    possible.  This reduces subsequent renames, and makes operations involving
-    lots of files faster.  This optimization is only possible if the parent
-    directory is created *before* creating any of its children, so avoid
-    creating children before parents, where possible.
-
-    Pending-deletion
-    ----------------
-    This temporary directory is used by _FileMover for storing files that are
-    about to be deleted.  In case of rollback, the files will be restored.
-    FileMover does not delete files until it is sure that a rollback will not
-    happen.
-    """
-
-    def __init__(self, tree, pb=None):
-        """Note: a tree_write lock is taken on the tree.
-
-        Use TreeTransform.finalize() to release the lock (can be omitted if
-        TreeTransform.apply() called).
-        """
-        tree.lock_tree_write()
-        try:
-            limbodir = urlutils.local_path_from_url(
-                tree._transport.abspath('limbo'))
-            osutils.ensure_empty_directory_exists(
-                limbodir,
-                errors.ExistingLimbo)
-            deletiondir = urlutils.local_path_from_url(
-                tree._transport.abspath('pending-deletion'))
-            osutils.ensure_empty_directory_exists(
-                deletiondir,
-                errors.ExistingPendingDeletion)
-        except BaseException:
-            tree.unlock()
-            raise
-
-        # Cache of realpath results, to speed up canonical_path
-        self._realpaths = {}
-        # Cache of relpath results, to speed up canonical_path
-        self._relpaths = {}
-        DiskTreeTransform.__init__(self, tree, limbodir, pb,
-                                   tree.case_sensitive)
-        self._deletiondir = deletiondir
-
-    def canonical_path(self, path):
-        """Get the canonical tree-relative path"""
-        # don't follow final symlinks
-        abs = self._tree.abspath(path)
-        if abs in self._relpaths:
-            return self._relpaths[abs]
-        dirname, basename = os.path.split(abs)
-        if dirname not in self._realpaths:
-            self._realpaths[dirname] = os.path.realpath(dirname)
-        dirname = self._realpaths[dirname]
-        abs = pathjoin(dirname, basename)
-        if dirname in self._relpaths:
-            relpath = pathjoin(self._relpaths[dirname], basename)
-            relpath = relpath.rstrip('/\\')
-        else:
-            relpath = self._tree.relpath(abs)
-        self._relpaths[abs] = relpath
-        return relpath
-
-    def tree_kind(self, trans_id):
-        """Determine the file kind in the working tree.
-
-        :returns: The file kind or None if the file does not exist
-        """
-        path = self._tree_id_paths.get(trans_id)
-        if path is None:
-            return None
-        try:
-            return file_kind(self._tree.abspath(path))
-        except errors.NoSuchFile:
-            return None
-
-    def _set_mode(self, trans_id, mode_id, typefunc):
-        """Set the mode of new file contents.
-        The mode_id is the existing file to get the mode from (often the same
-        as trans_id).  The operation is only performed if there's a mode match
-        according to typefunc.
-        """
-        if mode_id is None:
-            mode_id = trans_id
-        try:
-            old_path = self._tree_id_paths[mode_id]
-        except KeyError:
-            return
-        try:
-            mode = os.stat(self._tree.abspath(old_path)).st_mode
-        except OSError as e:
-            if e.errno in (errno.ENOENT, errno.ENOTDIR):
-                # Either old_path doesn't exist, or the parent of the
-                # target is not a directory (but will be one eventually)
-                # Either way, we know it doesn't exist *right now*
-                # See also bug #248448
-                return
-            else:
-                raise
-        if typefunc(mode):
-            osutils.chmod_if_possible(self._limbo_name(trans_id), mode)
-
-    def iter_tree_children(self, parent_id):
-        """Iterate through the entry's tree children, if any"""
-        try:
-            path = self._tree_id_paths[parent_id]
-        except KeyError:
-            return
-        try:
-            children = os.listdir(self._tree.abspath(path))
-        except OSError as e:
-            if not (osutils._is_error_enotdir(e) or
-                    e.errno in (errno.ENOENT, errno.ESRCH)):
-                raise
-            return
-
-        for child in children:
-            childpath = joinpath(path, child)
-            if self._tree.is_control_filename(childpath):
-                continue
-            yield self.trans_id_tree_path(childpath)
-
-    def _generate_limbo_path(self, trans_id):
-        """Generate a limbo path using the final path if possible.
-
-        This optimizes the performance of applying the tree transform by
-        avoiding renames.  These renames can be avoided only when the parent
-        directory is already scheduled for creation.
-
-        If the final path cannot be used, falls back to using the trans_id as
-        the relpath.
-        """
-        parent = self._new_parent.get(trans_id)
-        # if the parent directory is already in limbo (e.g. when building a
-        # tree), choose a limbo name inside the parent, to reduce further
-        # renames.
-        use_direct_path = False
-        if self._new_contents.get(parent) == 'directory':
-            filename = self._new_name.get(trans_id)
-            if filename is not None:
-                if parent not in self._limbo_children:
-                    self._limbo_children[parent] = set()
-                    self._limbo_children_names[parent] = {}
-                    use_direct_path = True
-                # the direct path can only be used if no other file has
-                # already taken this pathname, i.e. if the name is unused, or
-                # if it is already associated with this trans_id.
-                elif self._case_sensitive_target:
-                    if (self._limbo_children_names[parent].get(filename)
-                            in (trans_id, None)):
-                        use_direct_path = True
-                else:
-                    for l_filename, l_trans_id in (
-                            self._limbo_children_names[parent].items()):
-                        if l_trans_id == trans_id:
-                            continue
-                        if l_filename.lower() == filename.lower():
-                            break
-                    else:
-                        use_direct_path = True
-
-        if not use_direct_path:
-            return DiskTreeTransform._generate_limbo_path(self, trans_id)
-
-        limbo_name = pathjoin(self._limbo_files[parent], filename)
-        self._limbo_children[parent].add(trans_id)
-        self._limbo_children_names[parent][filename] = trans_id
-        return limbo_name
-
-    def apply(self, no_conflicts=False, precomputed_delta=None, _mover=None):
-        """Apply all changes to the inventory and filesystem.
-
-        If filesystem or inventory conflicts are present, MalformedTransform
-        will be thrown.
-
-        If apply succeeds, finalize is not necessary.
-
-        :param no_conflicts: if True, the caller guarantees there are no
-            conflicts, so no check is made.
-        :param precomputed_delta: An inventory delta to use instead of
-            calculating one.
-        :param _mover: Supply an alternate FileMover, for testing
-        """
-        for hook in MutableTree.hooks['pre_transform']:
-            hook(self._tree, self)
-        if not no_conflicts:
-            self._check_malformed()
-        with ui.ui_factory.nested_progress_bar() as child_pb:
-            if precomputed_delta is None:
-                child_pb.update(gettext('Apply phase'), 0, 2)
-                inventory_delta = self._generate_inventory_delta()
-                offset = 1
-            else:
-                inventory_delta = precomputed_delta
-                offset = 0
-            if _mover is None:
-                mover = _FileMover()
-            else:
-                mover = _mover
-            try:
-                child_pb.update(gettext('Apply phase'), 0 + offset, 2 + offset)
-                self._apply_removals(mover)
-                child_pb.update(gettext('Apply phase'), 1 + offset, 2 + offset)
-                modified_paths = self._apply_insertions(mover)
-            except BaseException:
-                mover.rollback()
-                raise
-            else:
-                mover.apply_deletions()
-        if self.final_file_id(self.root) is None:
-            inventory_delta = [e for e in inventory_delta if e[0] != '']
-        self._tree.apply_inventory_delta(inventory_delta)
-        self._apply_observed_sha1s()
-        self._done = True
-        self.finalize()
-        return _TransformResults(modified_paths, self.rename_count)
-
-    def _generate_inventory_delta(self):
-        """Generate an inventory delta for the current transform."""
-        inventory_delta = []
-        new_paths = self._inventory_altered()
-        total_entries = len(new_paths) + len(self._removed_id)
-        with ui.ui_factory.nested_progress_bar() as child_pb:
-            for num, trans_id in enumerate(self._removed_id):
-                if (num % 10) == 0:
-                    child_pb.update(gettext('removing file'),
-                                    num, total_entries)
-                if trans_id == self._new_root:
-                    file_id = self._tree.path2id('')
-                else:
-                    file_id = self.tree_file_id(trans_id)
-                # File-id isn't really being deleted, just moved
-                if file_id in self._r_new_id:
-                    continue
-                path = self._tree_id_paths[trans_id]
-                inventory_delta.append((path, None, file_id, None))
-            new_path_file_ids = dict((t, self.final_file_id(t)) for p, t in
-                                     new_paths)
-            for num, (path, trans_id) in enumerate(new_paths):
-                if (num % 10) == 0:
-                    child_pb.update(gettext('adding file'),
-                                    num + len(self._removed_id), total_entries)
-                file_id = new_path_file_ids[trans_id]
-                if file_id is None:
-                    continue
-                kind = self.final_kind(trans_id)
-                if kind is None:
-                    kind = self._tree.stored_kind(self._tree.id2path(file_id))
-                parent_trans_id = self.final_parent(trans_id)
-                parent_file_id = new_path_file_ids.get(parent_trans_id)
-                if parent_file_id is None:
-                    parent_file_id = self.final_file_id(parent_trans_id)
-                if trans_id in self._new_reference_revision:
-                    new_entry = inventory.TreeReference(
-                        file_id,
-                        self._new_name[trans_id],
-                        self.final_file_id(self._new_parent[trans_id]),
-                        None, self._new_reference_revision[trans_id])
-                else:
-                    new_entry = inventory.make_entry(kind,
-                                                     self.final_name(trans_id),
-                                                     parent_file_id, file_id)
-                try:
-                    old_path = self._tree.id2path(new_entry.file_id)
-                except errors.NoSuchId:
-                    old_path = None
-                new_executability = self._new_executability.get(trans_id)
-                if new_executability is not None:
-                    new_entry.executable = new_executability
-                inventory_delta.append(
-                    (old_path, path, new_entry.file_id, new_entry))
-        return inventory_delta
-
-    def _apply_removals(self, mover):
-        """Perform tree operations that remove directory/inventory names.
-
-        That is, delete files that are to be deleted, and put any files that
-        need renaming into limbo.  This must be done in strict child-to-parent
-        order.
-
-        If inventory_delta is None, no inventory delta generation is performed.
-        """
-        tree_paths = sorted(self._tree_path_ids.items(), reverse=True)
-        with ui.ui_factory.nested_progress_bar() as child_pb:
-            for num, (path, trans_id) in enumerate(tree_paths):
-                # do not attempt to move root into a subdirectory of itself.
-                if path == '':
-                    continue
-                child_pb.update(gettext('removing file'), num, len(tree_paths))
-                full_path = self._tree.abspath(path)
-                if trans_id in self._removed_contents:
-                    delete_path = os.path.join(self._deletiondir, trans_id)
-                    mover.pre_delete(full_path, delete_path)
-                elif (trans_id in self._new_name or
-                      trans_id in self._new_parent):
-                    try:
-                        mover.rename(full_path, self._limbo_name(trans_id))
-                    except errors.TransformRenameFailed as e:
-                        if e.errno != errno.ENOENT:
-                            raise
-                    else:
-                        self.rename_count += 1
-
-    def _apply_insertions(self, mover):
-        """Perform tree operations that insert directory/inventory names.
-
-        That is, create any files that need to be created, and restore from
-        limbo any files that needed renaming.  This must be done in strict
-        parent-to-child order.
-
-        If inventory_delta is None, no inventory delta is calculated, and
-        no list of modified paths is returned.
-        """
-        new_paths = self.new_paths(filesystem_only=True)
-        modified_paths = []
-        with ui.ui_factory.nested_progress_bar() as child_pb:
-            for num, (path, trans_id) in enumerate(new_paths):
-                if (num % 10) == 0:
-                    child_pb.update(gettext('adding file'),
-                                    num, len(new_paths))
-                full_path = self._tree.abspath(path)
-                if trans_id in self._needs_rename:
-                    try:
-                        mover.rename(self._limbo_name(trans_id), full_path)
-                    except errors.TransformRenameFailed as e:
-                        # We may be renaming a dangling inventory id
-                        if e.errno != errno.ENOENT:
-                            raise
-                    else:
-                        self.rename_count += 1
-                    # TODO: if trans_id in self._observed_sha1s, we should
-                    #       re-stat the final target, since ctime will be
-                    #       updated by the change.
-                if (trans_id in self._new_contents
-                        or self.path_changed(trans_id)):
-                    if trans_id in self._new_contents:
-                        modified_paths.append(full_path)
-                if trans_id in self._new_executability:
-                    self._set_executability(path, trans_id)
-                if trans_id in self._observed_sha1s:
-                    o_sha1, o_st_val = self._observed_sha1s[trans_id]
-                    st = osutils.lstat(full_path)
-                    self._observed_sha1s[trans_id] = (o_sha1, st)
-        for path, trans_id in new_paths:
-            # new_paths includes stuff like workingtree conflicts. Only the
-            # stuff in new_contents actually comes from limbo.
-            if trans_id in self._limbo_files:
-                del self._limbo_files[trans_id]
-        self._new_contents.clear()
-        return modified_paths
-
-    def _apply_observed_sha1s(self):
-        """After we have finished renaming everything, update observed sha1s
-
-        This has to be done after self._tree.apply_inventory_delta, otherwise
-        it doesn't know anything about the files we are updating. Also, we want
-        to do this as late as possible, so that most entries end up cached.
-        """
-        # TODO: this doesn't update the stat information for directories. So
-        #       the first 'bzr status' will still need to rewrite
-        #       .bzr/checkout/dirstate. However, we at least don't need to
-        #       re-read all of the files.
-        # TODO: If the operation took a while, we could do a time.sleep(3) here
-        #       to allow the clock to tick over and ensure we won't have any
-        #       problems. (we could observe start time, and finish time, and if
-        #       it is less than eg 10% overhead, add a sleep call.)
-        paths = FinalPaths(self)
-        for trans_id, observed in self._observed_sha1s.items():
-            path = paths.get_path(trans_id)
-            self._tree._observed_sha1(path, observed)
-
-
-class TransformPreview(DiskTreeTransform):
-    """A TreeTransform for generating preview trees.
-
-    Unlike TreeTransform, this version works when the input tree is a
-    RevisionTree, rather than a WorkingTree.  As a result, it tends to ignore
-    unversioned files in the input tree.
-    """
-
-    def __init__(self, tree, pb=None, case_sensitive=True):
-        tree.lock_read()
-        limbodir = osutils.mkdtemp(prefix='bzr-limbo-')
-        DiskTreeTransform.__init__(self, tree, limbodir, pb, case_sensitive)
-
-    def canonical_path(self, path):
-        return path
-
-    def tree_kind(self, trans_id):
-        path = self._tree_id_paths.get(trans_id)
-        if path is None:
-            return None
-        kind = self._tree.path_content_summary(path)[0]
-        if kind == 'missing':
-            kind = None
-        return kind
-
-    def _set_mode(self, trans_id, mode_id, typefunc):
-        """Set the mode of new file contents.
-        The mode_id is the existing file to get the mode from (often the same
-        as trans_id).  The operation is only performed if there's a mode match
-        according to typefunc.
-        """
-        # is it ok to ignore this?  probably
-        pass
-
-    def iter_tree_children(self, parent_id):
-        """Iterate through the entry's tree children, if any"""
-        try:
-            path = self._tree_id_paths[parent_id]
-        except KeyError:
-            return
-        try:
-            entry = next(self._tree.iter_entries_by_dir(
-                specific_files=[path]))[1]
-        except StopIteration:
-            return
-        children = getattr(entry, 'children', {})
-        for child in children:
-            childpath = joinpath(path, child)
-            yield self.trans_id_tree_path(childpath)
-
-    def new_orphan(self, trans_id, parent_id):
-        raise NotImplementedError(self.new_orphan)
-
-
-class _PreviewTree(inventorytree.InventoryTree):
-    """Partial implementation of Tree to support show_diff_trees"""
-
-    def __init__(self, transform):
-        self._transform = transform
-        self._final_paths = FinalPaths(transform)
-        self.__by_parent = None
-        self._parent_ids = []
-        self._all_children_cache = {}
-        self._path2trans_id_cache = {}
-        self._final_name_cache = {}
-        self._iter_changes_cache = dict((c.file_id, c) for c in
-                                        self._transform.iter_changes())
-
-    def supports_tree_reference(self):
-        # TODO(jelmer): Support tree references in _PreviewTree.
-        # return self._transform._tree.supports_tree_reference()
-        return False
-
-    def _content_change(self, file_id):
-        """Return True if the content of this file changed"""
-        changes = self._iter_changes_cache.get(file_id)
-        return (changes is not None and changes.changed_content)
-
-    def _get_repository(self):
-        repo = getattr(self._transform._tree, '_repository', None)
-        if repo is None:
-            repo = self._transform._tree.branch.repository
-        return repo
-
-    def _iter_parent_trees(self):
-        for revision_id in self.get_parent_ids():
-            try:
-                yield self.revision_tree(revision_id)
-            except errors.NoSuchRevisionInTree:
-                yield self._get_repository().revision_tree(revision_id)
-
-    def _get_file_revision(self, path, file_id, vf, tree_revision):
-        parent_keys = [
-            (file_id, t.get_file_revision(t.id2path(file_id)))
-            for t in self._iter_parent_trees()]
-        vf.add_lines((file_id, tree_revision), parent_keys,
-                     self.get_file_lines(path))
-        repo = self._get_repository()
-        base_vf = repo.texts
-        if base_vf not in vf.fallback_versionedfiles:
-            vf.fallback_versionedfiles.append(base_vf)
-        return tree_revision
-
-    def _stat_limbo_file(self, trans_id):
-        name = self._transform._limbo_name(trans_id)
-        return os.lstat(name)
-
-    @property
-    def _by_parent(self):
-        if self.__by_parent is None:
-            self.__by_parent = self._transform.by_parent()
-        return self.__by_parent
-
-    def _comparison_data(self, entry, path):
-        kind, size, executable, link_or_sha1 = self.path_content_summary(path)
-        if kind == 'missing':
-            kind = None
-            executable = False
-        else:
-            file_id = self._transform.final_file_id(self._path2trans_id(path))
-            executable = self.is_executable(path)
-        return kind, executable, None
-
-    def is_locked(self):
-        return False
-
-    def lock_read(self):
-        # Perhaps in theory, this should lock the TreeTransform?
-        return lock.LogicalLockResult(self.unlock)
-
-    def unlock(self):
-        pass
-
-    @property
-    def root_inventory(self):
-        """This Tree does not use inventory as its backing data."""
-        raise NotImplementedError(_PreviewTree.root_inventory)
-
-    def all_file_ids(self):
-        tree_ids = set(self._transform._tree.all_file_ids())
-        tree_ids.difference_update(self._transform.tree_file_id(t)
-                                   for t in self._transform._removed_id)
-        tree_ids.update(self._transform._new_id.values())
-        return tree_ids
-
-    def all_versioned_paths(self):
-        tree_paths = set(self._transform._tree.all_versioned_paths())
-
-        tree_paths.difference_update(
-            self._transform.trans_id_tree_path(t)
-            for t in self._transform._removed_id)
-
-        tree_paths.update(
-            self._final_paths._determine_path(t)
-            for t in self._transform._new_id)
-
-        return tree_paths
-
-    def _path2trans_id(self, path):
-        # We must not use None here, because that is a valid value to store.
-        trans_id = self._path2trans_id_cache.get(path, object)
-        if trans_id is not object:
-            return trans_id
-        segments = splitpath(path)
-        cur_parent = self._transform.root
-        for cur_segment in segments:
-            for child in self._all_children(cur_parent):
-                final_name = self._final_name_cache.get(child)
-                if final_name is None:
-                    final_name = self._transform.final_name(child)
-                    self._final_name_cache[child] = final_name
-                if final_name == cur_segment:
-                    cur_parent = child
-                    break
-            else:
-                self._path2trans_id_cache[path] = None
-                return None
-        self._path2trans_id_cache[path] = cur_parent
-        return cur_parent
-
-    def path2id(self, path):
-        if isinstance(path, list):
-            if path == []:
-                path = [""]
-            path = osutils.pathjoin(*path)
-        return self._transform.final_file_id(self._path2trans_id(path))
-
-    def id2path(self, file_id, recurse='down'):
-        trans_id = self._transform.trans_id_file_id(file_id)
-        try:
-            return self._final_paths._determine_path(trans_id)
-        except NoFinalPath:
-            raise errors.NoSuchId(self, file_id)
-
-    def _all_children(self, trans_id):
-        children = self._all_children_cache.get(trans_id)
-        if children is not None:
-            return children
-        children = set(self._transform.iter_tree_children(trans_id))
-        # children in the _new_parent set are provided by _by_parent.
-        children.difference_update(self._transform._new_parent)
-        children.update(self._by_parent.get(trans_id, []))
-        self._all_children_cache[trans_id] = children
-        return children
-
-    def extras(self):
-        possible_extras = set(self._transform.trans_id_tree_path(p) for p
-                              in self._transform._tree.extras())
-        possible_extras.update(self._transform._new_contents)
-        possible_extras.update(self._transform._removed_id)
-        for trans_id in possible_extras:
-            if self._transform.final_file_id(trans_id) is None:
-                yield self._final_paths._determine_path(trans_id)
-
-    def _make_inv_entries(self, ordered_entries, specific_files=None):
-        for trans_id, parent_file_id in ordered_entries:
-            file_id = self._transform.final_file_id(trans_id)
-            if file_id is None:
-                continue
-            if (specific_files is not None
-                    and self._final_paths.get_path(trans_id) not in specific_files):
-                continue
-            kind = self._transform.final_kind(trans_id)
-            if kind is None:
-                kind = self._transform._tree.stored_kind(
-                    self._transform._tree.id2path(file_id))
-            new_entry = inventory.make_entry(
-                kind,
-                self._transform.final_name(trans_id),
-                parent_file_id, file_id)
-            yield new_entry, trans_id
-
-    def _list_files_by_dir(self):
-        todo = [ROOT_PARENT]
-        ordered_ids = []
-        while len(todo) > 0:
-            parent = todo.pop()
-            parent_file_id = self._transform.final_file_id(parent)
-            children = list(self._all_children(parent))
-            paths = dict(zip(children, self._final_paths.get_paths(children)))
-            children.sort(key=paths.get)
-            todo.extend(reversed(children))
-            for trans_id in children:
-                ordered_ids.append((trans_id, parent_file_id))
-        return ordered_ids
-
-    def iter_child_entries(self, path):
-        trans_id = self._path2trans_id(path)
-        if trans_id is None:
-            raise errors.NoSuchFile(path)
-        todo = [(child_trans_id, trans_id) for child_trans_id in
-                self._all_children(trans_id)]
-        for entry, trans_id in self._make_inv_entries(todo):
-            yield entry
-
-    def iter_entries_by_dir(self, specific_files=None, recurse_nested=False):
-        if recurse_nested:
-            raise NotImplementedError(
-                'follow tree references not yet supported')
-
-        # This may not be a maximally efficient implementation, but it is
-        # reasonably straightforward.  An implementation that grafts the
-        # TreeTransform changes onto the tree's iter_entries_by_dir results
-        # might be more efficient, but requires tricky inferences about stack
-        # position.
-        ordered_ids = self._list_files_by_dir()
-        for entry, trans_id in self._make_inv_entries(ordered_ids,
-                                                      specific_files):
-            yield self._final_paths.get_path(trans_id), entry
-
-    def _iter_entries_for_dir(self, dir_path):
-        """Return path, entry for items in a directory without recursing down."""
-        ordered_ids = []
-        dir_trans_id = self._path2trans_id(dir_path)
-        dir_id = self._transform.final_file_id(dir_trans_id)
-        for child_trans_id in self._all_children(dir_trans_id):
-            ordered_ids.append((child_trans_id, dir_id))
-        path_entries = []
-        for entry, trans_id in self._make_inv_entries(ordered_ids):
-            path_entries.append((self._final_paths.get_path(trans_id), entry))
-        path_entries.sort()
-        return path_entries
-
-    def list_files(self, include_root=False, from_dir=None, recursive=True,
-                   recurse_nested=False):
-        """See WorkingTree.list_files."""
-        if recurse_nested:
-            raise NotImplementedError(
-                'follow tree references not yet supported')
-
-        # XXX This should behave like WorkingTree.list_files, but is really
-        # more like RevisionTree.list_files.
-        if from_dir == '.':
-            from_dir = None
-        if recursive:
-            prefix = None
-            if from_dir:
-                prefix = from_dir + '/'
-            entries = self.iter_entries_by_dir()
-            for path, entry in entries:
-                if entry.name == '' and not include_root:
-                    continue
-                if prefix:
-                    if not path.startswith(prefix):
-                        continue
-                    path = path[len(prefix):]
-                yield path, 'V', entry.kind, entry
-        else:
-            if from_dir is None and include_root is True:
-                root_entry = inventory.make_entry(
-                    'directory', '', ROOT_PARENT, self.path2id(''))
-                yield '', 'V', 'directory', root_entry
-            entries = self._iter_entries_for_dir(from_dir or '')
-            for path, entry in entries:
-                yield path, 'V', entry.kind, entry
-
-    def kind(self, path):
-        trans_id = self._path2trans_id(path)
-        if trans_id is None:
-            raise errors.NoSuchFile(path)
-        return self._transform.final_kind(trans_id)
-
-    def stored_kind(self, path):
-        trans_id = self._path2trans_id(path)
-        if trans_id is None:
-            raise errors.NoSuchFile(path)
-        try:
-            return self._transform._new_contents[trans_id]
-        except KeyError:
-            return self._transform._tree.stored_kind(path)
-
-    def get_file_mtime(self, path):
-        """See Tree.get_file_mtime"""
-        file_id = self.path2id(path)
-        if file_id is None:
-            raise errors.NoSuchFile(path)
-        if not self._content_change(file_id):
-            return self._transform._tree.get_file_mtime(
-                self._transform._tree.id2path(file_id))
-        trans_id = self._path2trans_id(path)
-        return self._stat_limbo_file(trans_id).st_mtime
-
-    def get_file_size(self, path):
-        """See Tree.get_file_size"""
-        trans_id = self._path2trans_id(path)
-        if trans_id is None:
-            raise errors.NoSuchFile(path)
-        kind = self._transform.final_kind(trans_id)
-        if kind != 'file':
-            return None
-        if trans_id in self._transform._new_contents:
-            return self._stat_limbo_file(trans_id).st_size
-        if self.kind(path) == 'file':
-            return self._transform._tree.get_file_size(path)
-        else:
-            return None
-
-    def get_file_verifier(self, path, stat_value=None):
-        trans_id = self._path2trans_id(path)
-        if trans_id is None:
-            raise errors.NoSuchFile(path)
-        kind = self._transform._new_contents.get(trans_id)
-        if kind is None:
-            return self._transform._tree.get_file_verifier(path)
-        if kind == 'file':
-            with self.get_file(path) as fileobj:
-                return ("SHA1", sha_file(fileobj))
-
-    def get_file_sha1(self, path, stat_value=None):
-        trans_id = self._path2trans_id(path)
-        if trans_id is None:
-            raise errors.NoSuchFile(path)
-        kind = self._transform._new_contents.get(trans_id)
-        if kind is None:
-            return self._transform._tree.get_file_sha1(path)
-        if kind == 'file':
-            with self.get_file(path) as fileobj:
-                return sha_file(fileobj)
-
-    def get_reference_revision(self, path):
-        trans_id = self._path2trans_id(path)
-        if trans_id is None:
-            raise errors.NoSuchFile(path)
-        reference_revision = self._transform._new_reference_revision.get(trans_id)
-        if reference_revision is None:
-            return self._transform._tree.get_reference_revision(path)
-        return reference_revision
-
-    def is_executable(self, path):
-        trans_id = self._path2trans_id(path)
-        if trans_id is None:
-            return False
-        try:
-            return self._transform._new_executability[trans_id]
-        except KeyError:
-            try:
-                return self._transform._tree.is_executable(path)
-            except OSError as e:
-                if e.errno == errno.ENOENT:
-                    return False
-                raise
-            except errors.NoSuchFile:
-                return False
-
-    def has_filename(self, path):
-        trans_id = self._path2trans_id(path)
-        if trans_id in self._transform._new_contents:
-            return True
-        elif trans_id in self._transform._removed_contents:
-            return False
-        else:
-            return self._transform._tree.has_filename(path)
-
-    def path_content_summary(self, path):
-        trans_id = self._path2trans_id(path)
-        tt = self._transform
-        tree_path = tt._tree_id_paths.get(trans_id)
-        kind = tt._new_contents.get(trans_id)
-        if kind is None:
-            if tree_path is None or trans_id in tt._removed_contents:
-                return 'missing', None, None, None
-            summary = tt._tree.path_content_summary(tree_path)
-            kind, size, executable, link_or_sha1 = summary
-        else:
-            link_or_sha1 = None
-            limbo_name = tt._limbo_name(trans_id)
-            if trans_id in tt._new_reference_revision:
-                kind = 'tree-reference'
-            if kind == 'file':
-                statval = os.lstat(limbo_name)
-                size = statval.st_size
-                if not tt._limbo_supports_executable():
-                    executable = False
-                else:
-                    executable = statval.st_mode & S_IEXEC
-            else:
-                size = None
-                executable = None
-            if kind == 'symlink':
-                link_or_sha1 = os.readlink(limbo_name)
-                if not isinstance(link_or_sha1, str):
-                    link_or_sha1 = link_or_sha1.decode(osutils._fs_enc)
-        executable = tt._new_executability.get(trans_id, executable)
-        return kind, size, executable, link_or_sha1
-
-    def iter_changes(self, from_tree, include_unchanged=False,
-                     specific_files=None, pb=None, extra_trees=None,
-                     require_versioned=True, want_unversioned=False):
-        """See InterTree.iter_changes.
-
-        This has a fast path that is only used when the from_tree matches
-        the transform tree, and no fancy options are supplied.
-        """
-        if (from_tree is not self._transform._tree or include_unchanged
-                or specific_files or want_unversioned):
-            return tree.InterTree(from_tree, self).iter_changes(
-                include_unchanged=include_unchanged,
-                specific_files=specific_files,
-                pb=pb,
-                extra_trees=extra_trees,
-                require_versioned=require_versioned,
-                want_unversioned=want_unversioned)
-        if want_unversioned:
-            raise ValueError('want_unversioned is not supported')
-        return self._transform.iter_changes()
-
-    def get_file(self, path):
-        """See Tree.get_file"""
-        file_id = self.path2id(path)
-        if not self._content_change(file_id):
-            return self._transform._tree.get_file(path)
-        trans_id = self._path2trans_id(path)
-        name = self._transform._limbo_name(trans_id)
-        return open(name, 'rb')
-
-    def get_file_with_stat(self, path):
-        return self.get_file(path), None
-
-    def annotate_iter(self, path,
-                      default_revision=_mod_revision.CURRENT_REVISION):
-        file_id = self.path2id(path)
-        changes = self._iter_changes_cache.get(file_id)
-        if changes is None:
-            get_old = True
-        else:
-            changed_content, versioned, kind = (
-                changes.changed_content, changes.versioned, changes.kind)
-            if kind[1] is None:
-                return None
-            get_old = (kind[0] == 'file' and versioned[0])
-        if get_old:
-            old_annotation = self._transform._tree.annotate_iter(
-                path, default_revision=default_revision)
-        else:
-            old_annotation = []
-        if changes is None:
-            return old_annotation
-        if not changed_content:
-            return old_annotation
-        # TODO: This is doing something similar to what WT.annotate_iter is
-        #       doing, however it fails slightly because it doesn't know what
-        #       the *other* revision_id is, so it doesn't know how to give the
-        #       other as the origin for some lines, they all get
-        #       'default_revision'
-        #       It would be nice to be able to use the new Annotator based
-        #       approach, as well.
-        return annotate.reannotate([old_annotation],
-                                   self.get_file(path).readlines(),
-                                   default_revision)
-
-    def get_symlink_target(self, path):
-        """See Tree.get_symlink_target"""
-        file_id = self.path2id(path)
-        if not self._content_change(file_id):
-            return self._transform._tree.get_symlink_target(path)
-        trans_id = self._path2trans_id(path)
-        name = self._transform._limbo_name(trans_id)
-        return osutils.readlink(name)
-
-    def walkdirs(self, prefix=''):
-        pending = [self._transform.root]
-        while len(pending) > 0:
-            parent_id = pending.pop()
-            children = []
-            subdirs = []
-            prefix = prefix.rstrip('/')
-            parent_path = self._final_paths.get_path(parent_id)
-            parent_file_id = self._transform.final_file_id(parent_id)
-            for child_id in self._all_children(parent_id):
-                path_from_root = self._final_paths.get_path(child_id)
-                basename = self._transform.final_name(child_id)
-                file_id = self._transform.final_file_id(child_id)
-                kind = self._transform.final_kind(child_id)
-                if kind is not None:
-                    versioned_kind = kind
-                else:
-                    kind = 'unknown'
-                    versioned_kind = self._transform._tree.stored_kind(
-                        self._transform._tree.id2path(file_id))
-                if versioned_kind == 'directory':
-                    subdirs.append(child_id)
-                children.append((path_from_root, basename, kind, None,
-                                 file_id, versioned_kind))
-            children.sort()
-            if parent_path.startswith(prefix):
-                yield (parent_path, parent_file_id), children
-            pending.extend(sorted(subdirs, key=self._final_paths.get_path,
-                                  reverse=True))
-
-    def get_parent_ids(self):
-        return self._parent_ids
-
-    def set_parent_ids(self, parent_ids):
-        self._parent_ids = parent_ids
-
-    def get_revision_tree(self, revision_id):
-        return self._transform._tree.get_revision_tree(revision_id)
-
-
 def joinpath(parent, child):
     """Join tree-relative paths, handling the tree root specially"""
     if parent is None or parent == "":
@@ -2614,11 +690,11 @@
         if wt.path2id('') != tree.path2id(''):
             wt.set_root_id(tree.path2id(''))
             wt.flush()
-    tt = wt.get_transform()
+    tt = wt.transform()
     divert = set()
     try:
         pp.next_phase()
-        file_trans_id[wt.path2id('')] = tt.trans_id_tree_path('')
+        file_trans_id[find_previous_path(wt, tree, '')] = tt.trans_id_tree_path('')
         with ui.ui_factory.nested_progress_bar() as pb:
             deferred_contents = []
             num = 0
@@ -2655,31 +731,30 @@
                         except errors.NotBranchError:
                             pass
                         else:
-                            divert.add(file_id)
-                    if (file_id not in divert
+                            divert.add(tree_path)
+                    if (tree_path not in divert
                         and _content_match(
                             tree, entry, tree_path, kind, target_path)):
                         tt.delete_contents(tt.trans_id_tree_path(tree_path))
                         if kind == 'directory':
                             reparent = True
-                parent_id = file_trans_id[entry.parent_id]
+                parent_id = file_trans_id[osutils.dirname(tree_path)]
                 if entry.kind == 'file':
                     # We *almost* replicate new_by_entry, so that we can defer
                     # getting the file text, and get them all at once.
                     trans_id = tt.create_path(entry.name, parent_id)
-                    file_trans_id[file_id] = trans_id
-                    tt.version_file(file_id, trans_id)
+                    file_trans_id[tree_path] = trans_id
+                    tt.version_file(trans_id, file_id=file_id)
                     executable = tree.is_executable(tree_path)
                     if executable:
                         tt.set_executability(executable, trans_id)
-                    trans_data = (trans_id, file_id,
-                                  tree_path, entry.text_sha1)
+                    trans_data = (trans_id, tree_path, entry.text_sha1)
                     deferred_contents.append((tree_path, trans_data))
                 else:
-                    file_trans_id[file_id] = new_by_entry(
+                    file_trans_id[tree_path] = new_by_entry(
                         tree_path, tt, entry, parent_id, tree)
                 if reparent:
-                    new_trans_id = file_trans_id[file_id]
+                    new_trans_id = file_trans_id[tree_path]
                     old_parent = tt.trans_id_tree_path(tree_path)
                     _reparent_children(tt, old_parent, new_trans_id)
             offset = num + 1 - len(deferred_contents)
@@ -2725,11 +800,11 @@
         unchanged = dict(unchanged)
         new_desired_files = []
         count = 0
-        for unused_tree_path, (trans_id, file_id, tree_path, text_sha1) in desired_files:
+        for unused_tree_path, (trans_id, tree_path, text_sha1) in desired_files:
             accelerator_path = unchanged.get(tree_path)
             if accelerator_path is None:
                 new_desired_files.append((tree_path,
-                                          (trans_id, file_id, tree_path, text_sha1)))
+                                          (trans_id, tree_path, text_sha1)))
                 continue
             pb.update(gettext('Adding file contents'), count + offset, total)
             if hardlink:
@@ -2745,7 +820,7 @@
                     tt.create_file(chunks, trans_id, sha1=text_sha1)
             count += 1
         offset += count
-    for count, ((trans_id, file_id, tree_path, text_sha1), contents) in enumerate(
+    for count, ((trans_id, tree_path, text_sha1), contents) in enumerate(
             tree.iter_files_bytes(new_desired_files)):
         if wt.supports_content_filtering():
             filters = wt._content_filter_stack(tree_path)
@@ -2880,11 +955,12 @@
     """Revert a working tree's contents to those of a target tree."""
     pb = ui.ui_factory.nested_progress_bar()
     try:
-        with target_tree.lock_read(), working_tree.get_transform(pb) as tt:
+        with target_tree.lock_read(), working_tree.transform(pb) as tt:
             pp = ProgressPhase("Revert phase", 3, pb)
             conflicts, merge_modified = _prepare_revert_transform(
                 working_tree, target_tree, tt, filenames, backups, pp)
             if change_reporter:
+                from . import delta
                 change_reporter = delta._ChangeReporter(
                     unversioned_filter=working_tree.is_ignored)
                 delta.report_changes(tt.iter_changes(), change_reporter)
@@ -2973,7 +1049,7 @@
                         new_trans_id = tt.create_path(wt_name, parent_trans_id)
                         if wt_versioned and target_versioned:
                             tt.unversion_file(trans_id)
-                            tt.version_file(file_id, new_trans_id)
+                            tt.version_file(new_trans_id, file_id=file_id)
                         # New contents should have the same unix perms as old
                         # contents
                         mode_id = trans_id
@@ -3011,7 +1087,7 @@
                 elif target_kind is not None:
                     raise AssertionError(target_kind)
             if not wt_versioned and target_versioned:
-                tt.version_file(file_id, trans_id)
+                tt.version_file(trans_id, file_id=file_id)
             if wt_versioned and not target_versioned:
                 tt.unversion_file(trans_id)
             if (target_name is not None
@@ -3156,7 +1232,7 @@
             if path_tree and path_tree.path2id('') == file_id:
                 # This is the root entry, skip it
                 continue
-            tt.version_file(file_id, conflict[1])
+            tt.version_file(conflict[1], file_id=file_id)
             new_conflicts.add((c_type, 'Versioned directory', conflict[1]))
         elif c_type == 'non-directory parent':
             parent_id = conflict[1]
@@ -3217,7 +1293,7 @@
                 raise errors.FileExists(to, str(e))
             # normal OSError doesn't include filenames so it's hard to see where
             # the problem is, see https://bugs.launchpad.net/bzr/+bug/491763
-            raise errors.TransformRenameFailed(from_, to, str(e), e.errno)
+            raise TransformRenameFailed(from_, to, str(e), e.errno)
         self.past_renames.append((from_, to))
 
     def pre_delete(self, from_, to):
@@ -3236,7 +1312,7 @@
             try:
                 os.rename(to, from_)
             except OSError as e:
-                raise errors.TransformRenameFailed(to, from_, str(e), e.errno)
+                raise TransformRenameFailed(to, from_, str(e), e.errno)
         # after rollback, don't reuse _FileMover
         self.past_renames = None
         self.pending_deletions = None
@@ -3256,7 +1332,7 @@
     :param target_tree: Tree to change
     :param source_tree: Tree to hard-link from
     """
-    with target_tree.get_transform() as tt:
+    with target_tree.transform() as tt:
         for change in target_tree.iter_changes(source_tree, include_unchanged=True):
             if change.changed_content:
                 continue
@@ -3268,3 +1344,184 @@
             tt.delete_contents(trans_id)
             tt.create_hardlink(source_tree.abspath(change.path[0]), trans_id)
         tt.apply()
+
+
+class PreviewTree(object):
+    """Preview tree."""
+
+    def __init__(self, transform):
+        self._transform = transform
+        self._parent_ids = []
+        self.__by_parent = None
+        self._path2trans_id_cache = {}
+        self._all_children_cache = {}
+        self._final_name_cache = {}
+
+    @property
+    def _by_parent(self):
+        if self.__by_parent is None:
+            self.__by_parent = self._transform.by_parent()
+        return self.__by_parent
+
+    def get_parent_ids(self):
+        return self._parent_ids
+
+    def set_parent_ids(self, parent_ids):
+        self._parent_ids = parent_ids
+
+    def get_revision_tree(self, revision_id):
+        return self._transform._tree.get_revision_tree(revision_id)
+
+    def is_locked(self):
+        return False
+
+    def lock_read(self):
+        # Perhaps in theory, this should lock the TreeTransform?
+        return lock.LogicalLockResult(self.unlock)
+
+    def unlock(self):
+        pass
+
+    def _path2trans_id(self, path):
+        # We must not use None here, because that is a valid value to store.
+        trans_id = self._path2trans_id_cache.get(path, object)
+        if trans_id is not object:
+            return trans_id
+        segments = osutils.splitpath(path)
+        cur_parent = self._transform.root
+        for cur_segment in segments:
+            for child in self._all_children(cur_parent):
+                final_name = self._final_name_cache.get(child)
+                if final_name is None:
+                    final_name = self._transform.final_name(child)
+                    self._final_name_cache[child] = final_name
+                if final_name == cur_segment:
+                    cur_parent = child
+                    break
+            else:
+                self._path2trans_id_cache[path] = None
+                return None
+        self._path2trans_id_cache[path] = cur_parent
+        return cur_parent
+
+    def _all_children(self, trans_id):
+        children = self._all_children_cache.get(trans_id)
+        if children is not None:
+            return children
+        children = set(self._transform.iter_tree_children(trans_id))
+        # children in the _new_parent set are provided by _by_parent.
+        children.difference_update(self._transform._new_parent)
+        children.update(self._by_parent.get(trans_id, []))
+        self._all_children_cache[trans_id] = children
+        return children
+
+    def get_file_with_stat(self, path):
+        return self.get_file(path), None
+
+    def is_executable(self, path):
+        trans_id = self._path2trans_id(path)
+        if trans_id is None:
+            return False
+        try:
+            return self._transform._new_executability[trans_id]
+        except KeyError:
+            try:
+                return self._transform._tree.is_executable(path)
+            except OSError as e:
+                if e.errno == errno.ENOENT:
+                    return False
+                raise
+            except errors.NoSuchFile:
+                return False
+
+    def has_filename(self, path):
+        trans_id = self._path2trans_id(path)
+        if trans_id in self._transform._new_contents:
+            return True
+        elif trans_id in self._transform._removed_contents:
+            return False
+        else:
+            return self._transform._tree.has_filename(path)
+
+    def get_file_sha1(self, path, stat_value=None):
+        trans_id = self._path2trans_id(path)
+        if trans_id is None:
+            raise errors.NoSuchFile(path)
+        kind = self._transform._new_contents.get(trans_id)
+        if kind is None:
+            return self._transform._tree.get_file_sha1(path)
+        if kind == 'file':
+            with self.get_file(path) as fileobj:
+                return osutils.sha_file(fileobj)
+
+    def get_file_verifier(self, path, stat_value=None):
+        trans_id = self._path2trans_id(path)
+        if trans_id is None:
+            raise errors.NoSuchFile(path)
+        kind = self._transform._new_contents.get(trans_id)
+        if kind is None:
+            return self._transform._tree.get_file_verifier(path)
+        if kind == 'file':
+            with self.get_file(path) as fileobj:
+                return ("SHA1", osutils.sha_file(fileobj))
+
+    def kind(self, path):
+        trans_id = self._path2trans_id(path)
+        if trans_id is None:
+            raise errors.NoSuchFile(path)
+        return self._transform.final_kind(trans_id)
+
+    def stored_kind(self, path):
+        trans_id = self._path2trans_id(path)
+        if trans_id is None:
+            raise errors.NoSuchFile(path)
+        try:
+            return self._transform._new_contents[trans_id]
+        except KeyError:
+            return self._transform._tree.stored_kind(path)
+
+    def _get_repository(self):
+        repo = getattr(self._transform._tree, '_repository', None)
+        if repo is None:
+            repo = self._transform._tree.branch.repository
+        return repo
+
+    def _iter_parent_trees(self):
+        for revision_id in self.get_parent_ids():
+            try:
+                yield self.revision_tree(revision_id)
+            except errors.NoSuchRevisionInTree:
+                yield self._get_repository().revision_tree(revision_id)
+
+    def get_file_size(self, path):
+        """See Tree.get_file_size"""
+        trans_id = self._path2trans_id(path)
+        if trans_id is None:
+            raise errors.NoSuchFile(path)
+        kind = self._transform.final_kind(trans_id)
+        if kind != 'file':
+            return None
+        if trans_id in self._transform._new_contents:
+            return self._stat_limbo_file(trans_id).st_size
+        if self.kind(path) == 'file':
+            return self._transform._tree.get_file_size(path)
+        else:
+            return None
+
+    def get_reference_revision(self, path):
+        trans_id = self._path2trans_id(path)
+        if trans_id is None:
+            raise errors.NoSuchFile(path)
+        reference_revision = self._transform._new_reference_revision.get(trans_id)
+        if reference_revision is None:
+            return self._transform._tree.get_reference_revision(path)
+        return reference_revision
+
+    def tree_kind(self, trans_id):
+        path = self._tree_id_paths.get(trans_id)
+        if path is None:
+            return None
+        kind = self._tree.path_content_summary(path)[0]
+        if kind == 'missing':
+            kind = None
+        return kind

=== modified file 'breezy/transport/http/__init__.py'
--- breezy/transport/http/__init__.py	2020-06-10 23:47:24 +0000
+++ breezy/transport/http/__init__.py	2020-07-18 23:14:00 +0000
@@ -227,7 +227,7 @@
     """
 
     # Some responses have bodies in which we have no interest
-    _body_ignored_responses = [301, 302, 303, 307, 400, 401, 403, 404, 501]
+    _body_ignored_responses = [301, 302, 303, 307, 308, 400, 401, 403, 404, 501]
 
     # in finish() below, we may have to discard several MB in the worst
     # case. To avoid buffering that much, we read and discard by chunks
@@ -879,7 +879,7 @@
 
         origin_req_host = req.origin_req_host
 
-        if code in (301, 302, 303, 307):
+        if code in (301, 302, 303, 307, 308):
             return Request(req.get_method(), newurl,
                            headers=req.headers,
                            origin_req_host=origin_req_host,
@@ -950,7 +950,7 @@
 
         return self.parent.open(redirected_req)
 
-    http_error_301 = http_error_303 = http_error_307 = http_error_302
+    http_error_301 = http_error_303 = http_error_307 = http_error_308 = http_error_302
 
 
 class ProxyHandler(urllib_request.ProxyHandler):
@@ -1860,10 +1860,10 @@
 
         code = response.code
         if (request.follow_redirections is False
-                and code in (301, 302, 303, 307)):
+                and code in (301, 302, 303, 307, 308)):
             raise errors.RedirectRequested(request.get_full_url(),
                                            request.redirected_to,
-                                           is_permanent=(code == 301))
+                                           is_permanent=(code in (301, 308)))
 
         if request.redirected_to is not None:
             trace.mutter('redirected from: %s to: %s' % (request.get_full_url(),
@@ -1982,11 +1982,9 @@
                     abspath, range_header,
                     'Server return code %d' % response.status)
             else:
-                raise errors.InvalidHttpResponse(
-                    abspath, 'Unexpected status %d' % response.status)
+                raise errors.BadHttpRequest(abspath, response.reason)
         elif response.status not in (200, 206):
-            raise errors.InvalidHttpResponse(
-                abspath, 'Unexpected status %d' % response.status)
+            raise errors.UnexpectedHttpStatus(abspath, response.status)
 
         data = handle_response(
             abspath, response.status, response.getheader, response)
@@ -2217,8 +2215,7 @@
             'POST', abspath, body=body_bytes,
             headers={'Content-Type': 'application/octet-stream'})
         if response.status not in (200, 403):
-            raise errors.InvalidHttpResponse(
-                abspath, 'Unexpected status %d' % response.status)
+            raise errors.UnexpectedHttpStatus(abspath, response.status)
         code = response.status
         data = handle_response(
             abspath, code, response.getheader, response)
@@ -2232,8 +2229,7 @@
         abspath = self._remote_path(relpath)
         response = self.request('HEAD', abspath)
         if response.status not in (200, 404):
-            raise errors.InvalidHttpResponse(
-                abspath, 'Unexpected status %d' % response.status)
+            raise errors.UnexpectedHttpStatus(abspath, response.status)
 
         return response
 
@@ -2499,9 +2495,8 @@
             t = self._http_transport_ref()
             code, body_filelike = t._post(bytes)
             if code != 200:
-                raise errors.InvalidHttpResponse(
-                    t._remote_path('.bzr/smart'),
-                    'Expected 200 response code, got %r' % (code,))
+                raise errors.UnexpectedHttpStatus(
+                    t._remote_path('.bzr/smart'), code)
         except (errors.InvalidHttpResponse, errors.ConnectionReset) as e:
             raise errors.SmartProtocolError(str(e))
         return body_filelike

=== modified file 'breezy/transport/http/response.py'
--- breezy/transport/http/response.py	2020-06-10 23:47:24 +0000
+++ breezy/transport/http/response.py	2020-07-18 23:14:00 +0000
@@ -394,7 +394,6 @@
                     url, 'Missing the Content-Range header in a 206 range response')
             rfile.set_range_from_header(content_range)
     else:
-        raise errors.InvalidHttpResponse(url,
-                                         'Unknown response code %s' % code)
+        raise errors.UnexpectedHttpStatus(url, code)
 
     return rfile

=== modified file 'breezy/tree.py'
--- breezy/tree.py	2020-02-18 01:57:45 +0000
+++ breezy/tree.py	2020-07-18 23:14:00 +0000
@@ -17,23 +17,8 @@
 """Tree classes, representing directory at point in time.
 """
 
-try:
-    from collections.abc import deque
-except ImportError:  # python < 3.7
-    from collections import deque
-
 from .lazy_import import lazy_import
 lazy_import(globals(), """
-
-from breezy import (
-    conflicts as _mod_conflicts,
-    debug,
-    delta,
-    filters,
-    revision as _mod_revision,
-    rules,
-    trace,
-    )
 from breezy.i18n import gettext
 """)
 
@@ -41,6 +26,8 @@
     errors,
     lock,
     osutils,
+    revision as _mod_revision,
+    trace,
     )
 from .inter import InterObject
 
@@ -269,6 +256,7 @@
 
         Each conflict is an instance of breezy.conflicts.Conflict.
         """
+        from . import conflicts as _mod_conflicts
         return _mod_conflicts.ConflictList()
 
     def extras(self):
@@ -688,6 +676,7 @@
             or None if unknown
         :return: the list of filters - [] if there are none
         """
+        from . import debug, filters
         filter_pref_names = filters._get_registered_names()
         if len(filter_pref_names) == 0:
             return []
@@ -722,6 +711,7 @@
         :return: an iterator of tuple sequences, one per path-name.
           See _RulesSearcher.get_items for details on the tuple sequence.
         """
+        from . import rules
         if _default_searcher is None:
             _default_searcher = rules._per_user_searcher
         searcher = self._get_rules_searcher(_default_searcher)
@@ -758,6 +748,10 @@
         """Check if this tree support versioning a specific file kind."""
         return (kind in ('file', 'directory', 'symlink', 'tree-reference'))
 
+    def preview_transform(self, pb=None):
+        """Obtain a transform object."""
+        raise NotImplementedError(self.preview_transform)
+
 
 class InterTree(InterObject):
     """This class represents operations taking place between two Trees.
@@ -784,78 +778,6 @@
         # it works for all trees.
         return True
 
-    def _changes_from_entries(self, source_entry, target_entry, source_path,
-                              target_path):
-        """Generate a iter_changes tuple between source_entry and target_entry.
-
-        :param source_entry: An inventory entry from self.source, or None.
-        :param target_entry: An inventory entry from self.target, or None.
-        :param source_path: The path of source_entry.
-        :param target_path: The path of target_entry.
-        :return: A tuple, item 0 of which is an iter_changes result tuple, and
-            item 1 is True if there are any changes in the result tuple.
-        """
-        if source_entry is None:
-            if target_entry is None:
-                return None
-            file_id = target_entry.file_id
-        else:
-            file_id = source_entry.file_id
-        if source_entry is not None:
-            source_versioned = True
-            source_name = source_entry.name
-            source_parent = source_entry.parent_id
-            source_kind, source_executable, source_stat = \
-                self.source._comparison_data(source_entry, source_path)
-        else:
-            source_versioned = False
-            source_name = None
-            source_parent = None
-            source_kind = None
-            source_executable = None
-        if target_entry is not None:
-            target_versioned = True
-            target_name = target_entry.name
-            target_parent = target_entry.parent_id
-            target_kind, target_executable, target_stat = \
-                self.target._comparison_data(target_entry, target_path)
-        else:
-            target_versioned = False
-            target_name = None
-            target_parent = None
-            target_kind = None
-            target_executable = None
-        versioned = (source_versioned, target_versioned)
-        kind = (source_kind, target_kind)
-        changed_content = False
-        if source_kind != target_kind:
-            changed_content = True
-        elif source_kind == 'file':
-            if not self.file_content_matches(
-                    source_path, target_path,
-                    source_stat, target_stat):
-                changed_content = True
-        elif source_kind == 'symlink':
-            if (self.source.get_symlink_target(source_path) !=
-                    self.target.get_symlink_target(target_path)):
-                changed_content = True
-        elif source_kind == 'tree-reference':
-            if (self.source.get_reference_revision(source_path)
-                    != self.target.get_reference_revision(target_path)):
-                changed_content = True
-        parent = (source_parent, target_parent)
-        name = (source_name, target_name)
-        executable = (source_executable, target_executable)
-        if (changed_content is not False or versioned[0] != versioned[1] or
-            parent[0] != parent[1] or name[0] != name[1] or
-                executable[0] != executable[1]):
-            changes = True
-        else:
-            changes = False
-        return TreeChange(
-            file_id, (source_path, target_path), changed_content,
-            versioned, parent, name, kind, executable), changes
-
     def compare(self, want_unchanged=False, specific_files=None,
                 extra_trees=None, require_versioned=False, include_root=False,
                 want_unversioned=False):
@@ -875,6 +797,7 @@
             a PathsNotVersionedError will be thrown.
         :param want_unversioned: Scan for unversioned paths.
         """
+        from . import delta
         trees = (self.source,)
         if extra_trees is not None:
             trees = trees + tuple(extra_trees)
@@ -920,237 +843,7 @@
             output. An unversioned file is defined as one with (False, False)
             for the versioned pair.
         """
-        if not extra_trees:
-            extra_trees = []
-        else:
-            extra_trees = list(extra_trees)
-        # The ids of items we need to examine to insure delta consistency.
-        precise_file_ids = set()
-        changed_file_ids = []
-        if specific_files == []:
-            target_specific_files = []
-            source_specific_files = []
-        else:
-            target_specific_files = self.target.find_related_paths_across_trees(
-                specific_files, [self.source] + extra_trees,
-                require_versioned=require_versioned)
-            source_specific_files = self.source.find_related_paths_across_trees(
-                specific_files, [self.target] + extra_trees,
-                require_versioned=require_versioned)
-        if specific_files is not None:
-            # reparented or added entries must have their parents included
-            # so that valid deltas can be created. The seen_parents set
-            # tracks the parents that we need to have.
-            # The seen_dirs set tracks directory entries we've yielded.
-            # After outputting version object in to_entries we set difference
-            # the two seen sets and start checking parents.
-            seen_parents = set()
-            seen_dirs = set()
-        if want_unversioned:
-            all_unversioned = sorted([(p.split('/'), p) for p in
-                                      self.target.extras()
-                                      if specific_files is None or
-                                      osutils.is_inside_any(specific_files, p)])
-            all_unversioned = deque(all_unversioned)
-        else:
-            all_unversioned = deque()
-        to_paths = {}
-        from_entries_by_dir = list(self.source.iter_entries_by_dir(
-            specific_files=source_specific_files))
-        from_data = dict(from_entries_by_dir)
-        to_entries_by_dir = list(self.target.iter_entries_by_dir(
-            specific_files=target_specific_files))
-        path_equivs = self.find_source_paths([p for p, e in to_entries_by_dir])
-        num_entries = len(from_entries_by_dir) + len(to_entries_by_dir)
-        entry_count = 0
-        # the unversioned path lookup only occurs on real trees - where there
-        # can be extras. So the fake_entry is solely used to look up
-        # executable it values when execute is not supported.
-        fake_entry = TreeFile()
-        for target_path, target_entry in to_entries_by_dir:
-            while (all_unversioned and
-                   all_unversioned[0][0] < target_path.split('/')):
-                unversioned_path = all_unversioned.popleft()
-                target_kind, target_executable, target_stat = \
-                    self.target._comparison_data(
-                        fake_entry, unversioned_path[1])
-                yield TreeChange(
-                    None, (None, unversioned_path[1]), True, (False, False),
-                    (None, None),
-                    (None, unversioned_path[0][-1]),
-                    (None, target_kind),
-                    (None, target_executable))
-            source_path = path_equivs[target_path]
-            if source_path is not None:
-                source_entry = from_data.get(source_path)
-            else:
-                source_entry = None
-            result, changes = self._changes_from_entries(
-                source_entry, target_entry, source_path=source_path, target_path=target_path)
-            to_paths[result.file_id] = result.path[1]
-            entry_count += 1
-            if result.versioned[0]:
-                entry_count += 1
-            if pb is not None:
-                pb.update('comparing files', entry_count, num_entries)
-            if changes or include_unchanged:
-                if specific_files is not None:
-                    precise_file_ids.add(result.parent_id[1])
-                    changed_file_ids.append(result.file_id)
-                yield result
-            # Ensure correct behaviour for reparented/added specific files.
-            if specific_files is not None:
-                # Record output dirs
-                if result.kind[1] == 'directory':
-                    seen_dirs.add(result.file_id)
-                # Record parents of reparented/added entries.
-                if not result.versioned[0] or result.is_reparented():
-                    seen_parents.add(result.parent_id[1])
-        while all_unversioned:
-            # yield any trailing unversioned paths
-            unversioned_path = all_unversioned.popleft()
-            to_kind, to_executable, to_stat = \
-                self.target._comparison_data(fake_entry, unversioned_path[1])
-            yield TreeChange(
-                None, (None, unversioned_path[1]), True, (False, False),
-                (None, None),
-                (None, unversioned_path[0][-1]),
-                (None, to_kind),
-                (None, to_executable))
-        # Yield all remaining source paths
-        for path, from_entry in from_entries_by_dir:
-            file_id = from_entry.file_id
-            if file_id in to_paths:
-                # already returned
-                continue
-            to_path = self.find_target_path(path)
-            entry_count += 1
-            if pb is not None:
-                pb.update('comparing files', entry_count, num_entries)
-            versioned = (True, False)
-            parent = (from_entry.parent_id, None)
-            name = (from_entry.name, None)
-            from_kind, from_executable, stat_value = \
-                self.source._comparison_data(from_entry, path)
-            kind = (from_kind, None)
-            executable = (from_executable, None)
-            changed_content = from_kind is not None
-            # the parent's path is necessarily known at this point.
-            changed_file_ids.append(file_id)
-            yield TreeChange(
-                file_id, (path, to_path), changed_content, versioned, parent,
-                name, kind, executable)
-        changed_file_ids = set(changed_file_ids)
-        if specific_files is not None:
-            for result in self._handle_precise_ids(precise_file_ids,
-                                                   changed_file_ids):
-                yield result
-
-    @staticmethod
-    def _get_entry(tree, path):
-        """Get an inventory entry from a tree, with missing entries as None.
-
-        If the tree raises NotImplementedError on accessing .inventory, then
-        this is worked around using iter_entries_by_dir on just the file id
-        desired.
-
-        :param tree: The tree to lookup the entry in.
-        :param path: The path to look up
-        """
-        # No inventory available.
-        try:
-            iterator = tree.iter_entries_by_dir(specific_files=[path])
-            return next(iterator)[1]
-        except StopIteration:
-            return None
-
-    def _handle_precise_ids(self, precise_file_ids, changed_file_ids,
-                            discarded_changes=None):
-        """Fill out a partial iter_changes to be consistent.
-
-        :param precise_file_ids: The file ids of parents that were seen during
-            the iter_changes.
-        :param changed_file_ids: The file ids of already emitted items.
-        :param discarded_changes: An optional dict of precalculated
-            iter_changes items which the partial iter_changes had not output
-            but had calculated.
-        :return: A generator of iter_changes items to output.
-        """
-        # process parents of things that had changed under the users
-        # requested paths to prevent incorrect paths or parent ids which
-        # aren't in the tree.
-        while precise_file_ids:
-            precise_file_ids.discard(None)
-            # Don't emit file_ids twice
-            precise_file_ids.difference_update(changed_file_ids)
-            if not precise_file_ids:
-                break
-            # If the there was something at a given output path in source, we
-            # have to include the entry from source in the delta, or we would
-            # be putting this entry into a used path.
-            paths = []
-            for parent_id in precise_file_ids:
-                try:
-                    paths.append(self.target.id2path(parent_id))
-                except errors.NoSuchId:
-                    # This id has been dragged in from the source by delta
-                    # expansion and isn't present in target at all: we don't
-                    # need to check for path collisions on it.
-                    pass
-            for path in paths:
-                old_id = self.source.path2id(path)
-                precise_file_ids.add(old_id)
-            precise_file_ids.discard(None)
-            current_ids = precise_file_ids
-            precise_file_ids = set()
-            # We have to emit all of precise_file_ids that have been altered.
-            # We may have to output the children of some of those ids if any
-            # directories have stopped being directories.
-            for file_id in current_ids:
-                # Examine file_id
-                if discarded_changes:
-                    result = discarded_changes.get(file_id)
-                    source_entry = None
-                else:
-                    result = None
-                if result is None:
-                    try:
-                        source_path = self.source.id2path(file_id)
-                    except errors.NoSuchId:
-                        source_path = None
-                        source_entry = None
-                    else:
-                        source_entry = self._get_entry(
-                            self.source, source_path)
-                    try:
-                        target_path = self.target.id2path(file_id)
-                    except errors.NoSuchId:
-                        target_path = None
-                        target_entry = None
-                    else:
-                        target_entry = self._get_entry(
-                            self.target, target_path)
-                    result, changes = self._changes_from_entries(
-                        source_entry, target_entry, source_path, target_path)
-                else:
-                    changes = True
-                # Get this parents parent to examine.
-                new_parent_id = result.parent_id[1]
-                precise_file_ids.add(new_parent_id)
-                if changes:
-                    if (result.kind[0] == 'directory' and
-                            result.kind[1] != 'directory'):
-                        # This stopped being a directory, the old children have
-                        # to be included.
-                        if source_entry is None:
-                            # Reusing a discarded change.
-                            source_entry = self._get_entry(
-                                self.source, result.path[0])
-                        precise_file_ids.update(
-                            child.file_id
-                            for child in self.source.iter_child_entries(result.path[0]))
-                    changed_file_ids.add(result.file_id)
-                    yield result
+        raise NotImplementedError(self.iter_changes)
 
     def file_content_matches(
             self, source_path, target_path,
@@ -1194,13 +887,7 @@
         :return: path in target, or None if there is no equivalent path.
         :raise NoSuchFile: If the path doesn't exist in source
         """
-        file_id = self.source.path2id(path)
-        if file_id is None:
-            raise errors.NoSuchFile(path)
-        try:
-            return self.target.id2path(file_id, recurse=recurse)
-        except errors.NoSuchId:
-            return None
+        raise NotImplementedError(self.find_target_path)
 
     def find_source_path(self, path, recurse='none'):
         """Find the source tree path.
@@ -1209,13 +896,7 @@
         :return: path in source, or None if there is no equivalent path.
         :raise NoSuchFile: if the path doesn't exist in target
         """
-        file_id = self.target.path2id(path)
-        if file_id is None:
-            raise errors.NoSuchFile(path)
-        try:
-            return self.source.id2path(file_id, recurse=recurse)
-        except errors.NoSuchId:
-            return None
+        raise NotImplementedError(self.find_source_path)
 
     def find_target_paths(self, paths, recurse='none'):
         """Find target tree paths.
@@ -1242,9 +923,6 @@
         return ret
 
 
-InterTree.register_optimiser(InterTree)
-
-
 def find_previous_paths(from_tree, to_tree, paths, recurse='none'):
     """Find previous tree paths.
 

=== modified file 'breezy/upstream_import.py'
--- breezy/upstream_import.py	2020-06-23 01:02:30 +0000
+++ breezy/upstream_import.py	2020-07-18 23:14:00 +0000
@@ -227,7 +227,7 @@
 
 
 def import_archive(tree, archive_file):
-    with tree.get_transform() as tt:
+    with tree.transform() as tt:
         import_archive_to_transform(tree, archive_file, tt)
         tt.apply()
 
@@ -285,7 +285,7 @@
         if tt.tree_file_id(trans_id) is None:
             name = basename(member.name.rstrip('/'))
             file_id = generate_ids.gen_file_id(name)
-            tt.version_file(file_id, trans_id)
+            tt.version_file(trans_id, file_id=file_id)
 
     for relative_path in implied_parents.difference(added):
         if relative_path == "":
@@ -294,7 +294,7 @@
         path = tree.abspath(relative_path)
         do_directory(tt, trans_id, tree, relative_path, path)
         if tt.tree_file_id(trans_id) is None:
-            tt.version_file(trans_id, trans_id)
+            tt.version_file(trans_id, file_id=trans_id)
         added.add(relative_path)
 
     for path in removed.difference(added):

=== modified file 'breezy/workingtree.py'
--- breezy/workingtree.py	2020-02-21 03:58:42 +0000
+++ breezy/workingtree.py	2020-07-18 23:14:00 +0000
@@ -1049,7 +1049,6 @@
             if file_id is None:
                 raise ValueError(
                     'WorkingTree.set_root_id with fileid=None')
-            file_id = osutils.safe_file_id(file_id)
             self._set_root_id(file_id)
 
     def _set_root_id(self, file_id):

=== modified file 'doc/en/release-notes/brz-3.1.txt'
--- doc/en/release-notes/brz-3.1.txt	2020-06-22 10:25:04 +0000
+++ doc/en/release-notes/brz-3.1.txt	2020-07-18 19:28:51 +0000
@@ -38,6 +38,14 @@
  * A new method ``MergeProposal.get_source_revision`` has been added.
    (Jelmer Vernooĳ)
 
+ * Add support for HTTP response code 308. (Jelmer Vernooĳ)
+
+ * Handle non-utf8 characters in Git repositories by surrogate escaping
+   them.  (Jelmer Vernooĳ, #1489872)
+
+ * ``BzrDir.sprout`` now correctly handles the ``revision_id``
+   argument when ``source_branch`` is None. (Jelmer Vernooĳ)
+
 Bug Fixes
 *********
 
@@ -47,6 +55,9 @@
  * Don't require ``ctypes.pythonapi`` to exist, as it's missing on newer
    versions of Pypy3. (Jelmer Vernooĳ)
 
+ * Prevent exceptions during commit when quilt is not installed but
+   the tree contains quilt patches. (Jelmer Vernooĳ)
+
 Documentation
 *************
 
@@ -61,6 +72,9 @@
  * ``BzrCommandError`` has been renamed to ``CommandError``.
    (Jelmer Vernooĳ)
 
+ * The ``osutils.safe_file_id`` and ``osutils.safe_revision_id`` methods
+   have been removed. (Jelmer Vernooĳ)
+
 Internals
 *********
 

